/*
$Header: /vol/baal-vol3/projekt98/quellen/XCTL_32/INCLUDE/DETECUSE/DETECGUI.H,v 1.21 2004/07/06 08:26:09 reinecke Exp $

Projekt   : XCTL
Subsystem : Detektoren
Autor     : Jan Picard <picard@informatik.hu-berlin.de> 2001-2002
          Institut fuer Informatik,
          Humboldt-Universitaet Berlin
Inhalt    : Interface-Deklaration fuer die Oberflaechenklassen des
          Detektoren-Subsystems
*/

#ifndef _DETECGUI_H
#define _DETECGUI_H

#include "utils\u_utils.h"
#include "datavisa\datavisa.h" // fr TCurve
#include "detecuse\detecmes.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// TCommonDevParamDlg
//##############################################################################

// Dialog fr die allgemeinen Einstellungen der Me-Einheiten
class TCommonDevParamDlg : public TModalDlg
{
public:
	TCommonDevParamDlg( TDetector* = 0);

private:
	HWND hChooseDetector;
	TDetector* Detector;
	BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	void Dlg_OnCommand( HWND, int, HWND, UINT );
	BOOL CanClose( void );
	void LeaveDialog( void );

	// event handler
	void OnParamSet();
	void OnShowCounter(HWND hwnd);
	void OnActivateChanges();
	void OnSoundSwitch();
	void OnSpecificParameters();
	void OnStaticFailure(HWND hwnd);
	void OnChooseDetector(HWND hwnd, UINT codeNotify);
};

//##############################################################################
// TPsdParametersDlg
//##############################################################################

// Dialog fr die Einstellungen des Psd 
class TPsdParametersDlg : public TModelessDlg
{
public:
	TPsdParametersDlg( TModelessDlg** );

	virtual BOOL Initialize ( HINSTANCE, HWND );

protected:
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	virtual void Dlg_OnCommand( HWND, int, HWND, UINT );

// event handler und Hilfemethoden zum Auswerten/ Aktualisieren des Bildschirms
private:
	void OnInitializePsd(HWND);
	void PSD2Screen(BOOL, BOOL, BOOL);
	bool Screen2PSD(BOOL, BOOL, BOOL);

private:
	TOneDimDetector* Detector;
	HWND m_Parent; // an dieses Fensterhandle wird beim 'Setzen'-Button eine Nachricht zum Update geschickt
	int OldId;
};

//##############################################################################
// TScsParametersDlg
//##############################################################################

// Parameter-Dialog
class TScsParametersDlg : public TModalDlg
{
public:
	TScsParametersDlg( TDetector*);

private:
	TRadicon *Radicon;
	BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	void Dlg_OnCommand( HWND, int, HWND, UINT );
	BOOL CanClose( void );

	// event handler
	void OnParamSet();
};

//##############################################################################
// TCalibratePsdDlg
//##############################################################################

// Struktur fr linken und rechten Peak
struct CPoint 
{
	float Pt[3];
	BOOL Valid;
};

// Dialog zum Kalibrieren des Psd
class TCalibratePsdDlg : public TModalDlg, public ITimer
{
// berschriebenes aus der (Fenster-)Basisklasse
public:
	TCalibratePsdDlg( TCurve& );

	//! setzt Defaultwerte der Dialogbox bei Aufruf der Box
	//! Rueckkehrcode immer true
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );

	//! behandelt Kommandos in der Dialogbox
	virtual void Dlg_OnCommand( HWND, int, HWND, UINT );

	//! behandelt alle Scrollbarereignisse
	virtual void Dlg_OnHScrollBar( HWND, HWND, UINT, int );

	//! wird bei verlassen der dialogbox aufgerufen
	virtual void LeaveDialog( void );

	virtual void OnTimer ( TBasicTimer *const );

// event handler
private:
	void OnIDOK();
	void OnCancel();
	double DoRefreshPosition(BOOL&);
	void OnParamSet();
	void OnCounterSet();
	void OnRightPositionReached();
	void OnLeftPositionReached();

// Hilfsmethoden zum Ermitteln der Scrollbarpositionen 
private:
	double GetBarScale ( void ) const;

	//! ermittelt Anzahl der Scrollbarschritte links bzw. rechts von der
	//! absoluten Null
	int GetBarEgde( BOOL ) const;

	//! liefert die Position des Scrollbar anhand der akt. Motorposition im
	//! Rueckkehrcode
	int GetBarPos() const;

private:
	TInterfaceTimer *m_lnkTimer; // POLLING-Timer

	//! gibt umrechnungsfactor an um Winkelsekunden in die jeweilige Einheit
	//! umzurechnen
	float fRelationFactor;

	//! wahrscheinlich ueberfluessig
	// 21.04.2004 BOOL bCalibrationOk;

	//! akt. angezeigte Kurve
	TCurve *Curve; // nicht im Destruktor freigeben

	//! gibt pos. des linken bzw. rechten Peaks an
	CPoint LeftPeak;
	CPoint RightPeak;

	//! gibt an ob rechter bzw. linker Peak schon erreicht ist
	BOOL bGetRightPeak;
	BOOL bGetLeftPeak;

	//! gibt an ob motor ueber groessere distanz faehrt
	BOOL bLongMove;

	// cm_CounterSet an dieses Fenster weiterleiten (AreaScan-Fenster)
	HWND m_OldControlWnd;

	//! scheinbar ueberfluessig
	// 21.04.2004 float fOldPsdStep;

	//! FIX Fehler 49
	//! beinhaltet die Schrittweite vor Aufruf der Calibr.-Box
	double dOldMotorWidth;

	//! Handle des Scrollbars
	HWND hBar, m_lastfocused;

	//! gibt an ob Abzahl der Scrollbarschritte kleiner als 20000 ist (1) oder
	//! nicht (2)
	int BarFactor;

	int nMotor;				//! enthaelt Index des Thetamotors in der Motorenliste
	float dStepWidth;    	//! Wert fuer Winkel pro Kanal
	TOneDimDetector *Psd;	//! Pointer auf den Psd
};

#endif // #ifndef _DETECGUI_H
