/*
$Header: /vol/baal-vol3/projekt98/quellen/XCTL_32/INCLUDE/DETECUSE/DETECUSE.H,v 1.28 2004/07/30 04:21:56 kullmann Exp $

Projekt   : XCTL
Subsystem : Detektoren
Autor     : Jan Picard <picard@informatik.hu-berlin.de> 2001-2002
          Institut fuer Informatik,
          Humboldt-Universitaet Berlin
Inhalt    : Deklaration des Interface des Detektoren-Subsystems
*/

#ifndef __DETECUSE_H
#define __DETECUSE_H

#if defined ( Build_Counters )
#define _COUNTERCLASS __declspec(dllexport)
#elif defined( Use_Counters )
#define _COUNTERCLASS __declspec(dllimport)
#else
#define _COUNTERCLASS
#endif

#include "utils\u_utils.h"
#include "detecuse\range.h"
#include "motrstrg\motrstrg.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

enum EDetectorType
{
	CounterDetector= 1, PsdDetector, GenericDetector, CCDDetector
};

enum EHowReadOutPsd
{
	FinalRead= 1, FirstRead, AccumulationRead, IntermediateRead
};

enum EPsdDataType
{
	PsdEnergyData= 3000, PsdPositionData= 3001, PsdUnknown= 0
};

//vormals in evrythng.h
#define  R_MeasOk          206



//! TExposureSettings kapselt Einstellungen fr die Belichtung
//! Alle Werte werden vor einer eventuellen Modifikation geprueft,
//! ob sie die entsprechenden Bereichsgrenzen einhalten.
//! Bei Verletzung der Bereichsgrenzen werden die neuen Werte in das
//! Intervall "gezwungen".
class _COUNTERCLASS TExposureSettings
{
public:
	TExposureSettings( float exposureTime= 0, DWORD exposureCounts= 0);

	float GetExposureTime() const
	{
		return fExposureTime;
	};
	DWORD GetExposureCounts() const
	{
		return dwExposureCounts;
	};

	void SetExposureTime(float exposureTime);
	void SetExposureCounts( DWORD exposureCounts );

#pragma warning( push )
#pragma warning( disable : 4251 )
	TRange<DWORD> CountBounds;
	TRange<float> TimeBounds;
#pragma warning( pop )

private:
	float fExposureTime;        // Belichtungszeit
	DWORD dwExposureCounts;     // Anzahl der zulssigen Photonen
};

// Struktur, die einen Schnappschuss der aktuellen Einstellungen eines
// Detektors speichern kann
struct _COUNTERCLASS TDSettings
{
	TExposureSettings exposure;
	int nAddedChannels;
};

// forward declaration
class TDetector;

//###########################################################################

//! TDetectorManager verwaltet alle nutzbaren Detektoren, die in der Datei
//! hardware.ini deklariert sind. TDetectorManager ist als Singleton
//! implementiert, so dass genau eine Instanz davon existiert.
//! TDetectorManager legt die entsprechenden TDetector-Objekte an und zerstoert
//! sie auch wieder.
class _COUNTERCLASS TDetectorManager
{
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Konstruktoren und Destruktor
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Der Klassendestruktor wird automatisch beim Programmende aufgerufen.
	//! Dazu muss er public deklariert sein.
	//! Er DARF NICHT direkt aufgerufen werden.
	virtual  ~TDetectorManager();

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Methoden, die kontrollierten Zugriff auf Attribute bieten
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Liefert die Singleton-Instanz
	static TDetectorManager& DetectorManager();

	//! liefert den "aktiven" Detektor
	TDetector* GetDetector( void ) const;

	//! liefert den n-ten Detektor der Dimension nFilterDimension
	TDetector* GetDetector( int n, int nFilterDimension= -1 ) const;

	//! Setzt den "aktiven" Detektor auf den n-ten Detektor
	//! der Dimension nFilterDimension
	BOOL SetDetector( int n, int nFilterDimension= -1 );

	//! Setzt den aktiven Detektor auf den Detektor, der als Parameter
	//! angegeben ist, WENN er ein vom DetektorManager verwalteter Detektor
	//! ist.
	BOOL SetDetector( const TDetector* );

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Allgemeine Hifsmethoden
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! GUI-Hilfsmethode, fuellt die "choose detector" ComboBox
	void FillDetectorComboBox( HWND comboBox, int nFilterDimension= -1 ) const;

	//! Ermittelt, ob mindestend ein Detektor der angegeben Dimension
	//! nutzbar ist
	BOOL DimAvailable(int dimension) const;

	//! Fuer TSteering-Kommandos: Ermittelt eine ID fuer einen
	//! Detektor aus seinem Namen
	int GetIdByDescription( LPCSTR detectorname ) const;

	//! Gibt dann TRUE zurueck, wenn n ein gueltiger Index eines nutzbaren
	//! Detektors ist
	BOOL IsValidId(int n) const;

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// private Konstruktoren
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
private:

	//! Singleton-Konstruktor
	TDetectorManager();

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Verbotene Operationen
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
private:

	TDetectorManager(const TDetectorManager&);
	TDetectorManager& operator=(const TDetectorManager&);

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	//   private-Methoden
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
private:

	//! Erstellt das Detektorobjekt, das dem Parameter entspricht.
	//! Wenn die Initialierung erfolgreich ist, wird es in den
	//! DetectorContainer eingefuegt.
	void CreateDetector(const char* type, int id);

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	//   private-Attribute
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
private:

	//! Zeigt auf den "aktiven" Detektor
	TDetector* ActiveDetector;
};

//###########################################################################

//! TDetector ist das abstrakte Interface zu den Funktionen, die
//! von allen Detektoren unterstuetzt werden.
class _COUNTERCLASS TDetector
{

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Konstruktoren und Destruktor
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	TDetector(int id);
	virtual ~TDetector( void );

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Methoden, die kontrollierten Zugriff auf Attribute bieten
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Gibt eine Identifikationsschluessel fuer den Detektor zurueck.
	//! Diese Methode stellt eine quasi-Serialisierung des Objektzeigers
	//! dar. Ein Detektor kann ueber eine numerische ID angeprochen werden.
	//! In erster Linie wird dazu die Nummer des ini-File-Abschittes benutzt,
	//! in dem dieser Detektor deklariert ist ( das X in [deviceX] )
	int GetId( void ) const
	{
		return nId;
	};

	//! Gibt zurueck, ob mit diesem Detektor gerade gemessen wird
	//! ( MeasureStart() aufgerufen wurde )
	BOOL IsActive( void ) const
	{
		return bActive;
	};

	//! Gibt den bei der letzten Messung fuer Sigma ermittelten Wert zurueck
	float GetSigma( void ) const
	{
		return fSigma;
	};

	//! Legt die Belichtungseinstellungen fuer den Detektor fest
	BOOL SetExposureSettings (const TExposureSettings& newExposureSettings);

	//! Gibt die Belichtungseinstellungen fuer den Detektor zurueck
	TExposureSettings& GetExposureSettings()
	{
		return exposure;
	};

	//! Gibt den im ini-File fuer diesen Detektor deklarierten Namen zurueck
	const char* const GetCharacteristic() const
	{
		return szCharacteristic;
	};

	//! Gibt zurueck, ob im Moment gueltige Messwerte vorliegen
	BOOL IsDataValid() const
	{
		return bDataValid;
	};

	//! Gibt die tatsaechlich zur Messung verwandte Zeit zurueck
	double GetRealTime() const
	{
		return dRealTime;
	};

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Interface-Methoden fuer alle Detektoren
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Initialisert den Detektor und prueft ( wenn vorhanden ), ob die
	//! Hardwarekommunikation mit dem Detektor moeglich ist
	virtual BOOL Initialize( void );

	//! Startet die Messung mit diesem Detektor
	virtual int MeasureStart( void )= 0;

	//! Stoppt die Messung mit diesem Detektor
	virtual int MeasureStop( void )= 0;

	//! Veranlasst den Detektor, die im letzte Messzyklus ermittelten Werte
	//! der Hardware zu lesen und als gueltigen Messwert bereitzustellen
	virtual int PollDetector( void ) { return R_Failure; }; // GR

	//! Gibt die im letzten Messzyklus ermittelten, integralen Messwerte zurueck
	virtual int GetIntegral( float& )= 0; // ehemalig GetData

	//! Veranlasst den Detektor, sich die aktuellen Einstellungen zu merken,
	//! damit sie zu einem spaeteren Zeitpunkt wieder restauriert werden
	//! koennen. Der Detektor kann sich genau einen Satz an Einstellungen
	//! merken.
	virtual BOOL PushSettings( void );

	//! restauriert die vorher mit PushSettings() gespeicherten Einstellungen
	virtual BOOL PopSettings( void );

	//! Veranlasst den Detektor, neue Einstellungen zu uebernehmen
	BOOL SetParameters( void );

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// GUI-Methoden
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Gibt das Handle des Zaehlerfensters zurueck
	HWND GetCounterWnd( void ) const
	{
		return hCounterWnd;
	};

	//! Wenn zu diesem Detektor das CounterWindow angezeigt wird, wird
	//! das entsprechende Fensterhandle zurueckgegeben, sonst 0
	void SetCounterWnd( HWND hwnd )
	{
		hCounterWnd= hwnd;
	};

	//! Wenn die Messdaten dieses Detektors von einem Scanfenster verarbeitet,
	//! werden, wird das entsprechende Fensterhandle zurueckgegeben,
	//! sonst 0	
	HWND GetControlWnd( void ) const
	{
		return hControlWnd;
	};

	//! Setzt das Handle des steuernden Fensters
	void SetControlWnd( HWND hwnd )
	{
		hControlWnd= hwnd;
	};

	//! Startet den Dialog fuer detektorspezifische Einstellungen,
	//! wenn ein solcher existiert
	// Rckgabewert gibt an, ob der Dialog jetzt angezeigt wird
	// FALSE -> modeless Dialog wurde versteckt oder es existiert kein Dialog
	virtual BOOL RunSpecificParametersDlg( HWND )
	{
		return FALSE;
	};

	//! Gibt zurueck, ob ein Dialog fuer detektorspezifische Einstellungen
	//! existiert
	virtual BOOL HasSpecificParametersDlg( void ) const
	{
		return FALSE;
	};

	//! Benachrichtigt das CounterWindow und, wenn nicht ausdruecklich anders
	//! gewuenscht, das ScanFenster, dass die Daten verarbeitet, ueber die
	//! jetzt vorliegenden neuen Messdaten
	void UpdateViews(BOOL bDisplayOnly= FALSE) const;

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Typ-Identifikation
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Gibt eine symbolische Konstante zurueck, die den Typ des Detektors
	//! identifiziert
	virtual EDetectorType GetDetectorType() const= 0;

	//! Liefert die Dimension der Messwerte, die dieser Detektor ermittelt.
	//! Kann als type identification genutzt werden
	virtual int GetDimension() const= 0;

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Allgemeine Hifsmethoden fuer alle Detektoren
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
protected:

	//! Rechnet Millisekunden in Sekunden um
	static double MillisecondsToSeconds( DWORD milliseconds);

	//! Berechnet Sigma aus der ermittelten Intensitaet
	static float CalculateSigma(float intensity, float factor= 1.0);

	//! Kapselung des gleichnamigen WINAPI-Rufs, bei dem einige Parameter,
	//! die fuer alle Detektoren gleichbleiben wuerden, erst von dieser
	//! Methode eingesetzt wird.
	DWORD GetPrivateProfileString( LPCSTR lpKeyName, LPCSTR lpDefault, LPSTR lpReturnedString, UINT iMaxLen= MaxString ) const;

	//! Kapselung des gleichnamigen WINAPI-Rufs, bei dem einige Parameter,
	//! die fuer alle Detektoren gleichbleiben wuerden, erst von dieser
	//! Methode eingesetzt wird.
	DWORD WritePrivateProfileString( LPCSTR lpKeyName, LPCSTR lpString) const;


	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Hook-Methoden, die jeder Detektor spezifisch definieren kann
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
private:

	//! Hook, mit dem Detektoren-Subklassen eigene Parameter einstellen
	virtual BOOL _SetParameters(void)
	{
		return TRUE;
	};

	//! Laedt die zu diesem Detektor gehoerenden Einstellungen aus der
	//! hardware.ini. Auf jeder Abstraktionsstufe werden die entsprechenden
	//! Einstellungen geladen.
	void LoadDetectorSettings();

	//! Speichert die zu diesem Detektor gehoerenden Einstellungen in die
	//! hardware.ini. Auf jeder Abstraktionsstufe werden die entsprechenden
	//! Einstellungen gespeichert.
	void SaveDetectorSettings() const;

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// protected-Attribute
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
protected:

	//! Speichert die Zeit, die tatsaechlich fuer die Messung aufgebracht
	//! wurde
	double dRealTime;

	//! Zustandsvariable. Zeigt an, dass gerade ein Messvorgang laeuft.
	//! Wird von MeasureStart auf TRUE und von MeasureStop auf FALSE gesetzt.
	BOOL bActive;

	//! Handle das Hauptfensters. Detektoren verwenden Timer, um die
	//! Messintervalle zu begrenzen. Da Windows-Timer aber nur an Fenster
	//! gesandt werden koennen, wird hier das Hauptfenster benachrichtigt.
	HWND hWndFrame;

	//! Zeigt an, ob der ini-File-Schluessel "Debug=" gesetzt ist
	BOOL bDebug;

	//! der im letzten Messzyklus ermittelte Messwert, bei mehrdimensionalen
	//! Detektoren das Integral ueber alle Kanaele
	float fIntensity;

	//! JP?
	float fSigma;

	//! Zustandsvariable. Zeigt an, ob im Moment gltige Messwerte vorliegen
	BOOL bDataValid;

	//! Laenge des Windows-Timer-Intervalls, fix oder aus ExposureTime berechnet
	UINT uAskTime;

	//! Startzeitpunkt einer Messung
	DWORD dwStartTime;

	//! Zwischenspeicher fuer Einstellungen, Einstellungen werden mit
	//! PushSettings() in den Zwischenspeicher geschrieben und mit
	//! PopSettings() wieder herausgeholt.
	TDSettings Settings;

	//! Der "Name="-Wert aus der hardware.ini, textuelle Beschreibung des
	//! Detektors, der auch in den Detektor-Auswahlboxen angezeigt wird.
	char szCharacteristic[ MaxString ];

	//! Einstellungen fuer die Belichtung
	TExposureSettings exposure;

	//! Der Detektor, der fuer den Event*-Mechanismus des ContinuousScans
	//! benutzt wird
	TDetector* EventDetector;

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Verbotene Operationen
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
private:

	//! Das Kopieren von Detektor-Objekten ist verboten
	TDetector(const TDetector&);

	//! Die Zuweisung ist fuer Detektor-Objekte verboten
	TDetector& operator=(const TDetector&);

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	//   private-Attribute
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
private:

	//! ID des Detektors, das in der hardware.ini als [DetektorX] fuer
	//! diesen Detektor deklarierte X
	int nId;

	//! Wenn zu diesem Detektor das CounterWindow angezeigt wird, dann
	//! enthaelt hDisplayWnd das entsprechende Fensterhandle, sonst 0
	HWND hCounterWnd;

	//! Wenn die Messdaten dieses Detektors von einem Scanfenster verarbeitet,
	//! werden, dann enthaelt hControlWnd das entsprechende Fensterhandle,
	//! sonst 0
	HWND hControlWnd;
};

//! TZeroDimDetector ist das abstrakte Interface fuer alle nulldimensionalen
//! Detektoren.
class _COUNTERCLASS TZeroDimDetector : public TDetector
{

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Konstruktoren und Destruktor
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	TZeroDimDetector(int id);
	virtual  ~TZeroDimDetector();

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Methoden, die kontrollierten Zugriff auf Attribute bieten
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Legt fest, ob die ermittelte Intensitaet auch akkustisch vermittelt
	//! werden soll
	void SetSound(BOOL);

	//! Gibt zurueck, ob im Moment die ermittelte Intensitaet als akkustisches
	//! Feedback geliefert wird
	BOOL GetSound(void) const
	{
		return bSound;
	};

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Interface-Methoden fuer alle Detektoren
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	int MeasureStart( void );
	int MeasureStop( void );
	virtual int GetIntegral( float& ); // ehemalig GetData

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// *Event*-Mechanismus, nur vom Continuous-Scan verwandt
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Methode, die von einem Windows-Timer periodisch aufgerufen wird
	//! ermittelt eine aktuelle Intensitaet fuer den Detektor
	static CALLBACK EventHandler( UINT, UINT, DWORD, DWORD, DWORD );

	//! Initialisiert ein periodisches Aufrufen der Methode EventHandler
	virtual BOOL InitializeEvent( HWND, int );

	//! liefert die durch EventHandler gemessene Intensitaet zurueck
	virtual BOOL GetEventIntensity( float& );

	//! macht dem Detektor das Fenster bekannt, an das er innerhalb von
	//! EventHandler seine Nachrichten schicken soll
	void SetEventHost(HWND);

	//! loescht Detektor-Event
	void KillEvent( void );

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Typ-Identifikation
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	EDetectorType GetDetectorType() const
	{
		return CounterDetector;
	};

	//! Liefert die Dimension der Messwerte, die dieser Detektor ermittelt.
	//! Kann als type identification genutzt werden
	int GetDimension() const
	{
		return 0;
	};

	BOOL Initialize( void );
	virtual int PollDetector( void )= 0;

// nicht berschrieben, wenn auch gleiche Signatur (weil in Basisklasse nicht virtuell)
private:
	void LoadDetectorSettings();
	void SaveDetectorSettings() const;

// ATTRIBUT(E)
private:
	//! Zustandsvariable. Gibt an, ob das Messergebnis auch akkustisch
	//! repraesentiert werden soll
	BOOL bSound;
};

// forward declaration
class TCurve;

//! TOneDimDetector ist das abstrakte Interface fuer alle eindimensionalen
//! Detektoren.
class _COUNTERCLASS TOneDimDetector : public TDetector
{

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Konstruktoren und Destruktor
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:
	TOneDimDetector(int id);
	virtual  ~TOneDimDetector( void );

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Methoden, die kontrollierten Zugriff auf Attribute bieten
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	//! Legt das zu verwendende Energiefenster fest
	virtual BOOL SetEnergyRange( UINT, UINT )
	{
		return TRUE;
	};

	//! Gibt das verwendete Energiefenster zurueck
	virtual void GetEnergyRange( UINT& low, UINT& high ) const
	{
		high= 1;
		low= 0;
	};

	//! Legt fest, wieviele Kanaele zu einer Gruppe zusammengefasst werden sollen,
	//! fuer die ein gemeinsamer Messwert ermittelt wird
	void SetAddedChannels( int ach );

	//! Gibt zurueck, wieviele Kanaele zu einer Gruppe zusammengefasst werden,
	//! fuer die ein gemeinsamer Messwert ermittelt wird
	int GetAddedChannels( void ) const
	{
		return nAddedChannels;
	};

	//! Gibt zurueck, ob die Hochspannungsregelung aktiv ist
	BOOL GetHVControl() const
	{
		return bHVControl;
	};

	//! Legt fest, ob die Hochspannungsregelung aktiv sein soll
	void SetHVControl(BOOL param)
	{
		bHVControl= param;
	};

	//! Gibt die Auslesereihenfolge zurueck,
	//! TRUE= von links nach rechts, FALSE= von rechts nach links
	BOOL GetReadLeftFirst() const
	{
		return bReadLeftFirst;
	};

	//! Legt die Kanal-Auslesereihenfolge fest,
	//! TRUE= von links nach rechts, FALSE= von rechts nach links
	void SetReadLeftFirst(BOOL param)
	{
		bReadLeftFirst= param;
	};

	//JP?
	void SetAngleRange( float );

	//! Legt fest, ob stufenweiser Signalaufbau stattfinden soll
	void SetSignalGrowUp( BOOL gup )
	{
		bSignalGrowUp= gup;
	};

	//! Gibt an, ob ein vorgegebenes Messwertlimit ueberschritten wurde
	BOOL IsSoftOverflow( void ) const
	{
		return bSoftOverflow;
	};

	//! Gibt an, ob in der Detektorhardware waehrend des Messvorgangs
	//! ein Ueberlauf aufgetreten ist.
	//! !!! Wird leider von keinem Subsystem abgefragt !!!
	BOOL IsHardOverflow( void ) const
	{
		return bHardOverflow;
	};

	// Einheit des Detectors
	EUnitType GetUnit ( void ) const
	{
		return eUnit;
	};
	LPCSTR GetUnitName ( void ) const
	{
		return szUnit;
	};	

	//! Winkelbereich fr einen Kanal 
	float GetAngleStep( void ) const;
	void SetAngleStep( float stp );
	//! Winkelbereich, der von einer Kanalgruppe abgedeckt wird
	float GetWidth() const;
	//! Winkelbereich, der von allen Kanaelen abgedeckt wird
	float GetAngleRange( void ) const;

	UINT GetAngleStepDigits( void ) const;
	void SetAngleStepDigits( UINT );

	//! Gibt eine "Kurve" mit den Messwerten der einzelnen Kanalgruppen zurueck
	virtual int FillInCurve( TCurve&, EUnitType ); // ehemalig GetData

	//! Gibt einen integralen/ aufsummierten Messwert ber alle Kanle zurueck
	virtual int GetIntegral( float& ); // ehemalig GetData

	int MeasureStart( void );
	int MeasureStop( void );
	int PollDetector( void );
	BOOL Initialize( void );
	BOOL PushSettings( void );
	BOOL PopSettings( void );

	BOOL RunSpecificParametersDlg( HWND );
	BOOL HasSpecificParametersDlg( void ) const
	{
		return TRUE;
	};

	//! Gibt die Nummer des Kanals zurueck, bei dem der groesste Messwert
	//! ermittelt wurde
	UINT GetMaximumChannel( void ) const;

	//! Gibt die groesste Einzelintensitaet zurueck, die bei einem Kanal
	//! ermittelt wurde
	float GetMaximumIntensity( void ) const
	{
		return float( dwMaxCounts );
	};

	//! Spezifische Init-Routine fuer eindimensionale Detektoren
	virtual int PsdInit( void )
	{
		return R_OK;
	};

	// ber welchen Antrieb wird Detektor bewegt (fr Kanaloffset bei Manueller Justage)
	EAxisType GetAxis() const
	{
		return m_Axis;
	};

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// Typ-Identifikation
	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:

	EDetectorType GetDetectorType() const
	{
		return PsdDetector;
	};

	//! Liefert die Dimension der Messwerte, die dieser Detektor ermittelt.
	//! Kann als type identification genutzt werden
	int GetDimension() const
	{
		return 1;
	};

	float CalculateValueInSecondsEx(EUnitType, float) const; // CalcValueInUnit, jedoch mit Hochrechnung auf <MaxChannels>-Kanle

protected:

	//! Prueft, ob am ein neuer Maximalwert fuer einen Kanal vorliegt
	void CheckMaximumChannel(UINT channel, DWORD counts); // ehemalig CheckMaxChannel (wurde umbenannt, um Verwechselungen mit MaxChannel zu verhindert)

	char szUnit[ _MAXLENUNIT + 1 ];

	//! Einheit, in der Werte zurueckgegeben werden
	//! wird nur einmal aus der hardware.ini initialisiert
	EUnitType eUnit;

	//JP?
	//21.08.2004 MEMCORRECT HGLOBAL hCountBuf;
	char *hCountBuf;

	//JP?
	int nCountBufItems;

	//! Der hchste Einzelwert, der in einem Kanal ermittelt wurde
	DWORD dwMaxCounts;

	//! Skalarer Messwert, der durch Integration ueber alle Kanalgruppen
	//! ermittelt wurde
	DWORD dwIntegratedCounts;

	//! Nummer des Kanals mit dem hoechsten Einzelmesswert
	UINT uMaximumChannel;

	//! Zustandsvariable. Ist auf TRUE gesetzt, wenn die Detektorhardware
	//! einen Ueberlauf festgestellt hat.
	BOOL bHardOverflow;

	//! Legt die Auslesereihenfolge der Kanaele fest. TRUE= von links nach rechts
	BOOL bReadLeftFirst;

	//! Zustandsvariable. Gibt an, ob stufenweise Signalaufbau erfolgen soll.
	BOOL bSignalGrowUp;

	//! Zustandsvariable. Gibt an, ob ein Auslesevorgang der erste ist
	//! Nur vom StoePsd richtig verwendet
	BOOL bFirstReadOut;

	//! Die tatsaechlich waehrend einer Messung verstrichene Zeit
	//JP? Vielleicht als Parameter uebergeben?
	DWORD dwElapsedTime;

// nicht berschrieben, wenn auch gleiche Signatur (weil in Basisklasse nicht virtuell)
private:
	void LoadDetectorSettings();
	void SaveDetectorSettings() const;

// ATTRIBUT(E)
private:
	//JP?
	float fAngleStep;
	UINT nDigits;

	//! Anzahl der Kanle, die zu einer Kanalgruppe zusammenschlossen werden
	int nAddedChannels;

	//! Hook, um die PSD-Hardware zum Start der Messung zu veranlassen
	virtual int PsdStart( void )
	{
		return R_OK;
	};

	//! Hook, um die PSD-Hardware zum Stop der Messung zu veranlassen
	virtual int PsdStop( void )
	{
		return R_OK;
	};

	//! Hook, um die Messwerte aus der PSD-Hardware zu lesen
	virtual int PsdReadOut( EHowReadOutPsd )= 0;

	//JP?
	virtual int PsdRead( int, LPWORD )
	{
		return R_OK;
	};

	//! Zustandsvariable. Gibt an, ob ein "weicher" Ueberlauf aufgetreten ist,
	//! d.h. ob beim Ermitteln der Messwerte ein Wert >= OverflowIntensity
	//! ermittelt wurde
	BOOL bSoftOverflow;

	//! Obergrenze eines Messwertes, wurde sie ueberschritten, ist ein
	//! SoftOverflow aufgetreten.
	//! Wird nur einmal aus dem ini-File initialisiert
	float OverflowIntensity;

	//! Zustandsvariable. Legt fest, ob die Hochspannungsregelung aktiv sein soll
	BOOL bHVControl;

	/// ber welchen Antrieb wird Detektor bewegt
	EAxisType m_Axis;

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// NEU: EnergyScale, PositionScale jetzt auch hier
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
public:
	//! legt fest, ob uEnergyScale oder uPositionScale (und ihre Wirkung auf First- und LastChannel, GetChannelNumber) verwendet werden soll
	virtual void SetDataType( EPsdDataType typeParam )
	{
		eDataType= typeParam;
	};


	// gibt <EnergyScale> zurck
	UINT GetEnergyScale() const {
		return uEnergyScale;
	};
	// setzt <EnergyScale>
	virtual UINT SetEnergyScale(UINT aEnergyScale);


	// gibt <PositionScale> zurck
	UINT GetPositionScale() const {
		return uPositionScale;
	};
	// setzt <PositionScale>
	virtual UINT SetPositionScale(UINT aPositionScale);


	//! Gibt die Groesse des zu reservierenden Datenpuffers (fuer das Auslesen der Messwerte) zurck
	// Muss unabhngig von gewhltem PositionScale und EnergyScale sein und stets den Maximalbedarf an Speicher decken knnen!
	virtual int GetBufferSize( void ) const
	{
		return nMaxChannels; // GetChannelNumber() nicht verwenden, weil sich dieser Wert vergrern knnte ... dann ist nicht gengend Speicher reserviert
	};

	// gibt die maximal mgliche Kanalanzahl bei angegebener Scale zurck
	int GetScaledMaxChannels(UINT aScale) const {
		return nMaxChannels >> aScale;
	};

// folgende sind von uPositionScale bzw. uEnergyScale abhngig
public:
	//! Ermitteln des ersten Kanals
	virtual int GetFirstChannel() const;

	//! Ermitteln des letzten Kanals
	virtual int GetLastChannel() const;

	//! Gibt die Anzahl der zu nutzenden Kanaele zurueck
	int GetChannelNumber( void ) const
	{
		return max(0, GetLastChannel() - GetFirstChannel() + 1);
	};

	//! Festlegen des ersten/ letzten Kanals
	void SetChannelRange(int first, int last);

protected:
	// legt fest ob PsdEnergyData oder PsdPositionData
	EPsdDataType eDataType;


	// Kanaladdition direkt im Detektor
	UINT uEnergyScale; // moeglich sind 00, 01, 02, 03 hex
	UINT uPositionScale; // moeglich sind 00, 01, 02, 03 hex


	// gibt die maximal mgliche Kanalanzahl bei PositionScale= 0 bzw. EnergieScale= 0 zurck
	int nMaxChannels;


	//! erster Kanal des PSDs, mit dem gemessen werden soll
	int nFirstChannel; // hochgerechnet auf <MaxChannels>-Kanle - siehe PsdReadOut
	//! letzter Kanal des PSDs, mit dem gemessen werden soll
	int nLastChannel; // hochgerechnet auf <MaxChannels>-Kanle - siehe PsdReadOut
};

//! Methoden-Objekt, namespace fuer Arme
//! Laesst zu, dass Dialogfenster aus anderen Subsystemen gestartet werden
//! koennen
class _COUNTERCLASS TDetectorGUI
{
public:
	//! Startet den Dialog fuer allgemeine Einstellungen fuer alle Detektoren
	static void RunCommonDevParamDialog(HWND);

	//! Startet den Dialog zum Kalibrieren von PSDs
	static void RunPsdCalibrateDialog(HWND, TCurve&);
};

#ifdef __cplusplus
extern "C"
{
#endif
	LPCSTR __declspec(dllexport) WINAPI dlGetVersion( void );
	HINSTANCE __declspec(dllexport) WINAPI dlGetInstance( void );
#ifdef __cplusplus
}

#endif

#endif // __DETECUSE_H

