//##############################################################################
//                                                                            //
// D_EXTERN.H                                                                 //
//                                                                            //
// Subsystem: Diffraktometrie/ Reflektometrie                                 //
//            DIE MDI-Fensterbasisklasse                                      //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//                                                                            //
//##############################################################################

#ifndef _D_EXTERN_H
#define _D_EXTERN_H

#include "internls\evrythng.h" // TMDIWindow

#include "difrkmty\d_intern.h" // TBitmapSource, TCoorSystem
#include "datavisa\datavisa.h" // TCurve, TOutputFormat, TScreen

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

const char EOL[]= "\r\n";
const int P_X= 0;
const int P_Y= 1;
const int P_Z= 2;

const int DATAOFFSET= 1000; // Anzahl von Zeichen vom Dateianfang bis zum [DATA]-Abschnitt; muss konstant und unverndert bleiden, damit der enthaltene [Header]-Abschnitt aktualisiert werden kann = Austausch durch einen neuen Block von <DATAOFFST>-Zeichen

typedef struct
{
	float x, y;
	int Ord;
} TKSystem[4];

enum EFileFormat 
{
	ffUnknown= 0, ffStandardFile= 1, ffShiftedStandard, ffTripel, ffTupel, ffBitmap, ffBkFile, ffDtnFile
	// 21.04.2004 ffShiftedBitmap, ffTiff8, ffTiff12, ffRAW8, ffRAW12
};

enum ExScanType
{
	stUnknown= 0, stStandardScan= 1, stOmega2ThetaScan, stContinuousScan, stPsdEnergyScan,
	stPsdPositionScan, stOmega2ThetaScanList, stScanList
};

enum EHAlign
{
	haLeft, haCenter, haRight
};

enum EVAlign
{
	vaTop, vaCenter, vaBottom
};

struct TKSProperties
{
	BOOL bLogarithmicY;
	float fMinX, fMaxX, fMinY, fMaxY;
};

struct TFloatPoint {
	float x, y;
};

//##############################################################################
// TPlotWindow
//##############################################################################
// stellt die Basisfunktionalitt zur Darstellung und Bewertung von Kurven dar

enum EWindowType {wtUnknown= 0, wtScan= 251, wtAreaScan= 252}; 

class TPlotWindow : public TFileWindow
{
	friend class TCurveShowParamDlg;
	friend class TCurveFreeScalDlg;
	friend class TBitmapSource;
	friend class TCurveFreeScalColor;

public:
	//! Konstruktor
	TPlotWindow( HINSTANCE );

	//! Destruktor
	virtual  ~TPlotWindow( void );

// berschriebenes aus der (Fenster-)Basisklasse; z.T. wieder in TAreaScanWindow und TScanWindow berschrieben
protected:
	virtual HICON GetIcon ( void ) const;

	//! welchen Mauscursor in diesem Fenster anzeigen?
	virtual HCURSOR GetCursor( void ) const
	{
		return LoadCursor(NULL, IDC_CROSS);
	};

	virtual BOOL New( void );

	virtual LRESULT OnEvent ( HWND, UINT, WPARAM, LPARAM );
	virtual LRESULT OnCommand ( WPARAM, LPARAM );

	// 21.04.2004 virtual void OnSize( WPARAM, LPARAM );

	//! wenn Darstellungstyp ein Bitmap, dann bereite Zwischenablage vor
	virtual void OnRenderFormat( WPARAM, LPARAM );
	
	//! raeumt Zwischenablage auf, wenn Besitzer der Zwischenablage tot
	virtual void OnRenderAllFormats( WPARAM, LPARAM );

	//! DoPaint wird von EvPaint aufgerufen und automatisch von Windows angefordert,
	//! um den Inhalt des Fensters neu anzuzeigen
	virtual void DoPaint( HDC, PAINTSTRUCT * );

	//! Kopieren mit Hilfe der Zwischenablage - Typen: CF_DIB, CF_PALETTE
	virtual void DoCopy( void );

// Mausbedienung
	//! behandelt Kommando: linke Maustaste wurde gedrueckt
	virtual void OnLButtonDown( WPARAM, LPARAM );

	//! wenn linke Maustaste gedrueckt gehalten und dabei bewegt wird,
	//! wird der Abstand zur Pos. des ersten Mausklicks berechnet
	virtual void OnMouseMove( WPARAM, LPARAM );

	//! gibt Handle zum zeichnen im Kindfenster zurck + linke Maustaste
	//! als nicht gedrueckt kennzeichnen
	virtual void OnLButtonUp( WPARAM, LPARAM );

// wird in TAreaScanWindow und TScanWindow wieder berschrieben
protected:
	// rechnet die Bildschirmkoordinaten in entsprechende Winkel/ Kanle/ Intensitten um
	void Screen2Title(float, float, // Bildschirmkoordinaten (x, y)
	                  float&, float&, float&, // Zahlenwerte (x, y, z), wobei z die zweizeilige Beschriftung bei DTN-File
					  bool); // Kanalausgabe in 5er Schritten runden

	// gibt die Beschriftung fr den jeweiligen Achsenabschnitt der X- und Y-Achse zurck
	virtual void GetTitleX(int, int, char *, bool);
	virtual void GetTitleY(int, int, char *, bool);

	virtual bool CanSwitchChannelAngle() { // Umschaltung Winkel-/ Kanaldarstellung erlaubt
		if ( VisualDB.bPsd ) return true; // nur wenn PSD angeschlossen
		else return false;
	}

	//! im Falle von cm_MoveScanReady
	virtual void SetKSProperties( TKSProperties* );

	//! Theta-Offset entfernt
	virtual double GetThetaOffset( int )
	{
		return 0.0;
	};

	virtual BOOL SaveFile(EAskType);

	//! fuelle TKSystem mit Standardwerten
	virtual BOOL SetMeasurementArea( TKSystem & );

	//! setze neue Koordinatensystembereiche
	virtual void SetRanges( void );

	//! sichert Daten in der SecondCurve
	//! im Falle von cm_MoveScanReady
	virtual void PickUpData( LPARAM );

// dismante curve
protected:
	//! fixiert die aktuelle Hauptkurve -> setzt Pointer von SecondCurve auf
	//! MainCurve
	void FreezeCurve( void );

	//! setzt SecondCurve auf Initialwerte zurueck (loeschen aller Metripel)
	//! zeichnet Scan-Fenster neu
	void KillSecondCurve( void );

	//! speichert fixierte Kurve (SecondCurve) unter auszuwaehlendem Dateinamen
	//! mit Extension *.bk
	void SaveSecondCurve( void );

	//! setzt den Filetyp, nach dem Laden von alten Daten
	void SetLoadFormat(EFileFormat);

// in TAreaScanWindow oder TScanWindow verwendet
protected:
	//! Funktion, die die bentigte Nachkommastellengenauigkeit
	//! berechnet und die ntigen Nachkommastellen zurckliefert
	int GetNachkommaStelle(TCoorSystem* pCoorSys, int x, int y);

// Zeichnen
private:
	// filtert Signal zur besseren Darstellung von Peaks
	// Staerke des Filters durch m_FilterStrength aenderbar
	void FilterPLine(LPPOINT, int &);

	// fllt die Struktur lpPoints mit den Werten fr eine Kurve
	// um damit spter dann die Kurve darzustellen
	void FormatCurveToPLine( TCurve *, LPPOINT, int & );

	// zeichnet eine Reihe von Punkten - im Gegensatz zu Polyline werden die Punkt nicht verbunden
	void Dottedline(HDC, const POINT *, int);

	//! setzt Variable bEnablePaint
	void EnablePaint( void )
	{
		bEnablePaint= TRUE;
		bPaintPoint= FALSE; // ALLES neu zeichnen
	};

	//! setzt Variable bEnablePaint
	void DisablePaint( void )
	{
		bEnablePaint= FALSE;
	};

	// wie ::TextOut bzw. ::GetTextExtentPoint, jedoch fr mehrzeilige Strings
	static BOOL GetTextExtentPointEx(HDC, LPCTSTR, int, LPSIZE);
	// die neuen letzten beiden Parameter geben an wie die x- und y-Koordinaten zu betrachten sind (bei center geben sie den Mittelpunkt des auszugebenden Textes an, etc.)
	static void TextOutEx(HDC, int, int, LPCTSTR, int, EHAlign= haLeft, EVAlign= vaTop);

	//! zeichnet Koord.System
	void DrawCoordinateSystem( HDC );

// ATTRIBUT(E)
public:
	// ehemalig extern-Verweise aus D_INTERN.CPP
	float tilesx; //! Standard Anzahl Abzissenbeschriftungen
	float tilesy; //! Standard Anzahl Ordinatenbeschriftungen
	int scrolldxposmin; //! Minimum Abzissenausdehnung
	int scrolldxposmax; //! Maximum Abzissenausdehnung
	int scrolldyposmin; //! Minimum Ordinatenausdehnung
	int scrolldyposmax; //! Maximum Ordinatenausdehnung
	int xBorder; //! der Rand zw. y-Achse und unterem Fensterrand
	int yBorder; //! der Rand zw. x-Achse und linkem Fensterrand
	WORD RLdx; //! RLdx ist die x-Punkte Ausdehnung
	WORD RLdy; //! RLdy ist die y-Punkte Ausdehnung - einstellbar im Visualising Options Menu	
	BOOL bIncreasePP; //! bIncreasePP ist die "groessere Punkte" Einstellmoeglichkeit Visualising Options Menu
	float MinIntensitaet;
	float MaxIntensitaet;	
	TFloatPoint mPoint1, mPoint2; //! zum Aufzeichnen der Mausbewegungen beim Ziehen einer Linie mit der Maus
	bool DataaquisitionActive;

	//! Abzissen/Scanachsebeschriftung
	char abzisse[ _MAXLENUNIT+1 ];

	//! Zeiger auf zugehoeries Bitmap
	TBitmapSource *BSource; // im Destruktor freigegeben

	BOOL m_Dotted; // Linien gepunktet zeichnen?
	BOOL m_HasFilter; // Low-Pass Filter verwenden?
	float m_FilterStrength; // Einfluss in Prozent?

protected:
	TCurve *MainCurve; // hier nicht im Destruktor freigeben: wird nur von TAreaScanWindow und TScanWindow initialisiert; darf bei TAreaScanWindow nicht im Destruktor freigegeben werden (wird von TDataBase verwaltet); bei TScanWindow wird MainCurve im Destruktor freigegeben
	TCurve *SecondCurve; // im Destruktor freigegeben

	// zur Umrechnung der Motoreinheiten beim AreaScan
	EUnitType m_MotorUnit, m_DetectorUnit;

	EWindowType WindowType;
	BOOL bMeasurementActive;

	//! gibt Darstellungstyp an
	EOutputType eOutputType;	

	//! FIX Fehler 39
	TDisplay VisualDB;

	//! beiinhaltet den geladenen FileTyp
	EFileFormat eLoadFormat;
	
	//! erstes und alternatives Koordinatensystem
	TCoorSystem CoorSys, AltCoorSys; // AltCoorSys sollte nicht zusammen mit otCurve verwendet werden, weil nach einmaligem Zeichnen auf CoorSys zurckgeschaltet wird (d.h. ein Fenster-Update zeichnet dann wieder mit CoorSys)

	BOOL bAltCoorSystem; // TRUE==beim nchsten Zeichnen AltCoorSys benutzen; siehe AltCoorSys

	//! MinIntensitaet, MaxIntensitaet
	float LowerBound, UpperBound;

	//! XY-Skalierung an/ aus
	BOOL bXY_Scaling;

	TScreen Screen;

	// Art der Beschriftung fr x-Achse, TRUE==Kanal, sonst Winkel 
	BOOL bSecondaryCoor; 

private:
	HFONT hFont;

	//! beinhaltet Handle auf ein DC (WinAPI)
	HDC DragDC;

	//! Darstellung zulassen oder nicht
	BOOL bEnablePaint;

	//! Mousebutton unten/oben
	BOOL bButtonDown;

	//! Koordinaten wohin mit der Maus geklickt wurde
	float xo, yo;
};

#endif // _D_EXTERN_H

//#############################################################################
// LastLine
//#############################################################################