/*//////////////////////////////////////////////////////////////////////////////
//                                                                            //
// MOTORCONTROLLER.H                                                          //
//                                                                            //
// Subsystem : Motorsteuerung / Hardwarezugriff								  //
//----------------------------------------------------------------------------//
// Autoren:  Ren Harder, Alexander Paschold (2003)		                      //
//                                                                            // 
// Stand  : 23.05.2003                                                        //
// letzter vollstndiger Test:                                    //
//                                                                            //
//////////////////////////////////////////////////////////////////////////////*/

#ifndef _MOTORCONTROLLER_H
#define _MOTORCONTROLLER_H

#include "hardware\hwio.h"

#ifndef _MOTORCLASS 
#if defined (Build_Motors)
#define _MOTORCLASS __declspec(dllexport)
#elif defined(Use_Motors)
#define _MOTORCLASS __declspec(dllimport)
#else
#define _MOTORCLASS
#endif
#endif

/////////////////////////////////////////////////////////////////
// Controller fr C-812 Motorsteuerkarte
class _MOTORCLASS TC_812Controller : public Controller
{

public:
	TC_812Controller(EDeviceType DeviceID, LPTSTR HardwareID, DeviceList* Devices);

	virtual BOOL Check();

	char Get(DWORD addr);
	void Put(DWORD addr, BYTE data);
	int PutChar(const char c);
	char GetChar(void);
	int ExecuteCmd(LPTSTR pString);

public:
	DWORD oFlag;
	DWORD oIn;
	DWORD oOut1;
	DWORD oOut2;
	DWORD oDPRam;
};

/////////////////////////////////////////////////////////////////
// Controller fr C-832 Motorsteuerkarte
class _MOTORCLASS TC_832Controller : public Controller
{

public:
	TC_832Controller(EDeviceType DeviceID, LPTSTR HardwareID, DeviceList* Devices);
	// Endlagen-Ueberwachung und Handling
	static void CALLBACK LimitWatch( UINT, UINT, DWORD, DWORD, DWORD );

	virtual BOOL Check();

public:
	BYTE activeConfig;
	WORD activeDrive;
	WORD baddr;
//	BYTE raddr;

	// Endlagen-Ueberwachung und Handling
	BOOL bLimitHit;
	BOOL bIdle;
	BOOL bIOActive;

private:
	const WORD CmdRegister;
	const WORD DataRegister;
	DWORD dwSimBasePort;

	//Hardwarezugriffsfunktionen (ehemals global)
private:
	int GetWord(WORD base, WORD regaddr, unsigned short drive);
	long GetDWord(WORD base, WORD regaddr, unsigned short drive);
	void PutWord(int data, WORD base, WORD regaddr, unsigned short drive);
	void PutDWord(long data, WORD base, WORD regaddr, unsigned short drive);


public:
	void UpdateController(BYTE activeConfig, WORD activeDrive);
	long Drive628c(BYTE cmd, WORD ctrl_word, long param, WORD base, WORD regaddr, unsigned short drive);
	void Put(unsigned port, int value);
	int Get(unsigned port);
	BOOL LM628Ready(WORD base, unsigned short drive);

};


#endif