// Makros zum Aufspren von MemoryLeaks

#ifndef _XCTL_CHECKMEMORY_H_
#define _XCTL_CHECKMEMORY_H_

#include <crtdbg.h>


//Memory states
_CrtMemState s1, s2, s3;


// This routine place comments at the head of a section of debug output
void OutputHeading( const char *explanation )
{
   _RPT1( _CRT_WARN, "\n\n-----%s----------------------------------------\n", explanation );
}

// This routine place an end-comment at the head of a section of debug output
void OutputEnd()
{
   _RPT0( _CRT_WARN, "\n---------------------------------------------\n\n");
}

// This routine checks memory and place comments at the head of a section of debug output
void CheckMemory(const char *explanation)
{
	OutputHeading(explanation);
	_CrtDumpMemoryLeaks();
	OutputEnd();
}

// This routine saves pre-execution memory state
void SavePreState()
{
	_CrtMemCheckpoint(&s1);
}

// This routine saves post-execution memory state
void SavePostState()
{
	_CrtMemCheckpoint(&s2);
}

// This routine dumps difference between pre- and post-execution memory state
void DumpMemoryState(const char *explanation)
{
	if (_CrtMemDifference(&s3,&s1,&s2))
	{
		OutputHeading(explanation);
		_CrtMemDumpStatistics( &s3 );
		OutputEnd();
	}
}

// This routine dumps all objects instantiated since pre-execution memory state
void DumpMemoryObjects(const char *explanation)
{
	OutputHeading(explanation);
	_CrtMemDumpAllObjectsSince( &s1);
	OutputEnd();
}


//
#ifdef _DEBUG
#define CHECKMEM(a) CheckMemory(a)
#define PRE SavePreState()
#define POST SavePostState()
#define DUMPDIFF(a) DumpMemoryState(a)
#define DUMPMEM(a) DumpMemoryObjects(a)
#else
#define CHECKMEM(a)
#define PRE
#define POST
#define DUMPDIFF(a)
#define DUMPMEM(a)
#endif //_DEBUG

#endif //_XCTL_CHECKMEMORY_H_

