// (C) 1993, 1994 by Heiko Damerow MPG AG "Roentgenbeugung"
//#############################################################################
//                                                                           //
// M_STEERG.H                                                                //
//                                                                           //
// Subsystem : Ablaufsteuerung - Kommandos                                   //
// Benutung durch andere Subsysteme erforderlich: NEIN                       //
// Implementation: M_STEERG.CPP                                              //
//---------------------------------------------------------------------------//
// letzte Bearbeitung: Thomas Kullmann, Gnther Reinecker                    //
// Stand:   28.07.2002                                                       //
//                                                                           //
//#############################################################################
//## Diese Kommentare (mit zwei Doppelkreuzen) sind von Marlies Gollnick ~:)

//! neu Kullmann+Reinecker:
// * Sinneinheiten zusammengefat
// * einheitlich strukturiert und formatiert
// * Toten Code auskommentiert und mit "//Datum: 28.07.2002" gekennzeichnet
//TCmd (und abgeleitete Klassen)
// * neue Methode GetName in TCmd (und abgeleiteten Klassen)

#ifndef __M_STEERG_H
#define __M_STEERG_H

#include "difrkmty\s_extern.h" // fr TScanWindow
#include "difrkmty\a_extern.h" // fr TAreaScanWindow

//#############################################################################
// globale Typedeklarationen
//#############################################################################

#define   nMaxArg   4

// Steuer-Codes fr Kommandos (Einzelschritte eines Kommandos)
enum ECmdCode 
{
	CReady= 1, CMeasure, CRecall, CReadyStep, CFirstStep, CControlStep, CStop //Datum: 28.07.2002 CFailure, CHalted
};


//#############################################################################
// TCmd
//#############################################################################

class TCmd
{
	friend class TSteering;

	//*****************************************************************************
	// Konstruktor-Destruktor
private:
	TCmd ( TCmd& ) {} // kein Copy-Konstruktor
	TCmd &operator= ( const TCmd& )
	{
		return *this;
	} // und auch kein Zuweisungsoperator

public:
	TCmd ( TCmdTag );
	virtual  ~TCmd ();

	//*****************************************************************************
	// Fortschritt- Kommandoinformtionen
public:
	virtual BOOL GetShowData ( LPSTR )
	{
		return TRUE;
	}
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "BASE" );
	} // ! neu Kullmann+Reinecker

	//*****************************************************************************
	// Schritte der Kommandoverarbeitung
public:
	//fhrt (abhngig von eStep) einen der folgenden Schritts aus
	ECmdCode DoAction ( void );

	virtual ECmdCode FirstStep ( void )
	{
		return CReady;
	};
	virtual ECmdCode ControlStep ( void )
	{
		return CReady;
	};
	virtual ECmdCode ReadyStep ( void )
	{
		return CReady;
	};
	virtual ECmdCode Ready ( void )
	{
		return CReady;
	};

	//*****************************************************************************
	// angegebenen Antrieb bewegen
	BOOL StartMove ( const int, double );

	//*****************************************************************************
	// Accessor-Methoden
public:
	virtual ECmdCode WakeUp ( void ); // der nchste zu verarbeitende Schritt

	BOOL IsPositionValid ( void )
	{
		return bPositionValid;
	};
	BOOL IsReady ( void )
	{
		return ( eStep == CReady );
	};
	ECmdId GetId ( void )
	{
		return eCmdId;
	};

	//Datum: 28.07.2002 virtual BOOL   DetectorRequest( void );
	//Datum: 28.07.2002 virtual LPSTR  GetFailure( void );

	//*****************************************************************************
	// in abgeleiteten Kommandos benutzte ATTRIBUTS
protected:
	ECmdCode eStep;
	BOOL bNoMeasure;   // normaly starts the detector after moving
	BOOL *bMoveOk;
	int nFailureId;
	//! FIX Fehler 72
	BOOL bRestart;

	//*****************************************************************************
	// PRIVATE ATTRIBUTS
private:
	ECmdId eCmdId;
	BOOL bPositionValid;
};

//#############################################################################
// TChooseAxisCmd
//#############################################################################

class TChooseAxisCmd : public TCmd
{
public:
	TChooseAxisCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "ChooseAxis" );
	} // ! neu Kullmann+Reinecker
};

//#############################################################################
// TSetWidthCmd
//#############################################################################

class TSetWidthCmd : public TCmd
{
public:
	TSetWidthCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "SetWidth" );
	} // ! neu Kullmann+Reinecker
};

//#############################################################################
// TLoadPointCmd
//#############################################################################

class TLoadPointCmd : public TCmd
{
public:
	TLoadPointCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "LoadPoint" );
	} // ! neu Kullmann+Reinecker
};

//#############################################################################
// TMoveToPointCmd
//#############################################################################

class TMoveToPointCmd : public TCmd
{
public:
	TMoveToPointCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "MoveToPoint" );
	} // ! neu Kullmann+Reinecker
	//Datum: 28.07.2002 ECmdCode        ReadyStep( void );
};

//#############################################################################
// TChooseDetectorCmd
//#############################################################################

class TChooseDetectorCmd : public TCmd
{
public:
	TChooseDetectorCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "ChooseDetector" );
	} // ! neu Kullmann+Reinecker
};

//#############################################################################
// TGotoIntensityCmd
//#############################################################################

class TGotoIntensityCmd : public TCmd
{
	//*****************************************************************************
	// Konstruktor-Destruktor
public:
	TGotoIntensityCmd ( TCmdTag );
	virtual  ~TGotoIntensityCmd ();

	//*****************************************************************************
	// Fortschritt- Kommandoinformtionen
public:
	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "GotoIntensity" );
	} // ! neu Kullmann+Reinecker

	//*****************************************************************************
	// Schritte der Kommandoverarbeitung
public:
	ECmdCode FirstStep ( void );
	ECmdCode ControlStep ( void );
	ECmdCode ReadyStep ( void );

	//*****************************************************************************
	// PRIVATE ATTRIBUTS
private:
	int nMotor;
	BOOL bSmallAngleSide;
	float fSearchIntensity;
	double *dDistance;
	float *fIntensity;
	ECmdParam nReadyAction;

	static const UINT nLifo;
};

//#############################################################################
// TGotoPeakCmd
//#############################################################################

class TGotoPeakCmd : public TCmd
{
	//*****************************************************************************
	// Konstruktor-Destruktor
public:
	TGotoPeakCmd ( TCmdTag );
	virtual  ~TGotoPeakCmd ();

	//*****************************************************************************
	// Fortschritt- Kommandoinformtionen
public:
	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "GotoPeak" );
	} // ! neu Kullmann+Reinecker

	//*****************************************************************************
	// Schritte der Kommandoverarbeitung
public:
	ECmdCode FirstStep ( void );
	ECmdCode ControlStep ( void );
	ECmdCode ReadyStep ( void );

	//*****************************************************************************
	// PRIVATE ATTRIBUTS
private:
	int nMotor;
	BOOL bSmallAngleSide;
	BOOL bFlankDetected;
	float fSearchIntensity;
	ECmdParam nReadyAction;
	double *dDistance;
	float *fIntensity;
	BOOL bBackMoveActive;
	//Datum: 28.07.2002 BOOL    bDynamicStepWidth;
	//Datum: 28.07.2002 BOOL    bDirectionChanged;

	static const UINT nLifo;
};

//#############################################################################
// TShowValueCmd
//#############################################################################

class TShowValueCmd : public TCmd
{
public:
	TShowValueCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "ShowValue" );
	} // ! neu Kullmann+Reinecker
};

//#############################################################################
// TCalculateCmd
//#############################################################################

class TCalculateCmd : public TCmd
{
public:
	TCalculateCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "Calculate" );
	} // ! neu Kullmann+Reinecker
};

//Datum: 28.07.2002
/*//#############################################################################
// TInquireCmd
//#############################################################################
 
class  TInquireCmd : public TCmd {
public:
  TInquireCmd( TCmdTag );
 
  virtual BOOL  GetShowData( LPSTR );
  virtual void  GetName ( LPSTR aName )  { strcpy( aName, "Inquire" ); }; // ! neu Kullmann+Reinecker
};*/

//#############################################################################
// TControlFlankCmd
//#############################################################################

class TControlFlankCmd : public TCmd
{
	//*****************************************************************************
	// Konstruktor-Destruktor
public:
	TControlFlankCmd ( TCmdTag );
	virtual  ~TControlFlankCmd ();

	//*****************************************************************************
	// Fortschritt- Kommandoinformtionen
public:
	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "ControlFlank" );
	} // ! neu Kullmann+Reinecker

	//*****************************************************************************
	// Schritte der Kommandoverarbeitung
public:
	ECmdCode FirstStep ( void );
	ECmdCode ControlStep ( void );
	ECmdCode ReadyStep ( void );

	//*****************************************************************************
	// PRIVATE ATTRIBUTS
private:
	int nMotor;
	float fControlRange;
	float fStartIntensity, fSearchIntensity;
	float fMinIntensity, fMaxIntensity, fIntensityRange;
	BOOL bSmallAngleSide;
	double *dDistance;
	float *fIntensity;

	static const UINT nLifo;
};

//#############################################################################
// TSetupScanCmd
//#############################################################################

class TSetupScanCmd : public TCmd
{
public:
	TSetupScanCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "Basis" );
	}; // ! neu Kullmann+Reinecker

private:
	TScanWindow *ScanW;
};

//#############################################################################
// TSetFileNameCmd
//#############################################################################

class TSetFileNameCmd : public TCmd
{
public:
	TSetFileNameCmd ( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "SetFileName" );
	}; // ! neu Kullmann+Reinecker

private:
	TScanWindow *ScanW;
	TAreaScanWindow *AreaScanW;
};

//#############################################################################
// TSaveDataCmd
//#############################################################################

class TSaveDataCmd : public TCmd
{
public:
	TSaveDataCmd( TCmdTag );

	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "SaveDataCmd" );
	} // ! neu Kullmann+Reinecker

private:
	TScanWindow *ScanW;
	TAreaScanWindow *AreaScanW;
};

//#############################################################################
// TScanCmd
//#############################################################################
//! kommentiert von der Diffr.-gruppe
//! Ablaufsteuerung fuer StepScan und AreaScan(mit Psd)

class TScanCmd : public TCmd
{
	//*****************************************************************************
	// Konstruktor-Destruktor
public:
	TScanCmd ( TCmdTag );

	//*****************************************************************************
	// Fortschritt- Kommandoinformtionen
public:
	//! taetigt ausgaben in die Statusleiste waehrend und bei ende des Scans; Rueckkehrcode immer True
	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "Scan" );
	} // ! neu Kullmann+Reinecker

	//*****************************************************************************
	// Schritte der Kommandoverarbeitung
public:
	//! setzt Initialwerte in erstem Schritt des Scans
	//! Rueckkehrcode immer CMeasure
	ECmdCode FirstStep ( void );

	//! wird vom 2. bis zum letztem Schritt des Scans aufgerufen
	//! Rueckkehrcode immer CRecall
	ECmdCode ControlStep ( void );

	//! wird nach letztem Schritt aufgerufen
	//! Rueckkehrcode immer CRecall
	ECmdCode ReadyStep ( void );

	//*****************************************************************************
	// PRIVATE ATTRIBUTS
private:
	//! Pointer auf zugehoeriges Scanfenster
	TScanWindow *ScanW; // nicht im Destruktor freigeben, wird vom Hauptprogramm verwaltet

	//! Pointer auf zugehoeriges AreaScanfenster
	TAreaScanWindow *AreaScanW; // nicht im Destruktor freigeben, wird vom Hauptprogramm verwaltet
	
	//! Zeiger auf Liste mit Inten. und zugeh. Schrittweiten
	TFloatList *Scaler; // nicht im Destruktor freigeben, wird von <AreaScanW> bernommen
	
	//! gibt Scanart an (ForScan= StepScan/ ForAreaScan= AreaScan)
	ECmdParam eCaller;

	//! gibt Scantyp an (Standard,Omega2Theta)
	ExScanType eScanType;

	//! Anzahl schon gemessener Punkte
	int nPointIdx,

	//! Anzahl der gesamten zu messenden Punkte
	nPointNumber;
	
	int nMotor, //! Nummer des akt. Motors
		nTheta, //! Nummer vom Theta-motor (nur bei Omega2Theta verwendet)
		nAbsorber; //! ?? (nur bei AreaScan verwendet)

	
	double dArgumentMin, //! Startwinkel des Scans
		   dArgumentMax, //! Endwinkel des Scans
		   dArgumentWidth; //! normale Schrittweite waehrend Scan
	
	//! ??(alle 3 nur bei AreaScan verwendet)
	double dThetaMin, //! Startwinkel des Thetamotors
		   dThetaMax, //! Endwinkel des Thetamotors
		   dThetaWidth; 	//! Schrittweite des Thetamotors
	

	
	double dStartPointTheta, //! ?? winkel von Theta bei Scanstart (nur bei Omega2Theta verwendet)
		   dStartPointAbsorber; //! ? winkel des Absorber bei Scanstart(nur bei AreaScan verwendet)

	BOOL bOmega2Theta; //! gibt an ob Omega2Theta-Scan

	//! gibt an ob Absorber benutzt
	BOOL bAbsorberUsed;

	//! gibt verhaeltnis an um das der Thetamotor bewegt werden soll, wenn sich
	//! der Omegamotor um einen Schritt bewegt (nur bei Omega2Theta verwendet)
	double dMoveRelation;

	//! gibt waehrend des Scans den naechsten anzufahrenden Winkel
	//! (Omega bei O2T-Scan)an
	double dArgument;

	//! ?? scheinbar ueberfluessig
	//21.04.2004 float fDriveInIntensity;
	//21.04.2004 float fDriveOutIntensity;
	//21.04.2004 float fIntensityRelation;
	//21.04.2004 BOOL bAbsorberActivated;
	//21.04.2004 BOOL bAbsorberMovedIn;

	//! gibt an ob dynamische Schrittweitensteuerung aktiv(nur bei StepScan
	//! verwendet)
	BOOL bDynamicStepWidth;

	//! Anzahl der verschied. Schrittweiten die benutzt werden
	//! (nur bei StepScan..)
	int nStepScaleLevels;

	//! gibt waehrend des Scans den naechsten anzufahrenden Winkel
	//! (Theta bei O2T-Scan)an
	double dArgument2;

	//! gibt fuer den Areascan im standardmodus an ob ein psd-offset verwendet
	//! werden soll in dem Fall muss eine startposition angefahren werden ..
	BOOL bChannelOffset;

	//! neu Kullmann+Reinecker
	DWORD dwStartTimeTicks;
};

//#############################################################################
// TAreaScanCmd
//#############################################################################
//! kommentiert von der Diffr.-gruppe
//! Ablaufsteuerung fuer AreaScan(mit 0.dim. Detektor)SLD

class TAreaScanCmd : public TCmd
{
	//*****************************************************************************
	// Konstruktor-Destruktor
public:
	TAreaScanCmd ( TCmdTag );

	//*****************************************************************************
	// Fortschritt- Kommandoinformtionen
public:
	//! realisiert die Ausgaben in die Statusleiste waehrend der Scan laeuft
	//! (sollte eigentlich nur den Ausg.string liefern die ausgabe sollte dann
	//! mit SendReport() getaetigt werden)
	//! 1. Param. Buffer enhaelt nach aufruf den ausgegeben String der Statusl.
	//! Rueckkehrcode immer true
	virtual BOOL GetShowData ( LPSTR );
	virtual void GetName ( LPSTR aName )
	{
		strcpy( aName, "AreaScanCmd" );
	} // ! neu Kullmann+Reinecker

	//*****************************************************************************
	// Schritte der Kommandoverarbeitung
public:
	//! aufruf aus DoAction() bei erstem Schritt des Scans
	//! Rueckkehrcode immer CMeasure
	ECmdCode FirstStep ( void );

	//! aufruf aus DoAction() waehrend Scan
	//! Rueckkehrcode immer CRecall
	ECmdCode ControlStep ( void );

	//! aufruf aus DoAction() wenn scan fertig ist
	//! Rueckkehrcode immer CRecall
	ECmdCode ReadyStep ( void );

	//*****************************************************************************
	// PRIVATE ATTRIBUTS
private:
	//! Pointer auf zugehoeriges Areascanfenster
	TAreaScanWindow *AreaScanW;

	//! Nummern des Omega- bzw. Thetamotors
	int nOmega, nTheta;

	//! Thetagesamtbereich und Thetaschrittweite
	double dThetaWindow, dThetaWidth;

	//! Omegastart- , Omegaendposition sowie Omegaschrittweite
	double dOmegaMin, dOmegaMax, dOmegaWidth;

	//! Omega-,Thetapos. vor Scanstart
	double dStartPointTheta, dStartPointOmega;

	//! gibt verhaeltnis an um das der Thetamotor bewegt werden soll, wenn sich
	//! der Omegamotor um einen Schritt bewegt (nur bei Omega2Theta verwendet)
	double dMoveRelation;

	//! Omega-,Thetaposition der naechsten Messung
	double dOmega, dTheta;

	//! letzer bzw. akt. Schritt fuer Theta bzw. Omega
	int nMaxThetaPoint, nThetaPoint;
	int nMaxOmegaPoint, nOmegaPoint;

	//! enthaelt Wert fuer 2Theta-Startposition des aktuellen Spektrums
	double d2ThetaStart;
};

/*27.04.2004
//#############################################################################
// TEditWindow
//#############################################################################

class TEditWindow : public TMDIWindow
{
public:
	TEditWindow( HINSTANCE, LPSTR aFilename );
	virtual  ~TEditWindow();

	char FileName[_MAX_PATH];
	BOOL bIsNewFile;
	BOOL bFileChanged;
	void OnCreate( );
	BOOL SetTitle( void );
	BOOL IsModified( void )
	{
		return bFileChanged;
	};
	BOOL CanClear( void );
	BOOL NewFile( void );
	BOOL SaveFile( EAskType );
	BOOL ReadFile( void );
	void ClearModify() {}
	virtual HCURSOR GetCursor( void ) const
	{
		return((HCURSOR)LoadImage(NULL,IDC_IBEAM,IMAGE_CURSOR,0,0,LR_DEFAULTCOLOR));
	}
	void OnSize( WPARAM, LPARAM );
	void DoPaint( HDC, PAINTSTRUCT* );
	void ClearWindow( void ) {}
	void DoSearch( void ) {}
	virtual LPCSTR ClassName( void )
	{
		return "Editor";
	}
	virtual LPCSTR GetCharacteristic( void )
	{
		return "Edit";
	}
};*/

#endif // __M_STEERG_H

//#############################################################################
// LASTLINE
//#############################################################################

