//#############################################################################
//                                                                           //
// WORKFLOW.H                                                                //
//                                                                           //
// Subsystem : Ablaufsteuerung  - Makroverarbeitung)                         //
// Benutung durch andere Subsysteme erforderlich: JA                         //
// Implementation: M_STEERG.CPP                                              //
//---------------------------------------------------------------------------//
// letzte Bearbeitung: Thomas Kullmann, Gnther Reinecker                    //
// Stand:   28.07.2002                                                       //
//                                                                           //
//#############################################################################
//! neu Kullmann+Reinecker:
// * Sinneinheiten zusammengefat
// * einheitlich strukturiert und formatiert
// * Toten Code auskommentiert und mit "//Datum: 28.07.2002" gekennzeichnet
//TSteering:
// * explizite inline Deklaration
// * strkere Datenkapselung und Accessor-/ Mutator-Methoden fr den Zugriff auf geschtzte Attribute
// * vormals globale Symbole in TSteering aufgenommen, teilweise static
// * neue POLLING-Methoden zum Auslesen der Status-/ und Fehlerinformationen
// * Absturzursache und sonstige Fehler beseitigt

#ifndef _WORKFLOW_H_
#define _WORKFLOW_H_

#include "detecuse\detecuse.h" // TDetector
//#include "datavisa\datavisa.h" // TPlotWindow

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//#############################################################################
// globale Typedeklarationen
//#############################################################################

const UINT nMaxArg= 4; // max. Anzahl von speicherbaren Zwischenergebnissen; siehe TSteering::dCalcArg[]

//Datum: 28.07.2002 neu Kullmann+Reinecker
const UINT _MAXMACROCOUNT= 20; // maximale Anzahl von Makros in aMacroList
const UINT _MAXLENP3= 50; // max. 50 Zeichen fr den dritten Kommandoparameter (TCmdTag::P3)
const UINT _MAXLENMAKFILENAME= 20; // max. 20 Zeichen fr den Dateinamen der mak-Datei (TMacroTag::Filename)
const UINT _MAXLENMACRONAME= 20; // max. 20 Zeichen fr die Bezeichnung des Makros (TMacroTag::Name)


// verfgbare Kommando's
enum ECmdId
{
	GotoPeak= 1, GotoIntensity, ChooseAxis, SetWidth,
	AreaScan, Scan, ControlFlank, SaveData, ChooseDetector,
	Calculate, LoadPoint, ShowValue, MoveToPoint, SetFileName,
	Inquire, GotoLine, Stop, SetupScan //Datum: 28.07.2002 GetHWB, ShowHWB, SetupAreaScan
};

// vordefinierte Kommando-Parameter
enum ECmdParam
{
	LargeSide= 1, SmallSide, ToSmallerAngle, ToLargerAngle,
	Equidistant, Interpolation, Reflection, LastGoal, Standard,
	DecreaseWidth, ThisDFPos, BackMove, MaximizeTilt,
	MaximizeCollimator, MaximizeGradient, DynamicWidth, Result,
	StaticStepWidth, IncreasePeak, Relative, AreaScanResult,
	AbsorberUsed, Hwb, Difference, ScanResult,
	Middle, Array, Start, Peak, Min, Max, List, Argument, Opposite,
	ForAreaScan, ForScan
};

// reprsentiert ein Kommando
struct TCmdTag
{
	ECmdId Id;        // Kommandotyp
	ECmdParam P1;                   // erster  Kommandoparameter
	ECmdParam P2;                   // zweiter
	char P3[ _MAXLENP3 + 1 ];    // dritter
};

// verfgbare Makro's
enum EMacroId
{
	InquireHwb= 1, SearchReflection,
	Test, AzimutalJustify, SetupTopography, ScanJob, AreaScanJob //Datum 28.07.2002 BatchMacro, MiddleOfValley
};

// reprsentiert ein Makro
struct TMacroTag
{
	EMacroId Id;             // Makrotyp
	BOOL bIsReady;       // TRUE <--> ist bereit
	char Name[ _MAXLENMACRONAME + 1 ];       // Makrobezeichnung
	char FileName[ _MAXLENMAKFILENAME + 1 ];   // Dateiname, aus der dieses Makro geladen wurde
	int Length;         // Anzahl der enthaltenen Kommandos
	TCmdTag *CmdList;        // Liste der Kommandos
};

//#############################################################################
// TSteering
//#############################################################################
// neu geordnet, Zugriffsschutz, Kommentare: Kullmann+Reinecker

class TSteering
{
	friend class TCmd;
	//Datum: 28.07.2002 friend class TMacroExecuteDlg;
	//Datum: 28.07.2002 friend TIMECALLBACK RecallSteering;

	//*****************************************************************************
	// Konstruktor-Destruktor
private:
	TSteering ( TSteering& ) {} //! neu Kullmann+Reinecker: kein Copy-Konstruktor
	TSteering& operator= ( const TSteering& )
	{
		return *this;
	} //! neu Kullmann+Reinecker: und auch kein Zuweisungsoperator

public:
	TSteering ( void );
	virtual  ~TSteering ( void ); // Testgruppe 21.10.2002
	//*****************************************************************************
	// Makros aus Dateien laden
public:
	// ldt Makros aus STANDARD.MAK
	BOOL Initialize ( const HWND, const int, TDetector* );

	// ldt Makros aus SCAN.MAK
	BOOL LoadMacroByUser ( void );

	//*****************************************************************************
	// Kommando-/ Makroverarbeitung vorbereiten
public:
	// setzt das Fenster, das fr die Aktualisierung des Detektors zustndig ist (cm_CounterSet)
	// gibt den zu verwendeten Antrieb und Detektor an
	BOOL StartUp ( const HWND, const int, TDetector* );

	// regelt wie Fortschrittsanzeigen (whrend der Kommando-/ Makroverarbeitung( auszugeben sind
	void Visualising ( BOOL, BOOL );

	//*****************************************************************************
	// Kommando-/ Makroverarbeitung starten
public:
	// startet die Verarbeitung des angegebenen Makros
	// an das (als zweiten Parameter bergebene) Fenster wird cm_SteeringReady geschickt, sobald die Makroverarbeitung beendet ist
	// Tipp: stattdessen kann die POLLING-Methode IsActive() eingesetzt werden, um zu ermitteln, wann das Makro "fertig" ist
	BOOL StartMacroExecution ( TMacroTag*, HWND );

	// erzeugt und startet ein neues Kommando (Parameter 1 bis 4 sind -typ und -parameter)
	// das bergebene Fenster, ist fr die Aktualisierung des Detektors zustndig (cm_CounterSet)
	int StartCmdExecution ( ECmdId, int, int, LPSTR, HWND );

	//*****************************************************************************
	// Kommando-/ Makroverarbeitung pausieren/ fortsetzen, stoppen
public:
	// pausiert die Makroverarbeitung bzw. setzt diese fort
	BOOL ToggleInterrupt ( void );

	// stoppt die Makroverarbeitung endgltig
	BOOL Reset ( BOOL=TRUE );

	//*****************************************************************************
	// Schnittstelle zu anderen Subsystemen
public:
	// vom Fenster aufzurufen, das cm_CounterSet empfangen hat
	BOOL DetectorRequest ( void );

	// vom Fenster aufzurufen, das wm_WakeUpSteering empfangen hat
	void WakeUp ( void );

	//*****************************************************************************
	// Makroliste bearbeiten: Makros auslesen, lschen und Makroanzahl ermitteln
public:
	// ==> Makro per Typ, Name oder Id zurckgeben; ggf. 0
	TMacroTag *GetMacroById ( EMacroId );
	TMacroTag *GetMacroByNumber ( int );
	TMacroTag *GetMacroByName ( LPSTR );

	// angebenes Makro lschen
	BOOL DeleteMacro ( TMacroTag* );

	// ==> Anzahl der gelandenen Makros
	int GetMacroCount ( void );

	//*****************************************************************************
	// POLLING zum Fortschritt der Makroverarbeitung ( ! neu Kullmann+Reinecker )
public:
	// ==> TRUE <--> Makro ODER Einzelkommando wird derzeit verarbeitet
	BOOL IsActive ( void );

	// ==> TRUE <--> Makro- ODER Einzelkommandoverarbeitung wurde abgebrochen (Reset)
	BOOL IsReset ( void );

	// NUR whrend der Makroverarbeitung werden Fortschrittsinfo's in den drei Parameterm zurckgegeben
	// * erster Parameter: Index des aktuellen Kommandos; Intervall [0 .. <dritter Parameter>-1]
	// * zweiter Parameter: Statusinformationen; wenn keine verfgbar: Bezeichnung des aktuellen Kommandos
	// * dritter Parameter: Anzahl der Kommandos whrend dieser Makrov- oder Einzelkommandoerarbeitung
	// ACHTUNG: Sprungkommandos beeinflussen den Kontrollfluss!
	//          D.h. der Index des aktuelles Kommdandos kann (auch um mehr als 1) erhht ODER erniedrigt werden!
	//          Fortschrittsanzeige demnach nicht mglich.
	// ==> IsActive()
	BOOL GetProgress ( int&, LPSTR, int& );

	//*****************************************************************************
	// Accessor-/ Mutator-Methoden (einiges neu oder vormals als inline deklariert)
public:
	// ==> TRUE <--> Positionierung des ausgewhlten Antriebs war erfolgreich
	BOOL IsPositionValid ( void );

	// aktuelle Absolutposition des ausgewhlten Antriebs
	double GetDistance ( void );

	// kann zum Zwischenspeichern der aktuellen Antriebsposition verwendet werden
	double GetStartPoint ( void );
	void SetStartPoint ( double );

	// aktuelle Intensitt des ausgewhlten Detektors
	float GetIntensity ( void );

	// Faktor (fSigma) des ausgewhlten Detektors
	float GetSigma ( void );

	// Rauschlevel des ausgewhlten Detektors
	float GetNoiseLevel ( void );
	void SetNoiseLevel ( float nl );

	// Divisor, der zur Korrektur der aktuellen Intensitt verwendet werden kann
	float GetNorm ( void );
	void SetNorm ( float norm );

	// sollen Meldung ausgegeben werden (Statuszeile oder Listenfeld - abhngig von Visualising)
	BOOL IsResponding ( void );
	void SetResponse ( BOOL sr );

	// ACHTUNG: Wenn die Halbwertsbreite bislang nicht berechnet wurde
	//          (CalculateCmd-Kommando NOCH nicht ausgefhrt ODER bislang kein Makro verarbeitet)
	//          ist fHWB == 0.
	double GetHwb ( void );
	void SetHwb ( double );

	//*****************************************************************************
	// PRIVATE METHODS
private:
	// bestimmtes Makro aus einer Datei laden
	BOOL LoadMacro ( LPSTR, LPSTR );

	// das angebene Einzelkommando ausfhren; am Ende NotifyCmdReady() aufrufen
	int StartCmdExecution ( TCmdTag );

	// fhrt das nchste Kommando der Makroverarbeitung ODER ruft NotifyCmdReady() oder NotifyMacroReady()
	BOOL ReadyReaction ( void );

	// fhrt das nchste Kommando der Makroverarbeitung aus; ggf. wird NotifyMacroReady() aufgerufen
	int ExecuteNextCmd ( void );

	// cm_SteeringReady an hHostWindow senden
	void NotifyCmdReady ( void ); // Einzelkommandoverarbeitung beendet
	void NotifyMacroReady ( void ); // Makroverarbeitung beendet

	// Kommandopausierung pausieren per Timer: Start, Stop
	BOOL StartTimer ( int );
	BOOL StopTimer ( void );
	static void CALLBACK RecallSteering ( UINT, UINT, DWORD, DWORD, DWORD );

	// Meldung MessageBuffer in der Statuszeile oder einem Listenfeld (je nach Visualising-Aufruf) ausgeben
	void SendReport ( void );

	// parsen von Makrobezeichnung, Kommandobezeichnung und -parametern
	BOOL ParsingMacroId ( TMacroTag&, LPSTR );
	BOOL ParsingCmd ( TCmdTag&, LPSTR, LPSTR, LPSTR, LPSTR );
	static ECmdParam ParsingCmdParam ( LPSTR param );

	//Datum: 04.08.2002 BOOL  GetFileLine( FILE*, LPSTR, int );
	//Datum: 28.07.2002 HWND  GetControlWnd ( void );
	//Datum: 28.07.2002 int   TimerRequest ( void );
	//Datum: 28.07.2002 BOOL  IsMacroCalled ( void );
	//Datum: 28.07.2002 int   ReturnMacroCall ( void );
	//Datum: 28.07.2002 int   CallMacro ( TMacroTag* );

	//*****************************************************************************
	// PUBLIC ATTRIBUTS NUR fr die von TCmd abgeleiteten Kommandos
public:
	// Info's ber Antriebspositionen
	double dGoalDistance;
	double dPeakPoint;
	double dStartPoint;

	// Info's ber Detektormessungen
	float fGoalIntensity;
	float fPeakIntensity;
	float fStartIntensity; // ! nur ein Lesezugriff

	// berechnete Werte
	double dCalcResult;
	double dCalcArg[nMaxArg];


	// Verknpfung auf einen Detektor fr TScanCmd, TAreaScanCmd und TGotoPeakCmd
	TDetector *Monitor;

	//*****************************************************************************
	// PRIVATE ATTRIBUTS
private:
	// Liste der Makros
	TMacroTag aMacroList[_MAXMACROCOUNT]; //Datum: 28.07.2002 neu Kullmann+Reinecker
	int nMacroNumber;

	// derzeit bearbeitetes Makro mit der Liste der enthaltenen Kommandos und dem Index des aktuellen Kommandos
	TMacroTag *TheMacro;
	TCmdTag *TheCmdList;
	int nCmdIdx;

	// derzeit ausgefhrtes [Einzel-]kommando
	TCmd *TheCmd;

	// Makro-/ Einzelkommandoverarbeitung pausiert oder beendet/ abgebrochen
	BOOL bInterrupted;
	BOOL bSteeringActive;
	BOOL bReset;

	// Handle zu dem Fenster mit dem Listenfeld (Ressourcen-Id id_Report), das bei cm_CounterSet DetectorRequest aufrufen muss
	HWND hControlWnd;
	// Handle zu dem Fenster, das ber die Windowsbotschaft cm_SteeringReady benachrichtigt werden soll, wenn die Makroverarbeitung beendet ist
	HWND hHostWindow;

	// zur Meldungsausgabe
	BOOL bSendReport;
	char MessageBuffer[MaxString];
	HWND hReportList;

	// Handle auf den Timer, der die Kommandopausierung erlaubt; 0 <--> Timer ist inaktiv
	// darf nicht static sein, weil RecallSteering sonst auf einem anderen nEvent arbeitet als StartTimer und StopTimer - Compiler-/Linkerfehler?!
	UINT nEvent;

	// gibt an, wie lange [in Millisekunden] die Kommandoverarbeitung dauern soll
	static const UINT nAskTicks;

	// Index des ausgewhlten Antriebs und ausgewhlter Detektor
	int nMotor;
	TDetector *Detector;

	// Detektorinfo's die nur in TSteering benutzt werden
	float fIncomingIntensity;
	float fIntensityNorm;
	float fNoiseLevel;

	// berechnete(r) Wert(e)
	float fHWB;
	BOOL bInquireResult; // ! nur ein Lesezugriff

	// speichert Detektorparameter vor Pausieren der Makroverarbeitung, um diese beim Fortsetzen wiederherstel-len zu knnen
	TExposureSettings exposure;

	//##die vier folgenden Varablen werden im Programmablauf zwar belegt, dann aber nicht weiter benutzt
	//Datum: 28.07.2002 BOOL  b_id_Angle;
	//Datum: 28.07.2002 BOOL  b_id_PeakIntensity;
	//Datum: 28.07.2002 BOOL  b_id_Intensity;
	//Datum: 28.07.2002 BOOL  b_id_HalfWidth;

	//Datum: 28.07.2002 BOOL  b_id_Report;
	//Datum: 28.07.2002 BOOL  bTimerActive;
	//Datum: 28.07.2002 int   nFailureId;
};

//#############################################################################
// TMacroExecuteDlg
//#############################################################################

class TMacroExecuteDlg : public TModelessDlg
{
public:
	TMacroExecuteDlg( TModelessDlg** );
private:
	TMacroTag *TheMacro;
	TDetector *Detector;
	HWND hMacroList;
	BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	void Dlg_OnCommand( HWND, int, HWND, UINT );
	BOOL CanClose( void );
	void LeaveDialog( void );
	int MessageLine;
	BOOL bInterrupted;
	BOOL bActionCaused;
	int SetupTask;
	BOOL bCheckMacro;
};

#endif //_WORKFLOW_H_

//#############################################################################
// LASTLINE
//#############################################################################
