//#############################################################################
//                                                                           //
// MJ_FUNK.CPP                                                               //
//                                                                           //
// Subsystem : neue Manuelle Justage / Funktionalitt                        //
// Benutzung durch andere Subsysteme erforderlich: NEIN                      //
//---------------------------------------------------------------------------//
// Autoren:  Thomas Kullmann, Gnther Reinecker                              //
// Stand  : 23.11.2002                                                       //
//                                                                           //
//#############################################################################

#include "utils\u_utils.h"
#include <assert.h> // ::assert()

#include "manjust\mj_funk.h" // SCHNITTSTELLE fr diese Datei

#include "datavisa\datavisa.h" // "difrkmty.h"
//#include "difrkmty\difrkmty.h" // TOneDimDetector
#include "workflow\workflow.h" // extern Steering

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// externe Abhngigkeiten
//##############################################################################

extern TSteering Steering;
__declspec(dllimport) LPMList lpMList; //Komposition aus TMotor-Objekten

//##############################################################################
// TManJustage
//##############################################################################

//------------------------------------------------------------------------------
// Konstruktor / Destruktor
//------------------------------------------------------------------------------
TManJustage::TManJustage ( void ) : m_MotorList(0)
{
	TMotor *Mtr;
	BOOL bValid;

	m_MotorCount= lpMList->GetAxisNumber(); // Anzahl der Motoren in lpMList
	m_MotorList= new TMotorData[ GetMotorCount() ];
	for ( UINT i= 0; i < GetMotorCount(); i++ )
	{
		Mtr= lpMList->MP( i );
		assert( Mtr ); // Motorliste muss stehts gltiges Motorobjekt zurckgeben
		m_MotorList[i].m_lnkMotor= Mtr;
		m_MotorList[i].m_AxisType= lpMList->ParsingAxis( Mtr->pCharacteristic() );
		sprintf( m_MotorList[i].m_MotorId, "motor%d", Mtr->GetId() ); // Index der ini-Sektion
		// die zuletzt verwendete Betriebsart;
		LONG idx= IniReadLong( GetHWFile(), m_MotorList[i].m_MotorId, "MJ_MotionType", 0 );
		if ( idx == 1 )
			m_MotorList[i].m_MotionType= mtDrive;
		else if ( idx == 2 )
			m_MotorList[i].m_MotionType= mtStep;
		else
			m_MotorList[i].m_MotionType= mtDirect; // default
		// die zuletzt verwendeten Betriebsparameter & Offset
		m_MotorList[i].m_AngleDest= IniReadDouble( GetHWFile(), m_MotorList[i].m_MotorId, "MJ_AngleDest", GetAngleMin(i, bValid), 0, GetAngleMax(i, bValid), GetDigits(i, bValid) ); // SollPos
		m_MotorList[i].m_Speed= IniReadDouble( GetHWFile(), m_MotorList[i].m_MotorId, "MJ_Speed", 0, Mtr->GetSpeed(), Mtr->GetMaxSpeed(), GetDigits(i, bValid) ); // Geschw.
		m_MotorList[i].m_AngleWidth= IniReadDouble( GetHWFile(), m_MotorList[i].m_MotorId, "MJ_AngleWidth", Mtr->GetMinAngleWidth(), Mtr->GetMinAngleWidth(), ( fabs(GetAngleMin(i, bValid)) + GetAngleMax(i, bValid) ), GetDigits(i, bValid) + 1 ); // Schrittweite
		Mtr->SetAngleBias( IniReadDouble( GetHWFile(), m_MotorList[i].m_MotorId, "MJ_Offset", -10000.0, 0.0, +10000.0, GetDigits(i, bValid) ) ); // Offset (keine Intervallgrenzen)
		// derzeit nicht in Bewegung
		m_MotorList[i].m_KzMoving= FALSE;
		// Kennzeichnen, dass bislang keine Daten zwischengespeichert werden
		m_MotorList[i].m_SpAngle= 0;  
		m_MotorList[i].m_HasAngle= FALSE; 
		m_MotorList[i].m_SpMoving= FALSE;  
		m_MotorList[i].m_HasMoving= FALSE; 
	}
	// erst danach darf der Detektor initialisiert werden
	TDetector *actDetector= TDetectorManager::DetectorManager().GetDetector(0, 1); // Detektor fr PSD-Offset
	if ( (actDetector) && (actDetector->GetDimension() == 1) && (actDetector->GetDetectorType() == PsdDetector))
	{
		m_lnkDetector= dynamic_cast<TOneDimDetector*>(actDetector); // muss eindimensional sein
		sprintf( m_DetectorId, "device%d", m_lnkDetector->GetId() ); // Index der ini-Sektion
		m_Channel= IniReadLong( GetHWFile(), m_DetectorId, "MJ_Channel", m_lnkDetector->GetFirstChannel(), m_lnkDetector->GetFirstChannel(), m_lnkDetector->GetLastChannel() ); // KanalOffset (keinesfalls SetChannel verwenden, weil dies Rckwirkungen auf AngleBias htte)
	}
	else 
	{ // keine ausgewhlter Detektor oder Detektor ist nicht eindimensional
		m_lnkDetector= 0; 
		strcpy(m_DetectorId, "");
		m_Channel= 0;
	}
	m_IsMeasuring= FALSE; // Halbwertsbreitenmessung wurde nicht von uns gestartet
	m_KzMeasuring= FALSE;
	// Speeding derzeit nicht aktiv
	m_UpdateCount= 0;
};
//----------------------------------------------------------------------------------------
// Destruktor

TManJustage::~TManJustage ( void )
{
	_FREELIST( m_MotorList );
};
//----------------------------------------------------------------------------------------
// Antriebssteuerung
//----------------------------------------------------------------------------------------
// Bewegung im Direktbetrieb

BOOL TManJustage::DoDirect ( const int aIndex )
{
	if ( !CanDoDirect( aIndex ) )
		return FALSE; // Direktbetrieb kann nicht durchgefhrt werden

	TMotor *Mtr= m_MotorList[aIndex].m_lnkMotor;
	BOOL bValid= FALSE;
	Mtr->SetSpeed( GetSpeed(aIndex, bValid) );  // akt. Geschw. setzen
	Mtr->ActivateDrive(); //Antrieb aktivieren
	if ( (!bValid) || (!Mtr->MoveToAngle( GetAngleDest(aIndex, bValid) )) )
		return FALSE; // anfahren der Sollposition
	// nicht MtrData->m_KzMoving auf TRUE setzen, sonst wird dies TManJustageDlg::OnMotionStarts vorweggeriffen
	return TRUE;
};
//----------------------------------------------------------------------------------------
// Bewegung im Fahrbetrieb

BOOL TManJustage::DoDrive ( const int aIndex, const EDirection aDirection )
{
	if ( !CanDoDrive( aIndex, aDirection ) )
		return FALSE; // Fahrbetrieb kann nicht durchgefhrt werden

	TMotor *Mtr= m_MotorList[aIndex].m_lnkMotor;
	BOOL bValid= FALSE;

	Mtr->SetSpeed( GetSpeed(aIndex, bValid) );
	Mtr->ActivateDrive(); //Antrieb aktivieren
	// zur maximalen Antriebsposition bewegen
	if ( (aDirection == dUp) && (!Mtr->MoveToAngle( GetAngleMax(aIndex, bValid) )) )
		return FALSE; // Motor konnte nicht gestartet werden
	// zur minimalen Antriebsposition bewegen
	if ( (aDirection == dDown) && (!Mtr->MoveToAngle( GetAngleMin(aIndex, bValid) )) )
		return FALSE; // Motor konnte nicht gestartet werden
	// nicht MtrData->m_KzMoving auf TRUE setzen, sonst wird dies TManJustageDlg::OnMotionStarts vorweggeriffen
	return TRUE;
};
//----------------------------------------------------------------------------------------
// Bewegung im Schrittbetrieb

BOOL TManJustage::DoStep ( const int aIndex, const EDirection aDirection )
{
	if ( !CanDoStep( aIndex, aDirection ) )
		return FALSE;  // Schrittbetrieb kann nicht durchgefhrt werden

	TMotor *Mtr= m_MotorList[aIndex].m_lnkMotor;
	BOOL bValid= FALSE;
	Mtr->SetSpeed( Mtr->GetMaxSpeed() );  // akt. Geschw. soll maximal sein
	TAngle AngleWidth= GetAngleWidth(aIndex, bValid);
	if ( !bValid )
		return FALSE;  // lesen der Schrittweite ist fehlgeschlagen

	Mtr->ActivateDrive(); //Antrieb aktivieren
	// zu <aktuelle Position> + <Schrittweite> bewegen
	if (( aDirection == dUp ) && ( !Mtr->MoveByAngle( AngleWidth ) ))
		return FALSE; // Motor konnte nicht gestartet werden
	// zu <aktuelle Position> - <Schrittweite> bewegen
	if (( aDirection == dDown ) && ( !Mtr->MoveByAngle( -AngleWidth ) ))
		return FALSE; // Motor konnte nicht gestartet werden
	// nicht MtrData->m_KzMoving auf TRUE setzen, sonst wird dies TManJustageDlg::OnMotionStarts vorweggeriffen
	return TRUE;
};
//----------------------------------------------------------------------------------------
// Stoppen der Bewegung, unabhngig von der Betriebsart

BOOL TManJustage::DoStop ( const int aIndex )
{
	if ( !CanDoStop( aIndex ) )
		return FALSE; // Bewegung kann nicht gestoppt werden

	TMotor *Mtr= m_MotorList[aIndex].m_lnkMotor;
	Mtr->StopDrive( TRUE ); //restartable
	// nicht MtrData->m_KzMoving auf FALSE setzen, sonst wird dies TManJustageDlg::OnMotionStops vorweggeriffen
	return TRUE;
};
//----------------------------------------------------------------------------------------
// Stoppen aller Antriebe, der Halbwertsbreitenmessung und der Detektormessung

void TManJustage::DoStopEverything ( void ) 
{
	//zuerst die Halbwertsbreitenmessung stoppen
	DoStopMeasure();
	//Detektor neu starten
	TDetector *Detector= GetDetector();
	if ( Detector ) 
  {
		Detector->MeasureStop();
		Detector->PopSettings();
		Detector->MeasureStart();
	};
	//letztendlich auch die Antriebe stoppen
	for (UINT i= 0; i < GetMotorCount(); i++) 
  {
		TMotor *Mtr= m_MotorList[i].m_lnkMotor;
		Mtr->SetCorrectionState( FALSE );
		DoStop(i);
	};
};
//----------------------------------------------------------------------------------------
// Statusinformationen
//----------------------------------------------------------------------------------------
// Test ob Index im Intervall

BOOL TManJustage::IsIndexValid ( const int aIndex ) const 
{
	return ( (0 <= aIndex) && (aIndex < (int)GetMotorCount()) );
};
//----------------------------------------------------------------------------------------
// Status des Offset fr <Antrieb>

BOOL TManJustage::HasOffset ( const int aIndex, BOOL &aValid ) const 
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return FALSE;

	aValid= TRUE;
	return m_MotorList[aIndex].m_lnkMotor->IsDistanceRelative();
};
//----------------------------------------------------------------------------------------
// Status der Bewegung

BOOL TManJustage::IsMoving ( const int aIndex, BOOL &aValid ) 
{
	if (m_UpdateCount>0) { // Speeding
		aValid= FALSE;
		if ( !IsIndexValid ( aIndex ) )
			return FALSE;

		// wenn ntig versuchen Antriebsstatus zwischenspeichern
		if ( !m_MotorList[aIndex].m_HasMoving ) 
		{
			m_MotorList[aIndex].m_SpMoving= DetermineMoving( aIndex, aValid );
			m_MotorList[aIndex].m_HasMoving= aValid;
		}
		// wenn mglich diesen zwischengespeicherten Antriebsstatus benutzen
		if ( m_MotorList[aIndex].m_HasMoving ) 
		{
			aValid= TRUE;
			return m_MotorList[aIndex].m_SpMoving;
		}
	}
	return DetermineMoving( aIndex, aValid );
};
//----------------------------------------------------------------------------------------
// Antrieb kalibriert?

BOOL TManJustage::IsCalibrated ( const int aIndex, BOOL &aValid ) const 
{
	aValid= FALSE;
	if ( !IsIndexValid (aIndex) )
		return FALSE;

	aValid= TRUE;
	return ( m_MotorList[aIndex].m_lnkMotor->IsCalibrated() );
};
//----------------------------------------------------------------------------------------
// GET-Methoden
//----------------------------------------------------------------------------------------
// Betriebsart

EMotionType TManJustage::GetMotionType ( const int aIndex, BOOL& aValid) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return mtDirect;

	aValid= TRUE;
	return m_MotorList[aIndex].m_MotionType; // lokalen Wert zurckgeben
};
//----------------------------------------------------------------------------------------
// Sollposition

TAngle TManJustage::GetAngleDest ( const int aIndex, BOOL &aValid) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return FALSE;

	TMotorData *MtrData= &m_MotorList[aIndex];
  TAngle result= MtrData->m_AngleDest;
  TMotor *Mtr= MtrData->m_lnkMotor;
  SetInRange( result, Mtr->GetAngleRelativeMin(), Mtr->GetAngleRelativeMax() ); // Anpassung an Intervallgrenzen
  aValid= TRUE;
  return result; // lokalen Wert zurckgeben
};
//----------------------------------------------------------------------------------------
// Geschwindigkeit

TSpeed TManJustage::GetSpeed ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return FALSE;

	aValid= TRUE;
	return m_MotorList[aIndex].m_Speed; // lokalen Wert zurckgeben
};
//----------------------------------------------------------------------------------------
// Schrittweite

TAngle TManJustage::GetAngleWidth ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return FALSE;

  aValid= TRUE;
  return m_MotorList[aIndex].m_AngleWidth; // lokalen Wert zurckgeben (muss nicht an Intervallgrenzen angepasst werden, weil sich diese nicht ndern; Anpassung kann bei SetAngleWidth erfolgen)
};
//----------------------------------------------------------------------------------------
// OnTimer() will auf Start/ Stop der Bewegung reagieren

BOOL TManJustage::GetKzMoving ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return FALSE;

	aValid= TRUE;
	return m_MotorList[aIndex].m_KzMoving; // lokalen Wert zurckgeben
};
//----------------------------------------------------------------------------------------
// Istposition als Winkelangabe

TAngle TManJustage::GetAngle ( const int aIndex, BOOL &aValid )
{
	if ( m_UpdateCount>0 ) { // Speeding
		aValid= FALSE;
		if ( !IsIndexValid ( aIndex ) )
			return FALSE;

		// wenn ntig versuchen letzte Antriebsposition zwischenspeichern
		if ( !m_MotorList[aIndex].m_HasAngle ) 
		{
			m_MotorList[aIndex].m_SpAngle= DetermineAngle( aIndex, aValid );
			m_MotorList[aIndex].m_HasAngle= aValid;

		}
		// wenn mglich diese zwischengespeicherte letzte Antriebsposition benutzen
		if ( m_MotorList[aIndex].m_HasAngle ) 
		{
			aValid= TRUE;
			return m_MotorList[aIndex].m_SpAngle;
		}
	}
	return DetermineAngle( aIndex, aValid );
};
//----------------------------------------------------------------------------------------
// Minimalposition als Winkelangabe

TAngle TManJustage::GetAngleMin ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return 0.0;

	aValid= TRUE;
	return min( m_MotorList[aIndex].m_lnkMotor->GetAngleRelativeMin(),   // 22.01.2003: auch Zahlendreher in ini-Datei untersttzen
				m_MotorList[aIndex].m_lnkMotor->GetAngleRelativeMax() );
};
//----------------------------------------------------------------------------------------
// Maximalposition als Winkelangabe

TAngle TManJustage::GetAngleMax ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return 0.0;

	aValid= TRUE;
	return max( m_MotorList[aIndex].m_lnkMotor->GetAngleRelativeMin(),   // 22.01.2003: auch Zahlendreher in ini-Datei untersttzen
				m_MotorList[aIndex].m_lnkMotor->GetAngleRelativeMax() );
};
//----------------------------------------------------------------------------------------
// Nachkommastellen

UINT TManJustage::GetDigits ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return 2;

	aValid= TRUE;
	return m_MotorList[aIndex].m_lnkMotor->GetDigits();
};
//----------------------------------------------------------------------------------------
// Einheit des Antriebs

LPCSTR TManJustage::GetUnit ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return "";

	aValid= TRUE;
	return m_MotorList[aIndex].m_lnkMotor->GetUnitName();
};
//----------------------------------------------------------------------------------------
// Anzahl der Antriebe in der Antriebsliste

UINT TManJustage::GetMotorCount ( void ) const
{
	return m_MotorCount;
};
//----------------------------------------------------------------------------------------
// Zeiger auf einen Eintrag in der Motorenliste

LPCSTR TManJustage::GetMotorName ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return "";

	aValid= TRUE;
	return m_MotorList[aIndex].m_lnkMotor->pCharacteristic();
};
//----------------------------------------------------------------------------------------
// Index des angegebenen Antriebs; ggf. -1

int TManJustage::GetMotorIdx ( const LPCSTR aName ) const
{
	BOOL bValid;
	for (UINT i= 0; i < GetMotorCount(); i++)
		if ( (strcmp( aName, GetMotorName(i, bValid)) == 0) && (bValid) )
			return i;
	return -1; // nicht gefunden
}
//----------------------------------------------------------------------------------------
// Index des angegebenen Antriebs; ggf. -1

int TManJustage::GetMotorIdx ( const EAxisType aAxisType ) const
{
	if ( aAxisType == (EAxisType)0 )
		return -1; // kann nicht eindeutig identifiziert werden
	for (UINT i= 0; i < GetMotorCount(); i++)
		if ( m_MotorList[i].m_AxisType == aAxisType )
			return i;
	return -1; // nicht gefunden
};
//----------------------------------------------------------------------------------------
// Antrieb der zur Halbwertsbreitenmessung verwendet wird
EAxisType TManJustage::GetHwbAxis() 
{ 
	if ( GetMotorIdx(DF) != -1 ) return DF;
	else if ( GetMotorIdx(Omega) != -1 ) return Omega;
	else return EAxisType(0);
}

//----------------------------------------------------------------------------------------
// standardisierte Antriebsbezeichnung

UINT TManJustage::ParsingAxis ( const LPCSTR aCharacteristic )
{
	return TMList::ParsingAxis( aCharacteristic );
};
//----------------------------------------------------------------------------------------
// SET-Methoden
//----------------------------------------------------------------------------------------
// Betriebsart

BOOL TManJustage::SetMotionType ( const int aIndex, EMotionType &aMotionType )
{
	if ( !CanSetMotionType( aIndex ) )
		return FALSE; // Betriebsart kann nicht gesetzt werden

	TMotorData *MtrData= &m_MotorList[aIndex];
	LONG idx;
	if ( aMotionType == mtDrive )
		idx= 1;
	else if ( aMotionType == mtStep )
		idx= 2;
	else
		idx= 0; // mtDirect
	IniWriteLong( GetHWFile(), MtrData->m_MotorId, "MJ_MotionType", idx );
	MtrData->m_MotionType= aMotionType; // lokal setzen
	return TRUE;
};
//----------------------------------------------------------------------------------------
// Sollposition

BOOL TManJustage::SetAngleDest ( const int aIndex, TAngle& aAngleDest )
{
	BOOL bValid= FALSE;
	if ( !CanSetAngleDest( aIndex ) )
		return FALSE; // Sollposition kann nicht gesetzt werden

	TMotorData *MtrData= &m_MotorList[aIndex];
	TMotor *Mtr= MtrData->m_lnkMotor;
	SetInRange( aAngleDest, Mtr->GetAngleRelativeMin(), Mtr->GetAngleRelativeMax() ); // Anpassung an Intervallgrenzen
	MtrData->m_AngleDest= aAngleDest; // lokal setzen
	IniWriteDouble( GetHWFile(), MtrData->m_MotorId, "MJ_AngleDest", aAngleDest, GetDigits(aIndex, bValid) );
	return bValid;
};
//----------------------------------------------------------------------------------------
// Geschwindigkeit

BOOL TManJustage::SetSpeed ( const int aIndex, TSpeed &aSpeed )
{
	BOOL bValid= FALSE;
	if ( !CanSetSpeed( aIndex ) )
		return FALSE; // Bewegungsgeschwindigkeit kann nicht gesetzt werden

	TMotorData *MtrData= &m_MotorList[aIndex];
	TMotor *Mtr= MtrData->m_lnkMotor;
	aSpeed= max( Mtr->GetMinSpeed(), min( fabs(aSpeed), Mtr->GetMaxSpeed() ) ); // Anpassung an Intervallgrenzen
	MtrData->m_Speed= aSpeed; // lokal setzen
	Mtr->SetSpeed( aSpeed );
	IniWriteDouble( GetHWFile(), MtrData->m_MotorId, "MJ_Speed", aSpeed, GetDigits(aIndex, bValid) );
	return bValid;
};
//----------------------------------------------------------------------------------------
// Schrittweite

BOOL TManJustage::SetAngleWidth ( const int aIndex, TAngle &aAngleWidth)
{
	BOOL bValid= FALSE;
	if ( !CanSetAngleWidth( aIndex ) )
		return FALSE; // Schrittweite kann nicht gesetzt werden

	TMotorData *MtrData= &m_MotorList[aIndex];
	TMotor *Mtr= MtrData->m_lnkMotor;
	Mtr->SetAngleWidth( aAngleWidth );  // im Motor setzen (hier auch Einpassung ins Intervall MinimalWith-MaximalWidth)
	aAngleWidth= Mtr->GetAngleWidth();  // korrigierten Wert lesen
	MtrData->m_AngleWidth= aAngleWidth; // lokal setzen
	IniWriteDouble( GetHWFile(), MtrData->m_MotorId, "MJ_AngleWidth", aAngleWidth, GetDigits(aIndex, bValid) + 1 );
	return bValid;
};
//----------------------------------------------------------------------------------------
// OnTimer() hat auf Start/ Stop der Bewegung reagiert

BOOL TManJustage::SetKzMoving ( const int aIndex, BOOL aIsMoving)
{
	if ( !IsIndexValid ( aIndex ) )
		return FALSE;

	m_MotorList[aIndex].m_KzMoving= aIsMoving;
	return TRUE;
};
//----------------------------------------------------------------------------------------
// Offset bzw relative Null
//----------------------------------------------------------------------------------------
// setzen der relativen Null

BOOL TManJustage::SetRelativeZero ( const int aIndex )
{
	BOOL bValid= FALSE;
	if ( !CanSetOffset(aIndex) )
		return FALSE; // darf nicht gesetzt werden

	TMotorData *MtrData= &m_MotorList[aIndex];
	TMotor *Mtr= MtrData->m_lnkMotor;
	ResetChannelOffset(aIndex); // hat Rckwirkungen auf SetRelativeZero, deshalb davor aufrufen
	if (!Mtr->SetRelativeZero())
		return FALSE; // TRUE wenn erfolgreich, d.h bei Stillstand des Antriebs

	IniWriteDouble( GetHWFile(), MtrData->m_MotorId, "MJ_Offset", Mtr->GetAngleBias(), GetDigits(aIndex, bValid) );
	return bValid;
};
//----------------------------------------------------------------------------------------
// aufheben der relativen Null

BOOL TManJustage::ResetRelativeZero ( const int aIndex )
{
	BOOL bValid= FALSE;
	if ( !CanResetRelativeZero(aIndex) )
		return FALSE; // darf nicht gesetzt werden

	TMotorData *MtrData= &m_MotorList[aIndex];
	TMotor *Mtr= MtrData->m_lnkMotor;
	ResetChannelOffset(aIndex); // hat Rckwirkungen auf ResetRelativeZero, deshalb davor aufrufen
	if ( !Mtr->ResetRelativeZero() )
		return FALSE; // TRUE wenn erfolgreich, d.h bei Stillstand des Antriebs

	IniWriteDouble( GetHWFile(), MtrData->m_MotorId, "MJ_Offset", 0, GetDigits(aIndex, bValid) );
	return bValid;
};
//----------------------------------------------------------------------------------------
// lesen des aktuellen Offset fr <Antrieb>

TAngle TManJustage::GetOffset ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return 0.0;

	aValid= TRUE;
	return m_MotorList[aIndex].m_lnkMotor->GetAngleBias();
};
//----------------------------------------------------------------------------------------
// setzen des Offset fr <Antrieb> direkt

BOOL TManJustage::SetOffset ( const int aIndex, TAngle &aOffset )
{
	BOOL bValid= FALSE;
	if ( !CanSetOffset( aIndex ) )
		return FALSE; // darf nicht gesetzt werden

	TMotorData *MtrData= &m_MotorList[aIndex];
	TMotor *Mtr= MtrData->m_lnkMotor;
	ResetChannelOffset(aIndex); // hat Rckwirkungen auf SetAngleBias, deshalb davor aufrufen
	if (!Mtr->SetAngleBias( aOffset ))
		return FALSE;  // an Intervallgrenzen anpassen und setzen

	IniWriteDouble( GetHWFile(), MtrData->m_MotorId, "MJ_Offset", aOffset, GetDigits(aIndex, bValid) );
	return bValid;
};
//----------------------------------------------------------------------------------------
// Berechnungen zum Offset fr <Antrieb>
// Werte werden nicht in den Antrieb bernommen, dafr SetOffset... verwenden
//----------------------------------------------------------------------------------------
// Position in Offset umrechnen

TAngle TManJustage::CalcOffsetFromAngle ( const int aIndex, TAngle &aAngle ) const
{
	BOOL bValid= FALSE;
	if ( !IsIndexValid( aIndex ) )
		return 0.0;

	TAngle Angle= m_MotorList[aIndex].m_lnkMotor->GetAngleAbs(FALSE, bValid); // absolute Position
	if (!bValid)
		return 0.0;  // an Intervallgrenzen anpassen

	return aAngle - Angle; // neuer Offset ist: neue Position - absoluter Position
};
//----------------------------------------------------------------------------------------
// Offset in Position umrechnen

TAngle TManJustage::CalcAngleFromOffset ( const int aIndex, TAngle &aOffset ) const
{
	BOOL bValid= FALSE;
	if ( !IsIndexValid( aIndex ) )
		return 0.0;

	TMotorData *MtrData= &m_MotorList[aIndex];
	TMotor *Mtr= MtrData->m_lnkMotor;
	TAngle AngleAbs= Mtr->GetAngleAbs(FALSE, bValid); // absolute Position
	if (!bValid)
		return 0.0;  // an Intervallgrenzen anpassen

	TAngle NewAngle= (aOffset + AngleAbs);
	SetInRange( NewAngle, Mtr->GetAngleMin(), Mtr->GetAngleMax() ); // Anpassung an Intervallgrenzen
	aOffset= NewAngle - AngleAbs;
	return NewAngle;  // neue Position ist: Offset + absoluter Antriebsposition
};
//----------------------------------------------------------------------------------------
// ChannelOffset
//----------------------------------------------------------------------------------------
// Index DES PSD-Antriebs; ggf. -1

int TManJustage::GetDetectorAxisIdx ( void ) const
{
	if ( m_lnkDetector != 0 )
	{
		EAxisType axis= m_lnkDetector->GetAxis();
		if ( axis != (EAxisType)0 )
			return GetMotorIdx(axis);
	}
	return -1;
};
//----------------------------------------------------------------------------------------
// Nachkommastellen der Kanalbreite

UINT TManJustage::GetDetectorDigits ( BOOL &aValid ) const
{
	aValid= ( m_lnkDetector != 0 );
	if ( aValid ) return m_lnkDetector->GetAngleStepDigits();

	return 5;
}
//----------------------------------------------------------------------------------------
// gibt die Bezeichnung des Detektors zurck; ggf. "PSD"

LPCSTR TManJustage::GetDetectorName ( BOOL &aValid ) const
{
	aValid= (m_lnkDetector != 0);
	if ( aValid ) return m_lnkDetector->GetCharacteristic();

	return "PSD";
}
//----------------------------------------------------------------------------------------
// 1dim Detector und dessen Achse vorhanden ?

BOOL TManJustage::HasDetectorAxis ( void ) const
{
	return ( (m_lnkDetector != 0) && (GetDetectorAxisIdx() != -1) );
};
//----------------------------------------------------------------------------------------
// lesen des aktuellen Kanals

int TManJustage::GetChannel ( void ) const
{
	if ( !HasDetectorAxis() )
		return 0;
	return m_Channel;
};
//----------------------------------------------------------------------------------------
// lesen des aktuellen Verhltnis: Winkel pro Kanal

TAngle TManJustage::GetAnglePerChannel ( void ) const
{
	if ( !HasDetectorAxis() )
		return 0;
	return CalcValueInUnit(m_lnkDetector->GetUnit(), m_lnkDetector->GetWidth());
};
//----------------------------------------------------------------------------------------
// setzen des aktuellen Kanals

BOOL TManJustage::SetChannel ( int &aChannel )
{
	BOOL bValid;
	if ( !HasDetectorAxis() )
	{
		m_Channel= 0;
		return FALSE;
	}

	int idx= GetDetectorAxisIdx();
	aChannel= max( m_lnkDetector->GetFirstChannel(), min( aChannel, m_lnkDetector->GetLastChannel() ) ); // Anpassung
	ResetChannelOffset(idx); // unbedingt vor GetOffset aufrufen, um das Antriebsoffset (ohne das aktuelle Kanaloffset!) auszulesen
	TAngle NewOffset= GetOffset(idx, bValid) + CalcChannelOffset(aChannel);
	if ( (!bValid) || (!SetOffset(idx, NewOffset)) ) // Offset konnte nicht gelesen oder gesetzt werden
		return FALSE;

	m_Channel= aChannel; // erst nach SetOffset, weil m_Channel dort auf FirstChannel gesetzt wird (ResetChannelOffset)
	IniWriteLong( GetHWFile(), m_DetectorId, "MJ_Channel", m_Channel ); // KanalOffset
	return TRUE;
};
//----------------------------------------------------------------------------------------
// Kanal-Offset Null setzen

BOOL TManJustage::ResetChannelOffset( const int aIndex )
{
	BOOL bValid= TRUE;
	if ( !HasDetectorAxis() )
		return FALSE; // ist kein Detektor-Antrieb vorhanden
	if ( (aIndex == -1) || (aIndex != GetDetectorAxisIdx()) )
		return FALSE; // der aktuelle ist nicht DER Detektor-Antrieb
    if ( m_Channel == m_lnkDetector->GetFirstChannel() )
		return FALSE; // ist bereits zurckgesetzt (wichtig, sonst Endlosrekursion bei *1*

	TAngle NewOffset= GetOffset(aIndex, bValid) - CalcChannelOffset(m_Channel); // Anteil des derzeit gesetzen Kanaloffsets subtrahieren
	m_Channel= m_lnkDetector->GetFirstChannel(); // erst NACH CalcChannelOffset, aber VOR SetOffset setzen (sonst Endlosrekursion bei *1*)
	if ( bValid ) SetOffset(aIndex, NewOffset); // *1*

	IniWriteLong( GetHWFile(), m_DetectorId, "MJ_Channel", m_Channel ); // KanalOffset
	return TRUE;
};
//----------------------------------------------------------------------------------------
// Berechnung des Kanal-Offset

TAngle TManJustage::CalcChannelOffset ( TChannel &aChannel ) const
{
	if ( !HasDetectorAxis() )
		return 0.0;
	aChannel= max( m_lnkDetector->GetFirstChannel(), min( aChannel, m_lnkDetector->GetLastChannel() ) ); // Anpassung
	return ( -1 * (aChannel-m_lnkDetector->GetFirstChannel()) * CalcValueInUnit(m_lnkDetector->GetUnit(), m_lnkDetector->GetWidth()) );
};
//----------------------------------------------------------------------------------------
// Berechnung der neuen Antriebsposition

TAngle TManJustage::CalcChannelAngle ( TChannel &aChannel ) const
{
	BOOL bValid= FALSE;
	if ( !HasDetectorAxis() )
		return 0.0;
	int idx= GetDetectorAxisIdx();
	if ( idx == -1 )
		return 0.0;

	TMotorData *MtrData= &m_MotorList[ idx ];
	TMotor *Mtr= MtrData->m_lnkMotor;
	TChannel ActChannel= m_Channel; // weil CalcChannelOffset eine Referenz erwartet (dies wrde im Wiederspruch zur const-Methodendeklaration stehen)
	return ( Mtr->GetAngle( FALSE, bValid) - CalcChannelOffset(ActChannel) + CalcChannelOffset(aChannel) );
};
//----------------------------------------------------------------------------------------
// Halbwertsbreitenmessung
//----------------------------------------------------------------------------------------

TDetector *TManJustage::GetDetector ( void ) const
{
	return TDetectorManager::DetectorManager().GetDetector();
};
//----------------------------------------------------------------------------------------

void TManJustage::UpdateDetector( void ) const
{
	GetDetector()->UpdateViews(TRUE);
	Steering.DetectorRequest();
};
//----------------------------------------------------------------------------------------
// starten der Messung

BOOL TManJustage::DoStartMeasure ( const HWND aControlWnd )
{
	if ( (!CanDoStartMeasure()) || (!aControlWnd) )
		return FALSE;  // Halbwertsbreitenmessung kann nicht gestartet werden

	Steering.Reset();  // aktive Makroverarbeitung abbrechnen
	Steering.StartUp( aControlWnd, 0, GetDetector() );  // Fensterhandle zum Fenster, das die Bildschirmaktualisierung des Zhlerfensters bernimmt; ausgewhlten Detektor setzen; zweiter Paramater ("Beugung fein" wird vom Makro selbst ausgewhlt) wird nicht bentigt
	Steering.Visualising( TRUE, FALSE );  //Fortschritt NUR in Statuszeile ausgeben
  if ( Steering.StartMacroExecution( Steering.GetMacroById(InquireHwb), 0 ) ) { // Makro starten; zweiter Parameter wird nicht bentigt
    m_IsMeasuring= TRUE;
    return TRUE;    
  } else return FALSE;
};
//----------------------------------------------------------------------------------------
// stoppen der Messung

BOOL TManJustage::DoStopMeasure ( void )
{
  if ( !CanDoStopMeasure() ) return FALSE;
	return Steering.Reset(); //zum Stoppen der Makroverarbeitung; ToggleInterrupt zum Pausieren
};
//----------------------------------------------------------------------------------------
// ==> TRUE <--> Messung aktiv

BOOL TManJustage::IsMeasuring ( void ) const
{
	return Steering.IsActive();
};
//----------------------------------------------------------------------------------------
// ==> Messung abgebrochen?

BOOL TManJustage::IsMeasureReset ( void ) const
{
	return Steering.IsReset();
};

//----------------------------------------------------------------------------------------
//OnTimer() will auf Start/ Stop der Makroverarbeitung reagieren

BOOL TManJustage::GetKzMeasuring ( void ) const
{
	return m_KzMeasuring;
};
//----------------------------------------------------------------------------------------
// ==> TRUE <--> Messung aktiv

BOOL TManJustage::GetMeasureProgress ( int &aCmdIdx, LPSTR aProgress, int &aCmdCount ) const
{
	aCmdIdx= 0;
	aProgress[0]= 0;
	aCmdCount= 0;
	return Steering.GetProgress( aCmdIdx, aProgress, aCmdCount );
};
//----------------------------------------------------------------------------------------
// ==> HWB

double TManJustage::GetMeasureHwb ( void ) const
{
	return Steering.GetHwb();
};
//----------------------------------------------------------------------------------------
//OnTimer() hat auf Start/ Stop der Makroverarbeitung reagiert

void TManJustage::SetKzMeasuring ( const BOOL aIsMeasuring )
{
	m_KzMeasuring= aIsMeasuring;
};
//----------------------------------------------------------------------------------------
// Zustand fr Bewegung ermitteln
//----------------------------------------------------------------------------------------
// ist Direktbetrieb mglich

BOOL TManJustage::CanDoDirect ( const int aIndex )
{
	BOOL bValid= FALSE;
	// TRUE <-> gltiger Antrieb, nicht in Bewegung (d.h. auch Halbwertsbreitenmessung ist inaktiv)
	if ( (IsMoving( aIndex, bValid)) || (!bValid) )
		return FALSE; // inclusive IsIndexValid

	TMotorData *MtrData= &m_MotorList[aIndex];
	// TRUE <--> im Direktbetrieb
	return ( MtrData->m_MotionType == mtDirect );
};
//----------------------------------------------------------------------------------------
// ist Fahrbetrieb mglich

BOOL TManJustage::CanDoDrive ( const int aIndex, const EDirection aDirection )
{
	BOOL bValid= FALSE;
	// TRUE <-> gltiger Antrieb, nicht in Bewegung (d.h. auch Halbwertsbreitenmessung ist inaktiv)
	if ( (IsMoving( aIndex, bValid)) || (!bValid) )
		return FALSE; // inclusive IsIndexValid

	TMotorData *MtrData= &m_MotorList[aIndex];
	// TRUE <-> im Fahrbetrieb
	if ( MtrData->m_MotionType != mtDrive )
		return FALSE;

	TMotor *Mtr= MtrData->m_lnkMotor;
	TAngle Angle= Mtr->GetAngle(FALSE, bValid);
	// TRUE <--> im Fahrbetrieb
	if ( !bValid )
		return FALSE;  // Auslesen der akt. Position ist fehlgeschlagen

	//Zielposition bereits erreicht?
	if ( aDirection == dUp )
		return ( Angle < GetAngleMax(aIndex, bValid) );
	else
		return ( GetAngleMin(aIndex, bValid) < Angle ); // aDirection==dDown
};
//----------------------------------------------------------------------------------------
// ist Schrittbetrieb mglich

BOOL TManJustage::CanDoStep ( const int aIndex, const EDirection aDirection )
{
	BOOL bValid= FALSE;
	// TRUE <-> gltiger Antrieb, nicht in Bewegung (d.h. auch Halbwertsbreitenmessung ist inaktiv)
	if ( (IsMoving( aIndex, bValid)) || (!bValid) )
		return FALSE; // inclusive IsIndexValid

	TMotorData *MtrData= &m_MotorList[aIndex];
	// TRUE <--> im Schrittbetrieb
	if ( MtrData->m_MotionType != mtStep )
		return FALSE;

	TMotor *Mtr= MtrData->m_lnkMotor;
	TAngle Angle= Mtr->GetAngle(FALSE, bValid);
	// TRUE <--> Auslesen der akt. Position erfolgreich
	if ( !bValid )
		return FALSE;

	TAngle AngleWidth= GetAngleWidth(aIndex, bValid);
	//Ist die Zielposition des nchsten Schrittes im Intervall
	if ( aDirection == dUp )
		return ( (Angle + AngleWidth) < GetAngleMax(aIndex, bValid) );
	else
		return ( GetAngleMin(aIndex, bValid) < (Angle - AngleWidth) ); // aDirection==dDown
};
//----------------------------------------------------------------------------------------
// ist Stoppen mglich

BOOL TManJustage::CanDoStop ( const int aIndex )
{
	BOOL bValid= FALSE;

	// TRUE <--> gltiger Antrieb, in Bewegung; Halbwertsbreitenmessung ist inaktiv
	return (IsMoving(aIndex, bValid)) && (bValid) && (!IsMeasuring() );
};
//----------------------------------------------------------------------------------------
// Zustand zum Setzen von Parametern ermitteln
//----------------------------------------------------------------------------------------
// kann Betriebsart gesetzt werden

BOOL TManJustage::CanSetMotionType ( const int aIndex )
{
	BOOL bValid= FALSE;

	// TRUE <--> gltiger Antrien, nicht in Bewegung (d.h. auch Halbwertsbreitenmessung ist inaktiv)
	return ( (!IsMoving( aIndex, bValid )) && (bValid) );
};
//----------------------------------------------------------------------------------------
// kann Sollpostion gesetzt werden

BOOL TManJustage::CanSetAngleDest ( const int aIndex )
{
	BOOL bValid= FALSE;
	// TRUE <-> gltiger Antrieb, nicht in Bewegung (d.h. auch Halbwertsbreitenmessung ist inaktiv)
	if ( (IsMoving( aIndex, bValid)) || (!bValid) )
		return FALSE; // inclusive IsIndexValid

	TMotorData *MtrData= &m_MotorList[aIndex];
	// TRUE <--> im Direktbetrieb
	return ( MtrData->m_MotionType == mtDirect );
};
//----------------------------------------------------------------------------------------
// kann Geschwindigkeit gesetzt werden

BOOL TManJustage::CanSetSpeed ( const int aIndex )
{
	BOOL bValid= FALSE;
	// TRUE <-> gltiger Antrieb, nicht in Bewegung (d.h. auch Halbwertsbreitenmessung ist inaktiv)
	if ( (IsMoving( aIndex, bValid)) || (!bValid) )
		return FALSE; // inclusive IsIndexValid

	TMotorData *MtrData= &m_MotorList[aIndex];
	// TRUE <--> im Direkt- ODER Fahrbetrieb
	return ( (MtrData->m_MotionType == mtDirect) || (MtrData->m_MotionType == mtDrive) );
};
//----------------------------------------------------------------------------------------
// kann Schrittweite gesetzt werden

BOOL TManJustage::CanSetAngleWidth ( const int aIndex )
{
	BOOL bValid= FALSE;
	// TRUE <-> gltiger Antrieb, nicht in Bewegung (d.h. auch Halbwertsbreitenmessung ist inaktiv)
	if ( (IsMoving( aIndex, bValid)) || (!bValid) )
		return FALSE; // inclusive IsIndexValid

	TMotorData *MtrData= &m_MotorList[aIndex];
	// TRUE <--> im Schrittbetrieb
	return ( MtrData->m_MotionType == mtStep );
};

//----------------------------------------------------------------------------------------
// setzen des Offset fr <Antrieb> mglich ?

BOOL TManJustage::CanSetOffset ( const int aIndex )
{
	BOOL bValid= FALSE;

	// TRUE <-> gltiger Antrieb, nicht in Bewegung (d.h. auch Halbwertsbreitenmessung ist inaktiv)
	return ( (!IsMoving(aIndex, bValid)) && (bValid) );
};
//----------------------------------------------------------------------------------------
// aufheben der Relativen Null

BOOL TManJustage::CanResetRelativeZero ( const int aIndex )
{
	BOOL bValid= FALSE;
	// TRUE <--> gltiger Antrieb, nicht in Bewegung, Offset <> 0
	return ( (CanSetOffset(aIndex)) && (HasOffset(aIndex, bValid)) && (bValid) );
};
//----------------------------------------------------------------------------------------
// kann Halbwertsbreitenmessung gestartet werden

BOOL TManJustage::CanDoStartMeasure ( void )
{
	BOOL bValid;
	// TRUE <--> Halbwertsbreitenmessung ist inaktiv
	if ( IsMeasuring() )
		return FALSE;

	// TRUE <--> Motor "Beugung Fein" bzw. "Omega" angeschlossenund Makro "Halbwertsbreite messen" sind vorhanden
	if ( (GetMotorIdx(GetHwbAxis()) == -1) || (!Steering.GetMacroById( InquireHwb )) )
		return FALSE;

	// TRUE <--> alle Motoren im Stillstand
	for (UINT i= 0; i < GetMotorCount(); i++ )
		if ( IsMoving( i, bValid ) )
			return FALSE;

	// wenn bis jetzt keine Einwnde, dann kanns ja losgehen
	return TRUE;
};
//----------------------------------------------------------------------------------------
// kann Halbwertsbreitenmessung gestoppt werden

BOOL TManJustage::CanDoStopMeasure ( void ) const
{
  return ( (IsMeasuring()) && (m_IsMeasuring) );
};

//------------------------------------------------------------------------------
// Speeding
//------------------------------------------------------------------------------
// m_UpdateCount erhhen 
UINT  TManJustage::BeginUpdate ( void ) {
  m_UpdateCount++; // erst danach m_UpdateCount erhhen; jetzt liefern GetAngle und IsMoving die zwischengespeicherten Werte
  if ( m_UpdateCount==1 ) for (int i=0; i<GetMotorCount(); i++) { // Kennzeichnen, dass bislang keine Daten zwischengespeichert wurden
    m_MotorList[i].m_SpAngle=  0;
	m_MotorList[i].m_HasAngle= FALSE;
    m_MotorList[i].m_SpMoving=  FALSE;
	m_MotorList[i].m_HasMoving= FALSE;
  }
  return m_UpdateCount;
}
//----------------------------------------------------------------------------------------
// m_UpdateCount verringern

UINT TManJustage::EndUpdate ( void ) {
  if ( m_UpdateCount>0 ) m_UpdateCount--;
  return m_UpdateCount;
}

//------------------------------------------------------------------------------
// Auslesen der wirklichen Antriebsdaten ohne Speeding (Aquivalent zu GetAngle und IsMoving)
//----------------------------------------------------------------------------------------
// aktuelle Antriebsposition beim Antrieb auslesen
TAngle TManJustage::DetermineAngle( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return 0.0;

	if (m_MotorList[aIndex].m_lnkMotor->IsMoveFinish())
		aValid= m_MotorList[aIndex].m_lnkMotor->GetAngle( 1 ); // mGetDistance
	else
		aValid= m_MotorList[aIndex].m_lnkMotor->GetAngle( 0 ); // mGetDistanceProcess
	return m_MotorList[aIndex].m_lnkMotor->GetAngle(); // aValid == TRUE <--> Position erfolgreich ausgelesen und umgerechnet
}
//----------------------------------------------------------------------------------------
// Antriebsstatus (in Bewegung?) beim Antrieb auslesen

BOOL TManJustage::DetermineMoving ( const int aIndex, BOOL &aValid) const 
{
	aValid= FALSE;
	if ( !IsIndexValid ( aIndex ) )
		return FALSE;

	aValid= TRUE;
/*Geschwindigkeitsoptimierung: Bei neuen Physikrechnern bitte unbedingt wieder einkommentieren!!! (--> Steigerung der Robustheit); Zudem sind die Testflle 
  Test_MJN.1.hts, Test_MJN.2.hts, Test_MJN.3.hts, Test_MJN.4.hts, Test_MJN.5.hts und Test_MJN.6.hts anzupassen (siehe dort "##Geschw."-Kommentar).
	// Beugung fein (DF) und Beugung grob (DC) drfen nicht gleichzeitig bewegt werden knnen
	int idx= -1;
	if ( m_MotorList[aIndex].m_AxisType == DC )
		idx= GetMotorIdx(DF);
	else if ( m_MotorList[aIndex].m_AxisType == DF )
		idx= GetMotorIdx(DC);
	if ( (idx != -1) && (!m_MotorList[idx].m_lnkMotor->IsMoveFinish()) )
		return TRUE; // der Partnerantrieb bewegt sich, also bewegen wir uns auch (nicht IsMoving benutzen. sonst Endlosrekursion)
*/
	return ( (IsMeasuring()) || (!m_MotorList[aIndex].m_lnkMotor->IsMoveFinish()) ); // TRUE <--> Halbwertsbreite nicht gemessen wird und Antrieb im Stillstand  
}

//----------------------------------------------------------------------------------------
// LastLine
