//#############################################################################
//                                                                           //
// MJ_OGUI.CPP                                                               //
//                                                                           //
// Subsystem : alte Manuelle Justage / Oberflche                            //
// Benutung durch andere Subsysteme erforderlich: JA                         //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand  : 12.10.2002                                                       //
//                                                                           //
//#############################################################################

#include "internls\evrythng.h" // GermanVersion
#include "winresrc\rc_def.h"   // Ressourcen-IDs
#include "help\help_def.h"     // Help-IDs

#include "manjust\mj_ogui.h" // SCHNITTSTELLE fr diese Datei

#include "workflow\workflow.h" // extern Steering

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// externe Abhngigkeiten
//##############################################################################

extern BOOL bManualMovesCorrected;

//#############################################################################
// sprachspezifische Konstanten
//#############################################################################

#ifdef GermanVersion
  static const char _NORMTITLE[]= "Manuelle Justage";   
  static const char _REFTITLE[]= "Manuelle Justage : Kein gltiger Referenzpunktlauf";   

  static const char szMsgLine500[]= "HWB= %.2f Sekunden";   
  static const char szMsgLine501[]= "&Messung abbrechen";   
  static const char szMsgLine502[]= "&Halbwertsbreite messen";   
#else
  static const char _NORMTITLE[]= "Manual Adjustment";   
  static const char _REFTITLE[]= "Manual Adjustment : No valid Reference Point";   

  static const char szMsgLine500[]= "HWB= %.2f arcsec";   
  static const char szMsgLine501[]= "Skip &Measurement";   
  static const char szMsgLine502[]= "Measure &Halfwidth";   
#endif

//#############################################################################
// TAngleCtlDlg
//#############################################################################

//*****************************************************************************
// KONSTRUKTOR-DESTRUKTOR
//*****************************************************************************

TAngleCtlDlg::TAngleCtlDlg ( void ) : TModalDlg("ANGLECONTROL", GetMainInstance() ), m_lnkAngleCtl(0), m_lnkTimer(0)
{
	m_lnkAngleCtl= new TAngleCtl();
	m_lnkTimer= new TInterfaceTimer( this, 100 ); //Observer, d.h. Timer benachrichtigt uns
	m_Selected= m_lnkAngleCtl->GetActualMotor();  // derzeit ausgewhlter Antrieb
	m_StepMode= new BOOL [m_lnkAngleCtl->GetMotorCount()]; // Liste der Betriebsparameter: Lnge=Anzahl der Motoren
	for (UINT i= 0; i < m_lnkAngleCtl->GetMotorCount(); i++)
		m_StepMode[i]= TRUE; // Schrittbetrieb "ein" und Fahrbetrieb "aus"
};
//-----------------------------------------------------------------------------

TAngleCtlDlg::~TAngleCtlDlg ( void )
{
	_FREEOBJ( m_lnkAngleCtl );
	_FREEOBJ( m_lnkTimer );
}

//*****************************************************************************
// EREIGNISSE DER BASISKLASSE
//*****************************************************************************

BOOL TAngleCtlDlg::Dlg_OnInit ( HWND, HWND, LPARAM )
{
	OnInit();

	// HotKey's Hilfe laden (nur Win32)
	LoadHotKeys( LoadAccelerators(GetMainInstance(), MAKEINTRESOURCE(ACC_AngleControl)) ); // Accelerator aus Ressourcen laden

	return TRUE;
};
//-----------------------------------------------------------------------------

void TAngleCtlDlg::Dlg_OnCommand ( HWND hwnd, int id, HWND hwndCtl, UINT codeNotify )
{
	int idx;
	if ( codeNotify == ONEXIT )
	{
		if ( id == id_NewAngle )
			OnNewAngleEntered( FALSE );
		else if ( id == id_AngleWidth )
			OnAngleWidthEntered( FALSE );
		else if ( id == id_SpeedValue )
			OnSpeedEntered( FALSE );
		return;
	}
	switch (id)
	{
		case cm_Help:
			WinHelp(GetHandle(), GetHelpFile(), HELP_CONTEXT, Help_AngleControlDlg);
			break;

		case cm_MotorInit:   // wird nur intern verschickt um die aktuellen Bewegungsparameter anzuzeigen und den Antrieb fr die Bewegung vorzubereiten
			OnMotorInit();
			break;

		case cm_ParamSet:   // wird nur intern verschickt um den aktuellen Antrieb samt Position und Einheit anzuzeigen
			OnParamSet();
			break;

		case cm_MoveButton:   // wird nur intern verschickt wenn sich der Antrieb in Bewegung gesetzt hat
			OnMotionStarts();
			break;

		case cm_SetAngleZero:   // Relative Null setzen
			OnSetAngleZero();
			break;

		case cm_CancelRelativeZero:   // Relative Null aufheben
			OnCancelRelativeZero();
			break;

		case cm_MeasureHWB:   // Halbwertsbreite messen
			if ( m_lnkAngleCtl->IsMeasuring() )
				OnStopMeasureHWB(); // Messung abbrechen (wenn aktiv)
			else
				OnStartMeasureHWB();
			break;

		case cm_SteeringReady:   // Botschaft wird durch das Steering-Objekt gesendet, wenn die Halbwertsbreitenmessung beendet ist
			OnSteeringReady();
			break;

		case cm_CounterSet:   // Botschaft wird vom Detektor gesendet, wenn neue Messwerte vorliegen
			OnCounterSet();
			break;

		case id_StepMode:   // Schrittbetrieb wurde ausgewhlt
			OnStepMode();
			break;

		case id_LongMoveMode:   // Fahrbetrieb wurde ausgewhlt
			OnLongMoveMode();
			break;

		case id_ChooseMotor:   // Antrieb wurde im Kombinationsfeld ausgewhlt
			if ( codeNotify != CBN_SELCHANGE )
				break; // wenn sich die Auswahl nicht gendert hat: Aufruf ignorieren

			idx= CtrlGetSel(id_ChooseMotor);
			if ( (m_Selected != idx) && (idx != CB_ERR) )
				OnChooseMotor(idx);
			break;

			case cm_SetFine:   // "Beugung fein" wurde mittels Schnellauswahl-Schaltflche ausgewhlt
			OnChooseMotor(Omega);
			break;

		case cm_SetTilt:   // Tilt wurde mittels Schnellauswahl-Schaltflche ausgewhlt
			OnChooseMotor(Psi);
			break;

		case cm_SetCollimator:   // Kollimator in Schnellauswahl auswhlen
			OnChooseMotor(Collimator);
			break;

		case IDOK:   // [ENTER] wurde gedrckt
			OnIDOK();
			break;

		default:
			TModalDlg::Dlg_OnCommand(hwnd, id, hwndCtl, codeNotify);
	}
};
//-----------------------------------------------------------------------------

void TAngleCtlDlg::Dlg_OnHScrollBar ( HWND, HWND, UINT code, int )
{
	switch (code)
	{
		case SB_LINEUP:
		case SB_LINEDOWN:
			OnScrollbarLine( code == SB_LINEDOWN ); // wenn Bewegung im Direktbetrieb stattfindet: nichts machen
			break;

		case SB_ENDSCROLL:   // Maustaste wurde losgelassen: Bewegung stoppen
			OnScrollbarEndscroll();
			break;
	}
};
//-----------------------------------------------------------------------------

void TAngleCtlDlg::Interrupt ( void )
{ // [ESC] wurde gedrckt
	OnInterrupt();
};
//-----------------------------------------------------------------------------

void TAngleCtlDlg::LeaveDialog ( void )
{
	OnLeave();
};
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnTimer ( TBasicTimer *const )
{
	m_lnkTimer->StopTimer();
	if ( OnSuperviseMove() )
		m_lnkTimer->StartTimer(); // weiter berwachen
};

//*****************************************************************************
// Methoden fr die bernahme der Bewegungsparameter
//******************************************************************************

void TAngleCtlDlg::OnNewAngleEntered ( const BOOL aStartMove )
{
	BOOL bValid;
	UINT iDigits= m_lnkAngleCtl->GetDigits(m_Selected, bValid);
	double angle= CtrlGetDouble(id_NewAngle, iDigits, _DECIMAL, bValid);
	if ( !bValid )
		return;

	BOOL moves= (m_lnkAngleCtl->IsMoving(m_Selected, bValid)) && (bValid);
	if ( moves )
		angle= m_lnkAngleCtl->GetAngle(m_Selected, bValid);
	CtrlSetDouble(id_NewAngle, angle, iDigits, _DECIMAL);

	// Bewegung nur starten, wenn [ENTER] gedrckt wurde
	if ( (aStartMove) && (m_lnkAngleCtl->DoDirect(m_Selected, angle)) )
		OnMotionStarts(); // wenn gestartet: Bewegung berwachen
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnSpeedEntered ( const BOOL aFocusCtrl )
{
	BOOL bValid;
	UINT iDigits= m_lnkAngleCtl->GetDigits(m_Selected, bValid);
	double speed= CtrlGetDouble(id_SpeedValue, iDigits, _DECIMAL, bValid);
	if ( !bValid )
		return;

	m_lnkAngleCtl->SetSpeed(m_Selected, speed);
	CtrlSetDouble(id_SpeedValue, speed, iDigits, _DECIMAL);
	if ( aFocusCtrl )
		SetFocus(m_BarHandle); // Antriebsbewegung nur per Bildlaufleiste mglich; siehe Dlg_OnHScrollBar
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnAngleWidthEntered ( const BOOL aFocusCtrl )
{
	BOOL bValid;
	UINT iDigits= m_lnkAngleCtl->GetDigits(m_Selected, bValid);
	double anglewidth= CtrlGetDouble(id_AngleWidth, iDigits + 1, _DECIMAL, bValid); // Schrittweite hat eine Nachkommastelle mehr als Winkel- und Geschwindigkeitsangaben
	if ( !bValid )
		return;

	m_lnkAngleCtl->SetAngleWidth(m_Selected, anglewidth);
	CtrlSetDouble(id_AngleWidth, anglewidth, iDigits + 1, _DECIMAL); // Schrittweite hat eine Nachkommastelle mehr als Winkel- und Geschwindigkeitsangaben
	if ( aFocusCtrl )
		SetFocus(m_BarHandle); // Antriebsbewegung nur per Bildlaufleiste mglich; siehe Dlg_OnHScrollBar
}

//*****************************************************************************
// potentielle Funktionalittskandidaten
//*****************************************************************************

void TAngleCtlDlg::OnInit ( void )
{
	m_BarHandle= GetDlgItem(GetHandle(), id_Bar); // Handle auf die Scrollbar
	BOOL bValid;
	for (UINT cnt= 0; cnt < m_lnkAngleCtl->GetMotorCount(); cnt++)
		CtrlAddString(id_ChooseMotor, m_lnkAngleCtl->GetMotorName(cnt, bValid)); // alle verfgbaren Achsen ins Kombinationsfeld eintragen
	m_lnkAngleCtl->DoInit(m_Selected);
	CtrlSetChecked(id_StepMode, m_StepMode[m_Selected]); // Kontrollfeld "Schritt" auswhlen, wenn Schrittbetrieb ausgewhlt ist
	CtrlSetChecked(id_LongMoveMode, !m_StepMode[m_Selected]); // Kontrollfeld "Fahren" auswhlen, wenn Fahrbetrieb ausgewhlt ist
	CtrlSetEnabled(id_AngleWidth, m_StepMode[m_Selected]); // Eingabefeld Schrittweite freigeben (wenn Schrittbetrieb ausgewhlt) bzw. sperren und ausgegrauen (SONST)
	//04.12.2002 Eingabe der Geschwindigkeit im Direktbetrieb ist nun mglich CtrlSetEnabled(id_SpeedValue, !m_StepMode[m_Selected]); // Eingabefeld Geschwindigkeit freigeben (wenn Fahrbetrieb ausgewhlt) bzw. sperren und ausgegrauen (SONST)
	if ( !m_lnkAngleCtl->CanDoStartMeasure() )
		CtrlSetEnabled(cm_MeasureHWB, FALSE); // wenn Omega nicht da oder Makro "InquireHWB" fehlt: "Halbwertsbreite messen" sperren und ausgrauen
	if ( m_lnkAngleCtl->GetMotorIdx(Omega) == -1 )
		CtrlSetEnabled(cm_SetFine, FALSE); // wenn Omega nicht da: Schnellauswahl-Schaltflche "Beugung fein" und Schaltflche "Halbwertsbreite messen" sperren und ausgrauen
	if ( m_lnkAngleCtl->GetMotorIdx(Psi) == -1 )
		CtrlSetEnabled(cm_SetTilt, FALSE);  // wenn Psi nicht da: Schnellauswahl-Schaltflche "Tilt" sperren und ausgrauen
	if ( m_lnkAngleCtl->GetMotorIdx(Collimator) == -1 )
		CtrlSetEnabled(cm_SetCollimator, FALSE); // wenn Kollimator nicht da: Schnellauswahl-Schaltflche "Kollimator" sperren und ausgrauen
	OnMotorInit();
	OnParamSet();
	m_lnkTimer->StartTimer();
};
//-----------------------------------------------------------------------------

BOOL TAngleCtlDlg::OnSuperviseMove ( void )
{
	BOOL bValid;
	if ( m_lnkAngleCtl->IsMeasuring() )
		return FALSE; // Positionierung nicht berwachen

	if ( (m_lnkAngleCtl->IsMoving(m_Selected, bValid)) && (bValid) )
	{
		OnMotionProgress();
		return TRUE; // Antrieb ist immernoch in Bewegung
	}
	else
	{
		OnMotionStops();
		return FALSE; // Antrieb steht jetzt
	}
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnMotorInit ( void )
{
	m_lnkAngleCtl->DoInitMotor( m_Selected, m_StepMode[m_Selected] );
	// Eingabefelder "Schrittweite" und "Geschwindigkeit" freigeben bzw. sperren und ausgrauen (in Abhngikeit davon, ob Schritt- oder Fahrbetrieb ausgewhlt sind)
	CtrlSetEnabled(id_AngleWidth, m_StepMode[m_Selected]);
	//04.12.2002 Eingabe der Geschwindigkeit im Direktbetrieb ist nun mglich CtrlSetEnabled(id_SpeedValue, !m_StepMode);
	CtrlSetChecked(id_StepMode, m_StepMode[m_Selected]); // Kontrollfeld "Schritt" auswhlen, wenn Schrittbetrieb ausgewhlt ist
	CtrlSetChecked(id_LongMoveMode, !m_StepMode[m_Selected]); // Kontrollfeld "Fahren" auswhlen, wenn Fahrbetrieb ausgewhlt ist
	// aktuelle Schrittweite und Geschwindigkeit anzeigen
	BOOL bValid;
	UINT iDigits= m_lnkAngleCtl->GetDigits(m_Selected, bValid);
	CtrlSetDouble(id_AngleWidth, m_lnkAngleCtl->GetAngleWidth(m_Selected, bValid), iDigits + 1, _DECIMAL); // Schrittweite hat eine Nachkommastelle mehr als Winkel- und Geschwindigkeitsangaben
	CtrlSetDouble(id_SpeedValue, m_lnkAngleCtl->GetSpeed(m_Selected, bValid), iDigits, _DECIMAL);
	if ( bManualMovesCorrected )
		m_lnkAngleCtl->SetCorrectionState(m_Selected); // berprfung der Kalibrierungskurven Correction aktivieren
	// Titel des Dialogs anpassen, je nachdem ob aktueller Antrieb kalibiert ist
	if ( m_lnkAngleCtl->IsCalibrated(m_Selected, bValid) )
		SetWindowText(GetHandle(), _NORMTITLE);
	else
		SetWindowText(GetHandle(), _REFTITLE);
	CtrlSetEnabled(cm_CancelRelativeZero, m_lnkAngleCtl->CanResetRelativeZero(m_Selected)); // Schaltflche "Relative Null aufheben" gdw. freigeben, wenn Relative Null gesetzt
	OnSuperviseMove();
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnParamSet ( void )
{
	BOOL bValid;

	CtrlSetEnabled(cm_CancelRelativeZero, m_lnkAngleCtl->HasOffset(m_Selected, bValid)); // Schaltflche "Relative Null aufheben" gdw. freigeben, wenn Relative Null gesetzt
	UINT iDigits= m_lnkAngleCtl->GetDigits(m_Selected, bValid);
	double dDistance= m_lnkAngleCtl->GetAngle(m_Selected, bValid);
	CtrlSetDouble(id_NewAngle, dDistance, iDigits, _DECIMAL);
	CtrlSetDouble(id_Angle, dDistance, iDigits, _DECIMAL);
	CtrlSetSelString(id_ChooseMotor, m_lnkAngleCtl->GetMotorName(m_Selected, bValid)); // aktuellen Antrieb im Kombinationsfeld auswhlen
	CtrlSetText(id_Unit, m_lnkAngleCtl->GetUnit(m_Selected, bValid)); // Einheit vom aktuellen Antrieb hinter Eingabefeld "Neuer Winkel" anzeigen
	CtrlSetText(id_StepUnit, m_lnkAngleCtl->GetUnit(m_Selected, bValid)); // Einheit vom aktuellen Antrieb hinter Eingabefeld "Schrittbetrieb" anzeigen

	SCROLLINFO scrInfo;
	scrInfo.cbSize= sizeof(SCROLLINFO);
	scrInfo.fMask= SIF_RANGE;
	scrInfo.nMin= GetBarEgde(TRUE);
	scrInfo.nMax= GetBarEgde(FALSE);
	SetScrollInfo(m_BarHandle,SB_CTL,&scrInfo,TRUE);
	SetFocus(m_BarHandle); // Bildlaufleiste fokussieren
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnSetAngleZero ( void )
{
	if ( !m_lnkAngleCtl->SetRelativeZero(m_Selected) )
		return;

	CtrlSetEnabled(cm_CancelRelativeZero, m_lnkAngleCtl->CanResetRelativeZero(m_Selected)); // Schaltflche "Relative Null aufheben" gdw. freigeben, wenn Relative Null gesetzt
	OnSuperviseMove();
	SetFocus(m_BarHandle); // Bildlaufleiste fokussieren
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnCancelRelativeZero ( void )
{
	if ( !m_lnkAngleCtl->ResetRelativeZero(m_Selected) )
		return;

	OnMeasureStops();
	SetFocus(m_BarHandle); // Bildlaufleiste fokussieren
};
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnStartMeasureHWB ( void )
{
	if ( !m_lnkAngleCtl->DoStartMeasure(GetHandle()) )
		return;

	OnMeasureStarts();
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnStopMeasureHWB ( void )
{
	if ( !m_lnkAngleCtl->DoStopMeasure(m_Selected) )
		return;

	OnMeasureStops();
	return;
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnSteeringReady ( void )
{
	if ( !m_lnkAngleCtl->MeasureStopped(m_Selected) )
		return;

	OnMeasureStops();
	MessageBeep(0);
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnCounterSet ( void )
{
	m_lnkAngleCtl->UpdateDetector();
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnStepMode ( void )
{
	BOOL bValid;
	if ( m_lnkAngleCtl->IsMoving(m_Selected, bValid) )
	{ // solange Antrieb in Bewegung: nichts machen
		CtrlSetChecked(id_StepMode, m_StepMode[m_Selected]);
		CtrlSetChecked(id_LongMoveMode, !m_StepMode[m_Selected]);
		return;
	}

	if ( !m_StepMode[m_Selected] )
	{ // wenn noch nicht in Schrittbetrieb
		m_StepMode[m_Selected]= TRUE;  // Flag setzen
		OnMotorInit();
		SetFocus(m_BarHandle);
		// Schrittbetrieb war schon ausgewhlt
	}
	else
		CtrlSetFocus(id_AngleWidth); // Eingabefeld Schrittbetrieb fokussieren
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnLongMoveMode ( void )
{
	BOOL bValid;
	if ( m_lnkAngleCtl->IsMoving(m_Selected, bValid) )
	{ // solange Antrieb in Bewegung: nichts machen
		CtrlSetChecked(id_StepMode, m_StepMode[m_Selected]);
		CtrlSetChecked(id_LongMoveMode, !m_StepMode[m_Selected]);
		return;
	}

	if ( m_StepMode[m_Selected] )
	{
		m_StepMode[m_Selected]= FALSE; // Flag setzen
		OnMotorInit();
		SetFocus(m_BarHandle);
		// Fahrbetrieb war schon ausgewhlt
	}
	else
		CtrlSetFocus(id_SpeedValue);
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnChooseMotor ( const int aIdx )
{
	BOOL bValid;
	if ( m_lnkAngleCtl->IsMeasuring() )
	{ // zuvor gewhten Antrieb wieder im Kombinationsfeld auswhlen
		CtrlSetSelString(id_ChooseMotor, m_lnkAngleCtl->GetMotorName(m_Selected, bValid));
		return;
	}

	m_Selected= aIdx;
	OnParamSet();
	OnMotorInit();
	m_lnkTimer->StartTimer(); //ggf. befindet sich der angegebene Antrieb noch in Bewegung
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnChooseMotor( const EAxisType aAxis )
{
	int temp= m_lnkAngleCtl->GetMotorIdx(aAxis);
	if ( (temp == -1) || (m_lnkAngleCtl->IsMeasuring()) )
		return;

	OnChooseMotor(temp);
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnIDOK ( void )
{
	if ( CtrlHasFocus(id_NewAngle) )
	{ // zuletzt war Eingabefeld "neuer Winkel: " fokussiert
		OnNewAngleEntered( TRUE );
	}
	else if ( CtrlHasFocus(id_AngleWidth) )
	{ // zuletzt war Eingabefeld "Schrittweire" fokussiert
		OnAngleWidthEntered( TRUE );
	}
	else if ( CtrlHasFocus(id_SpeedValue) )
	{ // zuletzt war Eingabefeld "Geschwindigkeit" fokussiert
		OnSpeedEntered( TRUE );
	};
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnScrollbarLine ( const BOOL aUp )
{
	if ( m_StepMode[m_Selected] )
	{ // Schrittbetrieb
		if ( !m_lnkAngleCtl->DoStep( m_Selected, aUp ) )
			return;
	}
	else
	{ // Fahrbetrieb durch Bewegung zur Extremposition emulieren
		if ( !m_lnkAngleCtl->DoDrive( m_Selected, aUp ) )
			return;
	}
	OnMotionStarts();
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnScrollbarEndscroll ( void )
{
	if ( !m_StepMode[m_Selected] )
		m_lnkAngleCtl->DoStop(m_Selected); // nur wenn im Fahrbetrieb
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnInterrupt ( void )
{
	m_lnkAngleCtl->DoStop(m_Selected);
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnLeave ( void )
{ // Detektormessung und Antriebe stoppen
	m_lnkTimer->StopTimer();
	m_lnkAngleCtl->DoStopEverything(m_Selected);
}

//*****************************************************************************
// Methoden zur Positionsumrechnung Scrollbar <-> Antriebsposition
//*****************************************************************************

double TAngleCtlDlg::GetBarScale ( void ) const 
{
	BOOL bValid;
	double angleMin= m_lnkAngleCtl->GetAngleMin( m_Selected, bValid );
	double angleMax= m_lnkAngleCtl->GetAngleMax( m_Selected, bValid );
	double diff= angleMax - angleMin;
	try 
	{
		return 32000 / diff;
	} 
	catch (...) 
	{
		return 0; // sonst div 0
	}		
}

int TAngleCtlDlg::GetBarEgde ( BOOL aLeft ) const 
{
	if ( !aLeft )
	{
		BOOL bValid;
		double angleMin= m_lnkAngleCtl->GetAngleMin( m_Selected, bValid );
		double angleMax= m_lnkAngleCtl->GetAngleMax( m_Selected, bValid );
		return (int) ( (angleMax-angleMin)*GetBarScale() );
	}
	else
		return 0;
};
//-----------------------------------------------------------------------------

int TAngleCtlDlg::GetBarPos ( void ) const 
{ 
	BOOL bValid;
	double angle= m_lnkAngleCtl->GetAngle( m_Selected, bValid );
	double angleMin= m_lnkAngleCtl->GetAngleMin( m_Selected, bValid );
	return (int) ( (angle-angleMin)*GetBarScale());
};

//*****************************************************************************
// Methoden zur Positionsumrechnung Scrollbar <-> Antriebsposition
//*****************************************************************************

void TAngleCtlDlg::OnMotionStarts ( void )
{
	SetDefaultCursor( IDC_WAIT );
	m_lnkTimer->StartTimer();
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnMotionProgress ( void )
{

	SCROLLINFO scrInfo;
	scrInfo.cbSize= sizeof(SCROLLINFO);
	scrInfo.fMask= SIF_POS;

	BOOL bValid;
	UINT iDigits= m_lnkAngleCtl->GetDigits(m_Selected, bValid);
	double dDistance= m_lnkAngleCtl->GetAngle(m_Selected, bValid);
	if ( !bValid )
	{ // Antriebsposition konnte nicht ermittelt werden
		SetDefaultCursor( IDC_WAIT );
		return;
	}
	CtrlSetDouble(id_Angle, dDistance, iDigits, _DECIMAL);
	scrInfo.nPos= GetBarPos();
	SetScrollInfo(m_BarHandle,SB_CTL,&scrInfo,TRUE);
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnMotionStops ( void )
{
	SCROLLINFO scrInfo;
	scrInfo.cbSize= sizeof(SCROLLINFO);
	scrInfo.fMask= SIF_POS;

	BOOL bValid;
	UINT iDigits= m_lnkAngleCtl->GetDigits(m_Selected, bValid);
	double dDistance= m_lnkAngleCtl->GetAngle(m_Selected, bValid);
	CtrlSetDouble(id_NewAngle, dDistance, iDigits, _DECIMAL);
	CtrlSetDouble(id_Angle, dDistance, iDigits, _DECIMAL);
	scrInfo.nPos= GetBarPos();
	SetScrollInfo(m_BarHandle,SB_CTL,&scrInfo,TRUE);
	SetDefaultCursor( IDC_ARROW );

}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnMeasureStarts ( void )
{
	CtrlSetText(cm_MeasureHWB, szMsgLine501);
	CtrlSetEnabled(cm_SetAngleZero, m_lnkAngleCtl->CanSetOffset(m_Selected)); 
	CtrlSetEnabled(cm_CancelRelativeZero, m_lnkAngleCtl->CanResetRelativeZero(m_Selected)); // Schaltflche "Relative Null aufheben" gdw. freigeben, wenn Relative Null gesetzt
}
//-----------------------------------------------------------------------------

void TAngleCtlDlg::OnMeasureStops ( void )
{
	OnMotorInit();
	OnParamSet();
	OnSuperviseMove();
	if ( !m_lnkAngleCtl->IsMeasureReset() )
	{ // Makroverarbeitung und Messung erfolgreich beendet
		double hwb= m_lnkAngleCtl->GetMeasureHwb();
		char buf[MaxString];
		sprintf(buf, szMsgLine500, hwb);
		SetInfo(buf);
	}
	CtrlSetText(cm_MeasureHWB, szMsgLine502);
	CtrlSetEnabled(cm_SetAngleZero, m_lnkAngleCtl->CanSetOffset(m_Selected)); 
	CtrlSetEnabled(cm_CancelRelativeZero, m_lnkAngleCtl->CanResetRelativeZero(m_Selected)); // Schaltflche "Relative Null aufheben" gdw. freigeben, wenn Relative Null gesetzt
	OnSuperviseMove();
}

//#############################################################################
// LastLine
//#############################################################################
