//#############################################################################
//                                                                           //
// MOTORS.CPP                                                                //
// Subsystem: Motorsteuerung                                                 //
// Benutung durch andere Subsysteme erforderlich: JA                         //
//                                                                           //
//---------------------------------------------------------------------------//
// Autoren: Heiko Damerow MPG AG "Roentgenbeugung" (C) 1993,95               //
// Stand:   05.08.2002                                                       //
//                                                                           //
//#############################################################################

#include "internls\evrythng.h" // GermanVersion
#include "winresrc\rc_def.h"   // Ressourcen-IDs
#include "help\help_def.h"     // Help-IDs
#pragma hdrstop

#include "motrstrg\motrstrg.h"
#include "motrstrg\m_layer.h"
#include "motrstrg\m_motcom.h"
#include "motrstrg\m_mothw.h"
#include "motrstrg\C_8x2.h"
#include "hardware\hwio.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

extern DeviceList MotorDrivers;
extern ControllerList MotorControllers;

#define DirectAccess 
// #define WriteLog
// #define CheckCorrect
// #define TESTLAUF

//vormals in evrythng.h
#define  R_TimeOut         214
#define  R_UnknownCmd      215

#ifdef GermanVersion
	char szMsgFailure[]= "Fehler";
	char szMessage[]= "Meldung";
	char szMsgLine001[]= "Initialisieren von %s";
	char szMsgLine002[]= "Probleme beim Initialisieren der Motor-Steuerung !\n\nAbbrechen ?";
	char szMsgLine003[]= "Startup der Motor-Steuerung";
	char szMsgLine004[]= "Sichern von Motor-Einstellungen %s                    ";
	char szMsgLine005[]= "Diese Aktion ist fr diesen Antrieb nicht vorbereitet !";
	char szMsgLine006[]= "Kein gltiger Motor !";
	char szMsgLine007[]= "Keine Kalibrierung erfolgt !";
	char szMsgLine008[]= "Absolute Null fr Motor %s";
	char szMsgLine009[]= "Kalibrierung beendet !";
	char szMsgLine010[]= "\n\nDie neuen Kalibrierungsdaten wurden gesichert !";
	// Testgruppe
	char szMsgLine011[]= "%s DCM-Parameter fr %s optimieren";
	//
	char szMsgLine012[]= "Fehler beim Initialisieren des Antrieb %s!";
#else
	char szMsgFailure[]= "Failure";
	char szMessage[]= "Message";
	char szMsgLine001[]= "Initializing of drive module %s";
	char szMsgLine002[]= "Problems with drive module startup !\n\nAbbort ?";
	char szMsgLine003[]= "Starting up the drive module";
	char szMsgLine004[]= "Save settings of %s                                   ";
	char szMsgLine005[]= "This operation shouldn't be used for this drive !";
	char szMsgLine006[]= "No valid drive !";
	char szMsgLine007[]= "Calibration isn't done !";
	char szMsgLine008[]= "Absolute zero for drive %s";
	char szMsgLine009[]= "Calibration ready !";
	char szMsgLine010[]= "\n\nThe new calibration data was saved !";
	// Testgruppe
	char szMsgLine011[]= "Optimize %s DCM-Parameter for %s";
	//
	char szMsgLine012[]= "Can't initialize the drive %s!";
#endif 
//------------------------------------------------------------------------------

static const int MAXIT= 30;    // maximal number of iterations

int TMotor::nMaxId= 0;

extern LPMList lpMList;
extern HINSTANCE hModuleInstance;

//Datum: 27.07.2002 extern TModalDlg* TheDialog;
//**** variables belonging C-832 ***********************************************
// LM628/629-commands exept RDSTAT
// Definition der Kommandos und des Typs TCSet in "C_8x2.inc"
static TCSet CmdSet[] =
	{
		{RESET, 0, 0, 0}, {PORT8, 0, 0, 0}, {PORT12, 0, 0, 0}, {DFH, 0, 0, 0},
		{SIP, 0, 0, 0}, {LPEI, 0, 1, 0}, {LPES, 0, 1, 0}, {SBPA, 0, 2, 1},
		{SBPR, 0, 2, 1}, {MSKI, 0, 1, 0}, {RSTI, 0, 1, 0}, {LFIL, 0, 4, 0},
		{UDF, 0, 0, 0}, {LTRJ, 0, 3, 1}, {STT, 0, 0, 0}, {RDSTAT, 1, 1, 0},
		{RDSIGS, 1, 1, 0}, {RDIP, 1, 1, 1}, {RDDP, 1, 1, 1}, {RDRP, 1, 1, 1},
		{RDDV, 1, 1, 1}, {RDRV, 1, 1, 0}, {RDSUM, 1, 1, 0}
	};

int nC812= -1;

//******************************************************************************
//***** definitions for motor simulation / hardware access *********************

static TSimulationType simulation_type= no_simulation;
static T812ISA_GET_CALLBACK c812isa_get_callback;
static T812ISA_PUT_CALLBACK c812isa_put_callback;
static T832_GET_CALLBACK c832_get_callback;
static T832_PUT_CALLBACK c832_put_callback;

void _MOTORCLASS WINAPI msSetSimulationType( TSimulationType t )
{
	simulation_type= t;
}
void _MOTORCLASS WINAPI msRegister_C812ISA_Get( T812ISA_GET_CALLBACK cb )
{
	c812isa_get_callback= cb;
}
void _MOTORCLASS WINAPI msRegister_C812ISA_Put( T812ISA_PUT_CALLBACK cb )
{
	c812isa_put_callback= cb;
}
void _MOTORCLASS WINAPI msRegister_C832_Get( T832_GET_CALLBACK cb )
{
	c832_get_callback= cb;
}
void _MOTORCLASS WINAPI msRegister_C832_Put( T832_PUT_CALLBACK cb )
{
	c832_put_callback= cb;
}
//******************************************************************************

TMList::TMList( int maxn ) // Kullmann, Reinecker (08.08.02) 2.Parameter, hier: HINSTANCE inst, entfernt
{
#ifdef WriteLog
	char buf[]= "Start of Program\r";
#endif

	AD_X= -1;
	AD_Y= -1;
	AD_Z= -1;
	AD_Rotation= -1;
	AD_Tilt= -1;
	AD_DC= -1;
	AD_Collimator= -1;
	AD_DF= -1;
	AD_Encoder= -1;
	AD_Absorber= -1;
	AD_Psi= -1;
	AD_Phi= -1;
	AD_Theta= -1;
	AD_Omega= -1;
	AD_Monochromator= -1;
	nActiveDrive= -1;
	nLastDrive= -1;
	nMaxNumber= maxn;
	//hMInstance      = inst;  ( Toter Code, am 08.08.02 auskommentiert)

#ifdef WriteLog

	FILE *LogFile;
	LogFile= fopen( "motor.log", "w" );
	if ( LogFile )
	{
		fseek( LogFile, 0, 2 );
		fwrite( buf, strlen( buf ), 1, LogFile );
		fclose( LogFile );
	}
#endif

	aMotor= new TMotor * [ nMaxNumber ];
};


TMList::~TMList( void )
{
	_FREELIST(aMotor);
};


BOOL TMList::InitializeModule( void )
{
	BOOL bNoFailure= TRUE;
	int id;
	char Ident[ MaxString ], ctype[ MaxString ];
	char buf[ MaxString ];

	//temp. Motorliste
	TMotor** initializedMotors= new TMotor* [nMaxNumber];


	//
	// erzeugung der motor objekte in der motoren liste entsprechen type-parameter
	// aus ini-file
	// beginnend bei [Motor0] werden solange die [MotorX]-sektionen gelesen bis
	// die erste gefunden wird die keinen oder einen leeren type-parameter hat
	//
	nLastDrive= -1;
	id= 0;
	do
	{
		sprintf( Ident, "Motor%d", id );
		GetPrivateProfileString( Ident, "Type", "", ctype, MaxString, GetHWFile( ) );
		if ( strlen( ctype ) == 0 )
			break;
		nLastDrive= id;

		if ( strcmpi( ctype, "C-812ISA" ) == 0 )
		{
			aMotor[ id ]= ( TMotor* )( TC_812ISA* ) new TC_812ISA();
			continue;
		}
		if ( strcmpi( ctype, "C-832" ) == 0)
		{
			aMotor[ id ]= ( TC_832 * ) new TC_832();
			continue;
		}
		// TMotor oder ungueltige type bezeichnung
		aMotor[ id ]= ( TMotor* ) new TMotor();
	} while ( id++ < nMaxNumber );

	// test ob wenigstens ein motor in motoren-liste
	// falls nicht wird ein dummy motor in die liste eingetragen
	if ( nLastDrive == -1 )
	{
		aMotor[0]= ( TMotor* ) new TMotor();
		nLastDrive= 0;
	}

	// achse 0 ist stets per default aktiviert
	nActiveDrive= 0;

	//********** initializion of all motors ************************************

	int initID= 0;
	for ( id= 0; id <= LastId(); id++ )
	{
		sprintf( buf, szMsgLine001, MP( id ) -> pCharacteristic());
		SetInfo( buf );
		// Initialisierung der Motor-Hardware
		if ( R_OK != MP( id ) -> Initialize() )
		{
			_FREEOBJ(aMotor[id]);
		}
		else
		{
			initializedMotors[initID++]= aMotor[id];
		}
	}
	SetInfo( szMsgLine003 );
	nLastDrive= initID -1;

	//hp test ob wenigstens ein motor erfolgreich initialisiert wurde
	// falls nicht wird ein dummy-motor initialisiert
	if ( nLastDrive == -1 )
	{
		initializedMotors[initID++]= ( TMotor* ) new TMotor();
		initializedMotors[initID-1]->Initialize();
		nLastDrive= 0;
	}

	//hp aufnehmen der initialisierten Motoren in Motorenliste
	for(id=0;id<initID;id++)
	{
		aMotor[id]= initializedMotors[id];
	}
	_FREELIST(initializedMotors);

	// Keine relativen Winkel setzen
	for ( id= 0;id <= LastId();id++ )
		MP( id ) -> SetAngleBias( 0.0 ); // Kullmann, Reinecker (04.08.02) dAngleBias ersetzt
	// HINWEIS zu SetAngleBias: SetAngleBias setzt dAngleBias nur, wenn der Antrieb steht,
	// whrend vorher in jedem Fall dAngleBias geschrieben wurde -> mgliche Nebeneffekte
	// Obwohl laut Spezifikation das Schreiben von dAngleBias NUR im Stillstand erlaubt ist

	// Die Positions-Register der Motor-Steuerung werden auf NULL gesetzt und
	// die abgespeicherten Positionen werden rechnerisch einbezogen
	// Die abgespeicherten Positionen werden nicht beruecksichtigt.
	// Es wird unterstellt, dass die Register der Motor-Steuerung noch
	// aktuell sind
	return bNoFailure;
};


EAxisType TMList::ParsingAxis( LPCSTR axisname)
{
	if ( !strcmp( axisname, "AzimutalRotation" ) ||
			!strcmp( axisname, "AZ" ) ||
			!strcmp( axisname, "Rotation" ) ||
			!strcmp( axisname, "Azimute" ) )
		return Rotation;

	if ( !strcmp( axisname, "X" ) ||
			!strcmp( axisname, "x" ) ||
			!strcmp( axisname, "Horizontal" ) ||
			!strcmp( axisname, "x-Achse" ) ||
			!strcmp( axisname, "x-Axis" ) )
		return X;

	if ( !strcmp( axisname, "Y" ) ||
			!strcmp( axisname, "y" ) ||
			!strcmp( axisname, "y-Achse" ) ||
			!strcmp( axisname, "y-Axis" ) )
		return Y;

	if ( !strcmp( axisname, "Z" ) ||
			!strcmp( axisname, "z" ) ||
			!strcmp( axisname, "Vertical" ) ||
			!strcmp( axisname, "z-Achse" ) ||
			!strcmp( axisname, "z-Axis" ) )
		return Z;

	if ( !strcmp( axisname, "Theta" ) )
		return Theta;

	if ( !strcmp( axisname, "Omega" ) )
		return Omega;

	if ( !strcmp( axisname, "Tilt" ) ||
			!strcmp( axisname, "TL" ) )
		return Tilt;

	if ( !strcmp( axisname, "DC" ) ||
			!strcmp( axisname, "Diffraction coarse" ) ||
			!strcmp( axisname, "Beugung Grob" ) ||   // neu 10.01.2002 Kullmann+Reinecker
			!strcmp( axisname, "Beugung grob" ) )
		return DC;

	if ( !strcmp( axisname, "DF" ) ||
			!strcmp( axisname, "Diffraction fine" ) ||
			!strcmp( axisname, "Beugung Fein" ) ||   // neu 10.01.2002 Kullmann+Reinecker
			!strcmp( axisname, "Beugung fein" ) )
		return DF;

	if ( !strcmp( axisname, "Psi" ) )
		return Psi;

	if ( !strcmp( axisname, "Phi" ) )
		return Phi;

	if ( !strcmp( axisname, "CC" ) ||
			!strcmp( axisname, "Collimator" ) ||
			!strcmp( axisname, "Kollimator" ) )
		return Collimator;

	if ( !strcmp( axisname, "Absorber" ) )
		return Absorber;

	if ( !strcmp( axisname, "Monochromator" ) )
		return Monochromator;

	if ( !strcmp( axisname, "Encoder" ) )
		return Encoder;

	return ( EAxisType ) 0;
};


BOOL TMList::SaveModuleSettings( void )
{
	int id;
	char buf[ MaxString ];

	for ( id= 0;id <= LastId( );id++ )
	{
		sprintf( buf, szMsgLine004, (LPSTR)MP( id ) -> pCharacteristic( ) );
		SetInfo( buf );
		MP( id ) -> SaveSettings( TRUE );
		MP( id ) -> StopDrive( FALSE );
	}
	return TRUE;
};


void TMList::SetAngleDefault( void )
{
	int id;

	for ( id= 0;id <= LastId( );id++ )
	{
		MP( id ) ->SetAngleBias( 0.0 ); // Kullmann, Reinecker (04.08.02) dAngleBias ersetzt
		// siehe oben: HINWEIS zu SetAngleBias
		MP( id ) ->SetAngleRange( );
	}
}

TMotor* TMList::MP( void )
{
	return aMotor[ nActiveDrive ];
};


TMotor* TMList::MP( int n )
{
	if ( ( n >= 0 ) && ( n <= nLastDrive ) )
		return aMotor[ n ];
	return aMotor[ nActiveDrive ];
};


void TMList::SetParametersDlg( void )
{
	TMotorParamDlg* dlg;

	dlg= new TMotorParamDlg( );
	//Datum: 27.07.2002 TheDialog= ( TMotorParamDlg * )  dlg;
	if ( dlg ) dlg->ExecuteDialog( GetFrameHandle( ) );
	_FREEOBJ(dlg);
};


void TMList::InquireReferencePointDlg( BOOL task ) // Kullmann, Reinecker (08.08.02) Parameter gendert int -> BOOL
{
	TModalDlg* dlg;

	dlg= ( TCalibrateDlg * ) new TCalibrateDlg( task );
	if ( dlg ) dlg->ExecuteDialog( GetFrameHandle( ) );
	_FREEOBJ(dlg);
};


BOOL TMList::SetAxis( int n )
{
	if ( ( n >= 0 ) && ( n <= nLastDrive ) )
	{
		// Motor, der zur Achse n gehoert, wird als aktiver Motor gekennzeichnet
		nActiveDrive= n;
		return TRUE;
	}
	else
	{
		nActiveDrive= 0;
		return FALSE;
	}
};


//**************************************************************************
//**************************************************************************
// Anfahren der Nullstellung : Referenzpunktlauf
TCalibrateDlg::TCalibrateDlg( BOOL task ) : TModalDlg( "CALIBRATEMOTORS", hModuleInstance ) // Kullmann, Reinecker (08.08.02) Parameter gendert int -> BOOL
{
	bZeroShouldSet= FALSE;
	nTask= task;
	lOldPos= new long [ lpMList->LastId( ) + 1 ];
	bZeroPosReached= new BOOL [ lpMList->LastId( ) + 1 ];
	bIndexPosReached= new BOOL [ lpMList->LastId( ) + 1 ];
	bTargetPosReached= new BOOL [ lpMList->LastId( ) + 1 ];
	// ANFANG: Kullmann, Reinecker (08.08.02)
	/*switch( nTask )
	{
		case 99:
			bAllMotors= TRUE;
			bHoldPosition= FALSE;
			bValidCalibrationData= TRUE;   
			break;

		default:
			bValidCalibrationData= TRUE;
			bAllMotors= FALSE;
			bHoldPosition= FALSE;
	}*/
	if ( nTask )
	{  // war 99
		bAllMotors= TRUE;
		bHoldPosition= FALSE;
		bValidCalibrationData= TRUE;
	}
	else
	{
		bValidCalibrationData= TRUE;
		bAllMotors= FALSE;
		bHoldPosition= FALSE;
	};
	// ENDE: Kullmann, Reinecker (08.08.02)
};


TCalibrateDlg::~TCalibrateDlg( void )
{
	_FREELIST(lOldPos);
	_FREELIST(bZeroPosReached);
	_FREELIST(bTargetPosReached);
	_FREELIST(bIndexPosReached);
};


BOOL TCalibrateDlg::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM )
{
	int i;

	// Initialisieren der Parameterfenster fuer einen Motor
	Motor= lpMList->MP( );
	SetDlgItemText( hwnd, id_Report, "" );
	CheckDlgButton( hwnd, id_ValidCalibrationData, bValidCalibrationData );
	hMotorList= GetDlgItem( hwnd, id_ChooseMotor );

	i= 0;
	while ( i <= lpMList->LastId( ) )
	{
		ComboBox_AddString( hMotorList, lpMList->MP( i++ ) ->pCharacteristic( ) );
	}
	ComboBox_SelectString( hMotorList, 0, Motor->pCharacteristic( ) );

	if ( nTask ) // Kullmann, Reinecker (08.08.02) nTask==99 ersetzt
		FORWARD_WM_COMMAND( hwnd, cm_CarryOnZero, hwndCtl, 0, PostMessage );

	// HotKey's Hilfe laden (nur Win32)
	LoadHotKeys( LoadAccelerators(GetInstance(), MAKEINTRESOURCE(ACC_CalibrateMotor)) ); // Accelerator aus Ressourcen laden

	return TRUE;
};


void TCalibrateDlg::Dlg_OnTimer( HWND hwnd, UINT id )
{
	int idx;
	BOOL AnyToDo= FALSE;

	if ( id != TimerIdInformation )
		return;
	KillTimer( hwnd, TimerIdInformation );

	for ( idx= 0;idx <= lpMList->LastId( );idx++ )
	{
		if ( !bIndexPosReached[ idx ] )
		{
			AnyToDo= TRUE;
			if ( lpMList->MP( idx ) -> IsIndexArrived( ) )
			{
				// Indexposition
				// bei C-812 meist die linke Endlage plus RemoveLimit
				// bei C-832 meist genau die Index-Position plus RemoveLimit
				// Es wurde noch kein Home neu gesetzt
				FORWARD_WM_COMMAND( hwnd, cm_IndexArrived, 0, idx, SendMessage );
				continue;
			}
			continue;
		}
		if ( !bZeroPosReached[ idx ] )
		{
			AnyToDo= TRUE;
			if ( lpMList->MP( idx ) ->IsMoveFinish( ) )
			{
				FORWARD_WM_COMMAND( hwnd, cm_DistanceSet, 0, idx, SendMessage );
				continue;
			}
		}
	}
	if ( AnyToDo )
		SetTimer( hwnd, TimerIdInformation, 130, NULL );
	return;
};


void TCalibrateDlg::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl, UINT codeNotify )
{
	int idx;
	char buf[ 2*MaxString ];
	int NextMotor;

	switch ( id )
	{
		case cm_Help:
			WinHelp(GetHandle(), GetHelpFile(), HELP_CONTEXT, Help_CalibrateMotorsDlg);
			break;

		case cm_CarryOnZero:
			if ( bZeroShouldSet )
			{
				MessageBeep( 0 );
				break;
			}
			//
			// Kalibrierungs-Bedarf festlegen
			//
			bHoldPosition= IsDlgButtonChecked( GetHandle( ), id_HoldPosition );
			bValidCalibrationData= IsDlgButtonChecked( GetHandle( ),
									id_ValidCalibrationData );

			if ( !bValidCalibrationData )
			{
				strcpy( buf, "Es wird davon ausgegangen, dass sich der Antrieb\r\n" );
				strcat( buf, "in der Position der physikalischen Null befindet.\r\n" );
				strcat( buf, "Es werden Kalibrierungsdaten geaendert !" );
				if ( IDYES != MessageBox( buf, "Meldung", MBASK ) )
					return;
			}
			for ( idx= 0;idx <= lpMList->LastId( );idx++ )
				bIndexPosReached[ idx ]= bZeroPosReached[ idx ]= TRUE;

			if ( !bAllMotors )
			{
				if ( lpMList->MP( ) -> IsInitialMoveReady() ) // Kullmann, Reinecker (04.08.02) bInitialMoveReady ersetzt
					bIndexPosReached[ Motor->GetId( ) ]= bZeroPosReached[ Motor->GetId( ) ]
														 = FALSE;
				else
					MessageBox( szMsgLine005, szMessage, MBINFO );
			}
			else
			{
				for ( idx= 0;idx <= lpMList->LastId( );idx++ )
					if ( lpMList->MP( idx ) -> IsInitialMoveReady() ) // Kullmann, Reinecker (04.08.02) bInitialMoveReady ersetzt
						bIndexPosReached[ idx ]= bZeroPosReached[ idx ]= FALSE;
			}

			//
			// RPL starten
			//
			for ( idx= 0;idx <= lpMList->LastId( );idx++ )
			{
				Motor= lpMList->MP( idx );
				if ( Motor->GetId( ) != idx )
					MessageBox( "Motor Id", szMsgFailure, MBINFO );
				if ( !bIndexPosReached[ idx ] )
				{
					Motor->PushSettings( );
					Motor->StartToIndex( lOldPos[ idx ] );  // old position will be returned
					bZeroShouldSet= TRUE;
				}
			}

			if ( bZeroShouldSet )
			{
				SetDlgItemText( GetHandle( ), id_Report, "" );
				EnableWindow( GetDlgItem( hwnd, id_ChooseMotor ), FALSE );
				EnableWindow( GetDlgItem( hwnd, cm_SetHome ), FALSE );
				EnableWindow( GetDlgItem( hwnd, cm_CarryOnZero ), FALSE );
				goto ExitWithTimer;
			}
			if ( nTask ) // Kullmann, Reinecker (08.08.02) nTask==99 ersetzt
				FORWARD_WM_COMMAND( hwnd, IDCANCEL, hwndCtl, 0, PostMessage );
			else
				SetDlgItemText( hwnd, id_Report, szMsgLine007 );
			return;

		case cm_IndexArrived:
			if ( !lpMList->SetAxis( codeNotify ) )
			{
				MessageBox( szMsgLine006, szMessage, MBINFO );
				return;
			}
			Motor= lpMList->MP( );

			// Alle fertigen Motoren nicht mehr behandeln
			if ( bZeroPosReached[ Motor->GetId( ) ] )
				goto ExitWithTimer;
			if ( bIndexPosReached[ Motor->GetId( ) ] )
				goto ExitWithTimer;

			// measure the difference between startpoint and index point
			if ( !bValidCalibrationData || bHoldPosition )
			{
				Motor->GetPosition( TRUE );
				lOldPos[ Motor->GetId( ) ] -= Motor->GetPosition(); // Kullmann, Reinecker (04.08.02) lPosition ersetzt
			}
			if ( !bValidCalibrationData )
				Motor->SetDistanceToZero( lOldPos[Motor->GetId()] ); // Kullmann, Reinecker (04.08.02) lDistanceToZero ersetzt

			// compute and set up the zero point
			Motor->SetHome( );
			// Motor->SetDeltaPosition()= -Motor->GetDistanceToZero();
			Motor->SetDeltaPosition( -Motor->GetDistanceToZero() ); // Kullmann, Reinecker (04.08.02) lDeltaPosition, lDistanceToZero ersetzt
			bIndexPosReached[ Motor->GetId( ) ]= TRUE;
			Motor->SetCalibrationState( TRUE ); // Kullmann, Reinecker (04.08.02) bIsCalibrated ersetzt
			sprintf( buf, "Index %s", Motor->pCharacteristic( ) );
			SetDlgItemText( GetHandle( ), id_Report, (LPSTR)buf );

			// start next motor to target position
			// if hold old position take the old position
			// in all other cases take the TargetAngle
			// notify old position
			if ( bHoldPosition )
				Motor->MoveByPosition( lOldPos[ Motor->GetId( ) ] );
			else
				Motor->MoveToAngle( Motor->GetInitialAngle() ); // Kullmann, Reinecker (04.08.02) dInitialAngle ersetzt
			goto ExitWithTimer;

		case cm_DistanceSet:
			if ( !lpMList->SetAxis( codeNotify ) )
			{
				MessageBox( szMsgLine006, szMsgFailure, MBINFO );
				return;
			}
			Motor= lpMList->MP( );
			if ( bHoldPosition )
				sprintf( buf, "Alte Position %s", Motor->pCharacteristic( ) );
			else
				sprintf( buf, "Target Pos %s", Motor->pCharacteristic( ) );
			SetDlgItemText( GetHandle( ), id_Report, (LPSTR)buf );
			bZeroPosReached[ Motor->GetId( ) ]= TRUE;
			Motor->GetAngle( 1 );
			Motor->PopSettings( ThisPosition );
			if ( !bValidCalibrationData )
				Motor->SaveSettings( FALSE );

			// Ist ein Motor noch nicht am Zielpunkt?
			NextMotor= FALSE;
			for ( idx= 0;idx <= lpMList->LastId( );idx++ )
				if ( !bZeroPosReached[ idx ] )
					NextMotor= TRUE;
			if ( NextMotor )
				goto ExitWithTimer;

			//*** Calibration finished
			if ( !bValidCalibrationData )
			{
				bValidCalibrationData= TRUE;
				CheckDlgButton( GetHandle( ), id_ValidCalibrationData, TRUE );
			}
			MessageBeep( 0 );
			if ( !nTask ) // Kullmann, Reinecker (08.08.02) nTask!=99 ersetzt
			{
				if ( !bValidCalibrationData )
				{
					strcpy( buf, szMsgLine009 );
					strcat( buf, szMsgLine010 );
					MessageBox( buf, szMessage, MBINFO );
				}
				else
					MessageBox( szMsgLine009, szMessage, MBINFO );

				bZeroShouldSet= FALSE;
				EnableWindow( GetDlgItem( GetHandle( ), id_ChooseMotor ), TRUE );
				EnableWindow( GetDlgItem( GetHandle( ), cm_SetHome ), TRUE );
				EnableWindow( GetDlgItem( GetHandle( ), cm_CarryOnZero ), TRUE );
			}
			else
				FORWARD_WM_COMMAND( hwnd, IDCANCEL, hwndCtl, 0, PostMessage );
			return;

		case cm_SetHome:
			Motor->SetHome( );
			Motor->GetAngle( 1 );
			sprintf( buf, szMsgLine008, Motor->pCharacteristic( ) );
			SetDlgItemText( GetHandle( ), id_Report, buf );
			return;

		case id_ChooseMotor:
			if ( codeNotify==CBN_SELCHANGE ) 
			{
				idx= ComboBox_GetCurSel( hMotorList );
				if ( ( Motor != lpMList->MP( idx ) ) && ( idx != CB_ERR ) )
				{
					lpMList->SetAxis( idx );
					Motor= lpMList->MP( );
				}
			}
			return;

		//Datum: 02.08.2002
		/*case cm_RotateMotor:
			lpMList->SetAxis( Motor->GetId( )  +1 );
			Motor= lpMList->MP( );
			ComboBox_SelectString( hMotorList,0,Motor->pCharacteristic( )  );
			return;*/

		case id_ValidCalibrationData:
			if ( bZeroShouldSet )
				break;
			bValidCalibrationData= IsDlgButtonChecked( GetHandle( ),
									id_ValidCalibrationData );
			if ( bValidCalibrationData )
			{
				EnableWindow( GetDlgItem( GetHandle( ), id_HoldPosition ), TRUE );
				EnableWindow( GetDlgItem( GetHandle( ), id_AllMotorsReset ), TRUE );
			}
			else
			{
				bHoldPosition= FALSE;
				EnableWindow( GetDlgItem( GetHandle( ), id_HoldPosition ), FALSE );
				EnableWindow( GetDlgItem( GetHandle( ), id_AllMotorsReset ), FALSE );
			}
			return;

		case id_AllMotorsReset:
			if ( bZeroShouldSet )
				break;
			bAllMotors= IsDlgButtonChecked( GetHandle( ), id_AllMotorsReset );
			if ( bAllMotors )
			{
				// Motor-Button deaktivieren
				EnableWindow( GetDlgItem( GetHandle( ), id_ChooseMotor ), FALSE );
				EnableWindow( GetDlgItem( GetHandle( ), cm_SetHome ), FALSE );
			}
			else
			{
				// Motor-Button aktivieren
				EnableWindow( GetDlgItem( GetHandle( ), id_ChooseMotor ), TRUE );
				EnableWindow( GetDlgItem( GetHandle( ), cm_SetHome ), TRUE );
			}
			return;

		default:
			TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
	}
	return;

ExitWithTimer:
	SetTimer( GetHandle( ), TimerIdInformation, 200, NULL );
	return;
};


BOOL TCalibrateDlg::CanClose( void )
{
	if ( bZeroShouldSet )
		return FALSE;
	// LimitWatch disable
	return TRUE;
};


void TCalibrateDlg::LeaveDialog( void )
{
	int id;
	KillTimer( GetHandle( ), TimerIdInformation );
	for ( id= 0;id <= lpMList->LastId( );id++ )
	{
		Motor= lpMList->MP( id );
		Motor->StopDrive( TRUE );
	}
};


//**************************************************************************
// Steuerung des Probentellers ueber die Position
void TMList::PositionControlDlg( void )
{
	TModalDlg* dlg;
	dlg= ( TPosControlDlg* ) new TPosControlDlg( );
	if ( dlg ) dlg->ExecuteDialog( GetFrameHandle( ) );
	_FREEOBJ(dlg);
};


TPosControlDlg::TPosControlDlg( ) : TModalDlg( "PositionControl", hModuleInstance )
{
	bMoveActive= FALSE;
};


BOOL TPosControlDlg::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM )
{
	int cnt;

	Motor= lpMList->MP( );
	BarHandle= GetDlgItem( GetHandle(), id_Bar );
	hMotorList= GetDlgItem( GetHandle(), id_ChooseMotor );
	cnt= 0;
	while ( cnt <= lpMList->LastId( ) )
		ComboBox_AddString( hMotorList, lpMList->MP( cnt++ ) ->pCharacteristic( ) );
	FORWARD_WM_COMMAND( GetHandle(), cm_ParamSet, hwndCtl, 0, SendMessage );
	SetFocus( BarHandle );
	return TRUE;
};


void TPosControlDlg::Dlg_OnTimer( HWND hwnd, UINT id )
{
	if ( id!=TimerIdInformation ) return;

	while ( !Motor->GetPosition( TRUE ) )
		/* wait for actual position */;
	CtrlSetDouble(id_Position, Motor->GetPosition(), 0, '.' );
	if ( Motor->IsMoveFinish( ) && bMoveActive )
	{
		KillTimer( hwnd, TimerIdInformation );
		bMoveActive= FALSE;
	}
	FORWARD_WM_COMMAND( hwnd, cm_SetupDlgItem, 0, 0, SendMessage );
};


void TPosControlDlg::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl, UINT codeNotify )
{
	int idx;
	BOOL bValid;

	SCROLLINFO scrInfo;
	scrInfo.cbSize= sizeof(SCROLLINFO);
	
	switch ( id )
	{
		case cm_SetupDlgItem:
			scrInfo.fMask= SIF_POS;
			scrInfo.nPos= ScrollbarCalculate(Motor, ePos);
			SetScrollInfo(BarHandle,SB_CTL,&scrInfo,TRUE);
			CtrlSetLong(id_NewPosition, Motor->GetPosition());
			return;

		case cm_ParamSet:
			// Initialisieren des Motor-Scroll-Bar's
			CtrlSetLong(id_Position, Motor->GetPosition());
			ComboBox_SelectString( hMotorList, 0, Motor->pCharacteristic( ) );
			BarHandle= GetDlgItem( GetHandle( ), id_Bar );
			scrInfo.fMask= SIF_RANGE;
			// ANFANG Kullmann+Reinecker (12.03.2004)
			/*scrInfo.nMin= -GetBarRange(Motor)/2;
			scrInfo.nMax= GetBarRange(Motor)/2;*/
			scrInfo.nMin= ScrollbarCalculate(Motor, eMin);
			scrInfo.nMax= ScrollbarCalculate(Motor, eMax);
			// ENDE Kullmann+Reinecker (12.03.2004)
			SetScrollInfo(BarHandle,SB_CTL,&scrInfo,TRUE);
			// Initialisieren aller Positionsfenster
			FORWARD_WM_COMMAND( GetHandle(), cm_SetupDlgItem, hwndCtl, 0, SendMessage );
			return;

		case id_ChooseMotor:
			if ( codeNotify!=CBN_SELCHANGE ) return;

			idx= ComboBox_GetCurSel( hMotorList );
			if ( ( Motor != lpMList->MP( idx ) ) && ( idx != CB_ERR ) )
			{
				Motor= lpMList->MP( idx );
				lpMList->SetAxis( Motor->GetId( ) );
				FORWARD_WM_COMMAND( hwnd, cm_ParamSet, hwndCtl, 0, SendMessage );
				FORWARD_WM_COMMAND( hwnd, cm_MotorInit, hwndCtl, 0, PostMessage );
			}
			return;

		//Datum: 02.08.2002
		/*case cm_RotateMotor:
			if( bMoveActive )
				break;
			lpMList->SetAxis( Motor->GetId( )  +1 );
			Motor= lpMList->MP( );
			FORWARD_WM_COMMAND( hwnd,cm_ParamSet,hwndCtl,0,SendMessage );
			return;*/

		case IDOK:
			// Der Nutzer hat mit Return eine neue Position eingegeben
			NewPosition= CtrlGetDouble(id_NewPosition, 0, '.', bValid);
			if ( NewPosition < Motor->GetPositionMin() ) //Neu : Kullmann, Reinecker (27.07.02) lPositionMin ersetzt
				NewPosition= Motor->GetPositionMin(); //Neu : Kullmann, Reinecker (27.07.02) lPositionMin ersetzt
			else
				if ( NewPosition > Motor->GetPositionMax() ) //Neu : Kullmann, Reinecker (27.07.02) lPositionMax ersetzt
					NewPosition= Motor->GetPositionMax(); //Neu : Kullmann, Reinecker (27.07.02) lPositionMax ersetzt
			FORWARD_WM_COMMAND( hwnd, cm_MoveButton, hwndCtl, 0, PostMessage );
			SetFocus( GetDlgItem( GetHandle( ), id_Bar ) );
			return;

		case cm_MoveButton:
			Motor->MoveToPosition( NewPosition );
			SetTimer( GetHandle( ), TimerIdInformation, 80, NULL );
			bMoveActive= TRUE;
			FORWARD_WM_COMMAND( hwnd, cm_SetupDlgItem, hwndCtl, 0, PostMessage );
			return;

		default:
			TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
	}
};


BOOL TPosControlDlg::HScrollBar( WPARAM wParam, LPARAM )
{
	switch ( wParam )
	{
		case SB_LINEUP:
			NewPosition= Motor->GetPosition() - Motor->GetPositionWidth(); //Neu : Kullmann, Reinecker (27.07.02) lPosition u. wPositionWidth ersetzt
			NewPosition= max( Motor->GetPositionMin(), NewPosition ); //Neu : Kullmann, Reinecker (27.07.02) lPositionMin ersetzt
			break;

		case SB_LINEDOWN:
			NewPosition= Motor->GetPosition() + Motor->GetPositionWidth(); //Neu : Kullmann, Reinecker (27.07.02) lPosition u. wPositionWidth ersetzt
			NewPosition= min( Motor->GetPositionMax() , NewPosition ); //Neu : Kullmann, Reinecker (27.07.02) lPositionMax ersetzt
			break;

		case SB_PAGEUP:
			NewPosition= Motor->GetPosition() - 5 * Motor->GetPositionWidth(); //Neu : Kullmann, Reinecker (27.07.02) lPosition u. wPositionWidth ersetzt
			NewPosition= max( Motor->GetPositionMin(), NewPosition ); //Neu : Kullmann, Reinecker (27.07.02) lPositionMin ersetzt
			break;

		case SB_PAGEDOWN:
			NewPosition= Motor->GetPosition() + 5 * Motor->GetPositionWidth(); //Neu : Kullmann, Reinecker (27.07.02) lPosition u. wPositionWidth ersetzt
			NewPosition= min( Motor->GetPositionMax(), NewPosition ); //Neu : Kullmann, Reinecker (27.07.02) lPositionMax ersetzt
			break;

		case SB_ENDSCROLL: // Bewegung ausloesen			
			FORWARD_WM_COMMAND( GetHandle( ), cm_MoveButton, 0, 0, PostMessage );
			break;
	}
	return TRUE;
};

// ANFANG Kullmann+Reinecker (12.03.2004)
int TPosControlDlg::ScrollbarCalculate( TMotor *act, EPosType aType ) {
	long ma= act->GetPositionMax();
	long mi= act->GetPositionMin();
	float skale;
	try 
	{
		skale= (float) ( 32000 / (float)(ma-mi)); 
	} 
	catch (...) 
	{
		skale= 0;
	}
	if ( aType==eMax ) return (int) ( (ma-mi)*skale );
	else if ( aType==ePos ) return (int) ( (act->GetPosition()-mi)*skale );
	else return 0; // eMin
}

/*int TPosControlDlg::GetBarRange( TMotor *act ) //Neu : Kullmann, Reinecker (14.11.02) const beim Parameter entfernt, sonst const-member-functions im Motor ntig
{
	try {
		//return ( int ) ( ( act->GetPositionMax() - act->GetPositionMin() ) / act->GetPositionWidth() ); //Neu : Kullmann, Reinecker (27.07.02) lPositionMax, lPositionMin, wPositionWidth ersetzt
	} catch (...) {
		return 0; //Neu : Kullmann+Reinecker: Div 0
	}
};


int TPosControlDlg::GetBarPos( TMotor *act ) //Neu : Kullmann, Reinecker (14.11.02) const beim Parameter entfernt, sonst const-member-functions im Motor ntig
{
	try {
		//return ( int ) ( ( act->GetPosition() - act->GetPositionMin() ) / act->GetPositionWidth() ); //Neu : Kullmann, Reinecker (27.07.02) lPositionMax, lPositionMin, wPositionWidth ersetzt
	} catch (...) { //Neu Kullmann+Reinecker: div 0
		return 0;
	}
};*/
// ENDE Kullmann+Reinecker (12.03.2004)

void TPosControlDlg::LeaveDialog( void )
{
	Motor->StopDrive( 1 );
	KillTimer( GetHandle( ), TimerIdInformation );
};


//*************************************************************************
//*** Allgemeine Einstellungen fuer Motoren
TMotorParamDlg::TMotorParamDlg( void ) : TModalDlg( "DRIVESETTINGS", hModuleInstance )
{
};



BOOL TMotorParamDlg::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM lParam )
{
	int i;

	// Initialisieren der Parameterfenster fuer einen Motor
	Motor= lpMList->MP( );
	hMotorList= GetDlgItem( GetHandle( ), id_ChooseMotor );
	i= 0;
	while ( i <= lpMList->LastId( ) )
		ComboBox_AddString( hMotorList, lpMList->MP( i++ ) ->pCharacteristic( ) );
	FORWARD_WM_COMMAND( hwnd, cm_ParamSet, hwndCtl, 0, SendMessage );
	SetFocus( GetDlgItem( GetHandle( ), id_Velocity ) );
	return TRUE;
};


void TMotorParamDlg::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl, UINT codeNotify )
{
	int idx;
	char buf[ MaxString ];

	switch ( id )
	{
		case cm_ParamSet: // Initialisieren der Parameterfenster fuer einen Motor			
			ComboBox_SelectString( hMotorList, 0, Motor->pCharacteristic( ) );
			sprintf( buf, Motor->pSF( ), Motor->GetSpeed( ) );
			SetDlgItemText( GetHandle( ), id_Velocity, ( LPSTR ) buf );
			SetDlgItemText( GetHandle( ), id_Limit, ( LPSTR ) ltoa( Motor->GetLimit( ),
							( LPSTR ) buf, 10 ) );
			SetDlgItemText( GetHandle( ), id_PositionMax,
							( LPSTR ) ltoa( Motor->GetPositionMax(), ( LPSTR ) buf, 10 ) ); // Kullmann, Reinecker (04.08.02) lPositionMax ersetzt
			SetDlgItemText( GetHandle( ), id_PositionMin,
							( LPSTR ) ltoa( Motor->GetPositionMin(), ( LPSTR ) buf, 10 ) ); // Kullmann, Reinecker (04.08.02) lPositionMin ersetzt
			SetDlgItemText( GetHandle( ), id_PositionWidth,
							( LPSTR ) ltoa( Motor->GetPositionWidth(), ( LPSTR ) buf, 10 ) ); // Kullmann, Reinecker (04.08.02) wPositionWidth ersetzt
			SetDlgItemText( GetHandle( ), id_Unit, ( LPSTR ) Motor->GetUnitName() ); // Kullmann, Reinecker (04.08.02) szUnit ersetzt
			sprintf( buf, Motor->pSF( ), Motor->GetAngleWidth( ) );
			SetDlgItemText( GetHandle( ), id_AngleWidth, ( LPSTR ) buf );
			sprintf( buf, Motor->pDF( ), Motor->GetAngleMin() ); // Kullmann, Reinecker (04.08.02) dAngleMin ersetzt
			SetDlgItemText( GetHandle( ), id_AngleMin, ( LPSTR ) buf );
			sprintf( buf, Motor->pDF( ), Motor->GetAngleMax() ); // Kullmann, Reinecker (04.08.02) dAngleMax ersetzt
			SetDlgItemText( GetHandle( ), id_AngleMax, ( LPSTR ) buf );
			return;

		case cm_SwitchLimitWatch:
			if ( !Motor->StartLimitWatch( ) )
				Motor->StopLimitWatch( );
			return;

		case id_ChooseMotor:
			switch ( codeNotify )
			{
				case CBN_EDITCHANGE:
					SetWindowText( hMotorList, Motor->pCharacteristic( ) );
					return;

				case CBN_SELCHANGE:
					idx= ComboBox_GetCurSel( hMotorList );
					if ( ( Motor != lpMList->MP( idx ) ) && ( idx != CB_ERR ) )
					{
						if ( CanClose( ) )
						{
							Motor= lpMList->MP( idx );
							lpMList->SetAxis( Motor->GetId( ) );
						}
						FORWARD_WM_COMMAND( hwnd, cm_ParamSet, 0, 0, SendMessage );
					}
					return;
			}
			return;

		//Datum: 02.08.2002
		/*case cm_RotateMotor:
			   // Erst die Einstellungen fuer den aktuellen Motor sichern
			   if( CanClose( )  ) {
			     lpMList->SetAxis( Motor->GetId( )  +1 );
			     Motor= lpMList->MP( );
			   }
			   FORWARD_WM_COMMAND( hwnd,cm_ParamSet,0,0,SendMessage );
			   return;*/

			default:
			TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
	}
};


BOOL TMotorParamDlg::CanClose( void )
{
	char buf[ MaxString ];
	BOOL FActivate= FALSE;
	BOOL FFail= FALSE;
	WORD valueW;
	DWORD valueDW;
	double valueF;

	for (;; )
	{
		GetDlgItemText( GetHandle( ), id_Velocity, ( LPSTR ) buf, 10 );
		if ( !buf )
		{
			FFail= TRUE;
			break;
		}
		valueF= atof( buf );
		if ( valueF == 0.0 )
		{
			FFail= TRUE;
			break;
		}
		if ( Motor->SetSpeed( valueF ) )
			FActivate= TRUE;
		GetDlgItemText( GetHandle( ), id_Limit, ( LPSTR ) buf, 10 );
		if ( !buf )
		{
			FFail= TRUE;
			break;
		}
		valueDW= ( DWORD ) atol( buf );
		if ( Motor->GetLimit( ) != valueDW )
			Motor->SetLimit( valueDW );
		break;
	}
	if ( FActivate )
		Motor->ActivateDrive( );
	if ( FFail )
		return FALSE;
	GetDlgItemText( GetHandle( ), id_PositionWidth, ( LPSTR ) buf, 10 );
	valueW= ( WORD ) atoi( buf );
	if ( 0 != valueW )
		Motor->SetPositionWidth( valueW ); // Kullmann, Reinecker (04.08.02) wPositionWidth ersetzt
	else
		return FALSE;

	GetDlgItemText( GetHandle( ), id_PositionMax, ( LPSTR ) buf, 10 );
	Motor->SetPositionMax( atol( buf ) ); // Kullmann, Reinecker (04.08.02) lPositionMax ersetzt
	GetDlgItemText( GetHandle( ), id_PositionMin, ( LPSTR ) buf, 10 );
	Motor->SetPositionMin( atol( buf ) ); // Kullmann, Reinecker (04.08.02) lPositionMin ersetzt
	GetDlgItemText( GetHandle( ), id_AngleMax, ( LPSTR ) buf, 10 );
	valueF= atof( buf );

	if ( ( 0.0 != valueF ) )
	{
		Motor->SetAngleMax( valueF ); // Kullmann, Reinecker (04.08.02) dAngleMax ersetzt
	}
	else
		return FALSE;

	GetDlgItemText( GetHandle( ), id_AngleMin, ( LPSTR ) buf, 10 );
	valueF= atof( buf );
	if ( ( 0.0 != valueF ) )
	{
		Motor->SetAngleMin( valueF ); // Kullmann, Reinecker (04.08.02) dAngleMin ersetzt
	}
	else
		return FALSE;
	GetDlgItemText( GetHandle( ), id_AngleWidth, ( LPSTR ) buf, 10 );
	valueF= atof( buf );
	if ( 0.0 != valueF )
	{
		Motor->SetAngleWidth( valueF );
	}
	else
		return FALSE;
	return TRUE;
};


//*************************************************************************

TOptimizeDC_832Dlg::TOptimizeDC_832Dlg( void ) : TModalDlg( "OptimizeDC_832", hModuleInstance )
{
	Drive= (TC_832*)lpMList->MP( );
	bCancel= TRUE;
};


BOOL TOptimizeDC_832Dlg::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM )
{
	old_vel= Drive->dwMaxVelocity;
	old_accel= Drive->dwAcceleration;
	old_kp= Drive->wKP;
	old_kd= Drive->wKD;
	old_ki= Drive->wKI;
	old_kl= Drive->wKL;
	old_poswidth= Drive->wPositionWidth;
	FORWARD_WM_COMMAND( hwnd, cm_ParamSet, hwndCtl, 0, SendMessage );
	return TRUE;
};

void TOptimizeDC_832Dlg::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl,
										UINT codeNotify )
{
	char buf[ MaxString ];

	switch ( id )
	{
		case cm_ParamSet:
			// Testgruppe
			sprintf( buf, szMsgLine011, "C832", Drive->pCharacteristic( ) );
			SetWindowText( hwnd, buf );
			//
			SetWindowText( GetHandle( ), buf );
			sprintf( buf, "%ld", Drive->dwMaxVelocity );
			SetDlgItemText( GetHandle( ), id_Velocity, ( LPSTR ) buf );
			sprintf( buf, "%ld", Drive->dwAcceleration );
			SetDlgItemText( GetHandle( ), id_Acceleration, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wKD );
			SetDlgItemText( GetHandle( ), id_kd, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wKP );
			SetDlgItemText( GetHandle( ), id_kp, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wKI );
			SetDlgItemText( GetHandle( ), id_ki, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wKL );
			SetDlgItemText( GetHandle( ), id_kl, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wPositionWidth );
			SetDlgItemText( GetHandle( ), id_PositionWidth, ( LPSTR ) buf );
			break;

		case cm_StartMoveScan:
			GetDlgItemText( GetHandle( ), id_Velocity, buf, MaxString );
			Drive->dwMaxVelocity= atol( buf );
			GetDlgItemText( GetHandle( ), id_Acceleration, buf, MaxString );
			Drive->dwAcceleration= atol( buf );
			GetDlgItemText( GetHandle( ), id_kd, buf, MaxString );
			Drive->wKD= ( WORD ) atoi( buf );
			GetDlgItemText( GetHandle( ), id_kp, buf, MaxString );
			Drive->wKP= ( WORD ) atoi( buf );
			GetDlgItemText( GetHandle( ), id_ki, buf, MaxString );
			Drive->wKI= ( WORD ) atoi( buf );
			GetDlgItemText( GetHandle( ), id_kl, buf, MaxString );
			Drive->wKL= ( WORD ) atoi( buf );
			GetDlgItemText( GetHandle( ), id_PositionWidth, buf, MaxString );
			Drive->wPositionWidth= ( WORD ) atoi( buf );
			Drive->ActivateFilterParameters( );
			mStartMoveScan( 10, 0 );
			return;

		default:
			TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
	}
	return;
};


BOOL TOptimizeDC_832Dlg::CanClose( void )
{
	bCancel= FALSE;
	Drive->SaveSettings( 0 );
	return TRUE;
};


void TOptimizeDC_832Dlg::LeaveDialog( void )
{
	if ( bCancel )
	{
		Drive->dwMaxVelocity= old_vel;
		Drive->dwAcceleration= old_accel;
		Drive->wKP= old_kp;
		Drive->wKD= old_kd;
		Drive->wKI= old_ki;
		Drive->wKL= old_kl;
		Drive->wPositionWidth= old_poswidth;
	}
	Drive->ActivateFilterParameters( );
};


//******************************************************************************
//******************************************************************************
void TC_812::OptimizingDlg( void )
{
	TModalDlg* dlg;

	dlg= ( TOptimizeDC_812Dlg * ) new TOptimizeDC_812Dlg( );
	if ( dlg ) dlg->ExecuteDialog( GetFrameHandle( ) );
	_FREEOBJ(dlg);
};


TOptimizeDC_812Dlg::TOptimizeDC_812Dlg( void ) : TModalDlg( "OptimizeDC_812", hModuleInstance )
{
	Drive= ( TC_812* ) lpMList->MP( );
	bCancel= TRUE;
};



BOOL TOptimizeDC_812Dlg::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM lParam )
{
	old_vel= Drive->dwMaxVelocity;
	old_accel= Drive->dwAcceleration;
	old_gain= Drive->wStaticGain;
	old_dgain= Drive->wDynamicGain;
	old_torque= Drive->wTorque;
	old_poswidth= Drive->wPositionWidth;
	FORWARD_WM_COMMAND( hwnd, cm_ParamSet, hwndCtl, 0, SendMessage );
	return TRUE;
};

void TOptimizeDC_812Dlg::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl,
										UINT codeNotify )
{
	char buf[ MaxString ];

	switch ( id )
	{
		case cm_ParamSet:
			// Testgruppe
			sprintf( buf, szMsgLine011, "C812", Drive->pCharacteristic( ) );
			SetWindowText( hwnd, buf );
			//
			sprintf( buf, "%ld", Drive->dwMaxVelocity );
			SetDlgItemText( hwnd, id_Velocity, ( LPSTR ) buf );
			sprintf( buf, "%ld", Drive->dwAcceleration );
			SetDlgItemText( hwnd, id_Acceleration, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wTorque );
			SetDlgItemText( hwnd, id_Torque, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wStaticGain );
			SetDlgItemText( hwnd, id_Gain, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wDynamicGain );
			SetDlgItemText( hwnd, id_DynamicGain, ( LPSTR ) buf );
			sprintf( buf, "%d", Drive->wPositionWidth );
			SetDlgItemText( hwnd, id_PositionWidth, ( LPSTR ) buf );
			break;

		case cm_StartMoveScan:
			GetDlgItemText( hwnd, id_Velocity, buf, MaxString );
			Drive->dwMaxVelocity= atol( buf );
			GetDlgItemText( hwnd, id_Acceleration, buf, MaxString );
			Drive->dwAcceleration= atol( buf );
			GetDlgItemText( hwnd, id_Gain, buf, MaxString );
			Drive->wStaticGain= ( WORD ) atoi( buf );
			GetDlgItemText( hwnd, id_DynamicGain, buf, MaxString );
			Drive->wDynamicGain= ( WORD ) atoi( buf );
			GetDlgItemText( hwnd, id_Torque, buf, MaxString );
			Drive->wTorque= ( WORD ) atoi( buf );
			GetDlgItemText( GetHandle( ), id_PositionWidth, buf, MaxString );
			Drive->wPositionWidth= ( WORD ) atoi( buf );
			Drive->ActivateFilterParameters( );
			mStartMoveScan( 15, 0 );
			break;

		default:
			TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
	}
};


BOOL TOptimizeDC_812Dlg::CanClose( void )
{
	char buf[ MaxString ];

	bCancel= FALSE;
	GetDlgItemText( GetHandle( ), id_Velocity, buf, MaxString );
	Drive->dwMaxVelocity= atol( buf );
	GetDlgItemText( GetHandle( ), id_Acceleration, buf, MaxString );
	Drive->dwAcceleration= atol( buf );
	GetDlgItemText( GetHandle( ), id_Gain, buf, MaxString );
	Drive->wStaticGain= ( WORD ) atoi( buf );
	GetDlgItemText( GetHandle( ), id_DynamicGain, buf, MaxString );
	Drive->wDynamicGain= ( WORD ) atoi( buf );
	GetDlgItemText( GetHandle( ), id_Torque, buf, MaxString );
	Drive->wTorque= ( WORD ) atoi( buf );
	Drive->ActivateFilterParameters( );
	Drive->SaveSettings( 0 );
	return TRUE;
};


void TOptimizeDC_812Dlg::LeaveDialog( void )
{
	if ( bCancel )
	{
		Drive->dwMaxVelocity= old_vel;
		Drive->dwAcceleration= old_accel;
		Drive->wTorque= old_torque;
		Drive->wStaticGain= old_gain;
		Drive->wDynamicGain= old_dgain;
		Drive->wPositionWidth= old_poswidth;
		Drive->ActivateFilterParameters( );
	}
};



//****************************************************************************
//****************************************************************************
TMotor::TMotor( )
{
	char Ident[ MaxString ];
	char ifile[ MaxString ];
	char buf[ MaxString ];

	nId= nMaxId++;
	dAngleBias= 0.0;
	dAngle= 0.0;
	lCorrPos= 0;
	fCorrBias= 0.0;
	bIndexLine= FALSE;
	bIndexDetected= FALSE;
	bControlBoardOk= FALSE;
	bLimitWatchActive= FALSE;
	eCorrStatus= CorrLinear;
	bIsCalibrated= FALSE;
	bInquireStatus= FALSE;
	bRangeHit= FALSE;
	bUpwards= TRUE;
	lDeltaPosition= 0;
	lPosition= 0;
	//  lPreviousPosition= 0;  Kullmann, Reinecker ( Toter Code, am 04.08.02 auskommentiert )


	strcpy( ifile, GetHWFile( ) );
	sprintf( Ident, "motor%d", nId );
	GetPrivateProfileString( Ident, "Name", "Motor", szCharacteristic, _MAXLENCHARACTERISTIC, GetHWFile( ) );
	WritePrivateProfileString( Ident, "Name", szCharacteristic, GetHWFile( ) );

	dwHysteresis= ( DWORD ) GetPrivateProfileInt( Ident, "Hysteresis", 0, GetHWFile( ) );
	WritePrivateProfileString( Ident, "Hysteresis", ltoa( dwHysteresis, buf, 10 ),
							   GetHWFile( ) );

	bUpwards= GetPrivateProfileInt( Ident, "Upwards", 1, GetHWFile( ) );
	WritePrivateProfileString( Ident, "Upwards", ltoa( bUpwards, buf, 10 ),
							   GetHWFile( ) );

	switch ( lpMList->ParsingAxis( szCharacteristic ) )
	{
		case Rotation:
		case Phi:
			lpMList->AD_Phi= nId;
			lpMList->AD_Rotation= nId;
			break;

		case X:
			lpMList->AD_X= nId;
			break;

		case Y:
			lpMList->AD_Y= nId;
			break;

		case Z:
			lpMList->AD_Z= nId;
			break;

		case Theta:
			lpMList->AD_Theta= nId;
			break;

		case Omega:
		case DF:
			lpMList->AD_Omega= nId;
			lpMList->AD_DF= nId;
			break;

		case Tilt:
		case Psi:
			lpMList->AD_Psi= nId;
			lpMList->AD_Tilt= nId;
			break;

		case DC:
			lpMList->AD_DC= nId;
			break;

		case Collimator:
			lpMList->AD_Collimator= nId;
			break;

		case Absorber:
			lpMList->AD_Absorber= nId;
			break;

		case Monochromator:
			lpMList->AD_Monochromator= nId;
			break;

		case Encoder:
			lpMList->AD_Encoder= nId;
			break;
	}
};


TMotor::Initialize( void )
{
	HCURSOR hCursorOld;
	float MaxFailure;
	char Ident[ MaxString + 1 ];
	char buf[ MaxString + 1 ];

	hCursorOld= SetCursor( LoadCursor(NULL,IDC_WAIT) );
	// Motor identifizieren
	sprintf( Ident, "motor%d", nId );

	GetPrivateProfileString( Ident, "MaxVelocity", "8000", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "MaxVelocity", buf, GetHWFile( ) );
	dwMaxVelocity= atol( buf );

	GetPrivateProfileString( Ident, "Velocity", "8000", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "Velocity", buf, GetHWFile( ) );
	dwVelocity= atol( buf );
	if ( dwVelocity == 0 )
		dwVelocity= dwMaxVelocity / 2;

	GetPrivateProfileString( Ident, "Unit", "Unit", szUnit, _MAXLENUNIT, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "Unit", szUnit, GetHWFile( ) );
	eUnit= UnitEnum( szUnit );
	strcpy( DFmt, "%.2lf" );
	strcpy( SFmt, "%.2lf" );

	GetPrivateProfileString( Ident, "Digits", "2", buf, MaxString, GetHWFile( ) );
	nDigits= atoi( buf ); // NEU: Kullmann, Reinecker (23.07.2002)
	DFmt[ 2 ]= ( char ) ( '0' + nDigits ); // NEU: Kullmann, Reinecker: atoi durch nDigits ersetzt (12.08.2002)
	SFmt[ 2 ]= ( char ) ( '1' + nDigits ); // NEU: Kullmann, Reinecker: atoi durch nDigits ersetzt (12.08.2002)
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "Digits", buf, GetHWFile( ) );

	GetPrivateProfileString( Ident, "MinimalWidth", "10", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "MinimalWidth", buf, GetHWFile( ) );
	wPositionMinWidth= ( WORD ) atoi( buf );

	GetPrivateProfileString( Ident, "MaximalWidth", "510", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "MaximalWidth", buf, GetHWFile( ) );
	wPositionMaxWidth= ( WORD ) atoi( buf );

	GetPrivateProfileString( Ident, "PositionMin", "-100", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "PositionMin", buf, GetHWFile( ) );
	lPositionMin= atol( buf );

	GetPrivateProfileString( Ident, "PositionMax", "100", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "PositionMax", buf, GetHWFile( ) );
	lPositionMax= atol( buf );
	if ( lPositionMax <= lPositionMin )
		lPositionMax= lPositionMin + 100;

	GetPrivateProfileString( Ident, "PositionWidth", "10", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "PositionWidth", buf, GetHWFile( ) );
	wPositionWidth= ( WORD ) atoi( buf );
	if ( wPositionWidth == 0 )
		wPositionWidth= 0.2;

	GetPrivateProfileString( Ident, "InitialMove", "0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "InitialMove", buf, GetHWFile( ) );
	bInitialMoveReady= atoi( buf );

	GetPrivateProfileString( Ident, "RestartPossible", "0", buf, MaxString, GetHWFile( ) );
	WritePrivateProfileString( Ident, "RestartPossible", "0", GetHWFile( ) ); // Programm als "nicht ordnungsgem beendet" kennzeichnen
	if ( bInitialMoveReady )
		SetCalibrationState( atoi( buf ) );
	else
		SetCalibrationState( TRUE );

	GetPrivateProfileString( Ident, "DeltaPosition", "0", buf, MaxString, GetHWFile( ) );
	lDeltaPosition= atol( buf );
	if ( IsCalibrated( ) )
	{
		if ( !bUpwards )
			lDeltaPosition += dwHysteresis;
	}
	dAngleBias= 0.0;
	lCorrPos= 0;

	GetPrivateProfileString( Ident, "DistanceToZero", "0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "DistanceToZero", buf, GetHWFile( ) );
	lDistanceToZero= atol( buf );

	GetPrivateProfileString( Ident, "RemoveLimit", "4000", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "RemoveLimit", buf, GetHWFile( ) );
	dwRemoveLimit= ( DWORD ) atol( buf );

	GetPrivateProfileString( Ident, "MoveFirstToLimit", "0", buf, MaxString,
							 GetHWFile( ) );
	bMoveFirstToLimit= atoi( buf );

	GetPrivateProfileString( Ident, "IndexLine", "0", buf, MaxString, GetHWFile( ) );
	bIndexLine= atoi( buf );

	GetPrivateProfileString( Ident, "InitialAngle", "0.0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "InitialAngle", buf, GetHWFile( ) );
	dInitialAngle= atof( buf );

	GetPrivateProfileString( Ident, "Acceleration", "10", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "Acceleration", buf, GetHWFile( ) );
	dwAcceleration= atol( buf );

	GetPrivateProfileString( Ident, "AngleMin", "-1.0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "AngleMin", buf, GetHWFile( ) );
	dAngleMin= atof( buf );

	GetPrivateProfileString( Ident, "AngleMax", "1.0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "AngleMax", buf, GetHWFile( ) );
	dAngleMax= atof( buf );

	GetPrivateProfileString( Ident, "AngleWidth", "0.1", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "AngleWidth", buf, GetHWFile( ) );
	dAngleWidth= atof( buf );

	GetPrivateProfileString( Ident, "Koeff_3", "0.0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "Koeff_3", buf, GetHWFile( ) );
	dKoeff_3= atof( buf );

	GetPrivateProfileString( Ident, "Koeff_2", "0.0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "Koeff_2", buf, GetHWFile( ) );
	dKoeff_2= atof( buf );

	GetPrivateProfileString( Ident, "Koeff_1", "1.0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "Koeff_1", buf, GetHWFile( ) );
	dKoeff_1= atof( buf );

	GetPrivateProfileString( Ident, "SpeedScale", "10.0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "SpeedScale", buf, GetHWFile( ) );
	fSpeedScale= atof( buf );
	SetCursor( hCursorOld );

	GetPrivateProfileString( Ident, "MaxFailure", "30.0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "MaxFailure", buf, GetHWFile( ) );
	MaxFailure= atof( buf );
	
	/*dwInterval= ( DWORD ) ( 3.0 * MaxFailure / dKoeff_1 );
	switch ( eUnit )
	{
		case utGrad:
			dwInterval *= 3600;
			break;

		case utMinuten:
			dwInterval *= 60;
			break;
	}*/
	dwInterval= (DWORD)CalcValueFromUnit(eUnit, DWORD(3.0 * MaxFailure / dKoeff_1));

	GetPrivateProfileString( Ident, "Correction", "0", buf, MaxString, GetHWFile( ) );
	if ( CreateDefaults( ) )
		WritePrivateProfileString( Ident, "Correction", buf, GetHWFile( ) );
	bCorrection= atoi( buf );
	SetCorrectionState( TRUE );
	return R_OK;
};


//*************** Element-Funktionen ******************************************
BOOL TMotor::SetAngleRange( void )
{
	double a1, a2;

	Translate( a1, lPositionMin );
	Translate( a2, lPositionMax );
	// 19.12.02 Kullmann, Reinecker : es soll auch a1 > a2 mglich sein, um die Bewegungsrichtung umzukehren
	dAngleMin= a1; // alt: min( a1,a2 );
	dAngleMax= a2; // alt: max( a1,a2 );
	return TRUE;
};

BOOL TMotor::PushSettings( void )
{
	Settings.dAngle= dAngle;
	Settings.dAngleMin= dAngleMin;
	Settings.dAngleMax= dAngleMax;
	Settings.dAngleWidth= dAngleWidth;
	Settings.dSpeed= dSpeed;
	return TRUE;
};

BOOL TMotor::PopSettings ( EMParameter param )
{
	if ( (dAngle != Settings.dAngle) && (param == OldPosition) )
		MoveToAngle( Settings.dAngle );
	dAngleMin= Settings.dAngleMin;
	dAngleMax= Settings.dAngleMax;
	SetAngleWidth( Settings.dAngleWidth );
	SetSpeed( Settings.dSpeed );
	return TRUE;
};

//ANFANG: neu Kullmann, Reinecker (04.05.2002) neue Methoden

//------------------------------------------------------------------------------
//Offset fr Relative Null vorhanden

BOOL TMotor::IsDistanceRelative( void )
{
	return ((int)( dAngleBias * pow( (float)10, (float)GetDigits() ) ) != 0);
};
//------------------------------------------------------------------------------
//Relative Null setzen (nur wenn Antrieb steht)

BOOL TMotor::SetRelativeZero( void )
{
	BOOL bValid= FALSE;
	if ( IsMoveFinish() )
		dAngleBias= -1 * GetAngleAbs(FALSE, bValid); //nur wenn der Antrieb steht: auf aktuellen Winkel setzen

	return IsDistanceRelative(); //relative Null gesetzt?
};
//------------------------------------------------------------------------------
//Relative Null aufheben (nur wenn Antrieb steht)

BOOL TMotor::ResetRelativeZero( void )
{
	if ( IsMoveFinish() )
		dAngleBias= 0.0; //nur wenn der Antrieb steht: Relative Null aufheben

	return !IsDistanceRelative(); //relative Null aufgehoben?
};

//ENDE: neu Kullmann, Reinecker (03.05.2002)

//------------------------------------------------------------------------------
// Umrechnung von Position in Winkel mit einem Polynom
// Umkehrfunktion mit Newton-Verfahren

void TMotor::funcd( double arg, double offset, double *f, double *df )
{
	*f= arg * ( dKoeff_1 + arg * ( dKoeff_2 + arg * dKoeff_3 ) ) - offset;
	*df= dKoeff_1 + arg * ( 2 * dKoeff_2 + arg * 3 * dKoeff_3 );
};


int TMotor::rtsave( double offset, double x1, double x2, double xacc, double *ret )
/*  Using a combination of Newton-Raphson and bisection, find the   */
/*  root of a function bracketed between x1 and x2. The root, re-   */
/*  turned as the function value rtsafe, will be refined until its  */
/*  accuracy is known within +-xacc. funcd is a user-supplied rou-  */
/*  tine that provides both the function value and the first deriv- */
/*  ative of the function.                                          */
{
	int j;
	double df, dx, dxold, f, fh, fl, xh, xl, rts;

	funcd( x1, offset, &fl, &df );
	funcd( x2, offset, &fh, &df );
	if ( ( fl ) *( fh ) >= 0.0 )
		return -1; // "Root must be bracketed in RTSAFE"
	if ( ( fl ) < 0.0 )   // Orient the search so that f(xl)<0.
	{
		xl= x1;
		xh= x2;
	}
	else
	{
		xh= x1;
		xl= ( fl );
		fl= fh;
		fh= xl;
		xl= x2;
	}
	rts= 0.5 * ( x1 + x2 );
	// Initialize the guess for root,
	dx= dxold= fabs( x2 - x1 );
	// the "step-size before last" and the last step.
	funcd( rts, offset, &f, &df );

	// Loop over allowed iterations.
	for ( j= 1;j <= MAXIT;j++ )
	{
		// Bisect if Newton out of range,
		if ( ( ( ( rts - xh ) *df - f ) *( ( rts - xl ) *df - f ) >= 0.0 ) || ( fabs( 2.0*f ) > fabs( dxold*df ) ) )
			// or not decreasing fast enough.
		{
			dxold= dx;
			dx= 0.5 * ( xh - xl );
			rts= xl + dx;
			if ( xl == rts )
			{
				//  Change in root is negligible.
				*ret= rts;
				return 0;
			}
		}
		else
			// Newton step acceptable. Take it.
		{
			dxold= dx;
			dx= f / df;
			df= rts;
			rts -= dx;
			if ( df == rts )
			{
				*ret= rts;
				return 0;
			}
		}
		if ( fabs( dx ) < xacc )
			// Convergence criterion.
		{
			*ret= rts;
			return 0;
		}
		funcd( rts, offset, &f, &df );
		// The one new function evaluation per iteration.
		if ( f < 0.0 )
			// Maintain the bracket on the root.
		{
			xl= rts;
			fl= f;
		}
		else
		{
			xh= rts;
			fh= f;
		}
	}
	return -2; // "Maximum number of iterations exceeded in RTSAFE"
};


BOOL TMotor::Translate( long &pos, double ang )
{
	// translate angle to position
	char buffer[ MaxString ];
	double l_ang;
	double l_pos;
	double xleft, xright;
	double temp;
	int ret;

	// Liegt Winkel im Bereich ?
	bRangeHit= FALSE;
	l_ang= ang - dAngleBias + fCorrBias;

	// 19.12.02 Kullmann, Reinecker
	// es ist bei einem negativen Koeff1 auch mglich das dAngleMin > dAngleMax ist
	// dies wrde hier immer zu einem RageHit fhren -> keine Antriebsbewegung mglich
	/*if( l_ang > dAngleMax )
	{
	bRangeHit= TRUE;
	l_ang= dAngleMax;
	}
	if( ang < dAngleMin )
	{
	bRangeHit= TRUE;
	l_ang= dAngleMin;
	}*/
	temp= l_ang;
	bRangeHit= !SetInRange( temp, dAngleMin, dAngleMax ); // bei Intervallberschreitung wird bRangeHit= TRUE

	// Uebersetzen des Winkels in eine Position
	/*switch ( eUnit )
	{
		case utGrad:
			l_ang= l_ang * 3600.0;
			break;

		case utMinuten:
			l_ang= l_ang * 60.0;
			break;
	}*/
	l_ang= CalcValueFromUnit(eUnit, l_ang);

	switch ( eCorrStatus )
	{
		case CorrPolynom:
			xleft= xright= l_ang / dKoeff_1;
			xright += dwInterval;
			xleft -= dwInterval;
			ret= rtsave( l_ang, xleft, xright, 1, &l_pos );
			if ( ret )
			{
				sprintf( buffer, "Newton %d !\n", ret );
				MessageBox( GetFocus( ), buffer, szMsgFailure, MBINFO );
				return FALSE;
			}
			break;

		default:
			l_pos= l_ang / dKoeff_1;
	}
	pos= (long)ceil(l_pos);
	bRangeHit= !SetInRange( pos , lPositionMax , lPositionMin ); // bei Intervallberschreitung wird bRangeHit= TRUE

	// 19.12.02 Kullmann, Reinecker
	// der selbe Grund wie oben in der Methode bei temp
	// pos= max( min( (long)ceil( l_pos ), lPositionMax ), lPositionMin );
	return TRUE;
};


BOOL TMotor::Translate( double &ang, long pos )
{
	// translate position to angle
	double l_ang;

	// Positions-Offset beruecksichtigen
	switch ( eCorrStatus )
	{
		case CorrPolynom:
			l_ang= pos * ( dKoeff_1 + pos * ( dKoeff_2 + pos * dKoeff_3 ) );
			break;

		default:
			l_ang= pos * dKoeff_1;
	}

	/*switch ( eUnit )
	{
		case utGrad:
			l_ang= l_ang / 3600.0;
			break;

		case utMinuten:
			l_ang= l_ang / 60.0;
			break;
	}*/
	l_ang= CalcValueInUnit(eUnit, l_ang);
	ang= l_ang + dAngleBias - fCorrBias;
	return TRUE;
};

// ANFANG: Kullmann, Reinecker (14.11.02) neu Hilfs-Methode zur Berechnung der absoluten Winkelposition
// ----------------------------------------------------------------------------
// translates position to angle

BOOL TMotor::TranslateAbs( double &ang, long pos )
{
	double l_ang;
	// Positions-Offset beruecksichtigen
	switch ( eCorrStatus )
	{
		case CorrPolynom:
			l_ang= pos * ( dKoeff_1 + pos * ( dKoeff_2 + pos * dKoeff_3 ) );
			break;

		default:
			l_ang= pos * dKoeff_1;
	}
	/*switch ( eUnit )
	{
		case utGrad:
			l_ang= l_ang / 3600.0;
			break;

		case utMinuten:
			l_ang= l_ang / 60.0;
			break;
	}*/
	l_ang= CalcValueInUnit(eUnit, l_ang);
	ang= l_ang - fCorrBias; // ang ist wahre absolute Winkelposition
	return TRUE;
};
// ENDE: Kullmann, Reinecker (14.11.02)

void TMotor::SetCorrectionState( BOOL onoff )
{
	long dcpos;
	float ang1, ang2;

	lCorrPos= 0;
	fCorrBias= 0.0;

	if ( !bCorrection || !bIsCalibrated )
	{
		eCorrStatus= CorrLinear;
		return;
	}
	if ( onoff )
	{
		if ( nId != lpMList->AD_DF )
		{
			eCorrStatus= CorrPolynom;
			return;
		}
		lpMList->MP( lpMList->AD_DC ) ->GetPosition( 1 );
		dcpos= lpMList->MP( lpMList->AD_DC ) ->lPosition;
		if ( dcpos == lCorrPos )
		{
			eCorrStatus= CorrPolynom;
			return;
		}
		GetAngle( 1 );
		ang1= dAngle;
		lCorrPos= dcpos;
		fCorrBias= 0.0;
		eCorrStatus= CorrPolynom;
		GetAngle( 1 ); // MS-Pos
		ang2= dAngle;
		fCorrBias= ang2 - ang1;
	}
	else
	{
		eCorrStatus= CorrLinear;
		lCorrPos= 0;
		fCorrBias= 0.0;
	}
};


BOOL TMotor::SetHome( void )
{
	bIsCalibrated= FALSE;
	SetCorrectionState( 0 );
	// lPreviousPosition= GetPosition( 1 );  Kullmann, Reinecker ( Toter Code, am 04.08.02 auskommentiert )
	lDeltaPosition= 0;
	lPosition= 0;
	dAngle= 0.0;
	dAngleBias= 0.0;
	return TRUE;
};


BOOL TMotor::MoveByPosition( long pos )
{
	lPosition += pos;
	return TRUE;
};


BOOL TMotor::MoveToPosition( long pos )
{
	lPosition= pos;
	return TRUE;
};


BOOL TMotor::MoveToAngle( double ang )
{
	long lpos;

	// Umwandlung von Winkel- in Positionsangaben
	if ( Translate( lpos, ang ) )
		// Aufruf der TDC_Drive-Funktion MoveToPosition() mit Pos.angabe
		if ( MoveToPosition( lpos ) )
			return TRUE;
	// bei Fehlschlag: FALSE
	return FALSE;
};


BOOL TMotor::MoveByAngle( double ang )
{
	long lpos;

	if ( !GetAngle( 1 ) )
		return FALSE;
	if ( Translate(lpos, (dAngle + ang)) )
		if ( MoveToPosition(lpos) )
			return TRUE;
	return FALSE;
};


BOOL TMotor::SetAngleWidth( double dang )
{
	double dposcmp, dpos;

	/*switch ( eUnit )
	{
		case utGrad:
			dposcmp= dpos= 3600.0 * fabs( dang / dKoeff_1 );
			break;

		case utMinuten:
			dposcmp= dpos= 60.0 * fabs( dang / dKoeff_1 );
			break;

		default:
			dposcmp= dpos= fabs( dang / dKoeff_1 );
			break;
	}*/
	dposcmp= dpos= CalcValueFromUnit(eUnit, fabs(dang / dKoeff_1));

	dpos= max( min( dpos, ( double ) wPositionMaxWidth ), ( double ) wPositionMinWidth );
	/*switch ( eUnit )
	{
		case utGrad:
			dAngleWidth= fabs( dKoeff_1 * dpos / 3600.0 );
			break;

		case utMinuten:
			dAngleWidth= fabs( dKoeff_1 * dpos / 60.0 );
			break;

		default:
			dAngleWidth= fabs( dKoeff_1 * dpos );
	}*/
	dAngleWidth= CalcValueInUnit(eUnit, dKoeff_1 * dpos);
	if ( dpos != dposcmp )
		return FALSE;
	return TRUE;
};


BOOL TMotor::GetAngle( BOOL saveyesno )
{
	double ang;

	// Holt Motorposition und speichert in IPosition,
	// Rueckgabewert bei C832 immer TRUE
	// nur bei C812 ohne DirectAccess FALSE moeglich
	if ( !GetPosition( saveyesno ) )
		return FALSE;
	// Falls keine Uebersetzung der IPosition in Winkel ang moeglich,
	// dann FALSE
	if ( !Translate( ang, lPosition ) ) // rechnet lPosition in ang um; bAngleBias wird beachtet
		return FALSE;
	// sonst Abspeichern des Winkels in Membervar dAngel und TRUE
	dAngle= ang;

	// Debug Code
#ifdef CheckCorrect

	if ( saveyesno )
	{
		char buf[ MaxString ];
		sprintf( buf, "P:%ld dP:%ld dW:%.2f", lPosition, lCorrPos, fCorrBias );
		SetInfo( buf );
	}
#endif

	return TRUE;
};

//ANFANG: neu Kullmann, Reinecker (04.08.2002) neue Methoden
//------------------------------------------------------------------------------
// aktuelle absolute Antriebsposition in Nutzereinheiten

double TMotor::GetAngleAbs( BOOL saveyesno, BOOL &aValid )
{
	double ang= 0.0;
	aValid= FALSE;
	// Holt Motorposition und speichert in IPosition,
	// Rueckgabewert bei C832 immer TRUE
	// nur bei C812 ohne DirectAccess FALSE moeglich
	if ( !GetPosition( saveyesno ) )
		return 0.0;
	// Falls keine Uebersetzung der IPosition in Winkel ang moeglich,
	// dann FALSE
	if ( !TranslateAbs( ang, lPosition ) )
		return 0.0;

	// Debug Code
#ifdef CheckCorrect

	if ( saveyesno )
	{
		char buf[ MaxString ];
		sprintf( buf, "P:%ld dP:%ld dW:%.2f", lPosition, lCorrPos, fCorrBias );
		SetInfo( buf );
	}
#endif
	aValid= TRUE;
	return ang;
};
//------------------------------------------------------------------------------
// aktuelle Antriebsposition in Nutzereinheiten

double TMotor::GetAngle( BOOL aStatus, BOOL &aValid )
{
	if ( !GetAngle( aStatus ) )
		aValid= FALSE;  // Auslesen und Umrechen

	aValid= TRUE;
	return dAngle;
};
//------------------------------------------------------------------------------
// minimale RELATIVE Antriebsposition in Nutzereinheiten

double TMotor::GetAngleRelativeMin( void )
{
	return dAngleMin + GetAngleBias();
};
//------------------------------------------------------------------------------
// maximale RELATIVE Antriebsposition in Nutzereinheiten

double TMotor::GetAngleRelativeMax( void )
{
	return dAngleMax + GetAngleBias();
};
//------------------------------------------------------------------------------
// minimale Schrittweite

double TMotor::GetMinAngleWidth()
{
	double a;
	/*if ( eUnit == utGrad )
		a= fabs( dKoeff_1 * wPositionMinWidth / 3600.0 );

	else if ( eUnit == utMinuten )
		a= fabs( dKoeff_1 * wPositionMinWidth / 60.0 );

	else
		a= fabs( dKoeff_1 * wPositionMinWidth );
	*/
	a= CalcValueInUnit(eUnit, dKoeff_1 * wPositionMinWidth);

	return a;
};
//------------------------------------------------------------------------------
// maximale Schrittweite

double TMotor::GetMaxAngleWidth()
{
	double a;
/*	if ( eUnit == utGrad )
		a= fabs( dKoeff_1 * wPositionMinWidth / 3600.0 );

	else if ( eUnit == utMinuten )
		a= fabs( dKoeff_1 * wPositionMinWidth / 60.0 );

	else
		a= fabs( dKoeff_1 * wPositionMinWidth );*/
	a= CalcValueInUnit(eUnit, dKoeff_1 * wPositionMinWidth);

	return a;
};
//------------------------------------------------------------------------------

double TMotor::GetAcceleration()
{
	double a;
	Translate(a, dwAcceleration);
	return fabsl(a);
};
//------------------------------------------------------------------------------
// minimale Antriebsgeschwindigkeit in Nutzereinheiten

double TMotor::GetMinSpeed()
{
	return (double)(1 / fSpeedScale);
};

//------------------------------------------------------------------------------
// maximale Antriebsgeschwindigkeit in Nutzereinheiten

double TMotor::GetMaxSpeed()
{
	return (double)(dwMaxVelocity / fSpeedScale);
};
//------------------------------------------------------------------------------
// setzt den Offset in Nutzereinheiten

BOOL TMotor::SetAngleBias( double aAngleBias )
{
	if ( !IsMoveFinish() )
		return FALSE; //nur bei Stillstand des Antriebs kann gesetzt werden
	dAngleBias= aAngleBias;
	return TRUE;
};
//------------------------------------------------------------------------------

UINT TMotor::GetDigits( void )
{
	return nDigits;
};

//ENDE  : neu Kullmann, Reinecker (04.08.2002)

//------------------------------------------------------------------------------

int TMotor::SaveSettings( BOOL LastSave )
{
	//*** Abspeichern der Motoreinstellungen in den Parameter-File.
	HCURSOR hCursorOld;
	char buf[ MaxString ];
	char Ident[ MaxString ];

	if ( !bControlBoardOk )
		return R_Failure;
	sprintf( Ident, "motor%d", nId );
	hCursorOld= SetCursor( LoadCursor(NULL,IDC_WAIT) );
	sprintf( buf, "%ld", lDistanceToZero );
	WritePrivateProfileString( Ident, "DistanceToZero", buf, GetHWFile( ) );
	if ( LastSave )
	{
		SetCorrectionState( FALSE );
		sprintf( buf, "%ld", dwVelocity );
		WritePrivateProfileString( Ident, "Velocity", buf, GetHWFile( ) );
		sprintf( buf, "%u", wPositionWidth );
		WritePrivateProfileString( Ident, "PositionWidth", buf, GetHWFile( ) );
		sprintf( buf, "%d", bUpwards );
		WritePrivateProfileString( Ident, "Upwards", buf, GetHWFile( ) );
		GetPosition( TRUE );
		sprintf( buf, "%ld", lPosition );
		WritePrivateProfileString( Ident, "DeltaPosition", buf, GetHWFile( ) );
		sprintf( buf, pDF( ), dAngleMin );
		WritePrivateProfileString( Ident, "AngleMin", buf, GetHWFile( ) );
		sprintf( buf, pSF( ), dAngleWidth );
		WritePrivateProfileString( Ident, "AngleWidth", buf, GetHWFile( ) );
		sprintf( buf, pDF( ), dAngleBias );
		WritePrivateProfileString( Ident, "AngleBias", buf, GetHWFile( ) );
		sprintf( buf, pDF( ), dAngleMax );
		WritePrivateProfileString( Ident, "AngleMax", buf, GetHWFile( ) );
		WritePrivateProfileString( Ident, "MJ_Offset", "0", GetHWFile( ) ); // 10.12.2002 Kullmann+Reinecker: Manuelle Justage Offset =0 setzen
		if ( bIsCalibrated )
			WritePrivateProfileString( Ident, "RestartPossible", "1", GetHWFile( ) );
	}
	SetCursor( hCursorOld );
	return R_OK;
};


//******************************************************************************
TDC_Drive::TDC_Drive( void ) : TMotor( )
{
	char Ident[ MaxString ];
	nMaxError= 2;

	sprintf( Ident, "motor%d", nId );
	wDeathBand= ( WORD ) GetPrivateProfileInt( Ident, "DeathBand", 1, GetHWFile( ) );
	wDeccelerationPoint= ( WORD ) GetPrivateProfileInt( Ident, "DeccelerationPoint",
						  20, GetHWFile( ) );
};


int TDC_Drive::SaveSettings( BOOL onlythis )
{
	return TMotor::SaveSettings( onlythis );
}


int TDC_Drive::Initialize( void )
{
	char buf[ MaxString ];

	if ( R_OK != TMotor::Initialize( ) )
		goto Exit;
	if ( CheckBoardOk( ) )
	{
		if ( !Reset( ) )
			goto Exit;
		if ( !ActivateFilterParameters( ) )
			goto Exit;
		if ( !ActivateDrive( ) )
			goto Exit;
	}
	else
	{
		sprintf( buf, szMsgLine012, pCharacteristic( ) );
		MessageBox( GetFocus( ), buf, szMsgFailure, MBSTOP );
#ifdef TESTLAUF

		return R_NoCommunication;
#endif
		return(FALSE);
	}
	return R_OK;

Exit:
#ifdef TESTLAUF

	return FALSE;
#else

	return R_OK;
#endif
};


BOOL TDC_Drive::PushSettings( void )
{
	return TRUE;
};


BOOL TDC_Drive::PopSettings( void )
{
	return TRUE;
};


BOOL TDC_Drive::MoveToPosition( long position )
{
	// Test auf Richtungswechsel zur Beruecksichtigen des Motorspiels
	if ( bUpwards && ( position < lPosition ) )
	{
		// ===> wechselt in <===
		lDeltaPosition += dwHysteresis;
		bUpwards= FALSE;
	}
	else
	{
		if ( !bUpwards && ( position > lPosition ) )
		{
			// <=== wechselt in ===>
			lDeltaPosition -= dwHysteresis;
			bUpwards= TRUE;
		}
	}

	// Beruecksichtigen der relativen Position nach Programm-Start
	// und Richtungswechsel
	position -= lDeltaPosition + lCorrPos;

	// Aufruf der motorabhaengigen Hardware-Funktion
	// TC_812::MoveAbsolute() bzw. TC_832::MoveAbsolute()
	// mit umgerechneter bzw. korrigierter Positionsangabe
	// (unter Bercksichtigung von Richtung, Spiel und relativer Position)
	return MoveAbsolute( position );
};


BOOL TDC_Drive::MoveByPosition( long distance )
{
	return MoveRelative( distance );
};


BOOL TDC_Drive::SetSpeed( double speed )
{
	DWORD velocity;

	velocity= fSpeedScale * speed;
	SetVelocity( velocity );
	dSpeed= (double) GetVelocity() / fSpeedScale;
	return TRUE;
};


double TDC_Drive::GetSpeed( void )
{
	DWORD vel;

	vel= GetVelocity( );
	if ( vel == 0 )
		return 0.0;
	dSpeed= (double) vel / fSpeedScale;
	return dSpeed;
};


BOOL TDC_Drive::GetPosition( BOOL bSave )
{
	long pos1, pos2;

	// Aufruf der Member-Fkt. _GetPosition
	// fuer die speziellen Karten (C812(ISA), C832)
	// Rueckgabewert bei C832 immer TRUE;
	// FALSE nur bei C812 ohne DirectAccess moeglich

	// Auslesen der 'real position'
	if ( _GetPosition( &pos1 ) )
	{
		if ( bSave )   // Wert speichern ?
		{
			// erneutes Auslesen der akt. Position
			if ( !_GetPosition( &pos2 ) )
				return FALSE;
			// bei Positionswertaenderung -> kein Speichern (Fahrbetrieb ?)
			if ( pos1 != pos2 )
				return FALSE;
		}
		// Beruecksichtigen der relativen Karten-Position
		lPosition= pos1 + lDeltaPosition + lCorrPos;
		return TRUE;
	}
	return FALSE;
};


long TDC_Drive::GetFailure( void )
{
	long failure;
	int check= 3;

	while ( !_GetFailure( &failure ) )
	{
		if ( 0 == check-- )
			return 100;
	}
	return failure;
};


//*************************************************************************
TC_812::TC_812( void ) : TDC_Drive( )
{
	char Ident[ MaxString ];

	sprintf( Ident, "motor%d", nId );
	nOnBoardId= GetPrivateProfileInt( Ident, "BoardId", 1, GetHWFile( ) );
	wStaticGain= ( WORD ) GetPrivateProfileInt( Ident, "Gain", 100, GetHWFile( ) );
	wDynamicGain= ( WORD ) GetPrivateProfileInt( Ident, "DynamicGain", 37, GetHWFile( ) );
	wMaxTorque= 127;
	nWaitTicks= 5;
	wTorque= ( WORD ) GetPrivateProfileInt( Ident, "Torque", 110, GetHWFile( ) );
};


int TC_812::SaveSettings( BOOL bLastSave )
{
	char buf[ MaxString ];
	char Ident[ MaxString ];

	if ( !bControlBoardOk )
		return R_Failure;
	sprintf( Ident, "motor%d", nId );
	sprintf( buf, "%u", wDynamicGain );
	WritePrivateProfileString( Ident, "DynamicGain", buf, GetHWFile( ) );
	sprintf( buf, "%u", wTorque );
	WritePrivateProfileString( Ident, "Torque", buf, GetHWFile( ) );
	sprintf( buf, "%u", wStaticGain );
	WritePrivateProfileString( Ident, "Gain", buf, GetHWFile( ) );
	sprintf( buf, "%lu", dwMaxVelocity );
	WritePrivateProfileString( Ident, "MaxVelocity", buf, GetHWFile( ) );
	sprintf( buf, "%lu", dwAcceleration );
	WritePrivateProfileString( Ident, "Acceleration", buf, GetHWFile( ) );
	sprintf( buf, "%lu", dwVelocity );
	WritePrivateProfileString( Ident, "Velocity", buf, GetHWFile( ) );
	return TDC_Drive::SaveSettings( bLastSave );
};


BOOL TC_812::ActivateFilterParameters( void )
{
	char cmd[ MaxString ];

	sprintf( cmd, "%uDB%u\r", nOnBoardId, wDeathBand );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	sprintf( cmd, "%uSQ%u,%uSG%u,%uSE%u\r", nOnBoardId, wTorque, nOnBoardId,
			 wStaticGain, nOnBoardId, wDynamicGain );

	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	SetVelocity( dwMaxVelocity );
	SetAcceleration( dwAcceleration );
	SetLimit( dwRemoveLimit );
	GetSpeed( );
	return TRUE;
};


BOOL TC_812::Reset( void )
{
	long lDeltaP= lDeltaPosition;
	BOOL bIsCal= IsCalibrated( );
	BOOL ret= SetHome( );
	lDeltaPosition= lDeltaP;
	SetCalibrationState( bIsCal );
	nC812= GetId( );
	return ret;
};


BOOL TC_812::SetMoment( float moment )
{
	// Dreh-Moment 0 .. 1.0
	if ( moment <= 1.0 )
		wTorque= 127 * moment;
	else
		wTorque= 60;
	SetTorque( wTorque );
	return TRUE;
};


float TC_812::GetMoment( void )
{
	WORD torque;
	// Dreh-Moment 0 .. 1.0
	torque= GetTorque( );
	return ( torque / wMaxTorque );
};


BOOL TC_812::SetLine( int number, BOOL status )
{
	char cmd[ MaxString + 1 ];

	if ( ( 1 > number ) && ( 16 < number ) )
		return FALSE;
	if ( status )
		sprintf( cmd, "%uCT%u,%uCN%u\r", nOnBoardId, number, nOnBoardId, number );
	else
		sprintf( cmd, "%uCT%u,%uCF%u\r", nOnBoardId, number, nOnBoardId, number );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	return TRUE;
};


BOOL TC_812::IsLimitHit( void )
{
	if ( GetStatus() & StatMLimitHit )
		return TRUE;
	else
		return FALSE;
};


BOOL TC_812::IsMoveFinish( void )
{
	if ( bInquireStatus )
	{
		if ( GetStatus() & StatMStopped )
			return TRUE;
	}
	else
	{
		if ( labs( GetFailure() ) <= (long) wDeathBand )
			return TRUE;
	}
	return FALSE;
};


BOOL TC_812::IsIndexArrived( void )
{
	if ( bIndexDetected && IsMoveFinish() )
		return TRUE;
	if ( IsLimitHit() )
	{
		if ( !bIndexDetected )
		{
			DelayTime( 20 );
			bIndexDetected= TRUE;
		}
	}
	return FALSE;
};


BOOL TC_812::StartToIndex( long &oldPos )
{
	SetSpeed( 10000.0 );
	lDeltaPosition= 0;
	SetCorrectionState( 0 );
	GetPosition( TRUE );
	oldPos= lPosition;
	bIndexDetected= FALSE;
	bIndexLine= FALSE;
	MoveToPosition( lPosition - 400000000l );
	// Die Motoren werden in Bewegung gesetzt
	// Erreichen der Stillstands-Position
	DelayTime( 100 );
	return TRUE;
};


BOOL TC_812::ActivateDrive( void )
{
	char cmd[ MaxString + 1 ];

	sprintf( cmd, "%uSP%u\r", nOnBoardId, wDeccelerationPoint );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	sprintf( cmd, "%uMN\r", nOnBoardId );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	return TRUE;
};


BOOL TC_812::StopDrive( BOOL bOffOn ) // Kullmann, Reinecker (08.08.02) Parameter gendert int -> BOOL
{
	char cmd[ MaxString + 1 ];

	if ( bOffOn )
	{
		sprintf( cmd, "%uAB\r", nOnBoardId );
		if ( R_OK != ExecuteCmd( cmd ) )
			return FALSE;
		// Einige Motoren koennen nach diesem Stop ihre Position
		// nicht mehr einregeln, deshalb ein zweites Kommando zum
		// Gleichsetzen von Target und Position
		Delay( 100 );
		
		sprintf( cmd, "%uAB\r", nOnBoardId );
		if ( R_OK != ExecuteCmd( cmd ) )
			return FALSE;
		sprintf( cmd, "%uMN\r", nOnBoardId );
		if ( R_OK != ExecuteCmd( cmd ) )
			return FALSE;
	}
	else
	{
		sprintf( cmd, "%uAB\r", nOnBoardId );
		if ( R_OK != ExecuteCmd( cmd ) )
			return FALSE;
	}
	return TRUE;
};


BOOL TC_812::SetDynamicGain( WORD dygain )
{
	char cmd[ MaxString + 1 ];

	dygain= min( dygain, ( WORD ) 255 );
	dygain= max( dygain, ( WORD ) 1 );
	wDynamicGain= dygain;

	sprintf( cmd, "%uSE%u\r", nOnBoardId, wDynamicGain );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	return TRUE;
};


WORD TC_812::GetDynamicGain( void )
{
	return wDynamicGain;
};


BOOL TC_812::SetTorque( WORD torque )
{
	char cmd[ MaxString + 1 ];

	wTorque= min( torque, ( WORD ) 127 );
	sprintf( cmd, "%uSQ%u\r", nOnBoardId, wTorque );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	return TRUE;
};


WORD TC_812::GetTorque( void )
{
	wTorque= min( wTorque, ( WORD ) 127 );
	return wTorque;
};


BOOL TC_812::SetVelocity( DWORD velocity )
{
	char cmd[ MaxString + 1 ];

	dwVelocity= ( velocity < dwMaxVelocity ) ? velocity : dwMaxVelocity;
	sprintf( cmd, "%uSV%lu\r", nOnBoardId, dwVelocity );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	sprintf( cmd, "%uCP\r", nOnBoardId );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	return TRUE;
};


DWORD TC_812::GetVelocity( void )
{
	dwVelocity= ( dwVelocity < dwMaxVelocity ) ? dwVelocity : dwMaxVelocity;
	return dwVelocity;
};


BOOL TC_812::SetAcceleration( DWORD acceleration )
{
	char cmd[ MaxString + 1 ];

	dwAcceleration= acceleration;
	sprintf( cmd, "%uSA%lu\r", nOnBoardId, dwAcceleration );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	sprintf( cmd, "%uSD%lu\r", nOnBoardId, dwAcceleration );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	sprintf( cmd, "%uCP\r", nOnBoardId );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	return TRUE;
};


DWORD TC_812::GetAcceleration( void )
{
	return dwAcceleration;
};


BOOL TC_812::SetLimit( DWORD limit )
{
	char cmd[ MaxString + 1 ];

	dwRemoveLimit= limit;
	sprintf( cmd, "%uLS%lu\r", nOnBoardId, dwRemoveLimit );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	return TRUE;
};


BOOL TC_812::SetHome( void )
{
	char cmd[ MaxString + 1 ];

	// changing K-System
	TMotor::SetHome( );
	sprintf( cmd, "%uDH\r", nOnBoardId );
	if ( R_OK != ExecuteCmd( cmd ) )
		return FALSE;
	return TRUE;
};


WORD TC_812::GetStatus( void )
{
	char buf[ MaxString + 1 ];
	char msg[ MaxString ];
	char *token;
	WORD _status;

	sprintf( buf, "%uTS\r", nOnBoardId );
	switch ( ExecuteCmd( buf ) )
	{
		case R_OK:
			token= strtok( buf, "\n\3\r" );
			if ( !token )
				return FALSE;
			if ( ( token[ 1 ] - '0' ) != nOnBoardId )
				return FALSE;
			if ( token[ 2 ] != 'S' )
				return FALSE;
			if ( strlen( token ) < 6 )
				return FALSE;
			sscanf( &( token[ 3 ] ), "%u", &_status );
			return _status;

		case R_TimeOut:
			sprintf( msg, "GetStatus: %s", buf );
			MessageBox( GetFocus( ), msg, "TimeOut MS", MBFAILURE );
			return -1;

		case R_UnknownCmd:
			sprintf( msg, "GetStatus: %s", buf );
			MessageBox( GetFocus( ), msg, "Fehler", MBFAILURE );
			return -1;

		default:
			return -1;
	}
};


BOOL TC_812::_GetPosition( long *position )
{
	char buf[ MaxString ];
	char msg[ MaxString ];
	char *token;

	sprintf( buf, "%uTP\r", nOnBoardId );
	switch ( ExecuteCmd( buf ) )
	{
		case R_OK:
			token= strtok( buf, "\n\3\r" );
			if ( !token )
				return FALSE;
			if ( ( token[ 1 ] - '0' ) != nOnBoardId )
				return FALSE;
			if ( token[ 2 ] != 'P' )
				return FALSE;
			sscanf( &( token[ 3 ] ), "%ld", position );
			return TRUE;

		case R_TimeOut:
			sprintf( msg, "GetPosition: %s", buf );
			MessageBox( GetFocus( ), msg, "TimeOut MS", MBFAILURE );
			return FALSE;

		case R_UnknownCmd:
			sprintf( msg, "GetPosition: %s", buf );
			MessageBox( GetFocus( ), msg, "Fehler", MBFAILURE );
			return FALSE;

		default:
			return FALSE;
	}
};


BOOL TC_812::_GetFailure( long *failure )
{
	char buf[ MaxString + 1 ];
	char msg[ MaxString ];
	char *token;

	sprintf( buf, "%uTE\r", nOnBoardId );
	switch ( ExecuteCmd( buf ) )
	{
		case R_OK:
			token= strtok( buf, "\n\3\r" );
			if ( !token )
				return FALSE;
			if ( ( token[ 1 ] - '0' ) != nOnBoardId )
				return FALSE;
			if ( token[ 2 ] != 'E' )
				return FALSE;
			if ( strlen( token ) < 12 )
			{
				nWaitTicks += 2;
				return FALSE;
			}
			sscanf( &( token[ 3 ] ), "%ld", failure );
			return TRUE;

		case R_TimeOut:
			sprintf( msg, "GetFailure: %s", buf );
			MessageBox( GetFocus( ), msg, "TimeOut MS", MBFAILURE );
			return FALSE;

		case R_UnknownCmd:
			sprintf( msg, "GetFailure: %s", buf );
			MessageBox( GetFocus( ), msg, "Fehler", MBFAILURE );
			return FALSE;

		default:
			return FALSE;
	}
};


BOOL TC_812::MoveRelative( long pos )
{
	char cmd[ MaxString + 1 ];

	sprintf( cmd, "%uMR%ld\r", nOnBoardId, pos );
	if ( R_OK == ExecuteCmd( cmd ) )
		return TRUE;
	return FALSE;
};


BOOL TC_812::MoveAbsolute( long pos )
{
	char cmd[ MaxString + 1 ];

	// Zusammensetzung des Motorkommandos "Move Absolute"
	// [boardID] MA [Position] [RETURN]
	sprintf( cmd, "%uMA%ld\r", nOnBoardId, pos );
	// Aufruf der Hardwarezugriffsfkt.
	// TC_812ISA::ExecuteCmd()
	if ( R_OK == ExecuteCmd( cmd ) )
		// bei korrekter Ausfuehrung des Kommandos: TRUE
		return TRUE;
	// wenn Fehler bei Kommandoausfuehrg.: FALSE
	return FALSE;
};

//############################################################################
//############### ISA Interface ##############################################
//############################################################################
//############################################################################

TC_812Controller::TC_812Controller(EDeviceType DeviceID, LPTSTR HardwareID, DeviceList* Devices)
					: Controller(C812,HardwareID,4),oDPRam(0),oFlag(0),oIn(0),oOut1(0),oOut2(0)
{
	char* szDeviceName= Devices->GetFileName(DeviceID, HardwareID);
	if(!szDeviceName)
	{
		if(simulation_type == simulation_only)
		{	//MsgBox bzgl. des fehlenden Treibers in der Simulation unterdrcken
			Hardware= new DummyIo(DeviceID, HardwareID, false);
		}
		else
		{
			Hardware= new DummyIo(DeviceID, HardwareID);
		}
	}
	else
	{
		Hardware= new HardwareIo(szDeviceName);
	}
}

BOOL TC_812Controller::Check()
{
	char  buf[ MaxString ];
	int   cnt= 60;
	int   retval;

	Put(oOut1,(BYTE)0x0D);
	Put(oOut1,(BYTE)0x0D);

	do
	{
		BYTE inByte= 0;

		DelayTime(4);
		inByte= Get(oIn);
		if(inByte == 0x0D)
			break;
		inByte= Get(oIn);
		if(inByte == 0x03)
			break;
	} while(--cnt);

	if( !cnt )
	{
		sprintf(buf,"No Board present at address 0x%s !",HardwareID);
		MessageBox(GetFocus( ),buf,"C-812",MBSTOP);
		return(FALSE);
	}

	strcpy(buf,"DM\r");
	retval= ExecuteCmd(buf);
	strcpy(buf,"EF\r");
	retval= ExecuteCmd(buf);
	if(retval != R_OK)
	{
		MessageBox(GetFocus(),buf,"C-812",MBFAILURE);
		return(FALSE);
	}

	return(TRUE);
}

char TC_812Controller::Get(DWORD addr)
{
  char return_val= 0;

  if ( simulation_type & 1 )
  {
    // case no_simulation: case test_simulation:
    // get the hardware generated return value
	Read(addr,(BYTE*)&return_val);

    if ( simulation_type & 2 )
    {
      // case test_simulation:
      // inform driver if possible, simulated value is ignored
      if ( c812isa_get_callback )
      {
		char* p= (char*)addr;
		c812isa_get_callback(p, return_val);

      }
    }
  }
  else
  {
    // case simulation_only:
    // let driver generate a simulated value
    if ( c812isa_get_callback )
    {
		char* p= (char*)addr;
	  return_val= c812isa_get_callback(p,0);
    }
  }

  return return_val;
}

void TC_812Controller::Put(DWORD addr, BYTE data)
{
  if ( simulation_type & 1 )
  {
    // case no_simulation: case test_simulation:
    // write the value to hardware
	Write(addr,data);
  }
  if ( simulation_type & 2 )
  {
    // case simulation_only: case test_simulation:
    // write the value to the simulation driver
    if ( c812isa_put_callback )
    {
      char* p= (char*)addr;
	  c812isa_put_callback(p, data);
    }
  }
}

int TC_812Controller::ExecuteCmd(LPTSTR pString)
{
	int   idx,cnt,retval= R_OK;
	char  chr;

	static BOOL bCmdActive= FALSE;

	// bereits im Kommandomodus ? ja -> Fehler
	if(bCmdActive)
		return R_Failure;

	// nun Umschalten in Kommandomodus
	bCmdActive= TRUE;

	cnt= 50; // Anzahl der zulaessigen Wiederholungen
	// Wiederholen bis keine Daten mehr zu lesen
	while ((chr= GetChar()) != '\x0')
	{// maximale Anzahl der Wiederholungsversuche erreicht => timeout
		if(!cnt--)
		{
			// timeout aufgetreten
			strcpy(pString,"TimeOut read");
			bCmdActive= FALSE;
			retval= R_TimeOut; // Rueckgabewert R_TimeOut
			return retval;
		}
	}

	// Befehls-String senden
	idx= 0;
	// solange Kommandostring noch nicht abgearbeitet
	while(pString[idx]!= 0)
	{
		// Zeichen des Kommando-Strings senden
		// Rueckgabe==0 ? d.h. Senden schlug fehl
		if(!PutChar(pString[idx]))
		{
			Delay(10); // 10 "delays" warten ??
			//Zeichensenden fehlgeschlagen ?
			if(!PutChar(pString[idx]))
			{
				// mgl.weise ex. noch auszulesendes Zeichen ?
				chr= GetChar();
				if(chr)
					continue; // falls ja: Sprung zum Anfang der While-Schleife
				// chr=0 -> Timeout beim Schreiben
				bCmdActive= FALSE;
				strcpy(pString,"TimeOut write");
				retval= R_TimeOut; // Rueckgabewert R_TimeOut
				return(retval);
			}
		}
		//naechstes Zeichen im Kommandostring
		idx++;
	}

	// recieve Request (Warten auf Uebertragungsende -> 0x03 an lpIn)
	Delay(50);
	idx= 0;
	cnt= 250; // max. Wiederholungen bis TimeOut

	while(idx < (20 * MaxString))
	{
		// Zeichen an lpIn auslesen
		chr= GetChar();
		if(!chr)
		{
			Delay(15);
			if(!cnt)
			{ // Timeout beim Lesen
				MessageBox(GetFocus(),"TimeOut read","C-812",MBINFO);
				// strcpy(pString,"TimeOut read");
				bCmdActive= FALSE;
				// bControlBoardOk= FALSE;
				retval= R_TimeOut; // Rueckgabewert R_TimeOut
				return retval;
			}
			if((cnt--)  && (idx == 0))
				continue; // zum Anfang der While-Schleife: auf den Request warten
			if(pString[idx-1] == '\x3')
				break; // Ende der Uebertragung erreicht
			continue; // zum Anfang der While-Schleife: auf den Request warten
		}
		// ausgelesene Zeichen an lpIn zu String zusammenfuegen
		pString[idx++]= chr;
	}

	// Nulltermierung des Strings
	pString[idx]= '\x0';
	// Kommandomodus verlassen
	bCmdActive= FALSE;

	// Fehlernachricht von Karten empfangen ?
	if(pString[0] == '?')
	{
		retval= R_Failure; // Rueckgabewert R_Failure
		return retval;
	}

	if(!cnt)
	{
		retval= R_Failure; // Rueckgabewert R_Failure
		return retval;
	}

	return R_OK;
}

int TC_812Controller::PutChar(const char c)
{
	int time1,time2;

	Delay(3); // wozu ????
	// solange Bit 0 (Busy-Flag) an Adresse lpFlag gesetzt (=1)
	// oder time1 > 0 wird die Schleife abgearbeitet (gewartet)
	// Timeout ist aufgetreten falls time1=0
	BYTE inByte= Get(oFlag);
	for(time1= 30000;(inByte & 0x01) && time1; time1--)
	{
		inByte= Get(oFlag);
	}

	// Test ob Timeout ( time1=0) aufgetreten
	// falls nicht (time1>0) und damit Bit 0 (Busy-Flag) an lpFlag=0
	if( time1 )   //time
	{
		Put(oOut1,(BYTE)c);		// write data
		Delay( 3 );				// Verzoegerung
		Put(oOut2,(BYTE)c);		// generate internal signal
	}

	// Warten bis Controller wieder schreibbereit
	// (Bit 0 (Busy-Flag) von lpFlag= 0)
	inByte= Get(oFlag);
	for(time2= 30000; (inByte & 0x00) && time2; time2--)
	{
		inByte= Get(oFlag);
	}
	Delay(3);  // Verzoegerung, wozu ???

	// Returnwert 0=timeout, sonst OK
	return time1;
}

char TC_812Controller::GetChar(void)
{
	char chr= 0;

	Delay( 1 );  // 1
	// Wenn Bit 1 gesetzt ist, so liegen Daten von der Steuerung an.
	BYTE inByte= Get(oFlag);
	if(!(inByte & 0x02))
		return(0);

	inByte= Get(oIn);
	chr= (char)inByte;

	Delay( 2 );                // wozu ???
	return chr;
}
// ph
/////////////////////////////////////////////////////////////////


TC_812ISA::TC_812ISA( void ) : TC_812( )
{
	char buf[ MaxString ];
	char Ident[ MaxString ];

	sprintf( Ident, "motor%d", nId );
	GetPrivateProfileString( Ident, "RamAddr", "0xD0000", buf, MaxString,
							 (LPSTR)GetHWFile( ) );
	sscanf( buf, "%x", &dwBaseAddr );

	//initialisieren des zugehrigen Controller-Objekts
	Hardware= dynamic_cast<TC_812Controller*>(GetController(C812,buf,&MotorControllers,&MotorDrivers));
	// Member des Controller-Objects initialisieren (ggf. schon initialisiert)
	
	Hardware->oOut1= 0x03FC;
	Hardware->oOut2= 0x03FF;
	Hardware->oFlag= 0x0800;
	Hardware->oIn= 0x03FE;
	Hardware->oDPRam= 0;
	//ph

	nWaitTicks= 5;
};


BOOL TC_812ISA::CheckBoardOk( void )
{
	char buf[ MaxString ];
	int cnt= 60;
	int retval;

	Hardware->Put(Hardware->oOut1,(BYTE)0x0D);
	Hardware->Put(Hardware->oOut1,(BYTE)0x0D);
	do
	{
		BYTE inByte= 0;

		DelayTime(4);
		inByte= Hardware->Get(Hardware->oIn);
		if(inByte == 0x0D)
			break;
		inByte= Hardware->Get(Hardware->oIn);
		if(inByte == 0x03)
			break;
	} while(--cnt);

	if ( !cnt )
	{
		//hp sprintf( buf, "No C-812 Board present at address %X !", dwBaseAddr );
		//hp MessageBox( GetFocus( ), buf, "C-812", MBSTOP );
#ifndef TESTLAUF

		return FALSE;
#endif

	}
	bControlBoardOk= TRUE;
	
	strcpy( buf, "DM\r" );
	retval= ExecuteCmd( buf );
	if ( retval != R_OK )
	{
		MessageBox( GetFocus( ), buf, "C-812", MBFAILURE );
#ifndef TESTLAUF

		bControlBoardOk= FALSE;
		return FALSE;
#endif
	}

	strcpy( buf, "EF\r" );
	retval= ExecuteCmd( buf );
	if ( retval != R_OK )
	{
		MessageBox( GetFocus( ), buf, "C-812", MBFAILURE );
#ifndef TESTLAUF

		bControlBoardOk= FALSE;
		return FALSE;
#endif
	}
	return TRUE;
};


BOOL TC_812ISA::SetHome( void )
{

	char cmd[ MaxString+1 ];

	// changing K-System
	TMotor::SetHome( );
	sprintf( cmd,"%uDH\r",nOnBoardId );
	if(R_OK != ExecuteCmd(cmd))
		return FALSE;

	int  addr= 0x010A + nOnBoardId - 1;
	Hardware->Put(Hardware->oDPRam + addr,(BYTE)0x00);
	Hardware->Put(Hardware->oDPRam + addr + 0x0004,(BYTE)0x00);
	Hardware->Put(Hardware->oDPRam + addr + 0x0008,(BYTE)0x00);
	Hardware->Put(Hardware->oDPRam + addr + 0x000C,(BYTE)0x00);
	//ph

	return TRUE;
};


BOOL TC_812ISA::IsMoveFinish( void )
{

	long fail1, fail2;

	_GetFailure( &fail1 );
	if ( labs( fail1 ) > (long)wDeathBand )
	{
		return FALSE;
	}
	else
	{
		// der Positionsfehler ist kleiner als Deathband, d.h. es kann sein,
		// das der Controller mit der aktuellen Position zufrieden ist (dicht genug
		// an der Sollposition)
		// um sicher zu gehen warten wie einen Moment; wenn der Positionsfehler dann
		// unverndert ist, steht der Motor
		DelayTime( 100 );
		_GetFailure( &fail2 );
		if ( fail1 - fail2 )
		{
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}

};


BOOL TC_812ISA::_GetPosition( long *position )
{
	tpos data;
	int addr= 0x010A + nOnBoardId - 1;

	data.byte[ 0 ]= Hardware->Get(Hardware->oDPRam + addr);
	data.byte[ 1 ]= Hardware->Get(Hardware->oDPRam + addr + 0x0004);
	data.byte[ 2 ]= Hardware->Get(Hardware->oDPRam + addr + 0x0008);
	data.byte[ 3 ]= Hardware->Get(Hardware->oDPRam + addr + 0x000C);
	*position= data.pos;
	return TRUE;
};


BOOL TC_812ISA::_GetFailure( long *failure )
{
	tpos data;
	int addr= 0x006C + nOnBoardId - 1;

	data.byte[ 0 ]= Hardware->Get(Hardware->oDPRam + addr);
	data.byte[ 1 ]= Hardware->Get(Hardware->oDPRam + addr + 0x0004);
	data.byte[ 2 ]= Hardware->Get(Hardware->oDPRam + addr + 0x0008);
	data.byte[ 3 ]= Hardware->Get(Hardware->oDPRam + addr + 0x000C);
	*failure= data.pos;
	return TRUE;

};


int TC_812ISA::ExecuteCmd( char *pString )
{
#ifndef TESTLAUF
	int idx, cnt, retval= R_OK;
	char chr;
#endif

	static BOOL bCmdActive= FALSE;

	// DC-Controller-Karte OK ?
	if ( !bControlBoardOk )
		return 0;
	// bereits im Kommandomodus ? ja -> Fehler
	if ( bCmdActive )
		return R_Failure;
	// nun Umschalten in Kommandomodus
	bCmdActive= TRUE;

#ifndef TESTLAUF

	cnt= 250; // Anzahl der zulaessigen Wiederholungen
	// Wiederholen bis keine Daten mehr zu lesen

	while ( ( chr= GetChar( ) ) != '\x0' )
	{
		// maximale Anzahl der Wiederholungsversuche erreicht => timeout
		if ( !cnt-- )
		{
			// timeout aufgetreten
			strcpy( pString, "TimeOut read" );
			bControlBoardOk= FALSE;
			bCmdActive= FALSE;
			retval= R_TimeOut; // Rueckgabewert R_TimeOut
			goto Exit;
		}
	}

#endif

	// Befehls-String senden
#ifdef WriteLog
	FILE *LogFile;
	LogFile= fopen( "\motor.log", "a" );
	if ( LogFile )
		fseek( LogFile, 0, 2 );
#endif

#ifndef TESTLAUF

	idx= 0;
	// solange Kommandostring noch nicht abgearbeitet

	while ( pString[ idx ] != 0 )
	{
		// Zeichen des Kommando-Strings senden
		// Rueckgabe==0 ? d.h. Senden schlug fehl
		if ( !PutChar( pString[ idx ] ) )
		{
			Delay( 10 ); // 10 "delays" warten ??
			//Zeichensenden fehlgeschlagen ?
			if ( !PutChar( pString[ idx ] ) )
			{
				// mgl.weise ex. noch auszulesendes Zeichen ?
				chr= GetChar( );
				if ( chr )
					continue; // falls ja: Sprung zum Anfang der While-Schleife
				// chr=0 -> Timeout beim Schreiben
				bCmdActive= FALSE;
				bControlBoardOk= FALSE;
				strcpy( pString, "TimeOut write" );
				retval= R_TimeOut; // Rueckgabewert R_TimeOut
				goto Exit;
			}
		}
		//naechstes Zeichen im Kommandostring
		idx++;
	}

#endif

#ifdef WriteLog
	if ( LogFile )
		fwrite( pString, strlen( pString ), 1, LogFile );
#endif

	// recieve Request (Warten auf Uebertragungsende -> 0x03 an lpIn)
	Delay( 50 );
#ifndef TESTLAUF

	idx= 0;
	cnt= 500; // max. Wiederholungen bis TimeOut

	while ( idx < ( 20 * MaxString ) )
	{
		// Zeichen an lpIn auslesen
		chr= GetChar( );
		if ( !chr )
		{
			Delay( 15 );
			if ( !cnt )
			{ // Timeout beim Lesen
				MessageBox( GetFocus( ), "TimeOut read", "C-812", MBINFO );
				// strcpy(pString,"TimeOut read");
				bCmdActive= FALSE;
				// bControlBoardOk= FALSE;
				retval= R_TimeOut; // Rueckgabewert R_TimeOut
				goto Exit;
			}
			if ( ( cnt-- ) && ( idx == 0 ) )
				continue; // zum Anfang der While-Schleife: auf den Request warten
			if ( pString[ idx - 1 ] == '\x3' )
				break; // Ende der Uebertragung erreicht
			continue; // zum Anfang der While-Schleife: auf den Request warten
		}
		// ausgelesene Zeichen an lpIn zu String zusammenfuegen


		pString[ idx++ ]= chr;
	}

	// Nulltermierung des Strings
	pString[ idx ]= '\x0';
	// Kommandomodus verlassen
	bCmdActive= FALSE;
	// Fehlernachricht von Karten empfangen ?
	if ( pString[ 0 ] == '?' )
	{
		retval= R_Failure; // Rueckgabewert R_Failure
		goto Exit;
	}
	if ( !cnt )
	{
		retval= R_Failure; // Rueckgabewert R_Failure
		goto Exit;
	}
#endif

Exit:

#ifdef WriteLog

	if ( LogFile )
		fclose( LogFile );
#endif

#ifndef TESTLAUF

	return retval; // Rueckgabe des jeweiligen Rueckgabewerts
#else

	bCmdActive= FALSE;
	pString[ 0 ]= '\x0';
	return R_OK;
#endif

};


int TC_812ISA::PutChar( const char c )
{
	int time1, time2;

	Delay( 3 ); // wozu ????
	// solange Bit 0 (Busy-Flag) an Adresse lpFlag gesetzt (=1)
	// oder time1 > 0 wird die Schleife abgearbeitet (gewartet)
	// Timeout ist aufgetreten falls time1=0
	BYTE inByte= Hardware->Get(Hardware->oFlag);
	for( time1= 300; (inByte & 0x01) && time1; time1--)
	{
		Delay(1);		//hp absolute Zeiteinheit warten
		inByte= Hardware->Get(Hardware->oFlag);
	}

	// Test ob Timeout ( time1=0) aufgetreten
	// falls nicht (time1>0) und damit Bit 0 (Busy-Flag) an lpFlag=0
	if( time1 )   //time
	{
		Hardware->Put(Hardware->oOut1,(BYTE)c);   // write data
		Delay( 3 );                 // Verzoegerung
		Hardware->Put(Hardware->oOut2,(BYTE)c);   // generate internal signal
	}
	else
		return(0);		//hp abbrechen, da weitere Versuche sinnlos
	// Warten bis Controller wieder schreibbereit
	// (Bit 0 (Busy-Flag) von lpFlag= 0)
	inByte= Hardware->Get(Hardware->oFlag);
	for( time2= 300; (inByte  & 0x00) && time2; time2--)
	{
		Delay(1);		//hp absolute Zeiteinheit warten
		inByte= Hardware->Get(Hardware->oFlag);
	}
	Delay( 3 );  // Verzoegerung, wozu ???
	// Returnwert 0=timeout, sonst OK
	return time1;
	//ph
};


char TC_812ISA::GetChar( void )
{
	char chr= 0;

	Delay( 1 );  // 1
	// Wenn Bit 1 gesetzt ist, so liegen Daten von der Steuerung an.
	BYTE inByte= Hardware->Get(Hardware->oFlag);
	if(!(inByte & 0x02))
		return(0);
  
	inByte= Hardware->Get(Hardware->oIn);
	chr= (char)inByte;
	
	/*if ( !( C812ISA_Get( lpFlag ) & 0x02 ) )      // text ready-flag
		return 0;              // no data available
	chr= C812ISA_Get( lpIn ); // read data
	*/
	//ph
	Delay( 2 );                // wozu ???
	return chr;
};




//############################################################################
//################### Interface ##############################################
//############################################################################
//############################################################################

TC_832::TC_832( void ) : TDC_Drive() ,CmdRegister(0) ,DataRegister(1)
{
	char Ident[ MaxString ], buf[ MaxString ];

	sprintf( Ident, "motor%d", nId );

	GetPrivateProfileString( Ident, "IOAddr", "0x210", (LPSTR)buf, MaxString,
							 GetHWFile( ) );
	sscanf( buf, "%x", &wBaseAddr );

	nOnBoardId= GetPrivateProfileInt( Ident, "BoardId", 0, GetHWFile( ) );

	wKI= (WORD)GetPrivateProfileInt( Ident, "IntegralGain", 10, GetHWFile( ) );
	wKL= (WORD)GetPrivateProfileInt( Ident, "IntegralLimit", 10, GetHWFile( ) );
	wKP= (WORD)GetPrivateProfileInt( Ident, "Gain", 100, GetHWFile( ) );
	wKD= (WORD)GetPrivateProfileInt( Ident, "DynamicGain", 37, GetHWFile( ) );

	cConfig= 0x80;
	if ( GetPrivateProfileInt( Ident, "DifferentialEncoder", 0, GetHWFile( ) ) )
	{
		if ( 0 == nOnBoardId )
			cConfig |= 0x10;
		else
			cConfig |= 0x20;
	}
	if ( GetPrivateProfileInt( Ident, "EnableInterrupts", 0, GetHWFile( ) ) )
	{
		cConfig |= 0x40;
	}
	wSample= 0x0100; // 512 s

	// initialisieren des zugehrigen Controller-Objekts (in buf steht wBaseAddr)
	Hardware= dynamic_cast<TC_832Controller*>(GetController(C832,buf,&MotorControllers,&MotorDrivers,
		                                       &ControllerIndex));
	//Member des Controller-Objects initialisieren (ggf. schon initialisiert)
	Hardware->activeConfig= cConfig;    // Konfigurations Register
	Hardware->activeDrive= nOnBoardId;
	Hardware->baddr= 0;  //BaseAddr ist im Treiber eingestellt -> hier nur Offsets
	baddr= 0;  //BaseAddr ist im Treiber eingestellt -> hier nur Offsets
	raddr= (cConfig | (nOnBoardId << 1));
	//ph

}
;


BOOL TC_832::Reset( void )
{
	// Reset setzt Position auf Null, falls Initialisierung nicht eindeutig genug
	// sollte hier erst Position gelesen werden und dann der Reset ausgefuehrt
	// werden.
	// changing K-System

	Hardware->UpdateController(cConfig,nOnBoardId);
	Drive628( RESET, 0, 0 );
	Drive628( RSTI, 0, 0 );
	Drive628( MSKI, 0, 0 );
	return TRUE;
};


void TC_832::OptimizingDlg( void )
{
	TModalDlg* dlg;

	dlg= (TOptimizeDC_832Dlg *) new TOptimizeDC_832Dlg( );
	if ( dlg ) dlg->ExecuteDialog( GetFrameHandle( ) );
	_FREEOBJ(dlg);
};


BOOL TC_832:: ActivateFilterParameters( void )
{

	Hardware->UpdateController(cConfig,nOnBoardId);
	// Static Gain entspricht dem P-Term
	Drive628( LFIL, wSample | LD_KP, wKP );
	// Dynamic Gain entspricht dem D-Term
	Drive628( LFIL, wSample | LD_KD, wKD );
	// Integral Gain entspricht dem I-Term
	Drive628( LFIL, wSample | LD_KI, wKI );
	// Integral-Limit entspricht dem L-Term
	Drive628( LFIL, wSample | LD_IL, wKL );
	Drive628( UDF, 0, 0 );
	SetVelocity( dwMaxVelocity );
	SetAcceleration( dwAcceleration );
	GetSpeed( );
	//StartLimitWatch();
	return TRUE;
};


int TC_832::SaveSettings( int LastSave )
{
	char buf[ MaxString ];
	char Ident[ MaxString ];

	Hardware->UpdateController(cConfig,nOnBoardId);
	sprintf( Ident, "motor%d", nId );
	sprintf( buf, "%u", wKD );
	WritePrivateProfileString( Ident, "DynamicGain", buf, GetHWFile( ) );
	sprintf( buf, "%u", wKP );
	WritePrivateProfileString( Ident, "Gain", buf, GetHWFile( ) );
	sprintf( buf, "%u", wKI );
	WritePrivateProfileString( Ident, "IntegralGain", buf, GetHWFile( ) );
	sprintf( buf, "%u", wKL );
	WritePrivateProfileString( Ident, "IntegralLimit", buf, GetHWFile( ) );
	sprintf( buf, "%lu", dwMaxVelocity );
	WritePrivateProfileString( Ident, "MaxVelocity", buf, GetHWFile( ) );
	sprintf( buf, "%lu", dwAcceleration );
	WritePrivateProfileString( Ident, "Acceleration", buf, GetHWFile( ) );
	sprintf( buf, "%lu", dwVelocity );
	WritePrivateProfileString( Ident, "Velocity", buf, GetHWFile( ) );
	return TDC_Drive::SaveSettings( LastSave );
};


BOOL TC_832::SetHome( void )
{


	Hardware->UpdateController(cConfig,nOnBoardId);

	TMotor::SetHome( );
	// changing K-System
	Drive628( DFH, 0, 0 );
	Drive628( LTRJ, LD_POS, 0 );
	Drive628( STT, 0, 0 );
	return TRUE;
};


BOOL TC_832::ActivateDrive( void )
{
	long pos;

	Hardware->UpdateController(cConfig,nOnBoardId);
	Drive628( UDF, 0, 0 );
	_GetPosition( &pos );
	Drive628( LTRJ, LD_POS, pos );
	Drive628( STT, 0, 0 );
	return TRUE;
};


BOOL TC_832::StopDrive( BOOL bOffOn )  // Kullmann, Reinecker (08.08.02) Parameter gendert int -> BOOL
{

	Hardware->UpdateController(cConfig,nOnBoardId);
	
	WORD ctr= 0x0400;  // Stop smoothly --> programming manual (page 29)

	Drive628( LTRJ, ctr, 0 );
	Drive628( STT, 0, 0 );
	while ( !IsMoveFinish( ) )
		DelayTime( 4 );
	if ( !bOffOn )
		return TRUE;
	Drive628( UDF, 0, 0 );
	Drive628( STT, 0, 0 );
	return TRUE;
};


BOOL TC_832::StartLimitWatch( void )
{
	Hardware->UpdateController(cConfig,nOnBoardId);

	if ( bLimitWatchActive )
		return FALSE;
	nEvent= timeSetEvent( 150,1,( LPTIMECALLBACK )  &TC_832Controller::LimitWatch,ControllerIndex,
                          TIME_PERIODIC );
	bLimitWatchActive= TRUE;
	return TRUE;
};


BOOL TC_832::StopLimitWatch( void )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	if ( !bLimitWatchActive )
		return FALSE;
	timeKillEvent( nEvent );
	bLimitWatchActive= FALSE;
	return TRUE;
};

BOOL TC_832::IsLimitHit( void )
{

  Hardware->UpdateController(cConfig,nOnBoardId);

  BYTE status;
  // Interrupt-Register
  Hardware->LM628Ready(CmdRegister,nOnBoardId);

  Hardware->Put(CmdRegister,cConfig | 0x07 );
  status= Hardware->Get(DataRegister);

  if( status & ( nOnBoardId+1 )  )
  {
    Drive628( LTRJ,0x400,0 );   // stop appruptly
    Drive628( STT,0,0 );
    if( bUpwards )
      Drive628( LTRJ,LD_RPOS,-labs( dwRemoveLimit )  );
    else
      Drive628( LTRJ,LD_RPOS,labs( dwRemoveLimit )  );
    bUpwards= !bUpwards;
    Drive628( STT,0,0 );
    while( !IsMoveFinish( )  )
      DelayTime( 10 );
    DelayTime( 100 );
    Hardware->LM628Ready(CmdRegister,nOnBoardId);

    Hardware->Put(CmdRegister,cConfig | 0x07 );
    status= Hardware->Get(DataRegister);
    Hardware->Put(DataRegister,status & ~( nOnBoardId+1 )  );
	DelayTime( 10 );
    Hardware->bLimitHit= bRangeHit= TRUE;
    return TRUE;
  }
  Hardware->bLimitHit= FALSE;
  return FALSE;


};


BOOL TC_832::IsIndexArrived( void )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	long ipos, pos;

	if ( IsLimitHit( ) )
	{
		Drive628( RSTI, 0, 0 );
		Drive628( MSKI, 0, 0 );
		Drive628( STT, 0, 0 );
		if ( bIndexLine )
		{
			Drive628( SIP, 0, 0 );
			Drive628( LTRJ, LD_RPOS, 400000000l );
			bUpwards= TRUE;
			Drive628( STT, 0, 0 );
			return FALSE;
		}
		return TRUE;
	}
	if ( bIndexLine && bIndexDetected && IsMoveFinish( ) )
		return TRUE;
	if ( !bIndexLine && bRangeHit && IsMoveFinish( ) )
		return TRUE;
	if ( !bRangeHit && bMoveFirstToLimit )
		return FALSE;
	if ( 0x08 & Drive628( RDSIGS, 0, 0 ) )
	{
		// stop drive
		Drive628( LTRJ, 0x400, 0 );
		Drive628( STT, 0, 0 );
		bIndexDetected= TRUE;
		Drive628( RSTI, 0, 0 );
		Drive628( MSKI, 0, 0 );
		ipos= Drive628( RDIP, 0, 0 );
		while ( !IsMoveFinish( ) )
			/* wait until finish */;
		pos= Drive628( RDRP, 0, 0 );
		bUpwards= ( ipos > pos );
		Drive628( LTRJ, LD_POS, ipos );
		Drive628( STT, 0, 0 );
		Delay( 150 );
		return FALSE;
	}
	return FALSE;
};


BOOL TC_832::StartToIndex( long &oldPos )
{
	
	Hardware->UpdateController(cConfig,nOnBoardId);

	SetSpeed( 10000.0 );
	bIndexDetected= FALSE;
	bRangeHit= FALSE;
	Hardware->bLimitHit= FALSE;
	lDeltaPosition= 0;
	SetCalibrationState( FALSE );
	SetCorrectionState( FALSE );
	lPosition= oldPos= Drive628( RDRP, 0, 0 );
	bUpwards= FALSE;
	if ( !bMoveFirstToLimit && bIndexLine )
	{
		Drive628( RSTI, 0, 0 );
		Drive628( SIP, 0, 0 );
	}
	Drive628( LTRJ, LD_RPOS, -400000000l );
	Drive628( STT, 0, 0 );
	// Die Motoren werden in Bewegung gesetzt
	// Erreichen der Stillstands-Position
	DelayTime( 150 );
	return TRUE;
};


BOOL TC_832::SetVelocity( DWORD velocity )
{
	Hardware->UpdateController(cConfig,nOnBoardId);

	dwVelocity= ( velocity < dwMaxVelocity ) ? velocity : dwMaxVelocity;
	if ( R_OK != Drive628( LTRJ, 0x0008, dwVelocity ) )
		return FALSE;
	return TRUE;
};


DWORD TC_832::GetVelocity( void )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	dwVelocity= ( dwVelocity < dwMaxVelocity ) ? dwVelocity : dwMaxVelocity;
	return dwVelocity;
};


BOOL TC_832::SetAcceleration( DWORD acceleration )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	dwAcceleration= acceleration;
	if ( R_OK != Drive628( LTRJ, 0x0020, dwAcceleration ) )
		return FALSE;
	return TRUE;
};


DWORD TC_832::GetAcceleration( void )
{
	Hardware->UpdateController(cConfig,nOnBoardId);

	return dwAcceleration;
};


BOOL TC_832::SetLimit( DWORD limit )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	dwRemoveLimit= limit;
	return TRUE;
};


WORD TC_832::GetStatus( void )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	return ( WORD ) Drive628( RDSTAT, 0, 0 );
};


BOOL TC_832::_GetPosition( long *position )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	*position= Drive628( RDRP, 0, 0 );
	return TRUE;
};


BOOL TC_832::_GetFailure( long *failure )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	*failure= ( Drive628( RDRP, 0, 0 ) - Drive628( RDDP, 0, 0 ) );
	return TRUE;
};


BOOL TC_832::MoveRelative( long position )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	long rpos;
	rpos= Drive628( RDRP, 0, 0 );
	rpos += position;
	Drive628( LTRJ, LD_POS, rpos );
	Drive628( STT, 0, 0 );
	return TRUE;
};


BOOL TC_832::MoveAbsolute( long pos )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

	// Aufruf der Hardwarezugriffsfkt.  TC_832::Drive628 und Drive628c
	// mit den Motorkommandos LTRJ und STT
	// LTRJ -> Load Trajectory Parameters mit Pos.Parameter
	//         (weitere Parameter z.B. pos, velo, accel, operation mode)
	// STT -> Start Motion Control: (execute selected Trajectory) )
	Drive628( LTRJ, LD_POS, pos );
	Drive628( STT, 0, 0 );
	// Rueckgabewert immer TRUE (Wieso ?)
	return TRUE;
};


BOOL TC_832::IsMoveFinish( void )
{
	Hardware->UpdateController(cConfig,nOnBoardId);

	DelayTime( 5 );
	if ( Drive628( RDSIGS, 0, 0 ) & ON_TARGET )
		return TRUE;
	else
		return FALSE;
};


BOOL TC_832::CheckBoardOk( )
{
	Hardware->UpdateController(cConfig,nOnBoardId);

	bControlBoardOk= FALSE;

#ifndef TESTLAUF 
	Hardware->Put(CmdRegister,cConfig | ( nOnBoardId << 1 )  );
	if(Hardware->Get(DataRegister) == 0xFF )
		return FALSE;
#endif


	bControlBoardOk= TRUE;
	return TRUE;
};


BOOL TC_832::ExecuteCmd( LPSTR )
{

	Hardware->UpdateController(cConfig,nOnBoardId);
	return R_OK;
};


long TC_832::Drive628( BYTE cmd, WORD ctrl_word, long param )
{

	Hardware->UpdateController(cConfig,nOnBoardId);

#ifdef WriteLog

	FILE *LogFile;
	char buf[ 30 ];

	LogFile= fopen( "\motor.log", "a" );
	if ( LogFile )
	{
		fseek( LogFile, 0, 2 );
		sprintf( buf, "%d\n", cmd );
		fwrite( buf, strlen( buf ), 1, LogFile );
		fclose( LogFile );
	}
#endif

	return(Hardware->Drive628c(cmd, ctrl_word, param,baddr,raddr,nOnBoardId));

};

/////////////////////////////////////////////////////////////////
TC_832Controller::TC_832Controller(EDeviceType DeviceID, LPTSTR HardwareID, DeviceList* Devices)
					: Controller(C832,HardwareID,2), bIdle(FALSE), bIOActive(FALSE),
					  bLimitHit(FALSE), activeConfig(0), activeDrive(0), baddr(0),
					  CmdRegister(0),DataRegister(1)
{
	char* szDeviceName= Devices->GetFileName(DeviceID, HardwareID);
	if(!szDeviceName)
	{
		if(simulation_type == simulation_only)
		{	//MsgBox bzgl. des fehlenden Treibers in der Simulation unterdrcken
			Hardware= new DummyIo(DeviceID, HardwareID, false);
		}
		else
		{
			Hardware= new DummyIo(DeviceID, HardwareID);
		}
	}
	else
	{
		Hardware= new HardwareIo(szDeviceName);
	}

	sscanf(HardwareID,"%x",&dwSimBasePort);
}

BOOL TC_832Controller::Check()
{
	Put(CmdRegister,activeConfig | (activeDrive << 1));
	if(Get(DataRegister) == 0xFF )
	  return(FALSE);

	return(TRUE);
}

void TC_832Controller::UpdateController(BYTE _activeConfig, WORD _activeDrive)
{
	activeConfig= _activeConfig;
	activeDrive= _activeDrive;
}

//ap
void CALLBACK TC_832Controller::LimitWatch(UINT IDEvent,UINT,DWORD dwUser,DWORD,DWORD)
// UINT IDEvent; identifies timer event
// UINT uReserved; not used
// DWORD dwUser; application-defined instance data  */
// DWORD dwReserved1; not used
// DWORD dwReserved2; not used
{
  BYTE status= 0;
  BYTE raddr= 0;
  static count= 0;
  static MessageIsPosted= 0;
  TC_832Controller* Hardware= dynamic_cast<TC_832Controller*>(MotorControllers.GetController(dwUser));

  Hardware->LM628Ready(Hardware->baddr,Hardware->activeDrive);
  Hardware->Put(Hardware->baddr,Hardware->activeConfig | 0x07 );
  status= Hardware->Get(Hardware->baddr + 1);

  Hardware->bLimitHit= 0;
  count++;
  PostMessage( GetFrameHandle( ),WM_COMMAND,cm_SetWatchIndicator,( LPARAM )  count );
  if( status & 0x01 )
  {
    raddr= (Hardware->activeConfig | 0x00);
    Hardware->Drive628c(LTRJ,0x400,0,Hardware->baddr,raddr,Hardware->activeDrive);
    Hardware->Drive628c(STT,0,0,Hardware->baddr,raddr,Hardware->activeDrive);
    Hardware->bLimitHit= 1;
  }
  if( status & 0x02 )
  {
    raddr= (Hardware->activeConfig | 0x02);
    Hardware->Drive628c(LTRJ,0x400,0,Hardware->baddr,raddr,Hardware->activeDrive);
    Hardware->Drive628c(STT,0,0,Hardware->baddr,raddr,Hardware->activeDrive);
    Hardware->bLimitHit= 2;
  }
  if(Hardware->bLimitHit && !MessageIsPosted)
    PostMessage(GetFrameHandle( ),WM_COMMAND,cm_LimitHitOccure,( LPARAM )Hardware->bLimitHit);
};

//******************************************************************************
//******************************************************************************
long TC_832Controller::Drive628c(BYTE cmd,WORD ctrl_word,long param,WORD base,WORD regaddr,unsigned short drive)
{
  TCSet  *pCmdDesc;

	if((base != 0))
	{
		MessageBox(0,"Ungltiger Offset fr C832.","Drive628c", MBFAILURE);
		return(0);
	}


  if(bIdle)           // Ist Drive nicht in Benutzung ?
  {
    bIdle= FALSE;  // Ja! Dann Status in "benutzt"
  }

  bIdle= TRUE;     // sonst, Status wird in "nicht benutzt" gesetzt
  // Wozu macht er das eigentlich ??
  // an dieser Stelle ist TC_832::bIdle immer True!

  // Kommandosatz durchgehen
  pCmdDesc= CmdSet;
  if( cmd==RDSTAT )            // Liegt Kommando "RDSTAT"(Read Status) vor ?
  {
    // Status
    Put(base,regaddr);          // Ja! Fordere Auslesen des Configuration
    // Data Registers
    bIdle= FALSE;       // Setze Status in "benutzt"
    return((long)Get(base + 1)); // Lese die entsprechenden Daten aus dem
    // Datenregister und gebe sie zurueck
  }
  else
    while(cmd != pCmdDesc->cmd)    // Wenn nicht RDSTAT, dann gehe Kommandosatz durch
      pCmdDesc++;

  LM628Ready(base,drive);              // Test ob Busy-Bit gesetzt,(Test ist aber
  //  sinnlos,wenn er nicht ausgewertet wird !!)
  Put(base + 1,cmd);            // Sende Kommando an Controller

  if( pCmdDesc->data )               // nur alle Data Kommandos
  {
    // read/write data
    if( ( cmd==LTRJ )   | ( cmd==LFIL )  )        // wenn "LTRJ"- oder "LFIL"-Kommando
      PutWord(ctrl_word,base,regaddr,drive); // dann schreibe
    if( pCmdDesc->report )                 // Liegt ein Report-Kommando vor?
    {
      // Ja! Report Kommando liegt vor
      if(pCmdDesc->length32)             // Hat es einen 32Bit Parameter ?
      {
        bIdle= FALSE;         // Setze Status in "benutzt"
        return GetDWord(base,regaddr,drive); // hole 32 Bit data
      }
      else                             // ansonsten
      {
        bIdle= FALSE;         // Setze Status in "benutzt"
        return (long)GetWord(base,regaddr,drive); // hole 16 Bit data
      }
    }
    else  // wenn kein Report Kommando
    {
      // dann nur noch Control Kommando moeglich
      if( pCmdDesc->length32 )             // 32Bit ?
        PutDWord(param,base,regaddr,drive);  // hole 32 Bit parameter
      else                             // ansonsten
        PutWord((int)param,base,regaddr,drive); // hole 16 Bit parameter
    }
  }

  bIdle= FALSE;    // Setze Status in "benutzt"
  return 0;
};

int TC_832Controller::GetWord(WORD base,WORD regaddr,unsigned short drive)
{
  union
  {
    int  integer;
    BYTE byte[ 2 ];
  } temp;

	if((base != 0))
	{
		MessageBox(0,"Ungltiger Offset fr C832.","GetWord", MBFAILURE);
		return(0);
	}

  LM628Ready(base,drive);

  Put(base,regaddr + 1);
  temp.byte[ 1 ]= Get(base + 1);
  temp.byte[ 0 ]= Get(base + 1);

  return temp.integer;
};

long TC_832Controller::GetDWord(WORD base,WORD regaddr,unsigned short drive)
{
  union
  {
    long  lval;
    BYTE  byte[ 4 ];
  } temp;

	if((base != 0))
	{
		MessageBox(0,"Ungltiger Offset fr C832.","GetDWord", MBFAILURE);
		return(0);
	}

  LM628Ready(base,drive);

  Put(base,regaddr + 1);
  temp.byte[ 3 ]= Get(base + 1);
  temp.byte[ 2 ]= Get(base + 1);
  LM628Ready(base,drive);
  Put(base,regaddr+1 );
  temp.byte[ 1 ]= Get(base + 1);
  temp.byte[ 0 ]= Get(base + 1);

  return temp.lval;
};

void TC_832Controller::PutWord(int data,WORD base, WORD regaddr, unsigned short drive)
{
  union
  {
    int  integer;
    BYTE byte[ 2 ];
  } temp;

	if((base != 0))
	{
		MessageBox(0,"Ungltiger Offset fr C832.","PutWord", MBFAILURE);
		return;
	}

  LM628Ready(base,drive);

  Put(base,regaddr + 1);
  Delay( 2 );
  temp.integer= data;
  Put(base + 1,temp.byte[ 1 ]);
  Put(base + 1,temp.byte[ 0 ]);

};

void TC_832Controller::PutDWord(long data,WORD base,WORD regaddr, unsigned short drive)
{
  union
  {
    long  lval;
    BYTE byte[ 4 ];
  } temp;

	if((base != 0))
	{
		MessageBox(0,"Ungltiger Offset fr C832.","PutDWord", MBFAILURE);
		return;
	}

  temp.lval= data;
  LM628Ready(base,drive);

 Put(base,regaddr+1 );
  Put(base + 1,temp.byte[ 3 ]);
  Put(base + 1,temp.byte[ 2 ]);
  LM628Ready(base,drive);
  Put(base,regaddr+1 );
  Put(base + 1,temp.byte[ 1 ]);
  Put(base + 1,temp.byte[ 0 ]);


};

int TC_832Controller::Get(unsigned port)
{
  int return_val= 0;

  if ( simulation_type & 1 )
  {
    // case no_simulation: case test_simulation:
    // get the hardware generated return valu
	  Read(port,(BYTE*)&return_val);

    if ( simulation_type & 2 )
    {
      // case test_simulation:
      // inform driver if possible, simulated value is ignored
      if ( c832_get_callback )
      {
		port += dwSimBasePort;	//erforderlich, da die Simulation mit absoluten Adressen arbeitet
        c832_get_callback( port, return_val );
      }
    }
  }
  else
  {
    // case simulation_only:
    // let driver generate a simulated value
    if ( c832_get_callback )
    {
	  port += dwSimBasePort;	//erforderlich, da die Simulation mit absoluten Adressen arbeitet
      return_val= c832_get_callback( port, 0 );
    }
  }

  return return_val;
}

void TC_832Controller::Put(unsigned port, int value)
{
  if ( simulation_type & 1 )
  {
    // case no_simulation: case test_simulation:
    // write the value to hardware
	Write(port,(BYTE)value);
  }
  if ( simulation_type & 2 )
  {
    // case simulation_only: case test_simulation:
    // write the value to the simulation driver
    if ( c832_put_callback ) 
	{
		port += dwSimBasePort;	//erforderlich, da die Simulation mit absoluten Adressen arbeitet
		c832_put_callback( port, value );
	}
  }
}



BOOL TC_832Controller::LM628Ready(WORD base, unsigned short drive)
{
  int   tt;
  int   time_out=1000;

	//ap -> Test,ob nur Offsets !!!
	if((base != 0))
	{
		MessageBox(0,"Ungltiger Offset fr C832.","LM628Ready",MBFAILURE);
		return(0);
	}

  // command register vom aktiven drive
  Put(base,activeConfig | (drive<<1));
  do
  {
    tt= Get(base+1);
    if( !( time_out-- )  )
      return FALSE;// wenn timeout, dann FALSE ->"nicht bereit"
  } while( tt & 0x01 );  // teste ob bit 0 von "tt" gesetzt (BUSY-Bit)
  if( tt & 0x02 )        // teste ob bit 1 von "tt" gesetzt, welche Bedeutung ?
  {
    time_out++;      // warum nochmal time_out um eins erhoehen ??
  }

  return TRUE;       // BUSY-Bit geloescht
}
;
// __LastLine__
