//******************************************************************************
//Datei     : TProtocol.cpp
//Projekt   : XCTL
//Subsystem : Protokollbuch
//Autor     : Jens Klier <klier@informatik.hu-berlin.de>
//            Institut fuer Informatik,
//            Humboldt-Universitaet Berlin
//Inhalt    : Implementation der Klasse TProtocol
//******************************************************************************
//!neu klier Protokollbuch
#include "internls\evrythng.h"
#include <ctype.h>
#include <commdlg.h>

// Bibliotheken zum Zugriff auf das Dateisystem (16/32bit)
// damit koennen die Dateiattribute der Protokolldatei geaendert werden
#include <winbase.h>

#include "protocol\TProtocolLanguage.h"
#include "protocol\TProtocol.h"



//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

#define Trennzeichen ";"

extern TProtocolLanguage *ProtocolLanguage;
TProtocol *Protocol;

int CALLBACK EnumMetafileProc( HDC hDC, HANDLETABLE *lpHTable, const ENHMETARECORD *lpMFR, int nObj, LPARAM lpData )
{
	char buf[MaxString], buf2[MaxString];
	int i;

	if ( (lpMFR->iType == 27)  || (lpMFR->iType == 54)  || (lpMFR->iType == 84)  ||
		 (lpMFR->iType == 531) || (lpMFR->iType == 532) || (lpMFR->iType == 1313))
	{
		if (lpMFR->iType == 531)
		{
			Protocol->SetXY('L', lpMFR->dParm[1], lpMFR->dParm[0]);
		}
		if (lpMFR->iType == 54)
		{
			Protocol->SetXY('L', lpMFR->dParm[0], lpMFR->dParm[1]);
		}
		if (lpMFR->iType == 532)
		{
			Protocol->SetXY('M', lpMFR->dParm[1], lpMFR->dParm[0]);
		}
		if (lpMFR->iType == 27)
		{
			Protocol->SetXY('M', lpMFR->dParm[0], lpMFR->dParm[1]);
		}
		if (lpMFR->iType == 1313)
		{
			strcpy(buf, "");
			for (i= 1; i <= ceil((float)(lpMFR->dParm[0] / 2)); i++)
			{
				sprintf(buf2, "%c%c", lpMFR->dParm[i] % 256, lpMFR->dParm[i] / 256);
				strcat( buf, buf2 );
			}
			if ((lpMFR->dParm[0] % 2) != 0)
			{
				sprintf(buf2, "%c", lpMFR->dParm[i] % 256);
				strcat( buf, buf2 );
			}
			Protocol->SetXY('T', lpMFR->dParm[lpMFR->nSize - 4], lpMFR->dParm[lpMFR->nSize - 5], buf);
		}
		if (lpMFR->iType == 84)
		{
			strcpy(buf, "");
			for (i= 1; i <= ceil((float)(lpMFR->dParm[9] / 2)); i++)
			{
				sprintf(buf2, "%c%c", lpMFR->dParm[i+16] % 65536, lpMFR->dParm[i+16] / 65536);
				strcat( buf, buf2 );
			}
			if ((lpMFR->dParm[9] % 2) != 0)
			{
				sprintf(buf2, "%c", lpMFR->dParm[i+16] % 65536);
				strcat( buf, buf2 );
			}
			Protocol->SetXY('T', lpMFR->dParm[0], lpMFR->dParm[1], buf);
		}
  }

	return 1;
}

TProtocol::TProtocol( int Feldelemente ) : Felder(Feldelemente),
		currentINDEX(NULL), lastINDEX(NULL), firstINDEX(NULL),
		currentPOINT(NULL), firstPOINT(NULL), lastPOINT(NULL),
		bActiv(FALSE), iProtocolOnStatus(Off), Parameter(0)
{
	Parameter= new TParameter[Feldelemente];
	if (!Parameter)
	{
		MessageBox(GetFocus(), ProtocolLanguage->GetMessage(1), ProtocolLanguage->GetFailure(), MBFAILURE);
		_FREELIST(Parameter);
	}

	SetAllParameterReadWrite();

	strcpy(CurrentLfdNr, "");
	strcpy(CopyLfdNr, "");
};

TProtocol::~TProtocol()
{
	freeINDEX();
	freePic();
	_FREELIST(Parameter);
};

// Initialisiert das Protokollbuch
BOOL TProtocol::Initialize()
{
	if (bActiv)
		return TRUE;
	if ( Parameter == NULL )
	{
		MessageBox(GetFocus(), ProtocolLanguage->GetMessage(4),
				   ProtocolLanguage->GetFailure(), MBFAILURE);
		return FALSE;
	}
	Protocol= this;
	if ( GetPrivateProfileInt("Protocol", "Debug", 0, GetCFile()) == 1)
		bDebug= TRUE;
	else
		bDebug= FALSE;
	if ( GetPrivateProfileInt(szProtocolSection, "NoSortNew", 0, GetCFile()) == 1)
		bNoSortNew= TRUE;
	else
		bNoSortNew= FALSE;
	if ( GetPrivateProfileInt(szProtocolSection, "SortDate", 0, GetCFile()) == 1)
		bSortDate= TRUE;
	else
		bSortDate= FALSE;
	bActiv= ReloadProtocol();
	return bActiv;
};

// Initialisiert das Protokollbuch neu
BOOL TProtocol::ReloadProtocol()
{
	FILE *out;
	OFSTRUCT of;
	OPENFILENAME ofn;
	char buf[MaxLine], buf2[_MAX_PATH];

	freeINDEX();
	freePic();
	ClearProtocolString();
	// Test ob die Protokolldatei existiert
	while (OpenFile(GetPFile(), &of, OF_EXIST) == HFILE_ERROR)
	{
		sprintf(buf, ProtocolLanguage->GetMessage(2), GetPFile());
		if (MessageBox(GetFocus(), buf, ProtocolLanguage->GetQuestion(), MBASK) == IDNO)
		{
			sprintf(buf, "%s%s", GetPName(), ".CSV");
			// Initialize the structure
			memset( &ofn, '\0', sizeof( OPENFILENAME ) );
			ofn.lpstrTitle= ProtocolLanguage->GetMessage(17);
			ofn.lStructSize= sizeof( OPENFILENAME );
			ofn.hwndOwner= GetFocus();
			ofn.lpstrFilter= "csv-Dateien (*.CSV)\0*.CSV\0\0";
			ofn.nFilterIndex= 1;
			ofn.lpstrFile= buf;
			ofn.nMaxFile= _MAX_PATH;
			ofn.lpstrInitialDir= GetPDirectory();
			ofn.Flags= OFN_HIDEREADONLY;
			if ( !GetOpenFileName(&ofn) )
			{
				MessageBox(GetFocus(), ProtocolLanguage->GetMessage(4),
						   ProtocolLanguage->GetFailure(), MBFAILURE);
				return FALSE;
			}
			SetPFile(buf);
		}
		else
		{
			if ( (out= fopen(GetPFile(), "wt")) == NULL )
			{
				sprintf(buf, ProtocolLanguage->GetMessage(3), GetPDirectory());
				MessageBox(GetFocus(), buf, ProtocolLanguage->GetFailure(), MBFAILURE);
				MessageBox(GetFocus(), ProtocolLanguage->GetMessage(4),
						   ProtocolLanguage->GetFailure(), MBFAILURE);
				return FALSE;
			}
			strcpy(buf, "");
			for (int i= 0; i < Felder; i++)
			{
				if (i > 0)
					strcat(buf, Trennzeichen);
				strcat(buf, GetParameterNamen(i, buf2));
			}
			strcat(buf, "\n");
			fputs(buf, out);
			fclose(out);

			SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);
		}
	}
	if ( !LoadProtocolFileIndex() )
	{
		MessageBox(GetFocus(), ProtocolLanguage->GetMessage(4),
				   ProtocolLanguage->GetFailure(), MBFAILURE);
		return FALSE;
	}
	sprintf(buf, "%s%s", GetPName(), ".BK5");
	strcpy(buf, GetProtocolBackupFile(buf));
	strcpy(buf2, buf);

	// Schreibschutzattribut der Protokolldatei wird gesetzt
	SetFileAttributes(buf, FILE_ATTRIBUTE_ARCHIVE);

	remove(buf);

	for (int i= 4; i > 0; i--)
	{
		buf2[strlen(buf2) - 1]--;
		rename(buf2, buf);
		buf[strlen(buf) - 1]--;
	}
	if (!BackupFile( GetPFile(), buf2))
	{
		MessageBox(GetFocus(), ProtocolLanguage->GetMessage(4),
				   ProtocolLanguage->GetFailure(), MBFAILURE);
		return FALSE;
	}
	if (!bNoSortNew)
		if (!SortProtocolFile())
		{
			MessageBox(GetFocus(), ProtocolLanguage->GetMessage(4),
					   ProtocolLanguage->GetFailure(), MBFAILURE);
			return FALSE;
		}
	return TRUE;
};

// Erzeugt eine Backupkopie einer Datei
BOOL TProtocol::BackupFile( LPCSTR File, LPCSTR BFile )
{
	FILE *in, *out;
	char buf[MaxLine];

	// Schreibschutzattribut der Protokolldatei wird entfernt
	SetFileAttributes(BFile, FILE_ATTRIBUTE_ARCHIVE);

	if ( (in= fopen(File, "rt")) == NULL)
		return FALSE;
	if ( (out= fopen(BFile, "wt")) == NULL )
		return FALSE;
	while (!feof(in))
	{
		if ( fgets( buf, MaxLine, in ) != NULL )
			fputs(buf, out);
	}
	fclose(in);
	fclose(out);

	// Schreibschutzattribut der Protokolldatei wird gesetzt
	SetFileAttributes(BFile, FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);

	return TRUE;
};

// Erzeugt aus der Protokolldatei einen Index im Speicher
// Die Lfd.-Nr. und die Position des Datensatzes innerhalb der Datei werden
// als Index im Speicher abgelegt.
BOOL TProtocol::LoadProtocolFileIndex( LPCSTR lLfdNr )
{
	freeINDEX(lLfdNr);
	return AppendProtocolFileIndex();
};

BOOL TProtocol::AppendProtocolFileIndex( void )
{
	FILE *in;
	int nr= 0;
	char buf[MaxLine], buf2[MaxLfdNr];
	long FilePos;

	bIsSortLfdNr= bIsSortDate= TRUE;
	currentINDEX= lastINDEX;
	if ( (in= fopen(GetPFile(), "rt")) == NULL )
		return FALSE;
	if (currentINDEX != NULL)
	{
		if ( fseek(in, currentINDEX->Pos, SEEK_SET) != 0 )
		{
			fclose(in);
			return FALSE;
		}
		if ( fgets( buf, MaxLine, in ) == NULL )
		{
			fclose(in);
			return FALSE;
		}
		nr= currentINDEX->Nr;
	}
	while (!feof(in))
	{
		FilePos= ftell(in);
		if ( fgets(buf, MaxLine, in ) != NULL )
		{
			if ( strncmp(buf, "Lfd", 3) != 0 )
			{
				if ( !StrSplit(buf) )
				{
					fclose(in);
					return FALSE;
				}
				lastINDEX= currentINDEX;

				currentINDEX= new TFileIndex;
				if(!currentINDEX)
				{
					fclose(in);
					MessageBox(GetFocus(), ProtocolLanguage->GetMessage(5), ProtocolLanguage->GetFailure(), MBFAILURE);
					return FALSE;
				}

				if (nr == 0)
					firstINDEX= lastINDEX= currentINDEX;
				currentINDEX->Nr= nr + 1;
				currentINDEX->Pos= FilePos;
				strcpy(currentINDEX->LfdNr, GetParameterLfdNr(buf2));
				MakeDateString(GetParameterDate(buf2), currentINDEX->Date);
				currentINDEX->Status= atoi(GetProtocolValue(Felder - 1, buf2));
				lastINDEX->next= currentINDEX;
				currentINDEX->last= lastINDEX;
				currentINDEX->next= NULL;
				if (nr++ == 0)
					firstINDEX->last= NULL;
				else
				{
					if (strcmp(currentINDEX->LfdNr, lastINDEX->LfdNr) < 0)
						bIsSortLfdNr= FALSE;
					if (strcmp(currentINDEX->Date, lastINDEX->Date) < 0)
						bIsSortDate= FALSE;
				}
			}
		}
	}
	fclose(in);
	lastINDEX= currentINDEX;
	currentINDEX= NULL;
	ReloadProtocolString(GetLfdNr(buf));
	return TRUE;
};

// Setzt den aktuellen Index auf NULL
void TProtocol::ResetIndex()
{
	currentINDEX= NULL;
};

// Loescht den Index und gibt den Speicher frei
void TProtocol::freeINDEX( LPCSTR lLfdNr )
{
	TFileIndex *tmpINDEX;

	if (firstINDEX == NULL)
		return;
	if (!SearchProtocolString(lLfdNr))
	{
		currentINDEX= firstINDEX;
		firstINDEX= tmpINDEX= NULL;
	}
	else
	{
		tmpINDEX= currentINDEX->last;
		if (tmpINDEX != NULL)
			tmpINDEX->next= NULL;
	}
	for (currentINDEX= currentINDEX->next; currentINDEX != NULL; currentINDEX= currentINDEX->next)
		_FREEOBJ(currentINDEX->last);
	_FREEOBJ(lastINDEX);
	currentINDEX= lastINDEX= tmpINDEX;
};

// Zerlegt den eingelesenen Protokollstring und traegt die Parameter in das
// Parameterfeld ein (Hochkomma am Anfang und am Ende eines Wertes werden entfernt)
BOOL TProtocol::StrSplit(LPCSTR sLine)
{
	char buf[MaxLine], buf2[MaxString], buf3[MaxString];
	char *token, *sepp, fieldsepp[]= Trennzeichen;
	int i, j, k, nfield, seppc;

	strcpy(buf, sLine);
	// Zeilenende entfernen
	if ( buf[strlen(buf) - 1] == '\r' || buf[strlen(buf) - 1] == '\n')
		buf[strlen(buf) - 1]= '\0';
	if ( buf[strlen(buf) - 1] == '\r' )
		buf[strlen(buf) - 1]= '\0';
	ClearProtocolString();  // Parameterfeld lschen
	token= buf;
	nfield= 0;
	do
	{
		if (*token == '"')
		{
			token++;  // erstes " berspringen
			for (i= j= 0; token[j] != '\0'; i++, j++)
			{
				if (token[j] == '"' && token[++j] != '"')
				{
					k= strcspn(token + j, fieldsepp);
					memmove(token + i, token + j, k);
					i += k;
					j += k;
					break;
				}
				token[i]= token[j];
			}
			token[i]= '\0';
			sepp= token + j;
		}
		else
			sepp= token + strcspn(token, fieldsepp);
		seppc= sepp[0];
		sepp[0]= '\0';  //Feld 0-terminiert
		if ( !SetProtocolValue(nfield++, token) )
		{
			sprintf(buf, ProtocolLanguage->GetMessage(15),
					GetParameterNamen(nfield - 1, buf2), token);
			MessageBox(GetFocus(), buf, ProtocolLanguage->GetFailure(), MBFAILURE);
			return FALSE;
		}
		token= sepp + 1;
	} while (seppc == fieldsepp[0] && nfield < Felder);
	GetParameterLfdNr(buf2);
	GetParameterDate(buf3);
	if (!TestLfdNr(buf2) || !TestDate(buf3))
	{
		sprintf(buf, ProtocolLanguage->GetMessage(16), buf2, buf3);
		MessageBox(GetFocus(), buf, ProtocolLanguage->GetFailure(), MBFAILURE);
		return FALSE;
	}
	return TRUE;
};

// Erzeugt aus dem Parameterfeld einen Protokollstring
// Enthaelt ein Parameter das Trennzeichen wird er in Hochkomma eingeschlossen
LPSTR TProtocol::StrMerge(LPSTR sLine)
{
	char buf[MaxLine];
	char fieldsepp[]= Trennzeichen;
	int i, j, k;

	strcpy(sLine, "");
	for (i= 0; i < Felder; i++)
	{
		for (j= k= 0; j < strlen(Parameter[i].Wert); j++)
		{
			buf[k]= Parameter[i].Wert[j];
			if (buf[k++] == '"')
				buf[k++]= '"';
		}
		buf[k]= '\0';
		if (strstr(buf, fieldsepp) != NULL)
		{
			strcat(sLine, "\"");
			strcat(sLine, buf);
			strcat(sLine, "\"");
		}
		else
			strcat(sLine, buf);
		if (i != Felder - 1)
			strcat(sLine, fieldsepp);
	}
	strcat(sLine, "\n");
	return sLine;
};

// Erzeugt aus einem Datum einen speziellen String fuer die Suche in der Protokolldatei
// 20.4.1999 -->  19990420
LPSTR TProtocol::MakeDateString( LPCSTR Date, LPSTR DateString )
{
	char *token;
	char buf[MaxString];

	if (strlen(Date) != 0)
	{
		strcpy(buf, Date);
		token= strtok(strrev(buf), ".");
		if (strlen(token) == 2)
			strcpy(DateString, "20");
		else
			strcpy(DateString, "");
		strcat(DateString, strrev(token));
		token= strtok(NULL, ".");
		if (strlen(token) == 1)
			strcat(DateString, "0");
		strcat(DateString, strrev(token));
		token= strtok(NULL, ".");
		if (strlen(token) == 1)
			strcat(DateString, "0");
		strcat(DateString, strrev(token));
	}
	else
		strcpy(DateString, "");
	return DateString;
};

// Prueft eine Zeichenkette ob sie ein gueltiges Datum enthaelt
BOOL TProtocol::TestDate( LPCSTR Date )
{
	char buf[MaxString / 4];
	char *token;
	int i;

	if ( (strlen(Date) < 6) || (strlen(Date) > 10) )
		return FALSE;
	strcpy(buf, Date);
	if ( (token= strtok(buf, ".")) == NULL )
		return FALSE;
	if ( strlen(token) != 1 && strlen(token) != 2 )
		return FALSE;
	for (i= 0; i < strlen(token); i++)
		if ( !isdigit(token[i]) )
			return FALSE;
	if ( (token= strtok(NULL, ".")) == NULL )
		return FALSE;
	if ( strlen(token) != 1 && strlen(token) != 2 )
		return FALSE;
	for (i= 0; i < strlen(token); i++)
		if ( !isdigit(token[i]) )
			return FALSE;
	if ( (token= strtok(NULL, ".")) == NULL )
		return FALSE;
	if ( strlen(token) != 2 && strlen(token) != 4 )
		return FALSE;
	for (i= 0; i < strlen(token); i++)
		if ( !isdigit(token[i]) )
			return FALSE;
	if ( strtok(NULL, ".") != NULL )
		return FALSE;
	return TRUE;
};

// ist TRUE wenn der aktuelle Protokollstring der Erste in der Protokolldatei ist
BOOL TProtocol::IsFirstProtocolString( )
{
	if (currentINDEX != NULL)
		if (currentINDEX->last == NULL)
			return TRUE;
	return FALSE;
};

// ist TRUE wenn der aktuelle Protokollstring der Erste in der Protokolldatei ist
BOOL TProtocol::IsLastProtocolString( )
{
	if (currentINDEX != NULL)
		if (currentINDEX->next == NULL)
			return TRUE;
	return FALSE;
};

// ist TRUE wenn der aktuelle Protokollstring der Erste in der Protokolldatei ist
BOOL TProtocol::IsNewProtocolString( )
{
	if (currentINDEX == NULL)
		return TRUE;
	return FALSE;
};

// ist TRUE wenn die Protokolldatei leer ist
BOOL TProtocol::IsEmptyProtocolFile( )
{
	if (firstINDEX == NULL)
		return TRUE;
	return FALSE;
};

BOOL TProtocol::InitializeSearchProtocolString( LPCSTR lLfdNr )
{
	iSearch= 0;
	if ( strlen(lLfdNr) > 0 )
		return SearchProtocolString(lLfdNr);
	currentINDEX= firstINDEX;
	return TRUE;
};

// sucht einen Protokollstring anhand der Lfd.-Nr. im Protokollindex
// wird die Lfd.-Nr. gefunden liefert die Methode TRUE zurueck
BOOL TProtocol::SearchProtocolString( LPCSTR lLfdNr )
{
	TFileIndex *curINDEX;

	if ( strlen(lLfdNr) == 0 )
		return FALSE;
	curINDEX= firstINDEX;
	while ( curINDEX != NULL )
	{
		if ( strlen(lLfdNr) > 6)
		{
			if ( strcmp(curINDEX->LfdNr, lLfdNr) == 0 )
			{
				currentINDEX= curINDEX;
				return TRUE;
			}
		}
		else
		{
			if ( strncmp(curINDEX->LfdNr, lLfdNr, strlen(lLfdNr)) == 0 )
			{
				currentINDEX= curINDEX;
				return TRUE;
			}
		}
		curINDEX= curINDEX->next;
	}
	return FALSE;
};

// speichert einen Protokollstring in der Protokolldatei
BOOL TProtocol::ImportProtocolFile( )
{
	FILE *in, *out, *log;
	char buf[MaxLine], buf2[MaxString], ImportFile[_MAX_PATH];
	char ImportLfdNr[MaxString], tmpLfdNr[MaxString]= "";
	OPENFILENAME ofn;
	OFSTRUCT of;
	int iCount= 0, iErrorLfdNr= 0, iErrorMaxLen= 0, iErrorDelete= 0;
	int iCountSkizze= 0, iErrorSkizze= 0, iErrorExistSkizze= 0;
	//Hilfsvariablen fr die Bestandteile einen Dateinamens
	char Drive[_MAX_DRIVE], Dir[_MAX_DIR], Name[_MAX_FNAME], Ext[_MAX_EXT], Path[_MAX_PATH];

	strcpy(ImportFile, "");
	// Initialize the structure
	memset( &ofn, '\0', sizeof( OPENFILENAME ) );
	ofn.lpstrTitle= ProtocolLanguage->GetMessage(17);
	ofn.lStructSize= sizeof( OPENFILENAME );
	ofn.hwndOwner= GetFocus();
	ofn.lpstrFilter= "csv-Dateien (*.CSV)\0*.CSV\0Backup-Dateien (*.BK*)\0*.BK*\0Alle Dateien (*.*)\0*.*\0\0";
	ofn.nFilterIndex= 1;
	ofn.lpstrFile= ImportFile;
	ofn.nMaxFile= _MAX_PATH;
	ofn.lpstrInitialDir= GetPDirectory();
	ofn.Flags= OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	if ( !GetOpenFileName( &ofn ) )
		return FALSE;
	if ( OpenFile(ImportFile, &of, OF_EXIST) == HFILE_ERROR)
		return FALSE;
	if ( (in= fopen(ImportFile, "rt")) == NULL )
		return FALSE;
	_splitpath( ImportFile, Drive, Dir, Name, Ext );
	sprintf( Path, "%s%s", Drive, Dir );
	BackupFile(GetPFile(), GetProtocolFile(GetPName(), ".tmp"));

	// Schreibschutzattribut der mProtokolldatei wird geloescht
	SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE);
	SetFileAttributes(GetProtocolFile(GetPName(), ".tmp"), FILE_ATTRIBUTE_ARCHIVE);

	if ( (out= fopen(GetPFile(), "at")) == NULL )
		return FALSE;
	if ( (log= fopen(GetProtocolFile("import.log"), "wt")) == NULL )
		return FALSE;
	if (currentINDEX != NULL)
		strcpy(tmpLfdNr, currentINDEX->LfdNr);
	else
		strcpy(tmpLfdNr, "");
	while (!feof(in))
	{
		if ( fgets( buf, MaxLine, in ) != NULL )
		{
			if ( strncmp(buf, "Lfd", 3) != 0 )
			{
				if (StrSplit(buf))
				{
					GetParameterLfdNr(ImportLfdNr);
					if ( strcmp(GetProtocolValue(Felder - 1, buf2), "2") != 0 )
					{
						if ( !SearchProtocolString(ImportLfdNr) )
						{
							fputs(buf, out);
							fprintf(log, "%s --> %d. LfdNr importiert", ImportLfdNr, ++iCount);
						}
						else
							fprintf(log, "%s --> %d. LfdNr existiert bereits", ImportLfdNr, ++iErrorLfdNr);
						if (OpenFile(GetSkizzeFile(ImportLfdNr), &of, OF_EXIST) == HFILE_ERROR)
						{
							sprintf( ImportFile, "%s%s.wmf", Path, ImportLfdNr);
							if (OpenFile(ImportFile, &of, OF_EXIST) == HFILE_ERROR)
							{
								sprintf( ImportFile, "%sskizzen\\%s.wmf", Path, ImportLfdNr);
								if (OpenFile(ImportFile, &of, OF_EXIST) != HFILE_ERROR)
								{
									CopyEnhMetaFile(GetEnhMetaFile(ImportFile), GetSkizzeFile(ImportLfdNr));

									fprintf(log, " --> %d. Skizze importiert\n", ++iCountSkizze);
								}
								else
									fprintf(log, " --> %d. Skizze existiert nicht\n", ++iErrorSkizze);
							}
							else
							{
								CopyEnhMetaFile(GetEnhMetaFile(ImportFile), GetSkizzeFile(ImportLfdNr));

								fprintf(log, " --> %d. Skizze importiert\n", ++iCountSkizze);
							}
						}
						else
							fprintf(log, " --> %d. Skizze existiert bereits\n", ++iErrorExistSkizze);
					}
					else
						fprintf(log, "%s --> %d. LfdNr ist als gelscht markiert\n", ImportLfdNr, ++iErrorDelete);
				}
				else
					fprintf(log, "!!!!!!!!!! --> %d. Messung enthlt fehlerhafte Daten\n", ++iErrorMaxLen);
			}
		}
	}
	fclose(in);
	fclose(out);
	fprintf(log, "\n\n%s%d\n%s%d\n\n%s%d\n%s%d\n%s%d\n\n%s%d\n%s%d", \
			ProtocolLanguage->GetMessage(8), iCount, \
			ProtocolLanguage->GetMessage(9), iCountSkizze, \
			ProtocolLanguage->GetMessage(10), iErrorLfdNr, \
			ProtocolLanguage->GetMessage(11), iErrorMaxLen, \
			ProtocolLanguage->GetMessage(12), iErrorDelete, \
			ProtocolLanguage->GetMessage(13), iErrorExistSkizze, \
			ProtocolLanguage->GetMessage(14), iErrorSkizze);
	fclose(log);

	if ( !LoadProtocolFileIndex() )
	{
		remove
			(GetPFile());
		rename(GetProtocolFile(GetPName(), ".tmp"), GetPFile());
		LoadProtocolFileIndex();
		if (strlen(tmpLfdNr) > 0)
			LoadProtocolString(tmpLfdNr);
		MessageBox(GetFocus(), ProtocolLanguage->GetMessage(7), ProtocolLanguage->GetFailure(), MBFAILURE);
		return FALSE;
	}
	remove
		(GetProtocolFile(GetPName(), ".tmp"));

	// Schreibschutzattribut der Protokolldatei wird gesetzt
	SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);

	SortProtocolFile();

	ReloadProtocolString(tmpLfdNr);
	sprintf(buf, "%s%d\n%s%d\n\n%s%d\n%s%d\n%s%d\n\n%s%d\n%s%d", \
			ProtocolLanguage->GetMessage(8), iCount, \
			ProtocolLanguage->GetMessage(9), iCountSkizze, \
			ProtocolLanguage->GetMessage(10), iErrorLfdNr, \
			ProtocolLanguage->GetMessage(11), iErrorMaxLen, \
			ProtocolLanguage->GetMessage(12), iErrorDelete, \
			ProtocolLanguage->GetMessage(13), iErrorExistSkizze, \
			ProtocolLanguage->GetMessage(14), iErrorSkizze);
	MessageBox(GetFocus(), buf, ProtocolLanguage->GetMessage(), MBINFO);
	return TRUE;
};

// speichert einen Protokollstring in der Protokolldatei
BOOL TProtocol::SaveProtocolString( )
{
	FILE *in, *out;
	char buf[MaxLine], buf2[MaxLine];
	long FilePos;

	GetParameterLfdNr(buf);
	GetParameterDate(buf2);
	if (!TestLfdNr(buf) || !TestDate(buf2))
	{
		sprintf(buf, ProtocolLanguage->GetMessage(16), buf, buf2);
		MessageBox(GetFocus(), buf, ProtocolLanguage->GetFailure(), MBFAILURE);
		return FALSE;
	}
	if ( strlen(GetLfdNr(buf)) <= 0 )
		return FALSE;

	// Schreibschutzattribut der Protokolldatei wird geloescht
	SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE);
	SetFileAttributes(GetProtocolFile(GetPName(), ".tmp"), FILE_ATTRIBUTE_ARCHIVE);

	if ( !SearchProtocolString(buf) )
	{
		if ( bNoSortNew || ((lastINDEX == NULL) || (strcmp(buf, lastINDEX->LfdNr) >= 0)) )
		{
			if ( (out= fopen(GetPFile(), "at")) == NULL )
				return FALSE;
			StrMerge(buf);
			fputs(buf, out);
			fclose(out);

			// Schreibschutzattribut der Protokolldatei wird gesetzt
			SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);

			AppendProtocolFileIndex();
		}
		else
		{
			currentINDEX= lastINDEX;
			while ((currentINDEX != NULL) && (strcmp(currentINDEX->LfdNr, buf) > 0))
				currentINDEX= currentINDEX->last;
			if (currentINDEX == NULL)
				currentINDEX= firstINDEX;
			else
				currentINDEX= currentINDEX->next;
			if ( (in= fopen(GetPFile(), "rt")) == NULL )
				return FALSE;
			if ( (out= fopen(GetProtocolFile(GetPName(), ".tmp"), "wt")) == NULL )
				return FALSE;
			StrMerge(buf);
			while (!feof(in))
			{
				FilePos= ftell(in);
				if ( fgets( buf2, MaxLine, in ) != NULL )
				{
					if (FilePos == currentINDEX->Pos)
						fputs(buf, out);
					fputs(buf2, out);
				}
			}
			fclose(out);
			fclose(in);
			remove
				(GetPFile());
			rename(GetProtocolFile(GetPName(), ".tmp"), GetPFile());

			// Schreibschutzattribut der Protokolldatei wird gesetzt
			SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);

			LoadProtocolFileIndex(currentINDEX->LfdNr);
		}
	}
	else
	{

		if (currentINDEX != NULL)
			currentINDEX->Status= atoi(GetProtocolValue(Felder - 1, buf));

		if ( (in= fopen(GetPFile(), "r+t")) == NULL )
			return FALSE;
		StrMerge(buf);
		FilePos= ftell(in);
		if ( fseek(in, currentINDEX->Pos, SEEK_SET) != 0 )
		{
			fclose(in);
			return FALSE;
		}
		if ( fgets( buf2, MaxLine, in ) == NULL )
		{
			fclose(in);
			return FALSE;
		}
		if ((strlen(buf) == strlen(buf2)) ||
				((currentINDEX == lastINDEX) && (strlen(buf) > strlen(buf2))) )
		{
			if ( fseek(in, currentINDEX->Pos, SEEK_SET) != 0 )
			{
				fclose(in);
				return FALSE;
			}
			fputs(buf, in);
			fclose(in);

			// Schreibschutzattribut der Protokolldatei wird gesetzt
			SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);

			LoadProtocolFileIndex(currentINDEX->LfdNr);
		}
		else
		{
			if ( fseek(in, FilePos, SEEK_SET) != 0 )
			{
				fclose(in);
				return FALSE;
			}
			if ( (out= fopen(GetProtocolFile(GetPName(), ".tmp"), "wt")) == NULL )
				return FALSE;
			while (!feof(in))
			{
				FilePos= ftell(in);
				if ( fgets( buf2, MaxLine, in ) != NULL )
				{
					if ( FilePos == currentINDEX->Pos )
						fputs(buf, out);
					else
						fputs(buf2, out);
				}
			}
			fclose(out);
			fclose(in);

			remove
				(GetPFile());
			rename(GetProtocolFile(GetPName(), ".tmp"), GetPFile());

			// Schreibschutzattribut der Protokolldatei wird gesetzt
			SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);

			LoadProtocolFileIndex(currentINDEX->LfdNr);
		}
	}
	if (!bNoSortNew)
		return SortProtocolFile();
	return TRUE;
};

// loescht alle Felder des Parameterfeldes
void TProtocol::ClearProtocolString( )
{
	ClearProtocolString(0, Felder);
};

// loescht alle Felder von a bis b des Parameterfeldes
void TProtocol::ClearProtocolString( int a, int b )
{
	if (a < 0)
		a= 0;
	if (b > Felder)
		b= Felder;
	for (int i= a; i < b; i++)
	{
		strcpy(Parameter[i].Wert, "");
		SetParameterReadWrite(i);
	}
};

// loescht alle Felder des Parameterfeldes und die Skizze
void TProtocol::NewProtocolString( )
{
	char buf[MaxLfdNr];

	strcpy(CopyLfdNr, "");
	ClearProtocolString();
	SetParameterDate(GetSysDate(buf));
	SetProtocolValue(Felder - 1, "0");
	SaveLfdNr(GetNewLfdNr(buf));
	ResetIndex();
	freePic();
};

// loescht alle Felder des Parameterfeldes und die Skizze
void TProtocol::CopyProtocolString( LPCSTR lLfdNr )
{
	char buf[MaxLfdNr];

	if (SearchProtocolString(GetLfdNr(buf)))
		strcpy(CopyLfdNr, buf);
	ClearOldParameter();
	SetParameterDate(GetSysDate(buf));
	SetProtocolValue(Felder - 1, "0");
	if (lLfdNr != NULL)
		SaveLfdNr(lLfdNr);
	else
		SaveLfdNr(GetNewLfdNr(buf));
	ResetIndex();
};

// laed einen Protokollstring aus der Protokolldatei und fuelt das Parameterfeld
// mit den entspraechenden Parametern
BOOL TProtocol::LoadProtocolString( long FilePos )
{
	FILE *in;
	char buf[MaxLine], buf2[MaxLfdNr];

	if ((in= fopen(GetPFile(), "rt")) == NULL)
		return FALSE;
	if ( fseek(in, FilePos, SEEK_SET) != 0 )
	{
		fclose(in);
		return FALSE;
	}
	if ( fgets( buf, MaxLine, in ) == NULL )
	{
		fclose(in);
		return FALSE;
	}
	fclose(in);
	StrSplit(buf);
	SetLfdNr(GetParameterLfdNr(buf2));
	return TRUE;
};

// laed einen Protokollstring aus der Protokolldatei und fuelt das Parameterfeld
// mit den entspraechenden Parametern
int TProtocol::LoadProtocolString( LPCSTR lLfdNr )
{
	if ( !SearchProtocolString(lLfdNr) )
	{
		if ( bDebug )
			MessageBox(GetFocus(), "LoadProtocolString NewProtocolString!", "Debug Info", MBINFO);
		NewProtocolString();
		return 1;
	}
	if ( !LoadProtocolString( currentINDEX->Pos) )
		return -1;
	return 0;
};

// laed einen Protokollstring aus der Protokolldatei und fuelt das Parameterfeld
// mit den entspraechenden Parametern
int TProtocol::ReloadProtocolString( LPCSTR lLfdNr )
{
	if ( strlen(lLfdNr) <= 0 )
	{
		NewProtocolString();
		return 2;
	}
	if ( !SearchProtocolString(lLfdNr) )
	{
		if (strlen(CopyLfdNr) > 0)
		{
			if (SearchProtocolString(CopyLfdNr))
			{
				LoadProtocolString(CopyLfdNr);
				CopyProtocolString(lLfdNr);
			}
			else
				NewProtocolString();
		}
		else
			NewProtocolString();
		Protocol->SetDefaultParameter();
		SaveLfdNr(lLfdNr);
		return 1;
	}
	if ( !LoadProtocolString( currentINDEX->Pos) )
		return -1;
	LoadPicFile(lLfdNr);
	return 0;
};

// laed den naechsen Protokollstring aus der Protokolldatei und fuelt das
// Parameterfeld mit den entspraechenden Parametern
int TProtocol::LoadNextProtocolString( )
{
	strcpy(CopyLfdNr, "");
	if ( currentINDEX == NULL )
		currentINDEX= firstINDEX;
	else
		currentINDEX= currentINDEX->next;
	while (currentINDEX != NULL && IsProtocolDelete())
		currentINDEX= currentINDEX->next;
	if ( currentINDEX == NULL )
	{
		NewProtocolString();
		return 1;
	}
	if ( LoadProtocolString(currentINDEX->Pos) )
		return 0;
	return -1;
};

// laed den vorherigen Protokollstring aus der Protokolldatei und fuelt das
// Parameterfeld mit den entspraechenden Parametern
int TProtocol::LoadLastProtocolString( )
{
	TFileIndex *curINDEX;

	strcpy(CopyLfdNr, "");
	if ( currentINDEX == NULL )
		currentINDEX= curINDEX= lastINDEX;
	else
	{
		curINDEX= currentINDEX;
		currentINDEX= currentINDEX->last;
	}
	while (currentINDEX != NULL && IsProtocolDelete())
		currentINDEX= currentINDEX->last;
	if (currentINDEX == NULL)
		currentINDEX= curINDEX;
	if ( LoadProtocolString(currentINDEX->Pos) )
	{
		if ( currentINDEX->last == NULL || (currentINDEX == curINDEX && currentINDEX != lastINDEX) )
			return 1;
		else
			return 0;
	}
	return -1;
};

// macht aus dem Systemdatum einen String
LPSTR TProtocol::GetSysDate( LPSTR Value )
{
	SYSTEMTIME d;

	GetLocalTime(&d);
	sprintf(Value, "%d.%d.%d", d.wDay, d.wMonth, d.wYear);
	return Value;
};

// der Wert eines Protokollparameters wird in das Protokollparameterfeld eingetragen
BOOL TProtocol::SetProtocolValue( int Nr, LPCSTR Value )
{
	char buf[MaxString], buf2[MaxString];

	if ( strlen(Value) > GetParameterMaxLen(Nr) )
	{
		if (bDebug)
		{
			sprintf(buf, "%s: %s (max. %d Zeichen!)", GetParameterNamen(Nr, buf2), Value, GetParameterMaxLen(Nr));
			MessageBox(GetFocus(), buf, "Debug Info", MBINFO);
		}
		return FALSE;
	}
	strcpy(Parameter[Nr].Wert, Value);
	return TRUE;
};

// der Wert eines Protokollparameters wird aus dem Protokollparameterfeld ausgelesen
LPSTR TProtocol::GetProtocolValue( int Nr, LPSTR Value )
{
	strcpy(Value, Parameter[Nr].Wert);
	return Value;
};

// die Variable fr die aktuelle Lfd.Nr. wird gesetzt
void TProtocol::SetLfdNr( LPCSTR Value )
{
	strcpy(CurrentLfdNr, Value);
};

// die Variable fr die aktuelle Lfd.Nr. wird ausgelesen
LPSTR TProtocol::GetLfdNr( LPSTR Value)
{
	strcpy(Value, CurrentLfdNr);
	return Value;
};

// der dynamische Speicher fr die Bildpunkte einer Skizze wird freigegeben
void TProtocol::freePic()
{
	if (firstPOINT == NULL)
		return;
	lastPOINT= currentPOINT= firstPOINT;
	while (currentPOINT != NULL)
	{
		lastPOINT= currentPOINT;
		currentPOINT= currentPOINT->next;
		_FREEOBJ(lastPOINT);
	}
	currentPOINT= lastPOINT= firstPOINT= NULL;
};

// die Variable fr den aktuellen Bildpunkt einer Skizze wird auf den ersten Bildpunkt gesetzt
void TProtocol::ResetXY()
{
	currentPOINT= firstPOINT;
	lastPOINT= NULL;
};

// liefert einen Bildpunkt einer Skizze zurck
// ist der letzte Bildpunkt erreicht wird der Rckgabewert der Funktion FALSE
BOOL TProtocol::GetXY( char &c, int &x, int &y, LPSTR t )
{
	if (currentPOINT == NULL)
	{
		if (lastPOINT != NULL)
			currentPOINT= lastPOINT->next;
		else
			currentPOINT= firstPOINT;
		return FALSE;
	}
	c= currentPOINT->c;
	x= currentPOINT->x;
	y= currentPOINT->y;
	strcpy(t, currentPOINT->t);
	if (currentPOINT->next != NULL)
		lastPOINT= currentPOINT;
	currentPOINT= currentPOINT->next;
	return TRUE;
};

// belegt neuen dynamischen Speicher fr einen Bildpunkt einer Skizze und
// initialisiert ihn mit den Wert eines Bildpunktes
void TProtocol::SetXY( char c, int x, int y, LPCSTR t )
{
	lastPOINT= currentPOINT;
	currentPOINT= new TMFPoint;
	if (!currentPOINT)
	{
		MessageBox(GetFocus(), ProtocolLanguage->GetMessage(6), ProtocolLanguage->GetFailure(), MBFAILURE);
		currentPOINT= lastPOINT;
		return;
	}

	if (firstPOINT == NULL)
		firstPOINT= currentPOINT;
	currentPOINT->c= c;
	currentPOINT->x= x;
	currentPOINT->y= y;
	strcpy(currentPOINT->t, t);
	currentPOINT->next= NULL;
	currentPOINT->last= lastPOINT;
	if (lastPOINT != NULL)
		lastPOINT->next= currentPOINT;
};

// loescht die letzten Bildpunkt einer Skizze
BOOL TProtocol::UndoXY()
{
	TMFPoint *tmpPOINT;

	if (currentPOINT != NULL)
	{
		do
		{
			if (currentPOINT->c == 'T')
			{
				currentPOINT= currentPOINT->last;
				break;
			}
			else
				currentPOINT= currentPOINT->last;
		} while ( (currentPOINT != NULL) && (currentPOINT->c != 'M') && (currentPOINT->c != 'T') );
		if ( (currentPOINT != NULL) && (currentPOINT->c == 'M') )
			currentPOINT= currentPOINT->last;
		if (currentPOINT == NULL)
		{
			tmpPOINT= firstPOINT;
			firstPOINT= NULL;
			lastPOINT= NULL;
		}
		else
		{
			tmpPOINT= currentPOINT->next;
			currentPOINT->next= NULL;
		}
		while (tmpPOINT != NULL)
		{
			lastPOINT= tmpPOINT;
			tmpPOINT= tmpPOINT->next;
			_FREEOBJ(lastPOINT);
		}
		if (currentPOINT != NULL)
			lastPOINT= currentPOINT->last;
	}
	if (currentPOINT == NULL)
		return FALSE;
	return TRUE;
};

// liest alle Bildpunkte einer Skizze aus einer Datei in den dynamischen Speicher
// die Bildpunkte werden in einer verketteten Liste gespeichert
BOOL TProtocol::LoadPicFile( LPCSTR lLfdNr )
{
	OFSTRUCT of;
	HENHMETAFILE hMeta;

	freePic();
	if (OpenFile(GetSkizzeFile(lLfdNr), &of, OF_EXIST) != HFILE_ERROR)
	{
		hMeta= GetEnhMetaFile(GetSkizzeFile(lLfdNr));

		if (hMeta != NULL)
		{
			EnumEnhMetaFile(0,hMeta,EnumMetafileProc,0,0);
			DeleteEnhMetaFile(hMeta);
			return TRUE;
		}
	}
	return FALSE;
};

// speichert alle Bildpunkte einer Skizze in einer Datei ab
BOOL TProtocol::SavePicFile( )
{
	OFSTRUCT of;
	HENHMETAFILE hMeta;
	HDC hdcMeta;
	char lLfdNr[MaxLfdNr];

	if ( strlen(GetLfdNr(lLfdNr)) <= 0 )
		return FALSE;
	if (firstPOINT == NULL)
	{
		DeletePicFile( lLfdNr );
		return FALSE;
	}
	if (OpenFile(GetSkizzeFile(lLfdNr), &of, OF_EXIST) != HFILE_ERROR)
	{
		hMeta= GetEnhMetaFile(GetSkizzeFile(lLfdNr));
		SetFileAttributes(GetSkizzeBackupFile(lLfdNr), FILE_ATTRIBUTE_ARCHIVE);

		if (hMeta != NULL)
		{
			CopyEnhMetaFile(hMeta, GetSkizzeBackupFile(lLfdNr));
		}

		// Schreibschutzattribut der Protokolldatei wird gesetzt
		SetFileAttributes(GetSkizzeBackupFile(lLfdNr), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);
	}

	// Schreibschutzattribut der Protokolldatei wird entfernt
	SetFileAttributes(GetSkizzeFile(lLfdNr), FILE_ATTRIBUTE_ARCHIVE);

	DeleteEnhMetaFile(hMeta);
	remove(GetSkizzeFile(lLfdNr));

	//if ( (hdcMeta= CreateMetaFile(GetSkizzeFile(lLfdNr))) == NULL )
	if((hdcMeta= CreateEnhMetaFile(NULL,GetSkizzeFile(lLfdNr),NULL,NULL)) == NULL)
		return FALSE;

	currentPOINT= lastPOINT= firstPOINT;
	while (currentPOINT != NULL)
	{

		if (hdcMeta != NULL)
		{
			switch (currentPOINT->c)
			{
				case 'M':
					MoveToEx(hdcMeta, currentPOINT->x, currentPOINT->y, NULL);
					break;

				case 'L':
					LineTo(hdcMeta, currentPOINT->x, currentPOINT->y);
					break;

				case 'T':
					TextOut(hdcMeta, currentPOINT->x, currentPOINT->y, currentPOINT->t, strlen(currentPOINT->t));
					break;
			}
		}

		if (currentPOINT->next != NULL)
			lastPOINT= currentPOINT;
		currentPOINT= currentPOINT->next;
	}
	currentPOINT= lastPOINT->next;

	CloseEnhMetaFile(hdcMeta);

	// Schreibschutzattribut der Protokolldatei wird gesetzt
	SetFileAttributes(GetSkizzeFile(lLfdNr), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);

	return TRUE;
};

// lscht eine Bilddatei und gibt den Speicher fr die Bildpunkte frei
void TProtocol::DeletePicFile( LPCSTR lLfdNr )
{
	freePic();

	// Schreibschutzattribut der Protokolldatei wird gesetzt
	SetFileAttributes(GetSkizzeFile(lLfdNr), FILE_ATTRIBUTE_ARCHIVE);

	remove(GetSkizzeFile(lLfdNr));
};

// erzeugt aus einer Lfd.Nr. einen Namen fr eine Bilddatei (PATH + Dateiname)
LPSTR TProtocol::GetSkizzeFile( LPCSTR Value )
{
	return GetProtocolFile( "SKIZZEN\\", Value, ".WMF" );
};

// erzeugt aus einer Lfd.Nr. einen Namen fr eine Bildbackupdatei (PATH + Dateiname)
LPSTR TProtocol::GetSkizzeBackupFile( LPCSTR Value )
{
	return GetProtocolBackupFile( "SKIZZEN\\", Value, ".BAK" );
};

// erzeugt einen Protokolldateinamen (PATH + Dateiname) aus mehreren Namensbestandteilen
LPSTR TProtocol::GetProtocolFile( LPCSTR Value1, LPCSTR Value2, LPCSTR Value3 )
{
	sprintf(szProtocolFile, "%s%s%s%s", GetPDirectory(), Value1, Value2, Value3);
	return szProtocolFile;
};

// erzeugt einen Protokollbackupdateinamen (PATH + Dateiname) aus mehreren Namensbestandteilen
LPSTR TProtocol::GetProtocolBackupFile( LPCSTR Value1, LPCSTR Value2, LPCSTR Value3 )
{
	sprintf(szProtocolFile, "%s%s%s%s", GetPBackupDirectory(), Value1, Value2, Value3);
	return szProtocolFile;
};

// speichert den aktuellen ProtokollDialog in einer Variable
void TProtocol::SetCurrentDlg( int idDlg )
{
	CurrentDlg= idDlg;
};

// gibt den aktuellen Protokolldialog aus einer Variable zurck
int TProtocol::GetCurrentDlg( )
{
	return CurrentDlg;
};

// liefert den Namen eines Protokollparamerters zurck
LPSTR TProtocol::GetParameterNamen( int Nr, LPSTR Value )
{
	strcpy(Value, Parameter[Nr].Name);
	return Value;
};

// liefert die maximale Lnge eines Protokollparamerters zurck
int TProtocol::GetParameterMaxLen( int Nr )
{
	return Parameter[Nr].MaxLen;
};

// liefert den Status eines Protokollparamerters
BOOL TProtocol::IsParameterReadOnly( int Nr )
{
	if (Parameter[Nr].Status == ReadOnly)
		return TRUE;
	return FALSE;
};

// setzt den Status eines Protokollparamerters
void TProtocol::SetParameterReadOnly( int Nr )
{
	Parameter[Nr].Status= ReadOnly;
};

// setzt den Status eines Protokollparamerters
void TProtocol::SetParameterReadWrite( int Nr )
{
	Parameter[Nr].Status= ReadWrite;
};

// liefert den Status des aktuellen Protokollstrings
BOOL TProtocol::IsProtocolReadOnly( void )
{
	if (currentINDEX != NULL)
		if (currentINDEX->Status > ReadWrite)
			return TRUE;
	return FALSE;
};

// setzt den Status des aktuellen Protokollstrings readonly
void TProtocol::SetProtocolReadOnly( void )
{
	char buf[MaxString];

	SetProtocolValue(Felder - 1, itoa(ReadOnly, buf , 10));
	SaveProtocolString();
};

// liefert den Status des aktuellen Protokollstrings
BOOL TProtocol::IsProtocolDelete( void )
{
	if (currentINDEX != NULL)
		if (currentINDEX->Status == Deleted)
			return TRUE;
	return FALSE;
};

// setzt den Status des aktuellen Protokollstrings delete
void TProtocol::SetProtocolDelete( void )
{
	char buf[MaxString];

	if (!IsProtocolDelete())
	{
		SetProtocolValue(Felder - 1, itoa(Deleted, buf , 10));
		SaveProtocolString();
	}
};

// ist TRUE wenn eine Messung aufgezeichnet wird
void TProtocol::SetProtocolOn( BOOL bStatus )
{
	if (bStatus && Initialize())
	{
		if (iProtocolOnStatus == Off)
		{
			iProtocolOnStatus= On;
			ClearProtocolString();
			if ( !((IsLastProtocolString() && !IsProtocolReadOnly()) ||
					IsNewProtocolString()) )
			{
				NewProtocolString();
			}
		}
	}
	else
		iProtocolOnStatus= Off;
};

// liefert den Status ob eine Messung aufgezeichnet wird
BOOL TProtocol::IsProtocolOn( )
{
	if (iProtocolOnStatus == Off)
		return FALSE;
	else
		return TRUE;
};

// ndert den Status eine Messung die aufgezeichnet wird
void TProtocol::ChangeProtocolOnStatus( void )
{
	if (IsProtocolOn())
		iProtocolOnStatus++;
};

// liefert den Status eine Messung die aufgezeichnet wird
BOOL TProtocol::IsProtocolOnBegin( )
{
	if (iProtocolOnStatus == Begin)
		return TRUE;
	else
		return FALSE;
};

void TProtocol::SetAllParameterReadWrite( void )
{
	for (int i= 0; i < Felder; i++)
		SetParameterReadWrite(i);
};

void TProtocol::SetAllParameterReadOnly( void )
{
	for (int i= 0; i < Felder; i++)
		SetParameterReadOnly(i);
};

LPCSTR TProtocol::GetProtocolSection( void )
{
	return szProtocolSection;
};

BOOL TProtocol::IsProtocolSortDate( void )
{
	return bSortDate;
};

void TProtocol::SetProtocolSortDate( BOOL bSet )
{
	if (bSortDate != bSet)
	{
		bSortDate= bSet;
		SortProtocolFile();
		if (bSortDate)
			WritePrivateProfileString(szProtocolSection, "SortDate", "1", GetCFile());
		else
			WritePrivateProfileString(szProtocolSection, "SortDate", "0", GetCFile());
	}
};

BOOL TProtocol::IsProtocolNoSortNew( void )
{
	return bNoSortNew;
};

void TProtocol::SetProtocolNoSortNew( BOOL bSet )
{
	bNoSortNew= bSet;
	if (bNoSortNew)
		WritePrivateProfileString(szProtocolSection, "NoSortNew", "1", GetCFile());
	else
		WritePrivateProfileString(szProtocolSection, "NoSortNew", "0", GetCFile());
};

// Sortiert den Protokolldateiindex
void TProtocol::QuickSortFileIndex(TFileIndex *von, TFileIndex *bis, int laenge)
{
	TFileIndex *links, *rechts, zwischenspeicher;
	char pivotelement[MaxLfdNr];
	int linc= 0;
	int rinc= 0;

	links= von;
	rechts= bis;
	if (bSortDate)
		strcpy(pivotelement, bis->Date);
	else
		strcpy(pivotelement, bis->LfdNr);
	do
	{
		if (bSortDate)
		{
			while (strcmp(links->Date, pivotelement) < 0)
			{
				links= links->next;
				linc++;
			}
			while (strcmp(pivotelement, rechts->Date) < 0)
			{
				rechts= rechts->last;
				rinc++;
			}
		}
		else
		{
			while (strcmp(links->LfdNr, pivotelement) < 0)
			{
				links= links->next;
				linc++;
			}
			while (strcmp(pivotelement, rechts->LfdNr) < 0)
			{
				rechts= rechts->last;
				rinc++;
			}
		}
		if (linc + rinc < laenge)
		{
			strcpy(zwischenspeicher.LfdNr, links->LfdNr);
			strcpy(zwischenspeicher.Date, links->Date);
			zwischenspeicher.Pos= links->Pos;

			strcpy(links->LfdNr, rechts->LfdNr);
			strcpy(links->Date, rechts->Date);
			links->Pos= rechts->Pos;

			strcpy(rechts->LfdNr, zwischenspeicher.LfdNr);
			strcpy(rechts->Date, zwischenspeicher.Date);
			rechts->Pos= zwischenspeicher.Pos;

			links= links->next;
			linc++;
			rechts= rechts->last;
			rinc++;
		}
	}
	while (linc + rinc < laenge);

	if (rinc < laenge - 1)
		QuickSortFileIndex(von, rechts, laenge - rinc);
	if (linc < laenge - 1)
		QuickSortFileIndex(links, bis, laenge - linc);
}

// Sortiert die Protokolldatei, wenn sie nicht sortiert ist (bIsSort),
// liefert TRUE wenn die Datei sortiert ist oder werden konnte
// bIsSort wird von LoadProtocolFileIndex() oder AppendProtocolFileIndex() gesetzt
BOOL TProtocol::SortProtocolFile( void )
{
	FILE *in, *out;
	char buf[MaxLine];

	if ( (!bIsSortLfdNr && !bSortDate) || (!bIsSortDate && bSortDate))
	{
		QuickSortFileIndex( firstINDEX, lastINDEX, lastINDEX->Nr);
		if ( (in= fopen(GetPFile(), "rt")) == NULL )
			return FALSE;
		if ( (out= fopen(GetProtocolFile(GetPName(), ".tmp"), "wt")) == NULL )
			return FALSE;
		if ( fgets( buf, MaxLine, in ) == NULL )
		{
			fclose(in);
			fclose(out);
			return FALSE;
		}
		if ( strncmp(buf, "Lfd", 3) == 0 )
			fputs(buf, out);
		currentINDEX= firstINDEX;
		while (currentINDEX != NULL)
		{
			if ( fseek(in, currentINDEX->Pos, SEEK_SET) != 0 )
			{
				fclose(in);
				fclose(out);
				return FALSE;
			}
			if ( fgets( buf, MaxLine, in ) == NULL )
			{
				fclose(in);
				fclose(out);
				return FALSE;
			}
			fputs(buf, out);
			currentINDEX= currentINDEX->next;
		}
		fclose(out);
		fclose(in);

		// Schreibschutzattribut der Protokolldatei wird entfernt
		SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE);

		remove(GetPFile());
		rename(GetProtocolFile(GetPName(), ".tmp"), GetPFile());

		// Schreibschutzattribut der Protokolldatei wird gesetzt
		SetFileAttributes(GetPFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);

		return LoadProtocolFileIndex();
	}
	return TRUE;
};


// LastLine


