//#############################################################################
//                                                                           //
// U_VALUES.CPP                                                              //
// Subsystem: Utilities - Tools fr Zahlen und Strings                       //
// Deklaration: U_VALUES.H                                                   //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand:   07.11.2002                                                       //
//                                                                           //
//#############################################################################

#include <stdio.h>  // sprintf
#include <string.h> // strlen, strcpy
#include <math.h> // pow, ceil, floor
#include <stdlib.h> // fr variable Parameteranzahl
#include <stdarg.h>

#include "utils\u_values.h" // SCHNITTSTELLE fr diese Datei
#include "utils\u_window.h" // MBINFO

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--


#ifdef _DEBUG
	UINT TLeakPolice::s_Counter= 0;
#endif

////////////////////////////////////////////////////////////////////////////////
// Trace
////////////////////////////////////////////////////////////////////////////////

void _CURVECLASS WINAPI TraceMsg ( LPCSTR aStr )
{
	MessageBox( NULL, aStr, "Trace", MBINFO );
}

void _CURVECLASS WINAPI TraceMsg ( const double aDouble )
{

	char buf[ _MAXLENDOUBLE + 1 ]; // + Nullterminierung
	sprintf( buf, "%.10f", aDouble );
	TraceMsg(buf);
}

void _CURVECLASS WINAPI Trace ( LPCSTR aStr )
{
	#ifdef _DEBUG
		const char prae[]= "TRACE: ";
		char *buf= new char[ strlen(prae) + strlen(aStr) + 3 ]; // + "\n" + Nullterminierung
		sprintf(buf, "%s%s\n", prae, aStr);
		OutputDebugString( buf );
		_FREEOBJ(buf);
	#endif // _DEBUG
}

void _CURVECLASS WINAPI TraceErr ( LPCSTR aStr )
{
	#ifdef _DEBUG
		const char prae[]= "FEHLER: ";
		char *buf= new char[ strlen(prae) + strlen(aStr) + 3 ]; // + "\n" + Nullterminierung
		sprintf(buf, "%s%s\n", prae, aStr);
		OutputDebugString( buf );
		_FREEOBJ(buf);
	#endif // _DEBUG
}

void _CURVECLASS WINAPI Trace ( const double aDouble )
{
	#ifdef _DEBUG
		char buf[ _MAXLENDOUBLE + 1 ]; // + Nullterminierung
		sprintf( buf, "%.10f", aDouble );
		Trace(buf);
	#endif // _DEBUG
}

////////////////////////////////////////////////////////////////////////////////
// Runden, Abrunden von Double-Werten
////////////////////////////////////////////////////////////////////////////////

double _CURVECLASS WINAPI DoubleRound ( const double aDouble, const UINT aDigits )
{
	if ( aDouble < 0 )
		return DoubleRoundOff( aDouble - 0.5, aDigits );
	else
		return DoubleRoundOff( aDouble + 0.5, aDigits );
}
//-----------------------------------------------------------------------------

double _CURVECLASS WINAPI DoubleRoundUp ( const double aDouble, const UINT aDigits )
{
	double corfak= pow( 10, aDigits ); //Multiplikation mit diesem Wert verschiebt das Komma nach rechts, Division nach links
	double temp= aDouble*corfak;
	double temp2= DoubleRoundOff(temp, 0);
	//Aufrunden durch berprfung der Nachkommastelle
	if ( aDouble < 0 ) {
		if ( temp2-temp>=0.09 ) temp2++;
	} else {
		if ( temp-temp2>=0.09 ) temp2++;
	}
	//Komma zurck an seine ursprngliche Position
	return temp2 / corfak;
}
//-----------------------------------------------------------------------------

double _CURVECLASS WINAPI DoubleRoundOff ( const double aDouble, const UINT aDigits )
{
	double temp, corfak= pow( 10, aDigits ); //Multiplikation mit diesem Wert verschiebt das Komma nach rechts, Division nach links
	//abrunden durch Verwerfen der Nachkommastellen
	if ( aDouble < 0 )
		temp= ceil( aDouble * corfak );
	else
		temp= floor( aDouble * corfak );
	//Komma zurck an seine ursprngliche Position
	return temp / corfak;
}
//-----------------------------------------------------------------------------

float _CURVECLASS WINAPI Double2Float ( const double aDouble, const UINT aDigits )
{
	double temp= aDouble * pow((double)10, (double)aDigits);
	float result= (float)temp;
	return result / powf((float)10, (float)aDigits);
}
//-----------------------------------------------------------------------------

double _CURVECLASS WINAPI Float2Double ( const float aFloat, const UINT aDigits )
{
	float temp= aFloat * powf((float)10, (float)aDigits);
	double result= (double)temp;
	return result / pow((double)10, (double)aDigits);
}

////////////////////////////////////////////////////////////////////////////////
// Stringmanipulation
////////////////////////////////////////////////////////////////////////////////

LPSTR _CURVECLASS WINAPI GetFormatedString( LPCSTR aFormat, ... ) 
{
	va_list args;
	va_start( args, aFormat );
	char *result= GetFormatedStringMaxLen( aFormat, 256, args );	
	va_end( args );
	return result;
}
LPSTR _CURVECLASS WINAPI GetFormatedString( LPCSTR aFormat, va_list &args ) 
{
	return GetFormatedStringMaxLen( aFormat, 256, args );
}
//-----------------------------------------------------------------------------

LPSTR _CURVECLASS WINAPI GetFormatedStringMaxLen( LPCSTR aFormat, const UINT aMaxLen, ... ) 
{
	va_list args;
	va_start( args, aMaxLen );
	char *buf= GetFormatedStringMaxLen( aFormat, aMaxLen, args );
	va_end( args );
	return buf;
}
LPSTR _CURVECLASS WINAPI GetFormatedStringMaxLen( LPCSTR aFormat, const UINT aMaxLen, va_list &args ) 
{
	char *buf= 0;
	buf= new char [ aMaxLen+1 ];
	vsprintf( buf, aFormat, args );	
	return buf;
}

////////////////////////////////////////////////////////////////////////////////
// Stringmanipulation und "Ist String ein Double?"
////////////////////////////////////////////////////////////////////////////////

void _CURVECLASS WINAPI StringReplace ( char *aString, const char aOld, const char aNew )
{
	if ( !aString )
		return; //Leerstring, nichts zu tun

	for ( UINT i= 0; i < strlen(aString); i++ )
		if ( aString[i] == aOld )
			aString[i]= aNew;
}
//-----------------------------------------------------------------------------

void _CURVECLASS WINAPI StringIgnore ( char *aString, const char aIgnore )
{
	if ( !aString || aIgnore=='\0' )
		return; // Leerstring oder zu ignorierendes Zeichen ist Nullterminierung: nichts zu tun

	for ( UINT i= 0; i < strlen(aString); )
		if ( aString[i] == aIgnore ) {
			for ( UINT j= i+1; j < strlen(aString); j++ )
				aString[j-1]= aString[j];
			aString[strlen(aString)-1]= '\0';
			// i nicht erhoehen, Stelle <i> nochmal prfen, wurde ja gendert
		} else i++;
}
//-----------------------------------------------------------------------------

int _CURVECLASS WINAPI DoubleErr ( LPCSTR aString, const char aDecimal, const char aThousand )
{
	if ( (!aString) || (strlen(aString)==0) )
		return 1; //Leerstring ist kein Double

	char *temp= new char[strlen(aString)+1]; // + Nullterminierung
	strcpy(temp, aString);
	if ( aThousand!='\0' ) StringIgnore( temp, aThousand );

	BOOL foundDecimal= FALSE;
	for ( UINT i= 0; i < strlen(temp); i++ )
	{
		if ( temp[i] == aDecimal )
		{
			if ( foundDecimal )
				return i; //zu viele Kommata
			else
				foundDecimal= TRUE;
		}
		else if ( temp[i] == '-' )
		{
			if ( i > 0 )
				return i; //ungltiges Minuszeichen
		}
		else if ( (temp[i] == '0') || (temp[i] == '1') || (temp[i] == '2') || (temp[i] == '3') || (temp[i] == '4') ||
				  (temp[i] == '5') || (temp[i] == '6') || (temp[i] == '7') || (temp[i] == '8') || (temp[i] == '9') )
		{
			//darf berall stehen
		}
		else
			return i; //ungltiges Zeichen
	}
	return -1;
}
//-----------------------------------------------------------------------------

BOOL _CURVECLASS WINAPI IsDouble ( LPCSTR aString, const char aDecimal, const char aThousand )
{
	return ( DoubleErr( aString, aDecimal, aThousand ) == -1 ); //keine fehlerhafte Stelle
}

////////////////////////////////////////////////////////////////////////////////
// Konvertierung zwischen String und Double
////////////////////////////////////////////////////////////////////////////////

double _CURVECLASS WINAPI String2Double ( LPCSTR aString, const UINT aDigits, const BOOL aRound, const char aDecimal, const char aThousand )
{
	char *temp= new char[ strlen(aString) + 2 ]; // + Nullterminierung und neue "0" vor dem Komma
	strcpy( temp, aString );
	StringReplace( temp, aDecimal, '.' );
	if ( aThousand!='\0' ) StringIgnore( temp, aThousand );
	double aResult= atof( temp ); //im Gegensatz zu sscanf mit Double-Genauigkeit
	_FREELIST(temp);
	if ( aRound )
		return DoubleRound( aResult, aDigits );
	else
		return DoubleRoundOff( aResult, aDigits );
}
//-----------------------------------------------------------------------------

	UINT GetDigits(const double aDouble)
	{
		if ( fabs(aDouble)>=100 ) return 0;
		else if ( fabs(aDouble)>=10 ) return 1;
		else if ( fabs(aDouble)>=1 ) return 2;
		else if ( fabs(aDouble)>=0.5 ) return 3;
		else if ( fabs(aDouble)>=0.25 ) return 4;
		else return 5;
	}

void _CURVECLASS WINAPI Double2String ( char *aReturn, const double aDouble, UINT aDigits, const BOOL aRound, const char aDecimal, const char aThousand )
{
	if ( aDigits>=NKAUTO ) aDigits= GetDigits(aDouble) + (aDigits-NKAUTO);
	double temp= aDouble;
	if ( !aRound )
		temp= DoubleRoundOff( aDouble, aDigits ); //sprintf rundet automatisch
	//in String konvertieren
	char format[6];
	sprintf( format, "%%.%df", aDigits ); //z.B. "%.3f" / "%.5f" -> Float-Werte mit drei / fnf Nachkommastellen
	sprintf( aReturn, format, temp ); //aValue konvertiert als String (Nachkommastellengenauigkeit: <aDigits>)
	StringReplace( aReturn, '.', aDecimal );
	if ( aThousand!='\0' ) { // Tausenderpunkte setzen
		// Position des Komma suchen (wenn keines, Ende der Zeichenkette nehmen)
		bool foundDecimal= false;
		UINT decimal= 0;
		for (UINT i= 0; i<strlen(aReturn); i++) if ( aReturn[i]==aDecimal ) {
			decimal= i;
			foundDecimal= true;
			break;
		}
		if ( !foundDecimal ) decimal= strlen(aReturn); // wenn kein Komma, letzte Position nehmen

		// Tausendertrennzeichen einfgen
		i= decimal;
		while (i>=0) {
			if ( aReturn[i]=='-' ) {
				break;
			} else if ( decimal-i==3 && i>0 && aReturn[i-1]!='-' ) { // Tausendertrennzeichen soll gesetzt werden (aber nicht direkt vor einem Minus!)
				aReturn[strlen(aReturn)+1]= '\0';
				for ( UINT j= strlen(aReturn); j>=i; j-- ) aReturn[j+1]= aReturn[j];
				aReturn[i]= aThousand;
				decimal= i; // so behandeln, als ob hier ein Komma steht; dann knnen wir einfach mit konstantem Anstand von 3 rechnen
			}
			if ( i>0 ) i--;
			else break;
		}
	}
}
////////////////////////////////////////////////////////////////////////////////
// Formatierte Zeitausgabe
////////////////////////////////////////////////////////////////////////////////

void _CURVECLASS WINAPI GetTimeString( char *aReturn, long aMillSec )
{
	sprintf( (LPSTR)aReturn, "%02d:%02d:%02d", (int)(aMillSec / 3600000l), (int)(aMillSec / 60000l) % 60, (int)(aMillSec / 1000l) % 60 );
}

//Datum: 16.08.2002
////////////////////////////////////////////////////////////////////////////////
// Min, Max
////////////////////////////////////////////////////////////////////////////////

BOOL _CURVECLASS WINAPI SetInRange( double &aValue, double aBorder1, double aBorder2 )
{
	if ( aBorder1 < aBorder2 )
	{
		if ( aValue < aBorder1 )
		{
			aValue= aBorder1;
			return FALSE;
		};
		if ( aValue > aBorder2 )
		{
			aValue= aBorder2;
			return FALSE;
		};
	}
	else
	{
		if ( aValue > aBorder1 )
		{
			aValue= aBorder1;
			return FALSE;
		};
		if (aValue < aBorder2)
		{
			aValue= aBorder2;
			return FALSE;
		};
	};
	return TRUE;
}
//-----------------------------------------------------------------------------

BOOL _CURVECLASS WINAPI SetInRange( long &aValue, long aBorder1, long aBorder2 )
{
	if ( aBorder1 < aBorder2 )
	{
		if ( aValue < aBorder1 )
		{
			aValue= aBorder1;
			return FALSE;
		};
		if ( aValue > aBorder2 )
		{
			aValue= aBorder2;
			return FALSE;
		};
	}
	else
	{
		if ( aValue > aBorder1 )
		{
			aValue= aBorder1;
			return FALSE;
		};
		if (aValue < aBorder2)
		{
			aValue= aBorder2;
			return FALSE;
		};
	};
	return TRUE;
}

/*
int _CURVECLASS WINAPI maxi(int value1, int value2) {
	return ( (value1 > value2) ? value1 : value2);
};
 
int _CURVECLASS WINAPI mini(int value1, int value2) {
	return ( (value1 < value2) ? value1 : value2);
};
 
long _CURVECLASS WINAPI maxl(long value1, long value2) {
	return ( (value1 > value2) ? value1 : value2);
};
 
long _CURVECLASS WINAPI minl(long value1, long value2) {
	return ( (value1 < value2) ? value1 : value2);
};
 
float _CURVECLASS WINAPI maxf(float value1, float value2) {
	return ( (value1 > value2) ? value1 : value2);
};
 
float _CURVECLASS WINAPI minf(float value1, float value2)	{
	return ( (value1 < value2) ? value1 : value2);
};
 
double _CURVECLASS WINAPI maxd(double value1, double value2) {
	return ( (value1 > value2) ? value1 : value2);
};
 
double _CURVECLASS WINAPI mind(double value1, double value2) {
	return ( (value1 < value2) ? value1 : value2);
};*/
