//#############################################################################
//                                                                           //
// U_WINDOW.CPP                                                              //
//                                                                           //
// Subsystem : Fenster-Templates: modale und modeless Dialog; MDI-Fenster    //
// Benutung durch andere Subsysteme erforderlich: JA                         //
// Deklaration: U_WINDOW.H                                                   //
//---------------------------------------------------------------------------//
// Autor(en): Thomas Kullmann, Gnther Reinecker                             //
// Quelle:    verschoben aus DLG_TPL.CPP                                     //
// Stand:     10.04.2002                                                     //
//                                                                           //
//#############################################################################

#include "utils\u_window.h" // SCHNITTSTELLE fr diese Datei
#include "internls\evrythng.h" // fr GermanVersion

//#############################################################################
// sprachspezifische Konstanten
//#############################################################################

#ifdef GermanVersion
	const char _MSGTITLE[]= "Meldung";
	const char _NONUMBER[]= "Der eingebene Wert ist keine Zahl!\nBitte prfen Sie Ihre Eingabe.";
	const char _NOWINDOW[]= "Fenster kann nicht erzeugt werden!";
	const char _ERRTITLE[]= "FEHLER";
#else
	const char _MSGTITLE[]= "Message";
	const char _NONUMBER[]= "This value is not a number!\nPlease check your input.";
	const char _NOWINDOW[]= "Window can't be created!";
	const char _ERRTITLE[]= "ERROR";
#endif

//#############################################################################
// global
//#############################################################################

BOOL _CURVECLASS WINAPI SetActivateHotKey ( HWND aWnd, const char aKey, const BOOL aShift, const BOOL aCtrl, const BOOL aAlt, const BOOL aExtended )
{
	// Taste    | HEX
	//----------+------
	// Shift    |  1
	// Ctrl     |  2
	// Alt      |  4
	// Extended |  8
	// Ctrl+Alt= sum(Ctrl, Alt)= 0x6
	//
	// 'P'      | 50
	// Ctrl+Alt+'P'= concat('6', '50')= 0x650
	UINT modifier= 0;
	if ( aShift )
		modifier += HOTKEYF_SHIFT;
	if ( aCtrl )
		modifier += HOTKEYF_CONTROL;
	if ( aAlt )
		modifier += HOTKEYF_ALT;
	if ( aExtended )
		modifier += HOTKEYF_EXT;
	// Keycode berechnen und registrieren
	WPARAM keycode= modifier;
	keycode *= 256; // shift modifier to the upper byte
	keycode += aKey;
	UINT iSetResult= SendMessage( aWnd, WM_SETHOTKEY, keycode, 0 );
	switch ( iSetResult )
	{
		case 2:   // ERFOLG: Tastenkombination bereits angemeldet
			return TRUE;

		case 1:   // ERFOLG
			return TRUE;

		case 0:   // ungltiges HotKey-Fenster
			return FALSE;

		case -1:   // ungltige Tastenkombination
			return FALSE;

		default:   // unbekannter Fehler
			return FALSE;
	}
}

//#############################################################################
// THotKey
//#############################################################################

THotKey::THotKey ( HWND aWindow, const char aKey, const BOOL aShift, const BOOL aCtrl, const BOOL aAlt, const BOOL aWin ) : m_Cmd( -1)
{
	m_Window= aWindow;
	UINT modifier= CalcModifier ( aShift, aCtrl, aAlt, aWin );
	m_KeyCode= CalcKeyCode ( aKey, aShift, aCtrl, aAlt, aWin );
	// eindeutigen Namen ermitteln
	char buf[51];
	sprintf(buf, "HOTKEYATOM%d%d", m_Window, m_KeyCode);
	m_Atom= GlobalAddAtom(buf);
	// registrieren
	if ( m_Atom )
		m_Registered= RegisterHotKey( m_Window, m_Atom, modifier, aKey );
	else
		m_Registered= FALSE;
}
//-----------------------------------------------------------------------------

THotKey::~THotKey ( void )
{
	if ( (m_Atom) && (m_Registered) )
	{
		UnregisterHotKey( m_Window, m_Atom );
	}
	if ( m_Atom )
	{
		GlobalDeleteAtom( m_Atom );
		m_Atom= 0;
	}
}
//-----------------------------------------------------------------------------

UINT THotKey::CalcModifier ( const BOOL aShift, const BOOL aCtrl, const BOOL aAlt, const BOOL aWin )
{
	UINT result= 0;
	if ( aShift )
		result += MOD_SHIFT;
	if ( aCtrl )
		result += MOD_CONTROL;
	if ( aAlt )
		result += MOD_ALT;
	if ( aWin )
		result += MOD_WIN;
	return result;
}
//-----------------------------------------------------------------------------

LPARAM THotKey::CalcKeyCode ( const char aKey, const BOOL aShift, const BOOL aCtrl, const BOOL aAlt, const BOOL aWin )
{
	LPARAM result= aKey;
	result *= long(65536); // shift key to the upper word
	result += CalcModifier( aShift, aCtrl, aAlt, aWin );
	return result;
}

//*****************************************************************************
// Accessormethode(n)
//*****************************************************************************

BOOL THotKey::IsEqual ( LPARAM lParam ) const
{
	return ( (IsRegistered()) && (m_KeyCode == lParam) );
};
//-----------------------------------------------------------------------------

BOOL THotKey::IsRegistered ( void ) const
{
	return ( m_Atom);
};

//#############################################################################
// TBasicWindow
//#############################################################################

TIndList TBasicWindow::s_WindowList(true, true, false, true); 
TBasicWindow *TBasicWindow::s_WaitingWindow= 0; // es gibt kein angemeldetes Fenster

//*****************************************************************************
// Konstruktor-Destruktor
//*****************************************************************************

TBasicWindow::TBasicWindow ( HINSTANCE aInstance ) 
{
	m_Window= 0; // kein Fensterhandle
	m_Instance= aInstance;
}
//-----------------------------------------------------------------------------

TBasicWindow::~TBasicWindow ( void )
{
	for (UINT i= 0; i < m_HotKeys.size(); i++ )
		_FREEOBJ(m_HotKeys[i]); // Tastenkombinationen unregistrieren und Objekte freigeben
	m_HotKeys.resize(0); // Borland 5.02 untertzt kein clear, obwohl von STL vorgeschrieben!

	UnregisterWindow(); // Fensterregistrierung beenden (sptestens jetzt wird das Fenster nicht mehr angezeigt)
};
//*****************************************************************************

char *TBasicWindow::LoadString ( const int aStrTbleId )  const 
{
	BOOL bValid;
	return LoadString( aStrTbleId, bValid );
}
//-----------------------------------------------------------------------------

char *TBasicWindow::LoadString ( const int aStrTbleId, BOOL &aValid )  const
{
	int actLen= 50;
	while (1) {		
		char *MsgString= new char[actLen+1];
		int len= ::LoadString( GetInstance(), aStrTbleId, MsgString, actLen );
		if ( len>=actLen-1 && actLen<512 ) { // in jedem Durchlauf 50 Zeichen mehr Platz gewhren
			_FREELIST(MsgString);
			actLen+= 50; 
			continue;
		}
		if ( len ) { // Eintrag existiert
			aValid= TRUE;
			char *result= new char[ strlen(MsgString)+1 ]; // + Nullterminierung
			strcpy(result, MsgString);
			_FREELIST(MsgString);
			return result;
		} else { // Eintrag ist nicht vorhanden
			_FREELIST(MsgString);
			aValid= FALSE;
			const char prae[]= "RESID-FEHLER!!! Fr RessourcenId %d ist kein StringTable-Eintrag verfgbar.";
			char *buf= new char[ strlen(prae)+_MAXLENDOUBLE+1 ];
			sprintf(buf, prae, aStrTbleId);	
			TraceErr(buf);
			_FREEOBJ(buf);

			char *result= new char[ 0+1 ]; // nur Nullterminierung
			strcpy(result, "");
			return result;
		}
	}
}
//*****************************************************************************

char  *TBasicWindow::GetWindowTxt ( HWND aWindow ) 
{
	int actLen= 50;
	while (1) {		
		char *MsgString= new char[actLen+1];
		int len= ::GetWindowText( aWindow, MsgString, actLen );
		if ( len>=actLen-1 && actLen<512 ) { // in jedem Durchlauf 50 Zeichen mehr Platz gewhren
			_FREELIST(MsgString);
			actLen+= 50; 
			continue;
		}
		if ( len ) { // Fenster existiert
			char *result= new char[ strlen(MsgString)+1 ]; // + Nullterminierung
			strcpy(result, MsgString);
			_FREELIST(MsgString);
			return result;
		} else { // Fenster ist nicht vorhanden oder hat keinen Titel
			_FREELIST(MsgString);
			const char prae[]= "ohne Titel";
			char *result= new char[ strlen(prae)+1 ];
			strcpy(result, prae);
			return result;
		}
	}
}
//-----------------------------------------------------------------------------

char  *TBasicWindow::GetWindowTxt()
{
	return GetWindowTxt(m_Window);
}
//*****************************************************************************

int TBasicWindow::MessageBox(LPCSTR aText, LPCSTR aCap, UINT uType)  const
{
	char *buf= new char[ max(strlen(_MSGTITLE), strlen(aCap)) ];
	if ( strlen(aCap)==0 ) strcpy(buf, _MSGTITLE);
	else strcpy(buf, aCap);
	if ( uType==0 ) uType= MB_OK;
	return ::MessageBox(GetHandle(), aText, buf, uType);
};
//-----------------------------------------------------------------------------

int TBasicWindow::MessageBox(const int aStrTbleId, LPCSTR aCap, UINT uType) const
{
	char *MsgString= LoadString( aStrTbleId ); // ggf. ""
	int result= MessageBox(MsgString, aCap, uType);
	_FREELIST(MsgString);
	return result;
}
//-----------------------------------------------------------------------------

int TBasicWindow::MessageBox(LPCSTR aText, const int aStrTbleId, UINT uType) const
{
	char *MsgString= LoadString( aStrTbleId ); // ggf. ""
	int result= MessageBox(aText, MsgString, uType);
	_FREELIST(MsgString);
	return result;
}
//-----------------------------------------------------------------------------

int TBasicWindow::MessageBox(const int aStrTbleId, const int aStrTbleId2, UINT uType) const
{
	char *MsgString= LoadString( aStrTbleId2 ); // ggf. ""
	int result= MessageBox(aStrTbleId, MsgString, uType);
	_FREELIST(MsgString);
	return result;
}
//-----------------------------------------------------------------------------

HCURSOR TBasicWindow::SetDefaultCursor(LPCTSTR aName) {
	return SetCursor( LoadCursor(NULL, aName) );
}

//*****************************************************************************
// Liste der registrierten Fenster; Zuordnung HWND <--> Fensterobjekt
//*****************************************************************************

void TBasicWindow::RegisterWindow ( HWND aWindow )
{
	if ( s_WaitingWindow == this )
		s_WaitingWindow= 0; // hier registrieren wir das Fenster, es ist also kein unregistriertes Fenster mehr

	// sind wir schon registriert?
	if ( !s_WindowList.Append(this) ) 
		return; // wir sind schon in der Liste (Abbruch)

	m_Window= aWindow;
}
//-----------------------------------------------------------------------------

void TBasicWindow::UnregisterWindow ( void )
{
	if ( s_WaitingWindow == this )
		s_WaitingWindow= 0; // falls das Fenster gar KEINE Windowsbotschaft erhalten haben sollte: jedenfalls wartet es nicht lnger auf ein Handle

	// aus der Liste der registrierten Fenster entfernen
	s_WindowList.RemoveData(this);
	m_Window= 0; // kein Fensterhandle mehr
}
//-----------------------------------------------------------------------------

TBasicWindow *TBasicWindow::Handle2Object ( HWND aWindow ) 
{
	// Objekt aus der Liste der registrierten Fenster ermitteln; damit endlich auch Zugriff auf Member
	for (TIndex i= 0; i<s_WindowList.GetCount(); i++) {
		TBasicWindow *result= (TBasicWindow *)s_WindowList[i];
		if ( result && result->m_Window==aWindow ) 
			return result; // gefunden :-)
	}
	return 0; // nicht gefunden
}
//-----------------------------------------------------------------------------

LRESULT CALLBACK TBasicWindow::EventHandlerEx ( HWND aWindow, UINT aId, WPARAM wParam, LPARAM lParam, BOOL &aValid )
{
	aValid= FALSE;
	// Objekt aus der Liste der registrierten Fenster ermitteln; damit endlich auch Zugriff auf Member
	TBasicWindow *self= Handle2Object(aWindow);

	// Wenn das Fensterhandle nicht in der Liste der registrierten Fenster ist, prfe ob sich ein
	// unregistriertes Fenster angemeldet hat und auf seine erste Windowsbotschaft (sein Handle) wartet.
	// Wenn JA, registriere dieses Fenster und hebe den Wartezustand auf
	if ( (!self) && (s_WaitingWindow) )
	{ // Problem besteht weil das zuletzt anzuzeigende, modale Dialogfenster NUR durch WM_INITDIALOG sein Handle erfahren kann - DialogBoxParam() gibt kein Handle zurck
		self= s_WaitingWindow;                      // Botschaft war fr's angemeldete Fenster
		s_WaitingWindow->RegisterWindow( aWindow ); // Fenster registrieren und Wartezustand aufheben (s_WaitingWindow= 0)
	};
	if ( !self ) { // weder registrierets noch angemeldetes Fenster: Botschaft nicht verarbeitet
		if ( aId==WM_GETTEXT ) return FALSE; // diese Nachricht wurde von uns und GetWindowTxt ausgelst; keine Meldung ausgeben !!!

		char *txt= GetWindowTxt(aWindow);
		const char prae[]= "Unbekannte Windows-Botschaft mit ID= %d, WPARAM= %d, LPARAM= %d, HWND= %d (%s) !";
		char *buf= new char[ strlen(prae) + 4*_MAXLENDOUBLE + strlen(txt) + 1 ];
		sprintf(buf, prae, aId, wParam, lParam, aWindow, txt);
		_FREELIST(txt);
		TraceErr(buf);
		_FREELIST(buf);
		return FALSE; 
	}

	// Botschaft nur an registrierte oder angemeldete Fenster weiterleiten
	aValid= TRUE;
	return self->OnEvent( aWindow, aId, wParam, lParam );
}
//-----------------------------------------------------------------------------

LRESULT CALLBACK TBasicWindow::EventHandler ( HWND aWindow, UINT aId, WPARAM wParam, LPARAM lParam )
{
	BOOL bValid= TRUE;
	return EventHandlerEx(aWindow, aId, wParam, lParam, bValid);
}

//*****************************************************************************
// Tastenkombinationen
//*****************************************************************************
BOOL TBasicWindow::SetActivateHotKey ( const char aKey, const BOOL aShift, const BOOL aCtrl, const BOOL aAlt, const BOOL aExtended )
{
	return ::SetActivateHotKey ( GetHandle(), aKey, aShift, aCtrl, aAlt, aExtended );
}
//-----------------------------------------------------------------------------

BOOL TBasicWindow::AddHotKey ( const char aKey, const BOOL aShift, const BOOL aCtrl, const BOOL aAlt, const BOOL aWin )
{
	THotKey *ht= new THotKey( GetHandle(), aKey, aShift, aCtrl, aAlt, aWin );
	m_HotKeys.push_back( ht );
	return ht->IsRegistered();
}
//-----------------------------------------------------------------------------

UINT TBasicWindow::LoadHotKeys ( HACCEL aAccel )
{
	// Accelerator aus Ressourcen laden und in <table> kopieren
	UINT count= CopyAcceleratorTable( aAccel, NULL, 0); // Anzahl der Tastenkombinatioen ermitteln
	ACCEL *table= new ACCEL[ count ]; // Tabelle der Tastenkombinationen erstellen
	CopyAcceleratorTable( aAccel, table, count ); // Tabelle mit Tastenkombinationen fllen

	// Tastenkombinationen registrieren; Informationen aus der "Tabelle mit Tastenkombinationen" entnehmen
	for (UINT i= 0; i < count; i++)
	{
		THotKey *ht= new THotKey( GetHandle(), (char)table[i].key, table[i].fVirt & FSHIFT, table[i].fVirt & FCONTROL, table[i].fVirt & FALT, FALSE ); // Windows-Taste wird in Ressourcen nicht untersttzt !
		ht->m_Cmd= table[i].cmd;
		m_HotKeys.push_back( ht );
	}

	_FREELIST(table); // "Tabelle mit Tastenkombinationen" freigeben
	return count;
}
//-----------------------------------------------------------------------------

BOOL TBasicWindow::IsHotKey ( LPARAM lParam, const char aKey, const BOOL aShift, const BOOL aCtrl, const BOOL aAlt, const BOOL aWin )
{
	return ( lParam == THotKey::CalcKeyCode( aKey, aShift, aCtrl, aAlt, aWin ) );
}

//#############################################################################
// TBasicDialog
//#############################################################################

TBasicDialog::TBasicDialog ( const char *aResName, HINSTANCE aInstance ) : TBasicWindow(aInstance), m_ResName(0)
{
	m_ResName= new char[ strlen(aResName) + 1 ]; // + Nullterminierung
	strcpy( m_ResName, aResName );
}
//-----------------------------------------------------------------------------

TBasicDialog::~TBasicDialog ( void )
{
	_FREELIST(m_ResName);
}

//*****************************************************************************
// EREIGNISSE
//*****************************************************************************

LRESULT TBasicDialog::OnEvent ( HWND aWindow, UINT aId, WPARAM wParam, LPARAM lParam )
{
	BOOL result= TRUE; // Botschaft durch HANDLE_MSG verarbeitet
	switch ( aId )
	{
		// Standardereignisse
		HANDLE_MSG( aWindow, WM_COMMAND, Dlg_OnCommand );
		HANDLE_MSG( aWindow, WM_INITDIALOG, Dlg_OnInit );
		HANDLE_MSG( aWindow, WM_HSCROLL, Dlg_OnHScrollBar );
		HANDLE_MSG( aWindow, WM_VSCROLL, Dlg_OnVScrollBar );
		HANDLE_MSG( aWindow, WM_TIMER, Dlg_OnTimer );

		// Mausereignisse (Klier)
		HANDLE_MSG( aWindow, WM_LBUTTONUP, Dlg_OnLButtonUp );
		HANDLE_MSG( aWindow, WM_LBUTTONDOWN, Dlg_OnLButtonDown );
		HANDLE_MSG( aWindow, WM_RBUTTONDOWN, Dlg_OnRButtonDown );
		HANDLE_MSG( aWindow, WM_MOUSEMOVE, Dlg_OnMouseMove );
		HANDLE_MSG( aWindow, WM_MOUSEACTIVATE, Dlg_OnMouseActivate );

		// Rest
		HANDLE_MSG( aWindow, WM_DESTROY, Dlg_OnDestroy );
		HANDLE_MSG( aWindow, WM_SETFOCUS, Dlg_OnSetFocus );

		case WM_HOTKEY:
		{
			for (UINT i= 0; i < m_HotKeys.size(); i++ )
				if ( GetFocus() )
				{ // nur wenn Fenster aktiv, sonst fangen wir die Tastenkombinationen auch wenn unsere Anwendung inaktiv ist!!!
					THotKey *hk= m_HotKeys[i];
					if ( (hk) && (hk->IsEqual(lParam)) )
					{
						Dlg_OnCommand( GetHandle(), hk->m_Cmd, 0, 0 );
						return TRUE; // Botschaft verarbeitet
					}
				} // sonst kann man in einer berschiebenen Methoden selbst auf WM_HOTKEY reagieren
			break;
		}

		// Systemcommand (klier 24.08.02)
		case WM_SYSCOMMAND:
			result= Dlg_OnSysCommand( aWindow, wParam, lParam );
			break;

		// Systemfarben haben sich gendert
		case WM_SYSCOLORCHANGE:
			//Datum: 27.07.2002 ! Ctl3dColorChange( );
			break;

		// Tastatur
		case WM_CHAR: // WM_KEYDOWN-Ergebnis bereits bersetzt in WM_CHAR (Taste mit ASCII-Code wurde gedrckt)		
			result= Dlg_OnChar( aWindow, wParam, lParam );
			break;

		//10.07.2003 klier
		// Auf nderungen an TabCtrl und ListView kann reagiert werden!
		case WM_NOTIFY:
			Dlg_OnNotify( aWindow, aId, wParam, lParam );
			return TRUE;
			break;

		default:
			result= TBasicWindow::OnEvent(aWindow, aId, wParam, lParam);
	} // switch
	return result;
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::Dlg_OnChar ( HWND, WPARAM aCharCode, LPARAM )
{
	if ( aCharCode == VK_ESCAPE )
	{ // ESC-Taste gedrckt und losgelassen
		// Datum: 27.07.2002 if(1 < (int)GET_WM_COMMAND_HWND(wParam,lParam))
		Interrupt( );
	}
	return TRUE; // Botschaft verarbeitet
}

//*****************************************************************************
// Verwaltung des Inhalts von Steuerelementen
//*****************************************************************************

UINT TBasicDialog::CtrlGetText ( const int aResId, char *aReturn, const int aMaxChars) const
{
	return GetDlgItemText( GetHandle(), aResId, aReturn, aMaxChars );
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetText ( const int aResId, LPCSTR aString ) const
{
	HWND ctrl= GetDlgItem( GetHandle(), aResId );
	SetDlgItemText( GetHandle(), aResId, aString );
	return ( ctrl != 0 );
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetText ( const int aResId, const int aStrTbleId ) const {
	char *MsgString= LoadString( aStrTbleId ); // ggf. ""
	BOOL result= CtrlSetText( aResId, MsgString );
	_FREELIST(MsgString);
	return result;
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetFormat ( const int aResId, LPCSTR aFormat, ... ) const
{
	va_list args;
	va_start( args, aFormat );	
	BOOL result= CtrlSetFormat( aResId, aFormat, args );
	va_end( args );
	return result;
}
BOOL TBasicDialog::CtrlSetFormat ( const int aResId, LPCSTR aFormat, va_list &args ) const
{
	char *buf= GetFormatedStringMaxLen( aFormat, 1024, args );
	BOOL result= CtrlSetText( aResId, buf );
	_FREELIST(buf);
	return result;
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetFormat ( const int aResId, const int aStrTbleIdFormat, ... ) const
{
	va_list args;
	va_start( args, aStrTbleIdFormat );	
	BOOL result= CtrlSetFormat( aResId, aStrTbleIdFormat, args );
	va_end( args );
	return result;
}
BOOL TBasicDialog::CtrlSetFormat ( const int aResId, const int aStrTbleIdFormat, va_list &args ) const
{
	char *MsgString= LoadString( aStrTbleIdFormat ); // ggf. ""
	BOOL result= CtrlSetFormat( aResId, MsgString, args );
	_FREELIST(MsgString);
	return result;
}
//*****************************************************************************

LONG TBasicDialog::CtrlGetLong ( const int aResId, BOOL &aValid ) const
{
	return (LONG)CtrlGetDouble( aResId, 0, '.', aValid );
}
//-----------------------------------------------------------------------------

LONG TBasicDialog::CtrlGetLong ( const int aResId, const LONG aMin, const LONG aMax, BOOL &aValid ) const
{
	LONG result= CtrlGetLong( aResId, aValid );
	return max( aMin, min( aMax, result ) );  // gelesenen Wert ins Intervall [<aMin>..<aMax>] einpassen
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetLong ( const int aResId, const LONG aLong ) const
{
	return CtrlSetDouble( aResId, (double)aLong, 0, '.' );
}
//*****************************************************************************

double TBasicDialog::CtrlGetDouble ( const int aResId, const UINT aDigits, const char aDecimal, BOOL &aValid ) const
{
	aValid= FALSE;
	char buf[ _MAXLENDOUBLE + 1 ]; // + Nullterminierung
	CtrlGetText(aResId, buf, _MAXLENDOUBLE);
	if ( !Ctrl(aResId) )
		return 0.0; // kein solches Steuerelement

	StringReplace( buf, aDecimal, '.' ); // damit sind Komma [,] und Punkt [.] zugelassen :-)
	if ( !IsDouble(buf, '.') )
	{  // ist kein Double-Wert: FEHLERMELDUNG
		MessageBox( _NONUMBER, _ERRTITLE, MBSTOP );
		CtrlSetFocus( aResId );		
		return 0.0;
	}
	aValid= TRUE;
	return String2Double( buf, aDigits, FALSE, '.' ); // bei zu vielen Nachkommastellen abschneiden
}
//-----------------------------------------------------------------------------

double TBasicDialog::CtrlGetDouble ( const int aResId, const double aMin, const double aMax, const UINT aDigits, const char aDecimal, BOOL &aValid ) const
{
	double result= CtrlGetDouble( aResId, aDigits, aDecimal, aValid );
	return max( aMin, min( aMax, result ) );  // gelesenen Wert ins Intervall [<aMin>..<aMax>] einpassen
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetDouble ( const int aResId, const double aDouble, const UINT aDigits, const char aDecimal ) const
{
	char buf[ _MAXLENDOUBLE + 1 ]; //  + Nullterminierung
	Double2String( buf, aDouble, aDigits, TRUE, aDecimal );
	return CtrlSetText(aResId, buf);
}
//*****************************************************************************

float TBasicDialog::CtrlGetFloat ( const int aResId, const UINT aDigits, const char aDecimal, BOOL &aValid ) const
{
	double temp= CtrlGetDouble( aResId, aDigits, aDecimal, aValid );
	return Double2Float( temp, aDigits );
}
//-----------------------------------------------------------------------------

float TBasicDialog::CtrlGetFloat ( const int aResId, const float aMin, const float aMax, const UINT aDigits, const char aDecimal, BOOL &aValid ) const
{
	float result= CtrlGetFloat( aResId, aDigits, aDecimal, aValid );
	return max( aMin, min( aMax, result ) );  // gelesenen Wert ins Intervall [<aMin>..<aMax>] einpassen
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetFloat ( const int aResId, const float aFloat, const UINT aDigits, const char aDecimal ) const
{
	double temp= Float2Double( aFloat, aDigits );
	return CtrlSetDouble ( aResId, temp, aDigits, aDecimal );
}
//*****************************************************************************

int TBasicDialog::CtrlAddString( const int aResId, LPCSTR aText ) const 
{
	HWND ctrl= Ctrl(aResId);
	if ( !ctrl ) return CB_ERR;
	return ComboBox_AddString( ctrl, aText );
}
//-----------------------------------------------------------------------------
	
int TBasicDialog::CtrlAddString( const int aResId, const int aStrTbleId ) const 
{
	BOOL bValid;
	int result= -1;
	char *MsgString= LoadString( aStrTbleId, bValid ); // ggf. ""
	if ( bValid ) result= CtrlAddString(aResId, MsgString);
	_FREELIST(MsgString);
	return result;
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlDelString( const int aResId, LPCSTR aText ) const 
{
	HWND ctrl= Ctrl(aResId);
	if ( !ctrl ) return CB_ERR;
	return ComboBox_DeleteString( ctrl, aText );
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlDelString( const int aResId, const int aStrTbleId ) const 
{
	BOOL bValid;
	int result= -1;
	char *MsgString= LoadString( aStrTbleId, bValid ); // ggf. ""
	if ( bValid ) result= CtrlDelString(aResId, MsgString);
	_FREELIST(MsgString);
	return result;
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlGetCount( const int aResId ) const 
{
	HWND ctrl= Ctrl(aResId);
	if ( !ctrl ) return CB_ERR;
	return ComboBox_GetCount( ctrl );
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlFindString( const int aResId, LPCSTR aText, const int aStartIdx ) const 
{
	HWND ctrl= Ctrl(aResId);
	if ( !ctrl ) return CB_ERR;
	return ComboBox_FindString( ctrl, aText, aStartIdx );
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlFindString( const int aResId, const int aStrTbleId, const int aStartIdx ) const 
{
	BOOL bValid;
	int result= CB_ERR;
	char *MsgString= LoadString( aStrTbleId, bValid ); // ggf. ""
	if ( bValid ) result= CtrlFindString( aResId, MsgString, aStartIdx );
	_FREELIST(MsgString);
	return result;
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlGetSel( const int aResId ) const 
{
	HWND ctrl= Ctrl(aResId);
	if ( !ctrl ) return CB_ERR;
	return ComboBox_GetCurSel( ctrl );
}
//-----------------------------------------------------------------------------

char *TBasicDialog::CtrlGetSelString( const int aResId ) const 
{
	HWND ctrl= Ctrl(aResId);
	if ( !ctrl ) {
		char *result= new char[ 0+1 ]; // nur Nullterminierung
		strcpy(result, "");
		return result;
	}
	int actLen= 50;
	while (1) {		
		char *MsgString= new char[actLen+1];
		int len= ComboBox_GetText( ctrl, MsgString, actLen );
		if ( len>=actLen && actLen<512 ) { // in jedem Durchlauf 50 Zeichen mehr Platz gewhren
			_FREELIST(MsgString);
			actLen+= 50; 
			continue;
		}
		if ( len!=CB_ERR ) { // Eintrag existiert
			char *result= new char[ strlen(MsgString)+1 ]; // + Nullterminierung
			strcpy(result, MsgString);
			return result;
		} else { // Eintrag ist nicht vorhanden
			char *result= new char[ 0+1 ]; // nur Nullterminierung
			strcpy(result, "");
			return result;
		}
	}
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlSetSel( const int aResId, const int aIdx ) const 
{
	HWND ctrl= Ctrl(aResId);
	if ( !ctrl ) return CB_ERR;
	return ComboBox_SetCurSel( ctrl, aIdx );
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlSetSelString( const int aResId, LPCSTR aString, const int aStartIdx ) const 
{
	HWND ctrl= Ctrl(aResId);
	if ( !ctrl ) return CB_ERR;
	return ComboBox_SelectString( ctrl, aStartIdx, aString );	
}
//-----------------------------------------------------------------------------

int TBasicDialog::CtrlSetSelString( const int aResId, const int aStrTbleId, const int aStartIdx ) const 
{
	BOOL bValid;
	int result= CB_ERR;
	char *MsgString= LoadString( aStrTbleId, bValid ); // ggf. ""
	if ( bValid ) result= CtrlSetSelString( aResId, MsgString, aStartIdx );
	else result= CtrlGetSel(aResId);
	_FREELIST(MsgString);
	return result;	
}
//*****************************************************************************

HWND TBasicDialog::Ctrl ( const int aResId ) const
{
	return GetDlgItem( GetHandle(), aResId );
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlHasFocus ( const int aResId ) const
{
	return ( GetFocus() == Ctrl(aResId) );
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetFocus ( const int aResId ) const
{
	BOOL result= ( SetFocus( Ctrl(aResId) ) != 0 );
	SendMessage( Ctrl(aResId), EM_SETSEL, 0, -1);
	return result;
	// SendMessage( Ctrl(aCtrl), EM_SETTABSTOPS, 0, 0); //TabStop-Reihenfolge aktualisieren
	// SendMessage( Ctrl(aCtrl), WM_ACTIVATE, 0, 0); //SelectAll
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlIsEnabled ( const int aResId ) const
{
	return IsWindowEnabled( Ctrl(aResId) );
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetEnabled ( const int aResId, const BOOL aEnable ) const
{
	return EnableWindow( Ctrl(aResId), aEnable );
}
//*****************************************************************************

BOOL TBasicDialog::CtrlIsChecked ( const int aResId ) const
{
	return ( CtrlGetState( aResId ) == BST_CHECKED );
}
//-----------------------------------------------------------------------------

BOOL TBasicDialog::CtrlSetChecked ( const int aResId, const BOOL aChecked ) const
{
	if ( aChecked )
		CtrlSetState( aResId, BST_CHECKED );
	else
		CtrlSetState( aResId, BST_UNCHECKED );
	return CtrlIsChecked( aResId );
}
//-----------------------------------------------------------------------------

UINT TBasicDialog::CtrlGetState ( const int aResId ) const
{
	return IsDlgButtonChecked( GetHandle(), aResId );
}
//-----------------------------------------------------------------------------

ECheckState TBasicDialog::CtrlGetCheckState ( const int aResId ) const 
{
	switch ( CtrlGetState(aResId) ) {
		case BST_CHECKED:
			return csChecked;

		case BST_UNCHECKED:
			return csUnchecked;

		case BST_INDETERMINATE:
			return csIndeterminate; 

		default:
			return csUnknown;
	}
};
//-----------------------------------------------------------------------------

UINT TBasicDialog::CtrlSetState ( const int aResId, const UINT aCheckState ) const
{
	CheckDlgButton( GetHandle(), aResId, aCheckState );
	return CtrlGetState( aResId );
}
//-----------------------------------------------------------------------------

ECheckState TBasicDialog::CtrlSetCheckState ( const int aResId, const ECheckState aCheckState ) const 
{
	switch ( aCheckState ) {
		case csChecked:
			CtrlSetState(aResId, BST_CHECKED);
			break;

		case csUnchecked:
			CtrlSetState(aResId, BST_UNCHECKED);
			break;

		case csIndeterminate:
			CtrlSetState(aResId, BST_INDETERMINATE);
			break; 
	}
	return CtrlGetCheckState(aResId);
}

//#############################################################################
// TModalDlg
//#############################################################################

TModalDlg::TModalDlg(const char *aResName, HINSTANCE aInstance) 
					: TBasicDialog(aResName, aInstance), 
					  m_Handler(0)
{
	m_Handler= (FARPROC)TBasicWindow::EventHandler;
}
//-----------------------------------------------------------------------------

TModalDlg::~TModalDlg ( void )
{
	UnregisterWindow(); // sptestens jetzt haben unser Handle verloren
	if ( m_Handler )
	{
		m_Handler= 0;
	}
};
//-----------------------------------------------------------------------------

int TModalDlg::ExecuteDialog ( HWND aParentWindow )
{
	TBasicWindow::s_WaitingWindow= this; // wartet auf sein Fensterhandle; wird nach der ersten Botschaft automatisch wieder 0 gesetzt
	int result= DialogBoxParam( GetInstance(), GetResName(), aParentWindow, (DLGPROC)m_Handler, 0l ); // pausiert, bis das Dialogfenster geschlossen wird; Rckgabewert ist der Schlieungsgrund
	UnregisterWindow(); // wir haben unser Handle soeben verloren
	return result;
};
//-----------------------------------------------------------------------------

void TModalDlg::Dlg_OnCommand ( HWND, int aId, HWND, UINT )
{
	if ( (aId == IDCLOSE) || (aId == IDCANCEL) || (aId == IDABORT) || ( (aId == IDOK) && (CanClose()) ) )
	{ // Dialogfenster schlieen
		LeaveDialog();
		EndDialog( GetHandle(), aId ); // schliet das Dialogfenster und gibt <id> als Schlieungsgrund an DialogBoxParam zurck
	}
};

//#############################################################################
// TModelessDlg
//#############################################################################

//-----------------------------------------------------------------------------
TModelessDlg::TModelessDlg(const char *aResName, TModelessDlg **aReference, HINSTANCE aInstance)
						: TBasicDialog(aResName, aInstance),
                          m_Handler(0),
						  m_Reference(0)
{
	m_Handler= (DLGPROC)(FARPROC)TBasicWindow::EventHandler;
	m_Reference= aReference;
}
//-----------------------------------------------------------------------------

TModelessDlg::~TModelessDlg ( void )
{
	if ( m_Window )
	{
		DestroyWindow( m_Window );
		UnregisterWindow(); // sptestens jetzt haben unser Handle verloren
		m_Window= 0;
	}
	if ( m_Handler )
	{
		m_Handler= 0;
	}
	*m_Reference= 0;
};
//-----------------------------------------------------------------------------

BOOL TModelessDlg::Initialize ( HINSTANCE aInstance, HWND aParentWnd )
{
	TBasicWindow::s_WaitingWindow= this; // wartet auf sein Fensterhandle; wird nach der ersten Botschaft automatisch wieder 0 gesetzt
	m_Window= CreateDialog( aInstance, GetResName(), aParentWnd, (DLGPROC)TBasicWindow::EventHandler );
	if ( m_Window == 0 )
	{
		MessageBox( _NOWINDOW, _ERRTITLE, MBSTOP);
		return FALSE;
	}
	ShowWindow( m_Window, SW_SHOW );
	return TRUE;
};
//-----------------------------------------------------------------------------

void TModelessDlg::Dlg_OnCommand ( HWND, int aId, HWND, UINT )
{
	if ( (aId == IDCANCEL) || (aId == IDABORT) || ( (aId == IDOK) && (CanClose()) ) )
	{ // Dialogfenster schlieen
		LeaveDialog();
		delete this; // schliet das Fenster und hebt seine Registrierung auf; es werden keine Botschaften mehr verschickt
	}
};

//#############################################################################
// TBasicMDIWindow
//#############################################################################

TBasicMDIWindow::TBasicMDIWindow ( HINSTANCE aInstance ) : TBasicWindow(aInstance)
{
	strcpy(m_Title, "");
	m_Showing= false;
}
//-----------------------------------------------------------------------------

TBasicMDIWindow::~TBasicMDIWindow ( void )
{
	if ( GetHandle() )
		SetWindowLong(GetHandle(), 1, 0); // UnregisterWindow bei ~TBasicWindow (mit UnregisterWindow ist GetHandle()==0)
}

//*****************************************************************************
// Anzeigen & Co
//*****************************************************************************

LRESULT TBasicMDIWindow::Show ( HWND aParent, const int aX, const int aY, const int aWidth, const int aHeight)
{
	if ( m_Showing ) return BringWindowToTop(GetHandle()); // Fensterklasse und Fenster nicht erneut regiestrieren!!!

	m_Showing= true;
	//Fensterklasse registrieren
	WNDCLASS wndClass, wndAsk;
	wndClass.style= CS_HREDRAW | CS_VREDRAW;
	wndClass.cbClsExtra= 0;
	wndClass.cbWndExtra= sizeof(HINSTANCE)+sizeof(LPARAM); // zu reservierender Speicher fr mdiCreate.hOwner und mdiCreate.lParam
	wndClass.hInstance= GetInstance();
	wndClass.hIcon= GetIcon();
	wndClass.hCursor= GetCursor();
	wndClass.hbrBackground= GetBkColor();
	wndClass.lpszMenuName= "";
	wndClass.lpszClassName= (LPSTR)ClassName();
	wndClass.lpfnWndProc= TBasicMDIWindow::EventHandler;
	if ( !GetClassInfo(wndClass.hInstance, wndClass.lpszClassName, &wndAsk) )
		RegisterClass(&wndClass);

	// Fenster anzeigen
	MDICREATESTRUCT mdiCreate;
	mdiCreate.szClass= (LPSTR)ClassName();
	mdiCreate.szTitle= (LPSTR)m_Title;
	mdiCreate.hOwner= GetInstance();
	mdiCreate.x= ( (aX != -1) ? aX : CW_USEDEFAULT );
	mdiCreate.y= ( (aY != -1) ? aY : CW_USEDEFAULT );
	mdiCreate.cx= ( (aWidth != -1) ? aWidth : CW_USEDEFAULT );
	mdiCreate.cy= ( (aHeight != -1) ? aHeight : CW_USEDEFAULT );
	mdiCreate.style= 0;
	mdiCreate.lParam= (LPARAM)this;
	s_WaitingWindow= this; // die eigentliche Registrierung erfolgt ber mdiCreate.lParam und WM_CREATE, davor werden jedoch auch WM_GETMINMAXINFO, WM_NCCREATE und WM_NCCALCSIZE versendet, die nicht ohne s_WaitingWindow zugeordnet werden knnten
	return SendMessage(aParent, WM_MDICREATE, NULL, (LPARAM)&mdiCreate);
}
//-----------------------------------------------------------------------------

BOOL TBasicMDIWindow::SetTitle ( void )
{
	if ( GetHandle() )
		SetWindowText(GetHandle(), (LPSTR)m_Title);
	return TRUE;
}

//*****************************************************************************
// EREIGNISSE
//*****************************************************************************

void TBasicMDIWindow::OnPaint ( void )
{
	PAINTSTRUCT ps;
	HDC hdc= BeginPaint(GetHandle(), &ps);
	DoPaint(hdc, &ps);
	EndPaint(GetHandle(), &ps);
}
//-----------------------------------------------------------------------------

void TBasicMDIWindow::OnHotKey ( LPARAM lParam ) 
{
	for (UINT i= 0; i < m_HotKeys.size(); i++ )
	{
		THotKey *hk= m_HotKeys[i];
		if ( (hk) && (hk->IsEqual(lParam)) )
		{
			OnCommand( hk->m_Cmd, 0 );
			return;
		}
	}
}
//-----------------------------------------------------------------------------

LRESULT TBasicMDIWindow::OnEvent ( HWND, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch (message)
	{
		case WM_SYSCOLORCHANGE:
			//! Ctl3dColorChange();
			return 0l;

		case WM_KEYDOWN:
			OnKeyDown(wParam, lParam);
			break; 

		case WM_LBUTTONDOWN:
			OnLButtonDown(wParam, lParam);
			break;

		case WM_LBUTTONUP:
			OnLButtonUp(wParam, lParam);
			break;

		case WM_RBUTTONDOWN:
			OnRButtonDown(wParam, lParam);
			break;

		case WM_MOUSEMOVE:
			OnMouseMove(wParam, lParam);
			break;

		case WM_COMMAND:
			OnCommand(wParam, lParam);
			break;

		case WM_INITMENUPOPUP:
			OnPopupMenuInit(wParam, lParam);
			break;

		case WM_MENUSELECT:
			OnMenuSelect(wParam, lParam);
			break;

		case WM_PAINT:
			OnPaint();
			break;

		case WM_SETFOCUS:
			OnFocus();
			break;

		case WM_SIZE:
			OnSize(wParam, lParam);
			break;

		case WM_TIMER:
			OnTimer(wParam, lParam);
			break;

		case WM_RENDERFORMAT:
			OnRenderFormat(wParam, lParam);
			break;

		case WM_RENDERALLFORMATS:
			OnRenderAllFormats(wParam, lParam);
			break;

		case WM_MDIDESTROY:
			CanClose(); // der Rckgabewert wird nicht ausgewertet?!
			break;

		case WM_DESTROY:
			// wenn noch nicht alle Daten gespeichert sind: ABBRUCH (sonst folgt WM_NCDESTROY)
			if ( !CanClose() )
				return 0l;
			break;
	} // end switch
	return FALSE; // d.h. DefMDIChildProc trotzdem aufrufen
}

//*****************************************************************************
//STATIC
//*****************************************************************************

LRESULT CALLBACK TBasicMDIWindow::EventHandler ( HWND hWindow, UINT message, WPARAM wParam, LPARAM lParam )
{
	BOOL aValid= TRUE;
	switch ( message )
	{
		case WM_CREATE:
			if ( lParam )
			{ // unser lParam ist ein Zeiger auf eine CREATESTRUCT ...
				CREATESTRUCT *a= (CREATESTRUCT *)lParam;
				if ( a )
				{ // ... im Member lpCreateParams liegt dort ein Zeiger eine MDICREATESTRUCT
					MDICREATESTRUCT *b= (MDICREATESTRUCT *)a->lpCreateParams;
					if ( b )
					{ // ... darin liegt im lParam unser Fensterobjekt :-)
						TBasicMDIWindow *self= (TBasicMDIWindow *)b->lParam;
						if ( (self) && (self->CanOpen()) )
						{
							SetWindowLong(hWindow, 1, (DWORD)self);
							self->RegisterWindow(hWindow);
							self->OnCreate();
							break;
						}
						else if ( self )
							self->OnNotCreated(); // CanOpen==FALSE
					}
				}
			}
			return -1;

		// Tastaturereignisse gehen NUR an das aktive Fenster
		case WM_HOTKEY:
		case WM_KEYDOWN:
		case WM_KEYUP:
		{
			// Objekt aus der Liste der registrierten Fenster ermitteln; damit endlich auch Zugriff auf Member
			TBasicMDIWindow *self= (TBasicMDIWindow *)Handle2Object(GetFocus()); // nur wenn Fenster aktiv, sonst fangen wir die Tastenkombinationen auch wenn unsere Anwendung inaktiv ist!!!
			if ( self ) switch ( message ) {
				case WM_HOTKEY:
					self->OnHotKey(lParam);
					break;

				case WM_KEYDOWN:
					self->OnKeyDown(wParam, lParam);
					break;

				case WM_KEYUP:
					self->OnKeyUp(wParam, lParam);
					break;
			}
			break;
		}

		case WM_NCDESTROY:
		{
			TBasicMDIWindow *self= (TBasicMDIWindow *)GetWindowLong(hWindow, 1);
			_FREEOBJ(self);
			break;
		}

		default:
			BOOL aValid= FALSE;
			LRESULT result;
			if ( (message != WM_CREATE) && (message != WM_NCDESTROY) )
				result= TBasicWindow::EventHandlerEx( hWindow, message, wParam, lParam, aValid );
			if ( (aValid) && (result != FALSE) )
				return result; // Botschaft wurde an OnEvent des entsprechenden Fensters weitergeleitet
	}
	return DefMDIChildProc(hWindow, message, wParam, lParam);
}

//#############################################################################
// LastLine
//#############################################################################
